@@ -24,8 +24,8 @@
 #include "model/wb_model_diagram_form.h"
 
 #include "grts/structs.workbench.model.h"
-#include "string_utilities.h"
-#include "file_functions.h"
+#include "base/string_utilities.h"
+#include "base/file_functions.h"
 
 using namespace wb;
 using namespace grt;
@@ -64,11 +64,6 @@ void WBComponentBasic::load_app_options(bool update)
   {
     grt::GRT *grt= _wb->get_grt_manager()->get_grt();
 
-    app_ToolbarRef tools_toolbar(app_ToolbarRef::cast_from(
-      grt->unserialize(make_path(_wb->get_datadir(), "data/tools_toolbar_basic.xml"))));
-
-    _toolbars[tools_toolbar->name()]= tools_toolbar;
-
     app_ToolbarRef options_toolbar;
 
     options_toolbar= app_ToolbarRef::cast_from(
@@ -85,7 +80,18 @@ void WBComponentBasic::load_app_options(bool update)
 }
 
 
+app_ToolbarRef WBComponentBasic::get_tools_toolbar()
+{
+  return app_ToolbarRef::cast_from(_wb->get_grt()->unserialize(make_path(_wb->get_datadir(),
+                                                                         "data/tools_toolbar_basic.xml")));
+}
 
+app_ToolbarRef WBComponentBasic::get_tool_options(const std::string &tool)
+{
+  if (_toolbars.find("options/"+tool) != _toolbars.end())
+    return _toolbars["options/"+tool];
+  return app_ToolbarRef();
+}
 
 void WBComponentBasic::setup_canvas_tool(ModelDiagramForm *view, const std::string &tool)
 {
@@ -144,24 +150,6 @@ void WBComponentBasic::setup_canvas_tool(ModelDiagramForm *view, const std::stri
 }
 
 
-grt::ListRef<app_ToolbarItem> WBComponentBasic::get_toolbar_items(const std::string &toolbar)
-{
-  if (toolbar == WB_TOOLBAR_OPTIONS)
-  {
-    ModelDiagramForm *form= dynamic_cast<ModelDiagramForm*>(_wb->get_active_form());
-
-    if (form)
-    {
-      if (_toolbars.find(toolbar+"/"+form->get_tool()) != _toolbars.end())
-        return _toolbars[toolbar+"/"+form->get_tool()]->items();
-    }
-    return grt::ListRef<app_ToolbarItem>();
-  }
-  else
-    return _toolbars[toolbar]->items();
-}
-
-
 std::vector<std::string> WBComponentBasic::get_command_dropdown_items(const std::string &option)
 {
   std::vector<std::string> items;
