@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,7 @@
 
 #include "wb_context_model.h"
 #include "workbench/wb_context_ui.h"
+#include "workbench/wb_command_ui.h"
 #include "wb_component.h"
 #include "wb_component_basic.h"
 #include "wb_component_physical.h"
@@ -32,21 +33,30 @@
 #include "model/wb_overview_physical.h"
 #include "wbcanvas/model_diagram_impl.h"
 #include "wbcanvas/workbench_physical_model_impl.h"
+#include "workbench/wb_model_file.h"
 
 #include "grt/clipboard.h"
 #include "grt/common.h"
-#include "string_utilities.h"
+#include "base/file_utilities.h"
+#include "base/file_functions.h"
+#include "base/string_utilities.h"
 #include "mforms/utilities.h"
 
 using namespace base;
 using namespace bec;
 using namespace wb;
 
+static std::map<std::string, std::string> auto_save_files;
+
 
 WBContextModel::WBContextModel(WBContextUI *wbui)
-: _wbui(wbui)
+: _wbui(wbui), _auto_save_point(0), _last_auto_save_time(0), _file(0)
+
 {  
   _overview= new PhysicalOverviewBE(_wbui->get_wb());
+  
+  _wbui->get_wb()->get_grt_manager()->get_clipboard()->signal_changed().connect(sigc::mem_fun(this, &WBContextModel::selection_changed));
+  _overview->signal_selection_changed().connect(sigc::mem_fun(this, &WBContextModel::selection_changed)); // make edit menu captions to update
 
   CommandUI *cmdui = wbui->get_command_ui();
   sigc::slot<bool> validate = sigc::mem_fun(this, &WBContextModel::has_selected_schema);
@@ -68,15 +78,136 @@ WBContextModel::WBContextModel(WBContextUI *wbui)
   
   // used to update the currentDiagram field
   _wbui->signal_main_form_change().connect(sigc::mem_fun(this, &WBContextModel::update_current_diagram));
+
+  // setup auto-save for model
+  int interval = _wbui->get_wb()->get_root()->options()->options().get_int("workbench:AutoSaveModelInterval", 60);
+  if (interval > 0)
+    _wbui->get_wb()->get_grt_manager()->run_every(sigc::mem_fun(this, &WBContextModel::auto_save_document), interval);
+  _auto_save_interval = interval;
+  
+  _wbui->get_wb()->get_root()->options()->signal_dict_changed().connect(sigc::mem_fun(this, &WBContextModel::option_changed));
 }
 
 
 WBContextModel::~WBContextModel()
 {
+  CommandUI *cmdui = _wbui->get_command_ui();
+  cmdui->remove_builtin_command("addModelDiagram");
+  cmdui->remove_builtin_command("addModelSchema");
+  cmdui->remove_builtin_command("addModelTable");
+  cmdui->remove_builtin_command("addModelView");
+  cmdui->remove_builtin_command("addModelRoutine");
+
+  _file= 0;
   delete _overview;
 }
 
 
+void WBContextModel::option_changed(grt::internal::OwnedDict*dict, bool, const std::string&key)
+{
+  if (key == "workbench:AutoSaveModelInterval" && dict == _wbui->get_wb()->get_wb_options().valueptr())
+  {
+    auto_save_document();
+  }
+}
+
+
+bool WBContextModel::auto_save_document()
+{
+  WBContext *wb= _wbui->get_wb();
+  int interval= wb->get_root()->options()->options().get_int("workbench:AutoSaveModelInterval", 60);
+  if (interval <= 0)
+    return false;
+  
+  workbench_DocumentRef doc(wb->get_document());
+    
+  mdc::Timestamp now= mdc::get_time();
+  if (now - _last_auto_save_time > interval
+      && _file
+      && doc.is_valid() 
+      && !wb->get_grt_manager()->get_dispatcher()->get_busy()
+      && wb->get_grt()->get_undo_manager()->get_latest_closed_undo_action() != _auto_save_point
+      && !wb->get_filename().empty())
+  {
+    _auto_save_point = wb->get_grt()->get_undo_manager()->get_latest_closed_undo_action();
+    _last_auto_save_time = now;
+    
+    try
+    {
+      // save the document in the same directory containing the expanded mwb file
+      _file->store_document_autosave(wb->get_grt(), doc);
+    } 
+    catch (std::exception &exc)
+    {
+      wb->show_exception(_("Could not store document data to autosave file."), exc);
+    }
+  }
+  
+  if (interval != _auto_save_interval)
+  {
+    // schedule new interval
+    wb->get_grt_manager()->run_every(sigc::mem_fun(this, &WBContextModel::auto_save_document), interval);
+    return false;
+  }
+  
+  return true;
+}
+
+
+void WBContextModel::detect_auto_save_files(const std::string &autosave_dir)
+{
+  std::map<std::string, std::string> files;
+
+  // look for .mwbd folders with autosave files
+  std::list<std::string> autosaves;
+  
+  try
+  {
+    autosaves = base::scan_for_files_matching(bec::make_path(autosave_dir, "*.mwbd*"));
+  }
+  catch (const std::runtime_error &)
+  {
+    return;
+  }
+
+  for (std::list<std::string>::const_iterator d = autosaves.begin(); d != autosaves.end(); ++d)
+  { 
+    if (base::LockFile::check(bec::make_path(*d, ModelFile::lock_filename.c_str())) != base::LockFile::NotLocked)
+      continue;
+    
+    if (g_file_test(bec::make_path(*d, MAIN_DOCUMENT_AUTOSAVE_NAME).c_str(), G_FILE_TEST_EXISTS))
+    {
+      gchar *orig_path;
+      gsize length;
+      if (g_file_get_contents(bec::make_path(*d, "real_path").c_str(),
+                              &orig_path, &length, NULL))
+      {
+        files[std::string(orig_path, length)] = *d;
+        g_free(orig_path);
+      }
+      else
+      {
+        std::string fname = g_basename(d->c_str());
+        fname = fname.substr(0, fname.rfind('.')).append(".mwb");
+        // if no real_path file in the autosave, this could be an autosave from an older version
+        files[fname] = *d;
+      }
+    }
+    else
+    {
+      g_message("Found model auto-save %s, but it is empty. Deleting it...", d->c_str());
+      base_rmdir_recursively(d->c_str());
+    }
+  }
+  ::auto_save_files = files;
+}
+
+std::map<std::string, std::string> WBContextModel::auto_save_files()
+{
+  return ::auto_save_files;
+}
+
+
 void WBContextModel::unrealize()
 {
   _page_settings_conn.disconnect();
@@ -92,49 +223,6 @@ void WBContextModel::unrealize()
 }
 
 
-grt::ListRef<app_ToolbarItem> WBContextModel::get_toolbar_items(const std::string &toolbar)
-{
-  WBContext *wb= _wbui->get_wb();
-  grt::ListRef<app_ToolbarItem> items;
-  
-  if (toolbar == WB_TOOLBAR_MAIN)
-  {
-    if (!_toolbar.is_valid())
-    {
-      _toolbar= app_ToolbarRef::cast_from(wb->get_grt()->unserialize(make_path(wb->get_datadir(), 
-                                                                               "data/model_toolbar.xml")));
-    }
-    
-    return _toolbar->items();
-  }
-  else if (toolbar.compare(0, sizeof(WB_TOOLBAR_TOOLS)-1, WB_TOOLBAR_TOOLS) == 0)
-  {
-    if (!_tools_toolbar.is_valid())
-      _tools_toolbar= app_ToolbarRef::cast_from(wb->get_grt()->unserialize(make_path(wb->get_datadir(),
-                                                                                     "data/tools_toolbar.xml")));
-   
-    items= grt::ListRef<app_ToolbarItem>(wb->get_grt());
-    grt::append_contents(items, _tools_toolbar->items());
-  }
-  else if (toolbar == WB_TOOLBAR_OPTIONS)
-  {
-    items= grt::ListRef<app_ToolbarItem>(wb->get_grt());
-  }
-  
-  // view specific contextual toolbars
-  if (_wbui->get_active_context() == WB_CONTEXT_MODEL)
-  {
-    grt::ListRef<app_ToolbarItem> model_items;
-      
-    // append model specific items
-    wb->foreach_component(sigc::compose(sigc::bind<0>(sigc::ptr_fun(grt::append_contents), items),
-                                        sigc::bind(sigc::mem_fun(&WBComponent::get_toolbar_items), toolbar)));
-  }
-  
-  return items;
-}
-
-
 model_DiagramRef WBContextModel::get_active_model_diagram(bool main_form)
 {
   bec::UIForm *form= main_form ? get_wbui()->get_active_main_form() : get_wbui()->get_active_form();
@@ -159,8 +247,9 @@ model_ModelRef WBContextModel::get_active_model(bool main_form)
 
 
 
-void WBContextModel::model_created(workbench_DocumentRef doc)
+void WBContextModel::model_created(ModelFile *file, workbench_DocumentRef doc)
 {
+  _file= file;
   _doc= doc;
 
   _wbui->get_wb()->get_component<WBComponentLogical>()->setup_logical_model(doc.get_grt(), _doc);
@@ -176,8 +265,9 @@ void WBContextModel::model_created(workbench_DocumentRef doc)
 }
 
 
-void WBContextModel::model_loaded(workbench_DocumentRef doc)
+void WBContextModel::model_loaded(ModelFile *file, workbench_DocumentRef doc)
 {
+  _file= file;
   _doc= doc;
   
   _wbui->get_wb()->foreach_component(sigc::mem_fun(&WBComponent::reset_document));
@@ -187,6 +277,18 @@ void WBContextModel::model_loaded(workbench_DocumentRef doc)
   _doc->physicalModels().get(0)->get_data()->set_delegate(this);
     
   _wbui->get_wb()->request_refresh(RefreshNewModel, "", 0);
+  
+  std::string temp_dir = _file->get_tempdir_path();
+  for (std::map<std::string, std::string>::iterator iter= ::auto_save_files.begin();
+       iter != ::auto_save_files.end(); ++iter)
+  {
+    if (iter->second == temp_dir)
+    {
+      ::auto_save_files.erase(iter);
+      _wbui->refresh_home_models();
+      break;
+    }
+  }
 }
 
 
@@ -277,21 +379,31 @@ mdc::CanvasView *WBContextModel::create_diagram(const model_DiagramRef &view)
                                                                                   (sigc::mem_fun(this, &WBContextModel::create_diagram_main), view));
 }
 
+//--------------------------------------------------------------------------------------------------
 
+/**
+ * Notification trigger for canvas view destructions. Can be called via two different paths.
+ * If called from the front end (because the UI caused closing the editor) then the diagram form
+ * is already unregistered and we don't need to call the UI again (it was the trigger after all).
+ */
 void WBContextModel::free_canvas_view(mdc::CanvasView *view)
 {
-  // this function is expected to be called from the main thread
-  notify_diagram_destroyed(get_diagram_form(view));
-  
-  // forward to frontend
-  if (_wbui->get_wb()->get_grt_manager()->in_main_thread())
-    _wbui->get_wb()->destroy_view(view);
-  else
-    _wbui->get_wb()->execute_in_main_thread<void>("destroy view", 
-                   sigc::bind<mdc::CanvasView*>(_wbui->get_wb()->destroy_view, view));
+  ModelDiagramForm* diagram = get_diagram_form(view);
+  if (diagram != NULL)
+  {
+    // This function is expected to be called from the main thread.
+    notify_diagram_destroyed(diagram);
+
+    // Notify front end so it can close its editor for this view.
+    if (_wbui->get_wb()->get_grt_manager()->in_main_thread())
+      _wbui->get_wb()->destroy_view(view);
+    else
+      _wbui->get_wb()->execute_in_main_thread<void>("destroy view", 
+      sigc::bind<mdc::CanvasView*>(_wbui->get_wb()->destroy_view, view));
+  }
 }
 
-
+//--------------------------------------------------------------------------------------------------
 
 mdc::CanvasView *WBContextModel::create_diagram_main(const model_DiagramRef &diagram_reference)
 {
@@ -325,13 +437,17 @@ mdc::CanvasView *WBContextModel::create_diagram_main(const model_DiagramRef &dia
   register_diagram_form(diagram);
   
   // Forward creation to front end.
-  mdc::CanvasView *view= wb->create_diagram(diagram_reference.id(), diagram_reference->name());
+  mdc::CanvasView *view= wb->create_diagram(diagram_reference);
   if (view)
   {
     diagram->attach_canvas_view(view);
     
     notify_diagram_created(diagram);
     
+    // use this signal instead of selection_change from canvas so that when the callback is called
+    // the grt diagram object already reflects the selection changes
+    diagram_reference->get_data()->signal_selection_changed().connect(sigc::hide(sigc::mem_fun(this, &WBContextModel::selection_changed)));
+
     wb->request_refresh(RefreshNewDiagram, diagram_reference.id(), (NativeHandle)view->get_user_data());
   }
   else
@@ -380,21 +496,23 @@ ModelDiagramForm *WBContextModel::get_diagram_form(mdc::CanvasView *view)
 
 void WBContextModel::notify_diagram_created(ModelDiagramForm *view)
 {
-  view->get_view()->get_selection()->signal_changed().connect(sigc::bind(sigc::mem_fun(_wbui, &WBContextUI::view_selection_changed), view));
-  
-  view->get_model_diagram()->signal_changed().
-  connect(sigc::bind(sigc::mem_fun(this, &WBContextModel::diagram_object_changed), view));
-  
+  ChangeSignal::iterator signal_id = view->get_model_diagram()->signal_changed().
+    connect(sigc::bind(sigc::mem_fun(this, &WBContextModel::diagram_object_changed), view));
+  view->change_signal_id(signal_id);
+    
   _wbui->get_physical_overview()->send_refresh_diagram(model_DiagramRef());
 }
 
 
 void WBContextModel::notify_diagram_destroyed(ModelDiagramForm *diagram)
 {
-  std::string id= diagram->get_model_diagram().id();
-  delete diagram;
-  _model_forms.erase(id);
-  
+  if (diagram != NULL)
+  {
+    std::string id= diagram->get_model_diagram().id();
+    delete diagram;
+    _model_forms.erase(id);
+  }
+
   _wbui->get_physical_overview()->send_refresh_diagram(model_DiagramRef());
 }
 
@@ -406,7 +524,9 @@ void WBContextModel::update_current_diagram(bec::UIForm *form)
   {
     model_DiagramRef diagram(dform->get_model_diagram());
     if (diagram.is_valid() && diagram->owner().is_valid())
-      diagram->owner()->currentDiagram(diagram);
+      diagram->owner()->currentDiagram(diagram);    
+    
+    _wbui->get_command_ui()->signal_validate_edit_menu_items().emit();
   }
 }
 
@@ -415,8 +535,11 @@ void WBContextModel::diagram_object_changed(const std::string &member, const grt
 {
   if (member == "name")
   {
-    _wbui->get_wb()->request_refresh(RefreshViewName, view->get_model_diagram()->name(), (NativeHandle)view->get_view()->get_user_data());
-    _wbui->get_physical_overview()->send_refresh_diagram(view->get_model_diagram());
+    if (view->get_model_diagram().is_valid())
+    {
+      _wbui->get_wb()->request_refresh(RefreshViewName, view->get_model_diagram()->name(), (NativeHandle)view->get_view()->get_user_data());
+      _wbui->get_physical_overview()->send_refresh_diagram(view->get_model_diagram());
+    }
   }
   else if (member == "zoom")
   {
@@ -496,13 +619,21 @@ void WBContextModel::update_plugin_arguments_pool(ArgumentPool &args)
     args.add_entries_for_object("activeCatalog", pmodel->catalog(), "db.Catalog");
   }
   
+  ModelDiagramForm *view;
   model_DiagramRef diagram(get_active_model_diagram(false));
+  if (!diagram.is_valid()) // in case an editor in a diagram tab is active
+  {
+    diagram = get_active_model_diagram(true);
+    view = dynamic_cast<ModelDiagramForm*>(_wbui->get_active_main_form());
+  }
+  else
+    view= dynamic_cast<ModelDiagramForm*>(_wbui->get_active_form());
+  
   if (diagram.is_valid())
   {
     args.add_entries_for_object("", diagram, "model.Diagram");
     args.add_entries_for_object("activeDiagram", diagram, "model.Diagram");
     
-    ModelDiagramForm *view= dynamic_cast<ModelDiagramForm*>(_wbui->get_active_form());
     if (view)
     {
       // diagram selection
@@ -514,7 +645,8 @@ void WBContextModel::update_plugin_arguments_pool(ArgumentPool &args)
       {
         model_ObjectRef object(selection[0]);
       
-        args.add_entries_for_object("", object, "model.Object");
+        //args.add_entries_for_object("", object, "model.Object");
+        args.add_entries_for_object("", object, "GrtObject");
         
         // check if object represented by this is wanted
         FOREACH_COMPONENT(_wbui->get_wb()->_components, iter)
@@ -540,7 +672,7 @@ void WBContextModel::update_plugin_arguments_pool(ArgumentPool &args)
     {
       GrtObjectRef object(selection[0]);
       
-      args.add_entries_for_object("", object, "db.DatabaseObject");
+      args.add_entries_for_object("", object, "GrtObject");
     }
   }
 }
@@ -724,3 +856,113 @@ int WBContextModel::add_object_plugins_to_popup_menu(const grt::ListRef<GrtObjec
   
   return count;
 }
+
+
+void WBContextModel::history_changed()
+{
+  std::string undo_description(_wbui->get_wb()->get_grt()->get_undo_manager()->undo_description());
+  std::string redo_description(_wbui->get_wb()->get_grt()->get_undo_manager()->redo_description());
+
+  std::list<bec::UIForm*> forms;
+  forms.push_back(_overview);
+  for (std::map<std::string, ModelDiagramForm*>::const_iterator iter= _model_forms.begin();
+       iter != _model_forms.end(); ++iter)
+    forms.push_back(iter->second);
+  
+  mforms::MenuItem *item;
+  
+  for (std::list<bec::UIForm*>::const_iterator iter = forms.begin(); iter != forms.end(); ++iter)
+  {
+    bec::UIForm *form = *iter;
+    mforms::MenuBar *menu = form->get_menubar();
+    if (menu)
+    {
+      item = menu->find_item("undo");
+      if (item)
+      {
+        if (!undo_description.empty())
+          item->set_title(strfmt(_("Undo %s"), undo_description.c_str()));
+        else
+          item->set_title(_("Undo"));
+      }
+      item = menu->find_item("redo");
+      if (item)
+      {
+        if (!redo_description.empty())
+          item->set_title(strfmt(_("Redo %s"), redo_description.c_str()));
+        else
+          item->set_title(_("Redo"));
+      }
+    }      
+  }
+}
+
+
+void WBContextModel::selection_changed()
+{
+  if (!_wbui->get_wb()->get_grt_manager()->in_main_thread())
+  {
+    _wbui->get_wb()->get_grt_manager()->run_when_idle(sigc::bind_return(sigc::mem_fun(this, &WBContextModel::selection_changed), false));
+    return;
+  }
+    
+  bec::Clipboard *clip = _wbui->get_wb()->get_clipboard();
+    
+  std::list<bec::UIForm*> forms;
+  forms.push_back(_overview);
+  for (std::map<std::string, ModelDiagramForm*>::const_iterator iter= _model_forms.begin();
+       iter != _model_forms.end(); ++iter)
+    forms.push_back(iter->second);
+  
+  mforms::MenuItem *item;
+
+  for (std::list<bec::UIForm*>::const_iterator iter = forms.begin(); iter != forms.end(); ++iter)
+  {
+    bec::UIForm *form = *iter;
+    mforms::MenuBar *menu = form->get_menubar();
+    if (menu)
+    {
+      std::string target(form->get_edit_target_name());
+      std::string content(clip->get_content_description());
+      
+      item = menu->find_item("copy");
+      if (item)
+      {
+        if (!target.empty())
+          item->set_title(strfmt(_("Copy %s"), target.c_str()));
+        else
+          item->set_title(_("Copy"));
+      }
+      
+      item = menu->find_item("cut");
+      if (item)
+      {
+        if (!target.empty())
+          item->set_title(strfmt(_("Cut %s"), target.c_str()));
+        else
+          item->set_title(_("Cut"));
+      }
+      
+      item = menu->find_item("delete");
+      if (item)
+      {
+        if (!target.empty())
+          item->set_title(strfmt(_("Delete %s"), target.c_str()));
+        else
+          item->set_title(_("Delete"));
+      }
+
+      item = menu->find_item("paste");
+      if (item)
+      {
+        if (!content.empty())
+          item->set_title(strfmt(_("Paste %s"), content.c_str()));
+        else
+          item->set_title(_("Paste"));
+      }
+    }
+  }
+  _wbui->get_command_ui()->signal_validate_edit_menu_items().emit();
+}
+
+
