@@ -24,7 +24,7 @@
 
 #include "wbcanvas/model_diagram_impl.h"
 #include "wbcanvas/model_figure_impl.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 // if you change these values, make sure to update the Diagram Options UIs
 // to allow different max values in the edit boxes (or query the max values)
