@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,7 @@
 #include "canvas_floater.h"
 
 #include "mdc_back_layer.h"
+#include "mforms/form.h"
 
 #include "model/wb_layer_tree.h"
 
@@ -41,7 +42,7 @@
 #include "workbench/wb_context_ui.h"
 
 #include "wb_physical_model_diagram_features.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #define DEFAULT_TOOL "basic/select"
 
@@ -71,7 +72,8 @@ static const double zoom_steps[]= {
 
 
 ModelDiagramForm::ModelDiagramForm(WBComponent *owner, const model_DiagramRef &view)
-: _view(0), _owner(owner), _model_diagram(view), _layer_tree(owner->get_wb()), _mini_view(0)
+: _view(0), _owner(owner), _model_diagram(view), _layer_tree(owner->get_wb()), _mini_view(0), _menu(0), 
+_toolbar(0), _tools_toolbar(0), _options_toolbar(0)
 {
   _drag_panning= false;
   _space_panning= false;
@@ -102,16 +104,330 @@ ModelDiagramForm::ModelDiagramForm(WBComponent *owner, const model_DiagramRef &v
   owner->get_wb()->get_clipboard()->signal_changed().connect(sigc::mem_fun(this, &ModelDiagramForm::clipboard_changed));
   
   _features= new PhysicalModelDiagramFeatures(this);
+  
+  _options_toolbar = new mforms::ToolBar(mforms::OptionsToolBar);
 }
 
 
 ModelDiagramForm::~ModelDiagramForm()
 {
+  delete _options_toolbar;
+  delete _toolbar;
+  delete _tools_toolbar;
+  delete _menu;
   sigc::connection(_signal_id).disconnect();
   delete _mini_view;
   delete _features;
 }
 
+mforms::ToolBar *ModelDiagramForm::get_toolbar()
+{
+  if (!_toolbar)
+  {
+    _toolbar = get_wb()->get_ui()->get_command_ui()->create_toolbar("data/model_diagram_toolbar.xml");
+  }
+  return _toolbar;
+}
+
+
+mforms::ToolBar *ModelDiagramForm::get_tools_toolbar()
+{
+  if (!_tools_toolbar)
+  {
+    _tools_toolbar = new mforms::ToolBar(mforms::ToolPickerToolBar);
+    app_ToolbarRef toolbar[3];
+    
+    toolbar[0]= app_ToolbarRef::cast_from(get_wb()->get_grt()->unserialize(make_path(get_wb()->get_datadir(),
+                                                                                     "data/tools_toolbar.xml")));
+    toolbar[1]= get_wb()->get_component_named("basic")->get_tools_toolbar();
+    toolbar[2]= get_wb()->get_component_named("physical")->get_tools_toolbar();
+    
+    
+    for (int t= 0; t < 3; t++)
+    {
+      for (size_t c = toolbar[t]->items().count(), i = 0; i < c; i++)
+      {
+        app_ToolbarItemRef titem(toolbar[t]->items()[i]);
+        mforms::ToolBarItem *item;
+        
+        if (titem->itemType() == "separator")
+          item = mforms::manage(new mforms::ToolBarItem(mforms::SeparatorItem));
+        else
+        {
+          item = mforms::manage(new mforms::ToolBarItem(mforms::ToggleItem));
+        
+          item->set_icon(IconManager::get_instance()->get_icon_path(*titem->icon()));
+          item->set_name((*titem->command()).substr(strlen("tool:")));
+          item->signal_activated().connect(sigc::bind(sigc::mem_fun(this, &ModelDiagramForm::set_tool), item->get_name()));
+        }
+        std::string shortcut;
+        for (std::vector<WBShortcut>::const_iterator iter= _shortcuts.begin(); iter != _shortcuts.end(); ++iter)
+        {
+          if (iter->command == *titem->command())
+          {
+            shortcut= iter->shortcut;
+            break;
+          }
+        }
+        if (shortcut.empty())
+          item->set_tooltip(titem->tooltip());
+        else
+          item->set_tooltip(strfmt("%s (Quick key - press %s)", titem->tooltip().c_str(), shortcut.c_str()));
+        
+        _tools_toolbar->add_item(item);
+        
+        if (item->get_name() == DEFAULT_TOOL)
+          item->set_checked(true);
+      }
+    }
+  }
+  return _tools_toolbar;
+}
+
+
+mforms::ToolBar *ModelDiagramForm::get_options_toolbar()
+{
+  update_options_toolbar();
+  return _options_toolbar;
+}
+
+
+mforms::MenuBar *ModelDiagramForm::get_menubar()
+{
+  if (!_menu)
+  {
+    _menu = _owner->get_wb()->get_ui()->get_command_ui()->create_menubar_for_context(WB_CONTEXT_MODEL);
+    _menu->signal_will_show().connect(sigc::hide(sigc::mem_fun(this, &ModelDiagramForm::revalidate_menu)));
+    
+    mforms::MenuItem *item = _menu->find_item("wb.edit.editSelectedFigure");
+    if (item)
+      item->set_validator(sigc::mem_fun(this, &ModelDiagramForm::has_selection));
+    item = _menu->find_item("wb.edit.editSelectedFigureInNewWindow");
+    if (item)
+      item->set_validator(sigc::mem_fun(this, &ModelDiagramForm::has_selection));
+  }
+  revalidate_menu();
+  return _menu;
+}
+
+
+void ModelDiagramForm::revalidate_menu()
+{
+  static const char *figure_notations[] = {
+    "workbench/default",
+    "workbench/simple",
+    "workbench/pkonly",
+    "classic",
+    "idef1x",
+    NULL
+  };
+  static const char *relationship_notations[] = {
+    "crowsfoot",
+    "classic",
+    "fromcolumn",
+    "uml",
+    "idef1x",
+    NULL
+  };
+  if (_menu)
+  {
+    bool has_selection_ = has_selection();
+    
+    _menu->set_item_enabled("wb.edit.goToNextSelected", has_selection_);
+    _menu->set_item_enabled("wb.edit.goToPreviousSelected", has_selection_);
+    _menu->set_item_enabled("wb.edit.selectSimilar", has_selection_);
+    _menu->set_item_enabled("wb.edit.selectConnected", get_selection().count() == 1);
+
+    _menu->set_item_checked("wb.edit.toggleGridAlign", get_diagram_options().get_int("AlignToGrid", 0) != 0);
+    _menu->set_item_checked("wb.edit.toggleGrid", get_diagram_options().get_int("ShowGrid", 1) != 0);
+    _menu->set_item_checked("wb.edit.togglePageGrid", get_diagram_options().get_int("ShowPageGrid", 1) != 0);
+    std::string notation = workbench_physical_ModelRef::cast_from(get_model_diagram()->owner())->figureNotation();
+    for (int i= 0; figure_notations[i]; i++)
+      _menu->set_item_checked(strfmt("wb.view.setFigureNotation:%s", figure_notations[i]), notation == figure_notations[i]);
+    
+    notation = workbench_physical_ModelRef::cast_from(get_model_diagram()->owner())->connectionNotation();
+    for (int i= 0; relationship_notations[i]; i++)
+      _menu->set_item_checked(strfmt("wb.view.setRelationshipNotation:%s", relationship_notations[i]), notation == relationship_notations[i]);
+
+
+    model_ModelRef model(get_model_diagram()->owner());    
+    for (int i= 1; i < 10; i++)
+    {
+      _menu->set_item_checked(strfmt("wb.view.setMarker:%i", i), false);
+      _menu->set_item_enabled(strfmt("wb.view.goToMarker:%i", i), false);
+    }
+    
+    for (size_t c= model->markers().count(), i= 0; i < c; i++)
+    {
+      _menu->set_item_checked(strfmt("wb.view.setMarker:%s", model->markers().get(i)->name().c_str()), true);
+      _menu->set_item_enabled(strfmt("wb.view.goToMarker:%s", model->markers().get(i)->name().c_str()), true);
+    }
+    
+    _menu->find_item("plugins")->validate();
+  }
+}
+
+
+static ToolbarItemType get_toolbar_item_type(const std::string &type)
+{
+  if (type == "action")
+    return ToolbarAction;
+  else if (type == "separator")
+    return ToolbarSeparator;
+  else if (type == "toggle")
+    return ToolbarToggle;
+  else if (type == "radio")
+    return ToolbarRadio;
+  else if (type == "dropdown")
+    return ToolbarDropDown;
+  else if (type == "label")
+    return ToolbarLabel;
+  else if (type == "check")
+    return ToolbarCheck;
+  else if (type == "search")
+    return ToolbarSearch;
+  else
+    throw std::runtime_error("invalid toolbar item type "+type);
+  return ToolbarAction;
+}
+
+
+void ModelDiagramForm::update_options_toolbar()
+{
+  app_ToolbarRef toolbar = get_wb()->get_component_named("basic")->get_tool_options(get_tool());
+
+  _options_toolbar->remove_all();
+
+  if (!toolbar.is_valid())
+    toolbar = get_wb()->get_component_named("physical")->get_tool_options(get_tool());
+  if (!toolbar.is_valid())
+    return;
+  
+  for (size_t c= toolbar->items().count(), i= 0; i < c; i++)
+  {
+    app_ToolbarItemRef titem(toolbar->items()[i]);
+    mforms::ToolBarItem *item;
+    
+    std::string type = titem->itemType();
+    if (type == "label")
+    {
+      item = mforms::manage(new mforms::ToolBarItem(mforms::LabelItem));
+      item->set_text(*titem->command());
+    }
+    else if (type == "dropdown")
+    {
+      std::string selected;
+      std::vector<std::string> items(get_dropdown_items(titem->name(), titem->command(), selected));
+      if (!items.empty() && !items.front().empty() && items.front()[0] == '#')
+        item = mforms::manage(new mforms::ToolBarItem(mforms::ColorSelectorItem));
+      else
+        item = mforms::manage(new mforms::ToolBarItem(mforms::SelectorItem));
+      item->set_selector_items(items);
+      item->set_text(selected);
+      item->signal_activated().connect(sigc::bind(sigc::mem_fun(this, &ModelDiagramForm::select_dropdown_item),
+                                                  titem->command(), item));
+    }
+    else if (type == "separator")
+      item = mforms::manage(new mforms::ToolBarItem(mforms::SeparatorItem));
+    else if (type == "action")
+    {
+      item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
+    }
+    else if (type == "check")
+    {
+      item = mforms::manage(new mforms::ToolBarItem(mforms::ToggleItem));
+      // icon text is used as label for check items
+      item->set_text(IconManager::get_instance()->get_icon_path(*titem->icon()));
+    }
+    else if (type == "toggle")
+    {
+      item = mforms::manage(new mforms::ToolBarItem(mforms::ToggleItem));
+    }
+    else
+      continue;
+
+    if (!titem->icon().empty())
+      item->set_icon(IconManager::get_instance()->get_icon_path(*titem->icon()));
+    if (!titem->altIcon().empty())
+      item->set_alt_icon(IconManager::get_instance()->get_icon_path(*titem->altIcon()));
+
+    item->set_name(titem->name());
+    item->set_tooltip(titem->tooltip()); 
+    //titem->command();
+    
+    _options_toolbar->add_item(item);
+  }
+}
+
+
+
+std::vector<std::string> ModelDiagramForm::get_dropdown_items(const std::string &name, const std::string &option, std::string &selected)
+{
+  std::vector<std::string> items;
+  WBComponent *compo;
+  
+  compo= get_wb()->get_component_named(base::split(name, "/")[0]);
+  if (compo)
+  {
+    std::string::size_type p = option.find(':');
+    if (p != std::string::npos)
+    {
+      std::string option_name = option.substr(p+1);
+      items= compo->get_command_dropdown_items(option_name);
+      selected= compo->get_command_option_value(option_name);
+    }
+  }
+  return items;
+}
+
+
+void ModelDiagramForm::select_dropdown_item(const std::string &option, mforms::ToolBarItem *item)
+{
+  WBComponent *compo;
+  
+  compo= get_wb()->get_component_named(base::split(item->get_name(), "/")[0]);
+  if (compo)
+  {
+    std::string::size_type p = option.find(':');
+    if (p != std::string::npos)
+    {
+      std::string option_name = option.substr(p+1);
+      compo->set_command_option_value(option_name, item->get_text());
+    }
+  }
+}
+
+
+void ModelDiagramForm::toggle_checkbox_item(const std::string &name, const std::string &option, bool state)
+{
+  WBComponent *compo;
+  
+  compo= get_wb()->get_component_named(base::split(name, "/")[0]);
+  if (compo)
+  {
+    std::string::size_type p = option.find(':');
+    if (p != std::string::npos)
+    {
+      std::string option_name = option.substr(p+1);
+      compo->set_command_option_value(option, state?"1":"0"); 
+    }
+  }
+}
+
+
+
+
+void ModelDiagramForm::selection_changed()
+{
+  get_wb()->request_refresh(RefreshSelection, "", 0);
+  
+  if (get_wb()->get_grt_manager()->in_main_thread())
+    revalidate_menu();
+  else
+    get_wb()->get_grt_manager()->run_when_idle(sigc::bind_return(sigc::mem_fun(this, &ModelDiagramForm::revalidate_menu), false));
+}
+
+
 void ModelDiagramForm::model_list_changed(grt::internal::OwnedList *list, bool added, const grt::ValueRef &value)
 {
   if (_update_count == 0)
@@ -149,16 +465,21 @@ void ModelDiagramForm::attach_canvas_view(mdc::CanvasView *cview)
 
   cview->get_selection()->signal_begin_dragging().connect(sigc::mem_fun(this, &ModelDiagramForm::begin_selection_drag));
   cview->get_selection()->signal_end_dragging().connect(sigc::mem_fun(this, &ModelDiagramForm::end_selection_drag));
-
+  _model_diagram->get_data()->signal_selection_changed().connect(sigc::hide(sigc::mem_fun(this, &ModelDiagramForm::selection_changed)));
+  
   _main_layer= _view->get_current_layer();
   _badge_layer= _view->new_layer("badges");
   _floater_layer= _view->new_layer("floater");
+  
+  // force updates
+  selection_changed();
 }
 
 
 void ModelDiagramForm::set_closed(bool flag)
 {
-  _model_diagram->closed(flag!=0);
+  if (_model_diagram.is_valid())
+    _model_diagram->closed(flag!=0);
 }
 
 
@@ -203,7 +524,8 @@ void ModelDiagramForm::enable_zoom_click(bool enable, bool zoomin)
     _reset_tool= _old_reset_tool;
     _handle_button= _old_handle_button;
     _handle_motion= _old_handle_motion;
-    _owner->get_wb()->tool_changed(_view);
+    
+    set_tool(_tool);
 
     return;
   }
@@ -222,7 +544,7 @@ void ModelDiagramForm::enable_zoom_click(bool enable, bool zoomin)
   WBComponent *compo= _owner->get_wb()->get_component_named("basic");
   compo->setup_canvas_tool(this, _tool);
 
-  _owner->get_wb()->tool_changed(_view);
+  set_tool(_tool);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -240,7 +562,7 @@ void ModelDiagramForm::enable_panning(bool flag)
     WBComponent *compo= _owner->get_wb()->get_component_named("basic");
     compo->setup_canvas_tool(this, _tool);
 
-    _owner->get_wb()->tool_changed(_view);
+    set_tool(_tool);
   }
   else
   {
@@ -250,7 +572,7 @@ void ModelDiagramForm::enable_panning(bool flag)
     _reset_tool= _old_reset_tool;
     _handle_button= _old_handle_button;
     _handle_motion= _old_handle_motion;
-    _owner->get_wb()->tool_changed(_view);
+    set_tool(_tool);
   }
 }
 
@@ -442,7 +764,13 @@ void ModelDiagramForm::handle_mouse_button(mdc::MouseButton button, bool press,
     {
       int x, y;
       _view->canvas_to_window(pos, x, y);
-      get_wb()->popup_menu(&items, x, y);
+      
+      _context_menu.clear();
+      _context_menu.add_items_from_list(items);
+      _context_menu.set_handler(sigc::mem_fun(get_wb()->get_ui()->get_command_ui(), 
+                                     (void (CommandUI::*)(const std::string&))&CommandUI::activate_command));
+      
+      _context_menu.popup_at(NULL, x, y);
     }
     return;
   }
@@ -529,7 +857,7 @@ bool ModelDiagramForm::handle_key(const mdc::KeyInfo &key, bool press, mdc::Even
     {
       if (iter->modifiers == state && iter->key == key)
       {
-        _owner->get_wb()->get_ui()->get_command_ui()->activate_command(iter->command);
+        set_tool(iter->command.substr(strlen("tool:")));
         return true;
       }
     }
@@ -582,9 +910,10 @@ bool ModelDiagramForm::handle_key(const mdc::KeyInfo &key, bool press, mdc::Even
 
 //--------------------------------------------------------------------------------------------------
 
-void ModelDiagramForm::set_tool(const std::string &tool)
+void ModelDiagramForm::set_tool(std::string tool)
 {
-  reset_tool(false);
+  if (_tool != DEFAULT_TOOL)
+    reset_tool(false);
 
   _tool= tool;
 
@@ -594,6 +923,17 @@ void ModelDiagramForm::set_tool(const std::string &tool)
   else
     throw std::runtime_error("Invalid tool "+tool);
 
+  std::vector<mforms::ToolBarItem*> items = _tools_toolbar->get_items();
+  for (std::vector<mforms::ToolBarItem*>::iterator item = items.begin(); item != items.end(); ++item)
+  {
+    if ((*item)->get_type() == mforms::ToggleItem)
+    {
+      if ((*item)->get_name() == _tool)
+        (*item)->set_checked(true);
+      else
+        (*item)->set_checked(false);
+    }
+  }
   _owner->get_wb()->tool_changed(_view);
 }
 
@@ -602,6 +942,16 @@ void ModelDiagramForm::reset_tool(bool notify)
 {
  // _tool_args.clear();
 
+  if (_tools_toolbar)
+  {
+    mforms::ToolBarItem *item = _tools_toolbar->find_item(_tool);
+    if (!_tool.empty() && item)
+      item->set_checked(false);
+    item = _tools_toolbar->find_item(DEFAULT_TOOL);
+    if (item)
+      item->set_checked(true);
+  }
+  
   _tool= DEFAULT_TOOL;
 
   if (_reset_tool)
@@ -1076,6 +1426,7 @@ static model_ObjectRef search_object_list(const grt::ListRef<model_Object> &obje
 
 bool ModelDiagramForm::search_and_focus_object(const std::string &text)
 {
+  if (text.empty()) return false;
   grt::ListRef<model_Object> selection(get_selection());
   model_ObjectRef selected, found_object;
   
@@ -1462,6 +1813,7 @@ std::string ModelDiagramForm::get_title()
  */
 void wb::ModelDiagramForm::change_signal_id(ChangeSignal::iterator signal_id)
 {
+  //XXX this is probably redundant, sigc::trackable will auto-disconnect the slot
   _signal_id = signal_id;
 }
 
