@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,9 +22,13 @@
 
 #include "workbench/wb_context.h"
 #include "model/wb_layer_tree.h"
-
+#include "grt/icon_manager.h"
+#include "base/ui_form.h"
 #include "mini_view.h"
 
+#include "mforms/menu.h"
+
+namespace mforms { class ToolBarItem; };
 
 namespace wb {
   class Floater;  
@@ -37,7 +41,9 @@ namespace wb {
     EditShiftTabPressed
   };
 
-  
+
+  typedef sigc::signal<void, const std::string&, const grt::ValueRef&> ChangeSignal;
+
   class ModelDiagramForm;
   class PhysicalModelDiagramFeatures;
   class LayerTreeBE;
@@ -124,9 +130,9 @@ namespace wb {
     void set_button_callback(const sigc::slot<bool, ModelDiagramForm*, mdc::MouseButton, bool, mdc::Point, mdc::EventState> &cb);
     void set_motion_callback(const sigc::slot<bool, ModelDiagramForm*, mdc::Point, mdc::EventState> &cb);
     void set_reset_tool_callback(const sigc::slot<void, ModelDiagramForm*> &cb);
-
+    
     std::string get_tool() { return _tool; }
-    void set_tool(const std::string &tool);
+    void set_tool(std::string tool);
     void reset_tool(bool notify);
     void set_tool_argument(const std::string &option, const std::string &value);
     std::string get_tool_argument(const std::string &option);
@@ -149,6 +155,7 @@ namespace wb {
     // events
     void handle_mouse_move(int x, int y, mdc::EventState state);
     void handle_mouse_button(mdc::MouseButton button, bool press, int x, int y, mdc::EventState state);
+    void handle_mouse_double_click(mdc::MouseButton button, int x, int y, mdc::EventState state);
     bool handle_key(const mdc::KeyInfo &key, bool press, mdc::EventState state);
 
     bool current_mouse_position(int &x, int &y);
@@ -188,6 +195,15 @@ namespace wb {
 
     void set_inline_editor_context(InlineEditContext *context);
 
+    void change_signal_id(ChangeSignal::iterator signal_id);
+
+    virtual mforms::ToolBar *get_toolbar();
+    mforms::ToolBar *get_tools_toolbar();
+    mforms::ToolBar *get_options_toolbar();
+    void update_options_toolbar();
+
+    virtual mforms::MenuBar *get_menubar();
+    void revalidate_menu();
   protected:
     struct OldPosition
     {
@@ -210,7 +226,7 @@ namespace wb {
     
     LayerTreeBE _layer_tree;
     MiniView *_mini_view;
-
+    
     PhysicalModelDiagramFeatures *_features;
 
     std::map<grt::internal::Value*, OldPosition> _old_positions;
@@ -219,6 +235,11 @@ namespace wb {
 
     double _paste_offset;
 
+    mforms::MenuBar *_menu;
+    mforms::ToolBar *_toolbar;
+    mforms::ToolBar *_tools_toolbar;
+    mforms::ToolBar *_options_toolbar;
+    
     sigc::signal<void, std::string> _tool_argument_changed;
 
     sigc::slot<bool, ModelDiagramForm*, mdc::MouseButton, bool, mdc::Point, mdc::EventState> _handle_button;
@@ -246,30 +267,40 @@ namespace wb {
     void diagram_changed(const model_ObjectRef &object);
     void diagram_objects_changed(grt::internal::OwnedList *list, bool added, const grt::ValueRef &value);
     
+    void selection_changed();
+    
     void model_list_changed(grt::internal::OwnedList *list, bool added, const grt::ValueRef &value);
+
+    std::vector<std::string> get_dropdown_items(const std::string &name, const std::string &option, std::string &selected);
+    void select_dropdown_item(const std::string &option, mforms::ToolBarItem *item);
+    void toggle_checkbox_item(const std::string &name, const std::string &option, bool state);
     
     //void hover_in_figure(mdc::CanvasItem *item, mdc::Point pos);
     //void hover_out_figure(mdc::CanvasItem *item);
     private:
       int _update_count; // If > 0 don't refresh depending structures.
+      ChangeSignal::iterator _signal_id; // The iterator which can be used to disconnect the form
+                                         // from its diagram change signal.
+      mforms::Menu _context_menu;
 
-    class UpdateLock
-    {
-    private:
-      ModelDiagramForm* _form;
-    public:
-      UpdateLock(ModelDiagramForm* form)
+      // Local class.
+      class UpdateLock
       {
-        _form = form;
-        _form->_update_count++;
-      };
-      ~UpdateLock()
-      {
-        if (_form->_update_count > 0)
-          _form->_update_count--;
-        _form->_layer_tree.tree_changed(); // Will only do anything if update count is 0.
-      };
-    };  
+      private:
+        ModelDiagramForm* _form;
+      public:
+        UpdateLock(ModelDiagramForm* form)
+        {
+          _form = form;
+          _form->_update_count++;
+        };
+        ~UpdateLock()
+        {
+          if (_form->_update_count > 0)
+            _form->_update_count--;
+          _form->_layer_tree.tree_changed(); // Will only do anything if update count is 0.
+        };
+      };  
   };
 
 };
