@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,6 +25,8 @@
 #include "wb_overview_physical_schema.h"
 
 #include "workbench/wb_context.h"
+#include "workbench/wb_context_ui.h"
+#include "workbench/wb_command_ui.h"
 #include "wb_component_physical.h"
 
 #include "grt/incremental_list_updater.h"
@@ -33,11 +35,13 @@
 
 #include "grt/icon_manager.h"
 #include "grt/clipboard.h"
-#include "ui_form.h"
+#include "base/ui_form.h"
 #include "grt/exceptions.h"
 
 #include "grts/structs.db.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
+
+#include "mforms/toolbar.h"
 
 /**
  * @file  wb_overview_physical.cpp
@@ -472,12 +476,27 @@ public:
 
 
 PhysicalOverviewBE::PhysicalOverviewBE(WBContext *wb)
-: OverviewBE(wb)
+: OverviewBE(wb), _menu(0), _toolbar(0)
 {
   _schemata_node_index= 1;
+  _menu = 0;
 }
 
 
+PhysicalOverviewBE::~PhysicalOverviewBE()
+{
+  delete _menu;
+}
+
+mforms::ToolBar *PhysicalOverviewBE::get_toolbar()
+{
+  if (!_toolbar)
+  {
+    _toolbar = _wb->get_ui()->get_command_ui()->create_toolbar("data/model_toolbar.xml");
+  }
+  return _toolbar;
+}
+
 OverviewBE::ContainerNode * PhysicalOverviewBE::create_root_node(workbench_physical_ModelRef model, PhysicalOverviewBE *owner)
 {
   return new PhysicalRootNode(model, owner);
@@ -532,6 +551,76 @@ model_ModelRef PhysicalOverviewBE::get_model()
   return _model;
 }
 
+
+static bool has_selection(PhysicalOverviewBE *overview)
+{
+  grt::ListRef<GrtObject> selection(overview->get_selection());
+  return selection.is_valid() && selection.count() > 0;
+}
+
+mforms::MenuBar *PhysicalOverviewBE::get_menubar()
+{
+  if (!_menu)
+  {
+    _menu = _wb->get_ui()->get_command_ui()->create_menubar_for_context(WB_CONTEXT_PHYSICAL_OVERVIEW);
+    
+    static const char* diagram_only_items[] = {
+      "diagram_size",
+      "fnotation",
+      "rnotation",
+      "wb.edit.goToNextSelected",
+      "wb.edit.goToPreviousSelected",
+      "wb.edit.selectSimilar",
+      "wb.edit.selectConnected",
+      "wb.edit.toggleGridAlign",
+      "wb.edit.toggleGrid",
+      "wb.edit.togglePageGrid",
+      "wb.view.zoomDefault",
+      "wb.view.zoomIn",
+      "wb.view.zoomOut",
+      "wb.view.setFigureNotation",
+      "wb.view.setRelationshipNotation",
+      "wb.view.setMarker:1",
+      "wb.view.setMarker:2",
+      "wb.view.setMarker:3",
+      "wb.view.setMarker:4",
+      "wb.view.setMarker:5",
+      "wb.view.setMarker:6",
+      "wb.view.setMarker:7",
+      "wb.view.setMarker:8",
+      "wb.view.setMarker:9",
+      "wb.view.goToMarker:1",
+      "wb.view.goToMarker:2",
+      "wb.view.goToMarker:3",
+      "wb.view.goToMarker:4",
+      "wb.view.goToMarker:5",
+      "wb.view.goToMarker:6",
+      "wb.view.goToMarker:7",
+      "wb.view.goToMarker:8",
+      "wb.view.goToMarker:9",
+      NULL
+    };
+    
+    for (int i= 0; diagram_only_items[i]; i++)
+      _menu->set_item_enabled(diagram_only_items[i], false);
+    
+    std::vector<mforms::MenuItem*> items(_menu->find_item("arrange")->get_subitems());
+    for (std::vector<mforms::MenuItem*>::const_iterator i = items.begin(); i != items.end(); ++i)
+    {
+      (*i)->set_enabled(false);
+    }
+    
+    mforms::MenuItem *item = _menu->find_item("wb.edit.editObject");
+    if (item)
+      item->set_validator(sigc::bind(sigc::ptr_fun(has_selection), this));
+    item = _menu->find_item("wb.edit.editObjectInNewWindow");
+    if (item)
+      item->set_validator(sigc::bind(sigc::ptr_fun(has_selection), this));    
+  }
+  return _menu;
+}
+
+
 internal::PhysicalSchemaNode *PhysicalOverviewBE::get_active_schema_node()
 {
   NodeId node(get_focused_child(NodeId(_schemata_node_index)));
