@@ -23,7 +23,7 @@
 #include "sqlide/wb_sql_editor_form.h"
 #include "sqlide/recordset_text_storage.h"
 #include "sqlide/recordset_sql_storage.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace mforms;
 using namespace base;
