@@ -17,6 +17,7 @@
  * 02110-1301  USA
  */
 
+
 #include "stdafx.h"
 
 #include "workbench/wb_context_ui.h"
@@ -26,7 +27,8 @@
 #include "sqlide/wb_sql_editor_snippets.h"
 #include "recordset_export_form.h"
 #include "grt/common.h"
-#include "string_utilities.h"
+#include "base/file_utilities.h"
+#include "base/string_utilities.h"
 
 #include "objimpl/db.query/db_query_Editor.h"
 #include "objimpl/db.query/db_query_Resultset.h"
@@ -38,6 +40,7 @@ using namespace wb;
 using namespace bec;
 using namespace base;
 
+static std::map<std::string,std::string> auto_save_sessions;
 
 //------------------------------------------------------------------------------------------------
 
@@ -46,34 +49,42 @@ class MYSQLWBBACKEND_PUBLIC_FUNC db_query_EditorConcreteImplData : public db_que
 {
   void sql_editor_list_changed(Sql_editor::Ref editor, bool added)
   {
-    if (added)
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)
     {
-      editor->grtobj()->owner(_self);
-      _self->queryBuffers().insert(editor->grtobj());
+      if (added)
+      {
+        editor->grtobj()->owner(_self);
+        _self->queryBuffers().insert(editor->grtobj());
+      }
+      else
+        _self->queryBuffers().remove_value(editor->grtobj());
     }
-    else
-      _self->queryBuffers().remove_value(editor->grtobj());
   }
 
   void recordset_list_changed(Recordset::Ref rset, bool added)
   {
-    if (added)
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)
     {
-      if (!rset->is_readonly())
-        _self->resultsets().insert(grtwrap_editablerecordset(_self, rset));
-      else
-        _self->resultsets().insert(grtwrap_recordset(_self, rset));
-    }
-    else
-    {
-      for (size_t c= _self->resultsets().count(), i= 0; i < c; i++)
+      if (added)
       {
-        db_query_ResultsetRef e(_self->resultsets()[i]);
-        
-        if (e->get_data()->recordset == rset)
+        if (!rset->is_readonly())
+          _self->resultsets().insert(grtwrap_editablerecordset(_self, rset));
+        else
+          _self->resultsets().insert(grtwrap_recordset(_self, rset));
+      }
+      else
+      {
+        for (size_t c= _self->resultsets().count(), i= 0; i < c; i++)
         {
-          _self->resultsets().remove(i);
-          break;
+          db_query_ResultsetRef e(_self->resultsets()[i]);
+          
+          if (e->get_data()->recordset == rset)
+          {
+            _self->resultsets().remove(i);
+            break;
+          }
         }
       }
     }
@@ -82,19 +93,23 @@ class MYSQLWBBACKEND_PUBLIC_FUNC db_query_EditorConcreteImplData : public db_que
   
   virtual void refresh_editor(Sql_editor::Ref editor)
   {
-    int oindex = _editor->active_sql_editor_index();
-    
-    for (size_t c= _self->queryBuffers().count(), i= 0; i < c; i++)
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)
     {
-      db_query_QueryBufferRef e(_self->queryBuffers()[i]);
-      if (e == editor->grtobj())
+      int oindex = ref->active_sql_editor_index();
+      
+      for (size_t c= _self->queryBuffers().count(), i= 0; i < c; i++)
       {
-        _editor->active_sql_editor_index(i);
-        _editor->do_partial_ui_refresh(Db_sql_editor::RefreshEditor);      
-        break;
+        db_query_QueryBufferRef e(_self->queryBuffers()[i]);
+        if (e == editor->grtobj())
+        {
+          ref->active_sql_editor_index(i);
+          ref->do_partial_ui_refresh(Db_sql_editor::RefreshEditor);      
+          break;
+        }
       }
+      ref->active_sql_editor_index(oindex);
     }
-    _editor->active_sql_editor_index(oindex);
   }
   
 public:
@@ -117,102 +132,139 @@ public:
   
   virtual db_mgmt_ConnectionRef connection() const
   {
-    return _editor->connection_descriptor();
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)
+      return _editor->connection_descriptor();
+    return db_mgmt_ConnectionRef();
   }
   
   virtual db_query_QueryBufferRef addQueryBuffer()
   {
-    _editor->new_sql_script_file();
-
-    return _editor->sql_editor()->grtobj();
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)
+    {
+      _editor->new_sql_script_file();
+      
+      return _editor->sql_editor()->grtobj();
+    }
+    return db_query_QueryBufferRef();
   }
   
   virtual grt::IntegerRef addToOutput(const std::string &text, long bringToFront)
   {
-    _editor->output_text_slot(text, bringToFront != 0);
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)
+      ref->output_text_slot(text, bringToFront != 0);
     
     return grt::IntegerRef(0);
   }
   
   virtual grt::ListRef<db_query_Resultset> executeScript(const std::string &sql)
   {
-    _editor->grt_manager()->replace_status_text("Executing query...");
-
-    std::vector<Recordset::Ref> rsets(_editor->exec_sql_returning_results(sql, true));
-    
     grt::ListRef<db_query_Resultset> result(_self->get_grt());
-    for (std::vector<Recordset::Ref>::const_iterator iter= rsets.begin(); iter != rsets.end(); ++iter)
-      result.insert(grtwrap_recordset(_self, *iter));
-
-    _editor->grt_manager()->replace_status_text("Query finished.");
-
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)
+    { 
+      ref->grt_manager()->replace_status_text("Executing query...");
+      
+      std::vector<Recordset::Ref> rsets(ref->exec_sql_returning_results(sql, true));
+      
+      for (std::vector<Recordset::Ref>::const_iterator iter= rsets.begin(); iter != rsets.end(); ++iter)
+        result.insert(grtwrap_recordset(_self, *iter));
+      
+      ref->grt_manager()->replace_status_text("Query finished.");
+    }
     return result;
   }
   
   virtual grt::IntegerRef executeScriptAndOutputToGrid(const std::string &sql)
   {
-    _editor->exec_sql_retaining_editor_contents(sql, true);
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)      
+      ref->exec_sql_retaining_editor_contents(sql, true);
 
     return grt::IntegerRef(0);
   }
   
   virtual db_query_EditableResultsetRef createTableEditResultset(const std::string &schema, const std::string &table, const std::string &where, bool showGrid)
   {
-    std::string query;
-    
-    query= base::strfmt("EDIT `%s`.`%s`", 
-                       base::escape_backticks(schema).c_str(),
-                       base::escape_backticks(table).c_str());
-    //XXX WHERE statement not handled for now, we have to make sure it doesn't contain anything dangerous
-    
-    if (showGrid)
-    {
-      executeScriptAndOutputToGrid(query);
-    }
-    else
-    {
-      std::vector<Recordset::Ref> rsets(_editor->exec_sql_returning_results(query, true));
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)
+    {      
+      std::string query;
       
-      if (rsets.size() == 1 && !rsets[0]->is_readonly())
-        return grtwrap_editablerecordset(_self, rsets[0]);
+      query= base::strfmt("EDIT `%s`.`%s`", 
+                          base::escape_backticks(schema).c_str(),
+                          base::escape_backticks(table).c_str());
+      //XXX WHERE statement not handled for now, we have to make sure it doesn't contain anything dangerous
+      
+      if (showGrid)
+      {
+        executeScriptAndOutputToGrid(query);
+      }
+      else
+      {
+        std::vector<Recordset::Ref> rsets(ref->exec_sql_returning_results(query, true));
+        
+        if (rsets.size() == 1 && !rsets[0]->is_readonly())
+          return grtwrap_editablerecordset(_self, rsets[0]);
+      }
     }
     return db_query_EditableResultsetRef();
   }
   
   virtual void activeSchema(const std::string &schema)
   {
-    _editor->active_schema(schema);
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)      
+      ref->active_schema(schema);
   }
   
   virtual std::string activeSchema()
   {
-    return _editor->active_schema();
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)      
+      return ref->active_schema();
+    return "";
   }
   
   virtual db_query_QueryBufferRef activeQueryBuffer()
   {
-    Sql_editor::Ref editor(_editor->sql_editor());
-    if (editor)
-      return editor->grtobj();
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)
+    {
+      Sql_editor::Ref editor(ref->sql_editor());
+      if (editor)
+        return editor->grtobj();
+    }
     return db_query_QueryBufferRef();
   }
   
   virtual db_query_ResultsetRef activeResultset()
   {
-    Recordset::Ref rset(_editor->active_recordset());
-    
-    if (rset)
+    boost::shared_ptr<Db_sql_editor> ref(_editor);
+    if (ref)
     {
-      for (size_t c= _self->resultsets().count(), i= 0; i < c; i++)
+      Recordset::Ref rset(ref->active_recordset());
+      
+      if (rset)
       {
-        db_query_ResultsetRef e(_self->resultsets()[i]);
-
-        if (e->get_data()->recordset == rset)
-          return e;
+        for (size_t c= _self->resultsets().count(), i= 0; i < c; i++)
+        {
+          db_query_ResultsetRef e(_self->resultsets()[i]);
+          
+          if (e->get_data()->recordset == rset)
+            return e;
+        }
       }
     }
     return db_query_ResultsetRef();
   }
+  
+  void detach()
+  {
+    _editor.reset();
+  }
     
 protected:  
   db_query_Editor *_self;
@@ -229,12 +281,6 @@ static void call_commit(wb::WBContextSQLIDE *sqlide)
     db_sql_editor->commit();
 }
 
-static bool validate_commit(wb::WBContextSQLIDE *sqlide)
-{
-  Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
-  return (db_sql_editor && !db_sql_editor->is_running_query() && !db_sql_editor->auto_commit());
-}
-
 
 static void call_rollback(wb::WBContextSQLIDE *sqlide)
 {
@@ -243,12 +289,6 @@ static void call_rollback(wb::WBContextSQLIDE *sqlide)
     db_sql_editor->rollback();  
 }
 
-static bool validate_rollback(wb::WBContextSQLIDE *sqlide)
-{
-  Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
-  return (db_sql_editor && !db_sql_editor->is_running_query() && !db_sql_editor->auto_commit());
-}
-
 
 static void call_autocommit(wb::WBContextSQLIDE *sqlide)
 {
@@ -257,15 +297,6 @@ static void call_autocommit(wb::WBContextSQLIDE *sqlide)
     db_sql_editor->auto_commit(!db_sql_editor->auto_commit());
 }
 
-static wb::CommandItemValidationState validate_autocommit(wb::WBContextSQLIDE *sqlide)
-{
-  Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
-  if (!db_sql_editor || db_sql_editor->is_running_query())
-    return wb::ItemDisabled;
-  // Note: meaning has been switched here. Auto commit on means the button is NOT checked.
-  return db_sql_editor->auto_commit() ? wb::ItemEnabled : wb::ItemEnabledAndChecked;
-}
-
 
 static void call_export(wb::WBContextSQLIDE *sqlide)
 {
@@ -290,13 +321,6 @@ static void call_cancel(wb::WBContextSQLIDE *sqlide)
   sqlide->get_active_sql_editor()->cancel_query();
 }
 
-static bool validate_cancel(wb::WBContextSQLIDE *sqlide)
-{
-  Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
-  return (db_sql_editor && db_sql_editor->is_running_query());
-}
-
-
 static void call_new_file(wb::WBContextSQLIDE *sqlide)
 {
   Db_sql_editor *editor= sqlide->get_active_sql_editor();
@@ -304,18 +328,13 @@ static void call_new_file(wb::WBContextSQLIDE *sqlide)
     editor->new_sql_script_file();
 }
 
-static void call_new_scratch(wb::WBContextSQLIDE *sqlide)
+static void call_new_query(wb::WBContextSQLIDE *sqlide)
 {
   Db_sql_editor *editor= sqlide->get_active_sql_editor();
   if (editor)
     editor->new_sql_scratch_area();
 }
 
-static bool validate_new_file(wb::WBContextSQLIDE *sqlide)
-{
-  return (NULL != sqlide->get_active_sql_editor());
-}
-
 
 static void call_open_file(wb::WBContextSQLIDE *sqlide)
 {
@@ -324,11 +343,6 @@ static void call_open_file(wb::WBContextSQLIDE *sqlide)
     sqlide->open_document(path);
 }
 
-static bool validate_open_file(wb::WBContextSQLIDE *sqlide)
-{
-  return (NULL != sqlide->get_active_sql_editor());
-}
-
 static void call_run_file(wb::WBContextSQLIDE *sqlide)
 {
   std::string path= sqlide->get_wbui()->get_wb()->show_file_dialog("open", _("Execute SQL Script"), "SQL Files (*.sql)|*.sql");
@@ -362,11 +376,43 @@ static void call_save_file_as(wb::WBContextSQLIDE *sqlide)
   }
 }
 
-static bool validate_save_file(wb::WBContextSQLIDE *sqlide)
+
+static void call_revert(wb::WBContextSQLIDE *sqlide)
 {
-  return (NULL != sqlide->get_active_sql_editor());
+  Db_sql_editor *editor = sqlide->get_active_sql_editor();
+  if (editor)
+  {
+    int i = editor->active_sql_editor_index();
+    if (i >= 0)
+    {
+      if (editor->sql_editor_dirty(i))
+      {
+        int rc = mforms::Utilities::show_message(_("Revert to Saved"),
+                            base::strfmt(_("Do you want to revert to the most recently saved version of '%s'?\nAny changes since them will be lost."),
+                                         editor->sql_editor_path(i).c_str()),
+                                                _("Revert"), _("Cancel"), "");
+        if (rc != mforms::ResultOk)
+          return;
+        
+        editor->revert_sql_script_file();
+      }
+    }
+  }
 }
 
+static bool validate_revert(wb::WBContextSQLIDE *sqlide)
+{
+  Db_sql_editor *editor = sqlide->get_active_sql_editor();
+  if (editor)
+  {
+    int i = editor->active_sql_editor_index();
+    if (i >= 0 && !editor->sql_editor_path(i).empty())
+      return !editor->sql_editor_is_scratch(i);
+  }
+  return false;
+}
+
+
 static void call_save_snippet(wb::WBContextSQLIDE *sqlide)
 {
   Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
@@ -382,14 +428,6 @@ static void call_save_snippet(wb::WBContextSQLIDE *sqlide)
 }
 
 
-static bool validate_save_snippet(wb::WBContextSQLIDE *sqlide)
-{
-  Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
-  // don't check contents as the toolbar is not updated when these change atm
-  return ((NULL != db_sql_editor)/* && !db_sql_editor->sql_editor()->sql().empty()*/);
-}
-
-
 static void call_continue_on_error(wb::WBContextSQLIDE *sqlide)
 {
   Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
@@ -397,15 +435,6 @@ static void call_continue_on_error(wb::WBContextSQLIDE *sqlide)
     db_sql_editor->continue_on_error(!db_sql_editor->continue_on_error());
 }
 
-static wb::CommandItemValidationState validate_continue_on_error(wb::WBContextSQLIDE *sqlide)
-{
-  Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
-  if (!db_sql_editor)
-    return wb::ItemDisabled;
-  return db_sql_editor->continue_on_error() ? wb::ItemEnabled : wb::ItemEnabledAndChecked;
-}
-
-
 static void call_recordsets_are_pinned_by_default(wb::WBContextSQLIDE *sqlide)
 {
   Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
@@ -413,15 +442,6 @@ static void call_recordsets_are_pinned_by_default(wb::WBContextSQLIDE *sqlide)
     db_sql_editor->recordsets_are_pinned_by_default(!db_sql_editor->recordsets_are_pinned_by_default());
 }
 
-static wb::CommandItemValidationState validate_recordsets_are_pinned_by_default(wb::WBContextSQLIDE *sqlide)
-{
-  Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
-  if (!db_sql_editor)
-    return wb::ItemDisabled;
-  return db_sql_editor->recordsets_are_pinned_by_default() ? wb::ItemEnabledAndChecked : wb::ItemEnabled;
-}
-
-
 static void call_explain_sql(wb::WBContextSQLIDE *sqlide)
 {
   Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
@@ -429,13 +449,6 @@ static void call_explain_sql(wb::WBContextSQLIDE *sqlide)
     db_sql_editor->explain_sql(false);
 }
 
-static bool validate_explain_sql(wb::WBContextSQLIDE *sqlide)
-{
-  Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
-  return (db_sql_editor && !db_sql_editor->is_running_query());
-}
-
-
 static void call_explain_current_statement(wb::WBContextSQLIDE *sqlide)
 {
   Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
@@ -443,13 +456,6 @@ static void call_explain_current_statement(wb::WBContextSQLIDE *sqlide)
     db_sql_editor->explain_current_statement();
 }
 
-static bool validate_explain_current_statement(wb::WBContextSQLIDE *sqlide)
-{
-  Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
-  return (db_sql_editor && !db_sql_editor->is_running_query());
-}
-
-
 static void call_reconnect(wb::WBContextSQLIDE *sqlide)
 {
   Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
@@ -457,12 +463,6 @@ static void call_reconnect(wb::WBContextSQLIDE *sqlide)
     db_sql_editor->connect();
 }
 
-static bool validate_reconnect(wb::WBContextSQLIDE *sqlide)
-{
-  Db_sql_editor *db_sql_editor= sqlide->get_active_sql_editor();
-  return (db_sql_editor && !db_sql_editor->is_running_query());
-}
-
 
 static void call_refresh_physical_overview(wb::WBContextSQLIDE *sqlide)
 {
@@ -471,11 +471,6 @@ static void call_refresh_physical_overview(wb::WBContextSQLIDE *sqlide)
     db_sql_editor->refresh_physical_overview(false);
 }
 
-static bool validate_refresh_physical_overview(wb::WBContextSQLIDE *sqlide)
-{
-  return (NULL != sqlide->get_active_sql_editor());
-}
-
 
 static void call_new_tab(wb::WBContextSQLIDE *sqlide)
 {
@@ -487,12 +482,6 @@ static void call_new_tab(wb::WBContextSQLIDE *sqlide)
   }
 }
 
-static bool validate_new_tab(wb::WBContextSQLIDE *sqlide)
-{
-  return (NULL != sqlide->get_active_sql_editor());
-}
-
-
 #ifdef _WIN32
 static void call_exec_sql(wb::WBContextSQLIDE *sqlide)
 {
@@ -560,7 +549,7 @@ static bool validate_discard_edits(wb::WBContextSQLIDE *sqlide)
 //--------------------------------------------------------------------------------------------------
 
 WBContextSQLIDE::WBContextSQLIDE(WBContextUI *wbui)
-: _wbui(wbui)
+: _wbui(wbui), _auto_save_active(false), _option_change_signal_connected(false)
 {
 }
 
@@ -572,6 +561,88 @@ WBContextSQLIDE::~WBContextSQLIDE()
 
 //--------------------------------------------------------------------------------------------------
 
+void WBContextSQLIDE::option_changed(grt::internal::OwnedDict*dict, bool, const std::string&key)
+{
+  if (key == "workbench:AutoSaveSQLEditorInterval" && dict == _wbui->get_wb()->get_wb_options().valueptr())
+  {
+    auto_save_workspaces();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool WBContextSQLIDE::auto_save_workspaces()
+{
+  WBContext *wb= _wbui->get_wb();
+  int interval= wb->get_root()->options()->options().get_int("workbench:AutoSaveSQLEditorInterval", 60);
+  if (interval <= 0 || !_auto_save_active)
+    return false;
+
+  for (std::list<boost::weak_ptr<Db_sql_editor> >::const_iterator iter = _open_editors.begin();
+       iter != _open_editors.end(); ++iter)
+  {
+    Db_sql_editor::Ref editor((*iter).lock());
+    try 
+    {
+      if (editor)
+        editor->auto_save();
+    }
+    catch (const std::exception &exception)
+    {
+      g_warning("Exception during auto-save of SQL Editors: %s", exception.what());
+      wb->get_grt_manager()->replace_status_text(base::strfmt("Error during auto-save of SQL Editors: %s", exception.what()));
+    }
+  }
+  
+  
+  if (interval != _auto_save_interval)
+  {
+    // schedule new interval
+    wb->get_grt_manager()->run_every(sigc::mem_fun(this, &WBContextSQLIDE::auto_save_workspaces), interval);
+    return false;
+  }
+  
+  return true;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void WBContextSQLIDE::detect_auto_save_files(const std::string &autosave_dir)
+{
+  // look for SQLEditor autosave workspace folders
+  std::list<std::string> autosaves;
+  try
+  {
+    autosaves = base::scan_for_files_matching(bec::make_path(autosave_dir, "sql_workspaces/*.autosave"));
+  }
+  catch (const std::runtime_error)
+  {
+  }
+
+  for (std::list<std::string>::const_iterator d = autosaves.begin(); d != autosaves.end(); ++d)
+  {
+    gchar *conn_id;
+    gsize length;
+    if (g_file_get_contents(bec::make_path(*d, "connection_id").c_str(),
+                            &conn_id, &length, NULL))
+    {
+      ::auto_save_sessions[std::string(conn_id, length)] = *d;
+      g_free(conn_id);
+      g_message("Found auto-save workspace %s", d->c_str());
+    }
+    else
+      g_warning("Found incomplete auto-save workspace %s", d->c_str());
+  }
+}
+
+
+std::map<std::string, std::string> WBContextSQLIDE::auto_save_sessions()
+{
+  return ::auto_save_sessions;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 bec::GRTManager *WBContextSQLIDE::get_grt_manager()
 {
   return _wbui->get_wb()->get_grt_manager();
@@ -582,63 +653,48 @@ void WBContextSQLIDE::init()
 {
   DbSqlEditorSnippets::setup(this, bec::make_path(get_grt_manager()->get_user_datadir(), "snippets"));
   
+  _wbui->get_wb()->signal_app_closing().connect(sigc::mem_fun(this, &WBContextSQLIDE::finalize));
+
+  
   // register the builtin module
   /*WbSqlIdeImpl *module= */_wbui->get_wb()->get_grt()->get_native_module<WbSqlIdeImpl>();
   _wbui->signal_main_form_change().connect(sigc::mem_fun(this, &WBContextSQLIDE::active_form_changed));
   
   // setup some builtin commands handled by ourselves for the SQL IDE
-  _wbui->get_command_ui()->add_builtin_command("query.commit", sigc::bind(sigc::ptr_fun(call_commit), this), 
-                                              sigc::bind(sigc::ptr_fun(validate_commit), this));
-  _wbui->get_command_ui()->add_builtin_command("query.rollback", sigc::bind(sigc::ptr_fun(call_rollback), this), 
-                                              sigc::bind(sigc::ptr_fun(validate_rollback), this));
-  _wbui->get_command_ui()->add_builtin_toggle_command("query.autocommit",
-                                               sigc::bind(sigc::ptr_fun(call_autocommit), this),
-                                               sigc::bind(sigc::ptr_fun(validate_autocommit), this));
+  _wbui->get_command_ui()->add_builtin_command("query.commit", sigc::bind(sigc::ptr_fun(call_commit), this));
+  _wbui->get_command_ui()->add_builtin_command("query.rollback", sigc::bind(sigc::ptr_fun(call_rollback), this)); 
+  _wbui->get_command_ui()->add_builtin_command("query.autocommit", sigc::bind(sigc::ptr_fun(call_autocommit), this));
 
-  _wbui->get_command_ui()->add_builtin_command("query.savesnippet", sigc::bind(sigc::ptr_fun(call_save_snippet), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_save_snippet), this));
+  _wbui->get_command_ui()->add_builtin_command("query.savesnippet", sigc::bind(sigc::ptr_fun(call_save_snippet), this));
 
-  _wbui->get_command_ui()->add_builtin_command("query.newtab", sigc::bind(sigc::ptr_fun(call_new_tab), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_new_tab), this));
+  _wbui->get_command_ui()->add_builtin_command("query.newtab", sigc::bind(sigc::ptr_fun(call_new_tab), this));
 
-  _wbui->get_command_ui()->add_builtin_command("query.newFile", sigc::bind(sigc::ptr_fun(call_new_file), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_new_file), this));
-  _wbui->get_command_ui()->add_builtin_command("query.newScratch", sigc::bind(sigc::ptr_fun(call_new_scratch), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_new_file), this));
-  _wbui->get_command_ui()->add_builtin_command("query.openFile", sigc::bind(sigc::ptr_fun(call_open_file), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_open_file), this));
-  _wbui->get_command_ui()->add_builtin_command("query.saveFile", sigc::bind(sigc::ptr_fun(call_save_file), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_save_file), this));
-  _wbui->get_command_ui()->add_builtin_command("query.saveFileAs", sigc::bind(sigc::ptr_fun(call_save_file_as), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_save_file), this));
-  _wbui->get_command_ui()->add_builtin_command("query.runFile", sigc::bind(sigc::ptr_fun(call_run_file), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_open_file), this));
+  _wbui->get_command_ui()->add_builtin_command("query.newQuery", sigc::bind(sigc::ptr_fun(call_new_query), this));
+  _wbui->get_command_ui()->add_builtin_command("query.newFile", sigc::bind(sigc::ptr_fun(call_new_file), this));
+  _wbui->get_command_ui()->add_builtin_command("query.openFile", sigc::bind(sigc::ptr_fun(call_open_file), this));
+  _wbui->get_command_ui()->add_builtin_command("query.saveFile", sigc::bind(sigc::ptr_fun(call_save_file), this));
+  _wbui->get_command_ui()->add_builtin_command("query.saveFileAs", sigc::bind(sigc::ptr_fun(call_save_file_as), this));
+  _wbui->get_command_ui()->add_builtin_command("query.revert", sigc::bind(sigc::ptr_fun(call_revert), this), 
+                                               sigc::bind(sigc::ptr_fun(validate_revert), this));
+
+  _wbui->get_command_ui()->add_builtin_command("query.runFile", sigc::bind(sigc::ptr_fun(call_run_file), this));
 
   _wbui->get_command_ui()->add_builtin_command("query.export", sigc::bind(sigc::ptr_fun(call_export), this), 
                                               sigc::bind(sigc::ptr_fun(validate_export), this));
   
-  _wbui->get_command_ui()->add_builtin_command("query.cancel", sigc::bind(sigc::ptr_fun(call_cancel), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_cancel), this));
+  _wbui->get_command_ui()->add_builtin_command("query.cancel", sigc::bind(sigc::ptr_fun(call_cancel), this));
 
-  _wbui->get_command_ui()->add_builtin_command("query.reconnect", sigc::bind(sigc::ptr_fun(call_reconnect), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_reconnect), this));
+  _wbui->get_command_ui()->add_builtin_command("query.reconnect", sigc::bind(sigc::ptr_fun(call_reconnect), this));
 
-  _wbui->get_command_ui()->add_builtin_command("query.refresh_physical_overview", sigc::bind(sigc::ptr_fun(call_refresh_physical_overview), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_refresh_physical_overview), this));
+  _wbui->get_command_ui()->add_builtin_command("query.refresh_physical_overview", sigc::bind(sigc::ptr_fun(call_refresh_physical_overview), this));
 
-  _wbui->get_command_ui()->add_builtin_toggle_command("query.stopOnError",
-                                               sigc::bind(sigc::ptr_fun(call_continue_on_error), this),
-                                               sigc::bind(sigc::ptr_fun(validate_continue_on_error), this));
-  _wbui->get_command_ui()->add_builtin_toggle_command("query.recordsets_are_pinned_by_default",
-                                               sigc::bind(sigc::ptr_fun(call_recordsets_are_pinned_by_default), this), 
-                                               sigc::bind(sigc::ptr_fun(validate_recordsets_are_pinned_by_default), this));
+  _wbui->get_command_ui()->add_builtin_command("query.stopOnError", sigc::bind(sigc::ptr_fun(call_continue_on_error), this));
+  _wbui->get_command_ui()->add_builtin_command("query.recordsets_are_pinned_by_default",
+                                               sigc::bind(sigc::ptr_fun(call_recordsets_are_pinned_by_default), this));
 
-  _wbui->get_command_ui()->add_builtin_command("query.explain",
-                                               sigc::bind(sigc::ptr_fun(call_explain_sql), this),
-                                               sigc::bind(sigc::ptr_fun(validate_explain_sql), this));
+  _wbui->get_command_ui()->add_builtin_command("query.explain", sigc::bind(sigc::ptr_fun(call_explain_sql), this));
   _wbui->get_command_ui()->add_builtin_command("query.explain_current_statement",
-                                               sigc::bind(sigc::ptr_fun(call_explain_current_statement), this),
-                                               sigc::bind(sigc::ptr_fun(validate_explain_current_statement), this));
+                                               sigc::bind(sigc::ptr_fun(call_explain_current_statement), this));
 
   // other commands declared in frontend code
 #ifdef _WIN32
@@ -654,24 +710,61 @@ void WBContextSQLIDE::init()
 }
 
 
-grt::ListRef<app_ToolbarItem> WBContextSQLIDE::get_toolbar_items(const std::string &name)
+void WBContextSQLIDE::finalize()
 {
-  WBContext *wb= _wbui->get_wb();
- 
-  if (name == "main")
+  std::list<Db_sql_editor::Ptr>::iterator next, ed = _open_editors.begin(); 
+  while (ed != _open_editors.end())
   {
-    if (!_toolbar.is_valid())
-      _toolbar = app_ToolbarRef::cast_from(wb->get_grt()->unserialize(make_path(wb->get_datadir(), "data/dbquery_toolbar.xml")));
-
-    return _toolbar->items();
+    next = ed;
+    ++next;
+    if (!ed->expired())
+    {
+      ed->lock()->close();
+    }
+    ed = next;
   }
-  return grt::ListRef<app_ToolbarItem>();
 }
 
 
+static void *connect_editor(Db_sql_editor::Ref editor)
+{
+  try
+  {
+    editor->connect();
+  }
+  catch (std::exception &exc)
+  {
+    return new std::string(exc.what());
+  }
+  return new std::string();
+}
+
+static bool cancel_connect_editor()
+{
+  return true;
+}
+
 Db_sql_editor::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_ConnectionRef &conn)
 {
   Db_sql_editor::Ref editor(Db_sql_editor::create(this, conn));
+  /*
+  std::string *result;
+  result = (std::string*)mforms::Utilities::run_cancellable_wait_message_task(sigc::bind(sigc::ptr_fun(connect_editor), editor),
+                                                                              sigc::ptr_fun(cancel_connect_editor));
+  if (!result)
+    throw grt::user_cancelled("cancelled");
+  
+  if (result->empty())
+    delete result;
+  else
+  {
+    std::string tmp(*result);
+    delete result;
+    throw std::runtime_error(tmp);
+  }
+  
+  editor->finish_startup();
+    */
   {
     // create entry for grt tree
     db_query_EditorRef object(_wbui->get_wb()->get_grt());
@@ -685,6 +778,28 @@ Db_sql_editor::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
     
   _open_editors.push_back(editor);
 
+  // setup auto-save for model
+  if (!_auto_save_active)
+  {
+    _auto_save_active= true;
+    int interval = _wbui->get_wb()->get_root()->options()->options().get_int("workbench:AutoSaveSQLEditorInterval", 60);
+    if (interval > 0)
+      _wbui->get_wb()->get_grt_manager()->run_every(sigc::mem_fun(this, &WBContextSQLIDE::auto_save_workspaces), interval);
+    _auto_save_interval = interval;
+
+    if (!_option_change_signal_connected)
+    {
+      _wbui->get_wb()->get_root()->options()->signal_dict_changed().connect(sigc::mem_fun(this, &WBContextSQLIDE::option_changed));
+      _option_change_signal_connected= true;
+    }
+  }
+  
+  if (::auto_save_sessions.find(conn.id()) != ::auto_save_sessions.end())
+  {
+    ::auto_save_sessions.erase(conn.id());
+    _wbui->refresh_home_connections();
+  }
+  
   return editor;
 }
 
@@ -784,7 +899,15 @@ void WBContextSQLIDE::editor_will_close(Db_sql_editor* editor)
       {
         db_query_EditorRef editor(editors[i]);
         
+        // remove all recordsets
+        while (editor->resultsets().count() > 0)
+        {
+          editor->resultsets()[0]->owner(GrtObjectRef());
+          editor->resultsets().remove(0);
+        }
+        
         editor->reset_references();
+        dynamic_cast<db_query_EditorConcreteImplData*>(editors[i]->get_data())->detach();
         
         editors.remove(i);
         break;
@@ -792,6 +915,9 @@ void WBContextSQLIDE::editor_will_close(Db_sql_editor* editor)
     }
 
     _open_editors.erase(iter);
+    
+    if (_open_editors.empty())
+      _auto_save_active = false;
   }
 }
 
@@ -813,14 +939,17 @@ void WBContextSQLIDE::update_plugin_arguments_pool(bec::ArgumentPool &args)
   if (editor_ptr)
   {
     db_query_EditorRef editor(get_grt_editor_object(editor_ptr));
-    db_query_ResultsetRef rset(editor->activeResultset());
-
-    args.add_entries_for_object("activeSQLEditor", editor);
-    args.add_entries_for_object("activeQueryBuffer", editor->activeQueryBuffer());
-    args.add_entries_for_object("", editor->activeQueryBuffer());
-
-    if (rset.is_valid())
-      args.add_entries_for_object("activeResultset", rset, "db.query.Resultset");
+    if (editor.is_valid())
+    {
+      db_query_ResultsetRef rset(editor->activeResultset());
+      
+      args.add_entries_for_object("activeSQLEditor", editor);
+      args.add_entries_for_object("activeQueryBuffer", editor->activeQueryBuffer());
+      args.add_entries_for_object("", editor->activeQueryBuffer());
+      
+      if (rset.is_valid())
+        args.add_entries_for_object("activeResultset", rset, "db.query.Resultset");
+    }
   }
 }
 
