@@ -34,21 +34,27 @@ namespace wb
 {
   class WBContextUI;
   
-  class MYSQLWBBACKEND_PUBLIC_FUNC WBContextSQLIDE
+  class MYSQLWBBACKEND_PUBLIC_FUNC WBContextSQLIDE : public sigc::trackable
   {
     WBContextUI *_wbui;
     std::list<boost::weak_ptr<Db_sql_editor> > _open_editors;
     app_ToolbarRef _toolbar;
 
+    int _auto_save_interval;
+    bool _auto_save_active;
+    bool _option_change_signal_connected;
+    
   public:
     WBContextSQLIDE(WBContextUI *wbui);
     ~WBContextSQLIDE();
     void init();
+    void finalize();
+    
+    static void detect_auto_save_files(const std::string &autosave_dir);
+    static std::map<std::string, std::string> auto_save_sessions();
     
     boost::shared_ptr<Db_sql_editor> create_connected_editor(const db_mgmt_ConnectionRef &conn);
 
-    grt::ListRef<app_ToolbarItem> get_toolbar_items(const std::string &name);
-
     WBContextUI *get_wbui() { return _wbui; }
     bec::GRTManager *get_grt_manager();
     
@@ -71,6 +77,9 @@ namespace wb
     db_query_EditorRef get_grt_editor_object(Db_sql_editor *editor);
   private:
     void active_form_changed(bec::UIForm *form);
+    
+    bool auto_save_workspaces();
+    void option_changed(grt::internal::OwnedDict*dict, bool, const std::string&key);
   };
 };
                       
