@@ -1,27 +1,27 @@
 /* 
- * (c) 2009-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
 
 #include "wb_live_schema_tree.h"
 #include "grtdb/charset_utils.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace wb;
 using namespace bec;
@@ -52,8 +52,8 @@ static std::string opt_quote_ident(const std::string &ident)
 }
 
 
-LiveSchemaTree::LiveSchemaTree(bec::GRTManager *grtm)
-: _grtm(grtm), _schemata_vector_serial(0), _is_schema_contents_enabled(true)
+LiveSchemaTree::LiveSchemaTree(grt::GRT* grt)
+: _grt(grt), _schemata_vector_serial(0), _is_schema_contents_enabled(true)
 {
 }
 
@@ -198,7 +198,7 @@ int LiveSchemaTree::count_children(const bec::NodeId &node)
   else if (node.depth() == 3)
   {
     if (ObjectNode *obj_node= get_object_node_by_index(node))
-       return obj_node->column_names.size();
+       return obj_node->columns.size();
   }
   return 0;
 }
@@ -216,6 +216,11 @@ bool LiveSchemaTree::get_field(const bec::NodeId &node, int column, std::string
   if (node.depth() == 1)
   {
     value= _schemata[node[0]].name;
+    if (column == 4242) // hack to return formatted column text in linux
+    {
+      if (value == _active_schema)
+        value = "<b>"+value+"</b>";
+    }
 #if defined(_WIN32)
     SchemaNode &snode(_schemata[node[0]]);
     if (snode.fetching)
@@ -253,7 +258,7 @@ bool LiveSchemaTree::get_field(const bec::NodeId &node, int column, std::string
   {
     if (ObjectNode *obj_node= get_object_node_by_index(node))
     {
-      value= obj_node->column_names[node[3]];
+      value= obj_node->columns[node[3]].name;
       return true;
     }
     return true;
@@ -276,30 +281,67 @@ std::string LiveSchemaTree::get_field_description(const bec::NodeId &node, int c
 
 bec::IconId LiveSchemaTree::get_field_icon(const bec::NodeId &node, int column, bec::IconSize size)
 {
+  std::string suffix;
+#ifdef _WIN32
+  suffix = "win.$.png";
+#else
+  suffix = "$.png";
+#endif
+
   if (node.depth() == 1)
   {
-    return bec::IconManager::get_instance()->get_icon_id("db.Schema.$.png", bec::Icon16);
+    return bec::IconManager::get_instance()->get_icon_id("db.Schema.side." + suffix, bec::Icon16);
   }
   else if (node.depth() == 2)
   {
+    switch (node[1])
+    {
+      case 0: return bec::IconManager::get_instance()->get_icon_id("db.Table.many.side." + suffix, bec::Icon16);
+      case 1: return bec::IconManager::get_instance()->get_icon_id("db.View.many.side." + suffix, bec::Icon16);
+      case 2: return bec::IconManager::get_instance()->get_icon_id("db.Routine.many.side." + suffix, bec::Icon16);
+    }
     return 0;
   }
   else if (node.depth() == 3)
   {
     switch (node[1])
     {
-      case 0: return bec::IconManager::get_instance()->get_icon_id("db.Table.$.png", bec::Icon16);
-      case 1: return bec::IconManager::get_instance()->get_icon_id("db.View.$.png", bec::Icon16);
-      case 2: return bec::IconManager::get_instance()->get_icon_id("db.Routine.$.png", bec::Icon16);
+      case 0: return bec::IconManager::get_instance()->get_icon_id("db.Table.side." + suffix, bec::Icon16);
+      case 1: return bec::IconManager::get_instance()->get_icon_id("db.View.side." + suffix, bec::Icon16);
+      case 2: return bec::IconManager::get_instance()->get_icon_id("db.Routine.side." + suffix, bec::Icon16);
     }
   }
   else if (node.depth() == 4)
   {
-    return bec::IconManager::get_instance()->get_icon_id("db.Column.$.png", bec::Icon16);
+    ObjectNode *obj_node= get_object_node_by_index(node);
+    if (obj_node)
+    {
+      if (obj_node->columns[node[3]].is_pk)
+        return bec::IconManager::get_instance()->get_icon_id("db.Column.pk.side." + suffix, bec::Icon16);
+      //else if (obj_node->columns[node[3]].is_fk)
+      //  return bec::IconManager::get_instance()->get_icon_id("db.Column.fk.side." + suffix, bec::Icon16);
+      else
+        return bec::IconManager::get_instance()->get_icon_id("db.Column.side." + suffix, bec::Icon16);
+    }
   }
   return 0;
 }
 
+void LiveSchemaTree::set_active_schema(const std::string &schema)
+{
+  _active_schema = schema;
+}
+
+bool LiveSchemaTree::get_field(const bec::NodeId &node, int column, int &value)
+{
+  value = 0;
+  if (node.depth() == 1)
+  {
+    if (_active_schema == _schemata[node[0]].name)
+      value = 1;
+  }
+  return true;
+}
 
 bec::NodeId LiveSchemaTree::get_child(const bec::NodeId &node, int index)
 {
@@ -428,7 +470,7 @@ void LiveSchemaTree::object_details_arrived(const SchemaNode &schema_node, const
           if (iter->name == obj_node.name)
           {
             iter->details= obj_node.details;
-            iter->column_names= obj_node.column_names;
+            iter->columns= obj_node.columns;
             iter->fetched= true;
             iter->fetching= false;
           }
@@ -443,7 +485,7 @@ void LiveSchemaTree::object_details_arrived(const SchemaNode &schema_node, const
     std::vector<ObjectNode> * obj_nodes= get_object_nodes_by_type(snode, obj_type);
     ObjectNode *onode= &(*obj_nodes)[obj_index];
     onode->details= obj_node.details;
-    onode->column_names= obj_node.column_names;
+    onode->columns= obj_node.columns;
     onode->fetched= true;
     onode->fetching= false;
   }
@@ -537,7 +579,7 @@ bool LiveSchemaTree::activate_node(const bec::NodeId &node)
   {
   case 1:
     {
-      activate_object_signal.emit("activate", Schema, _schemata[node[0]].name, "");
+      activate_object_signal.emit("activate", Schema, _schemata[node[0]].name, "");      
       return true;
     }
     break;
@@ -560,7 +602,7 @@ bool LiveSchemaTree::activate_node(const bec::NodeId &node)
     {
       if (ObjectNode *obj_node= get_object_node_by_index(node))
       {
-        std::string column_name= obj_node->column_names[node[3]];
+        std::string column_name= obj_node->columns[node[3]].name;
         column_name= opt_quote_ident(column_name);
         sql_editor_text_insert_signal(column_name);
         return true;
@@ -588,7 +630,7 @@ bec::MenuItemList LiveSchemaTree::get_popup_items_for_nodes(const std::vector<be
     {
       std::string caption= _("Select Rows");
       {
-        DictRef options= DictRef::cast_from(_grtm->get_grt()->get("/wb/options/options"));
+        DictRef options= DictRef::cast_from(_grt->get("/wb/options/options"));
         bool limit_rows= (0 != options.get_int("SqlEditor:LimitRows"));
         int limit_rows_count= options.get_int("SqlEditor:LimitRowsCount");
         if (limit_rows && (limit_rows_count <= 0))
@@ -1065,3 +1107,22 @@ void LiveSchemaTree::is_schema_contents_enabled(bool value)
 {
   _is_schema_contents_enabled= value;
 }
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Returns true if the given node corresponds to the currently active schema.
+ */
+bool LiveSchemaTree::is_highlighted(const NodeId& node)
+{
+  if (node.depth() > 1)
+    return false;
+
+  if (node[0] > -1 && node[0] < (int) _schemata.size())
+    if (_active_schema == _schemata[node[0]].name)
+      return true;
+
+  return false;
+}
+
+//--------------------------------------------------------------------------------------------------
