@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,8 +24,9 @@
 #include "grtpp.h"
 #include "grts/structs.app.h"
 
+#include "mdc_image_manager.h"
 #include "home_screen.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #include "mforms/menu.h"
 
@@ -654,10 +655,10 @@ void WorkbenchCentral::mouse_move(int x, int y)
 
 //----------------- ActionLink ---------------------------------------------------------------------
 
-ActionLink::ActionLink(const std::string& icon_name, const std::string& title, const std::string& description,
+ActionLink::ActionLink(cairo_surface_t *icon, const std::string& title, const std::string& description,
                        const grt::ValueRef &object, HomeScreenAction action, bool enabled)
 {
-  _icon= Utilities::load_icon(icon_name);
+  _icon= cairo_surface_reference(icon);
   _title= title;
   _description= description;
   _last_text_width= 0;
@@ -863,10 +864,22 @@ ActionLinkBox::~ActionLinkBox()
 
 //--------------------------------------------------------------------------------------------------
 
-void ActionLinkBox::add_link(const std::string& icon_name, const std::string& title,
+void ActionLinkBox::add_link(cairo_surface_t *icon, const std::string& title,
                              const std::string& description, const grt::ValueRef &object, HomeScreenAction action, bool enabled)
 {
-  ActionLink* link= new ActionLink(icon_name, title, description, object, action, enabled);
+  ActionLink* link= new ActionLink(icon, title, description, object, action, enabled);
+  _links.push_back(link);
+  set_layout_dirty(true);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void ActionLinkBox::add_link(const std::string &icon_name, const std::string& title,
+                             const std::string& description, const grt::ValueRef &object, HomeScreenAction action, bool enabled)
+{
+  cairo_surface_t *icon = Utilities::load_icon(icon_name);
+  ActionLink* link= new ActionLink(icon, title, description, object, action, enabled);
+  delete_surface(icon);
   _links.push_back(link);
   set_layout_dirty(true);
 }
@@ -1120,10 +1133,10 @@ void ActionLinkBox::layout()
   if (is_layout_dirty())
   {
     set_layout_dirty(false);
-    
+
     _layout_height= _top_spacing;
     _layout_width= _left_spacing + _right_spacing;
-    
+
     if (_links.size() > 0)
       _layout_height += _links.size() * ACTION_LINK_HEIGHT + (_links.size() - 1) * ACTION_LINK_SPACING;
     if (_image != NULL)
@@ -1131,11 +1144,11 @@ void ActionLinkBox::layout()
       _layout_height += BOX_IMAGE_SPACING + image_height(_image);
       _layout_width += image_width(_image);
     }
-    
+
     if (_layout_height < ACTION_LINK_HEIGHT)
       _layout_height= ACTION_LINK_HEIGHT;
     _layout_height += _bottom_spacing;
-    
+
     delete_pattern(_background);
     if (_gradient != NULL)
       _background = _gradient->create_pattern(_layout_height);
@@ -1221,10 +1234,11 @@ void GradientBox::repaint(cairo_t *cr, int areax, int areay, int areaw, int area
 
 ActionList::ActionList(HomeScreen* owner, int left_spacing, int right_spacing, WorkspaceGradient* spacer_gradient,
                        const std::string& icon_name, HomeScreenAction action)
-: Box(true), _panel(mforms::ScrollPanelDrawBackground), _panel_border(false), _icon_name(icon_name)
+: Box(true), _panel(mforms::ScrollPanelDrawBackground), _panel_border(false)
 {
   _owner= owner;
   _action= action;
+  _icon = Utilities::load_icon(icon_name);
 
   _left_spacer= new GradientBox(spacer_gradient);
   _left_spacer->set_size(left_spacing, -1);
@@ -1237,10 +1251,12 @@ ActionList::ActionList(HomeScreen* owner, int left_spacing, int right_spacing, W
   _link_box->single_click(false);
   _link_box->set_back_color(0xeef1f5);
 
-  int w, h;
+  int w = -1, h = -1;
   _link_box->get_layout_size(&w, &h);
-  _link_box->set_size(w, h);
-  
+  _link_box->set_size(-1, -1); // by mx: w and h set to -1, because without this Windows does not show 
+                               // scrollbars, and with w and h set to > 0, linux does not show
+                               // scrollbars
+
   _panel.add(_link_box);
   _panel.set_back_color("#eef1f5");
   _panel.set_visible_scrollers(true, false);
@@ -1264,6 +1280,7 @@ ActionList::ActionList(HomeScreen* owner, int left_spacing, int right_spacing, W
 
 ActionList::~ActionList()
 {
+  delete_surface(_icon);
   delete _link_box;
   delete _left_spacer;
   delete _right_spacer;
@@ -1279,14 +1296,11 @@ void ActionList::set_spacer_back_color(int color)
 
 //--------------------------------------------------------------------------------------------------
 
-void ActionList::add_entry(const std::string& title, const std::string& description, const grt::ValueRef &object)
+void ActionList::add_entry(const std::string& title, const std::string& description, const grt::ValueRef &object,
+                           cairo_surface_t *icon)
 {
-  _link_box->add_link(_icon_name, title, description, object, _action, true);
-  
-  // Explicitly set the size of the link box, as this doesn't happen automatically.
-  int w, h;
-  _link_box->get_layout_size(&w, &h);
-  _link_box->set_size(w, h);
+  _link_box->add_link(icon ? icon : _icon, title, description, object, _action, true);
+  _link_box->set_needs_repaint();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1308,11 +1322,6 @@ ActionLink* ActionList::selected()
 void ActionList::clear()
 {
   _link_box->clear();
-
-  // Explicitly set the size of the link box, as this doesn't happen automatically.
-  int w, h;
-  _link_box->get_layout_size(&w, &h);
-  _link_box->set_size(w, h);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1380,14 +1389,6 @@ Workspace::Workspace(HomeScreen* owner)
     "db_connection_34x34.png", ActionOpenConnectionFromList);
   _connections->set_spacer_back_color(0xf4f7fc);
   _connections_menu = 0;
-  /*
-  _connections_menu = new Menu();
-  _connections_menu->set_handler(sigc::mem_fun(this, &Workspace::handle_command));
-  _connections_menu->add_item("Delete Connection...", "delete_connection");
-  _connections_menu->add_separator();
-  _connections_menu->add_item("Delete All Connections...", "delete_all_connections");
-  _connections->set_context_menu(_connections_menu);
-  */
   _left_column.set_back_color("#f4f7fc");
   _left_column.add(_sql_top, false, true);
   _left_column.add(_connections, true, true);
@@ -1416,14 +1417,6 @@ Workspace::Workspace(HomeScreen* owner)
   _models= new ActionList(_owner, COLUMN_LIST_LEFT_SPACING, COLUMN_LIST_RIGHT_SPACING, _separator_gradient,
     "db_model_34x34.png", ActionOpenEERModelFromList);
   _models_menu = 0;
-  /*
-  _models_menu = new Menu();
-  _models_menu->set_handler(sigc::mem_fun(this, &Workspace::handle_command));
-  _models_menu->add_item("Remove Model File from List", "remove_file");
-  _models_menu->add_separator();
-  _models_menu->add_item("Clear List", "clear_mru_list");
-  _models->set_context_menu(_models_menu);
-   */
 
   _middle_column.set_back_color("#f4f7fc");
   _middle_column.add(_eer_top, false, true);
@@ -1457,13 +1450,6 @@ Workspace::Workspace(HomeScreen* owner)
     "db_server_34x34.png", ActionManageInstanceFromList);
   _server_instances->set_spacer_back_color(0xf4f7fc);
   _servers_menu = 0;
-  /*_servers_menu = new Menu();
-  _servers_menu->set_handler(sigc::mem_fun(this, &Workspace::handle_command));
-  _servers_menu->add_item("Delete Server Instance...", "delete_instance");
-  _servers_menu->add_separator();
-  _servers_menu->add_item("Delete All Server Instances...", "delete_all_instances");
-  _server_instances->set_context_menu(_servers_menu);
-  */
   _right_column.set_back_color("#f4f7fc");
   _right_column.add(_admin_top, false, true);
   _right_column.add(_server_instances, true, true);
@@ -1499,18 +1485,22 @@ Workspace::~Workspace()
 
 //--------------------------------------------------------------------------------------------------
 
-void Workspace::add_list_entry(HomeScreenListType list, const std::string& title, const std::string& description, const grt::ValueRef &object)
+void Workspace::add_list_entry(HomeScreenListType list, const std::string& title, 
+                               const std::string& description, const grt::ValueRef &object,
+                               const std::string &icon_name)
 {
+  cairo_surface_t *icon = icon_name.empty() ? 0 : mdc::ImageManager::get_instance()->get_image(icon_name);
+
   switch (list)
   {
     case HomeScreenSQLDevList:
-      _connections->add_entry(title, description, object);
+      _connections->add_entry(title, description, object, icon);
       break;
     case HomeScreenModellingList:
-      _models->add_entry(title, description, object);
+      _models->add_entry(title, description, object, icon);
       break;
     case HomeScreenAdministrationList:
-      _server_instances->add_entry(title, description, object);
+      _server_instances->add_entry(title, description, object, icon);
       break;
   }
 }
@@ -1556,21 +1546,11 @@ void Workspace::clear_list(HomeScreenListType list)
 static Menu *mforms_menu_from_menu_item_list(const bec::MenuItemList &menu_items)
 {
   Menu *menu = new Menu();
-  
-  for (bec::MenuItemList::const_iterator item = menu_items.begin(); item != menu_items.end(); ++item)
-  {
-    if (item->type == bec::MenuAction)
-    {
-      int i = menu->add_item(item->caption, item->name);
-      menu->set_item_enabled(i, item->enabled);
-    }
-    else if (item->type == bec::MenuSeparator)
-      menu->add_separator();
-  }
-
+  menu->add_items_from_list(menu_items);
   return menu;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void Workspace::set_context_menu_items(HomeScreenListType list, const bec::MenuItemList &menu_items)
 {
@@ -1618,10 +1598,12 @@ void Workspace::handle_command(const std::string& command, HomeScreenListType li
 
 //----------------- HomeScreen ---------------------------------------------------------------------
 
-HomeScreen::HomeScreen(grt::GRT* grt)
+#include "workbench/wb_command_ui.h"
+
+HomeScreen::HomeScreen(CommandUI *cmdui, grt::GRT *grt)
 : AppView(false, "home", true), _workbench_central_section(true, "Workbench Central", true),
-_workspace_section(false, "Workspace"), _callback(NULL)
-{
+_workspace_section(false, "Workspace"), _callback(NULL), _user_data(0), _auto_save_warning(0)
+{  
   // TODO: cannot set a minimum size here as this doesn't work on OS X (box layouter is buggy).
   //set_size(1024, 671); // Minimum size for this entire page.
   _central_content= new WorkbenchCentral(this, grt);
@@ -1635,18 +1617,88 @@ _workspace_section(false, "Workspace"), _callback(NULL)
   _workspace_section.set_content(_workspace_content);
   _workspace_section.set_name("workspace section");
   add(&_workspace_section, true, true);
+  
+  _menubar = cmdui->create_menubar_for_context(WB_CONTEXT_HOME_GLOBAL);
 }
 
 //--------------------------------------------------------------------------------------------------
 
 HomeScreen::~HomeScreen()
 {
+  delete _auto_save_warning;
   delete _central_content;
   delete _workspace_content;
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void HomeScreen::set_wb_central_expanded(bool flag)
+{
+  _workbench_central_section.set_expanded(flag);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool HomeScreen::get_wb_central_expanded()
+{
+  return _workbench_central_section.get_expanded();
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void HomeScreen::show_autosave_warning(int model, int sqleditor)
+{
+  if (model == 0 && sqleditor == 0)
+  {
+    if (_auto_save_warning)
+    {
+      remove(_auto_save_warning);
+      delete _auto_save_warning;
+      _auto_save_warning= 0;
+    }
+  }
+  else if (!_auto_save_warning)
+  {
+    Box *box = _auto_save_warning = manage(new Box(true));
+    box->set_padding(12);
+    box->set_spacing(12);
+    
+    ImageBox *icon = manage(new ImageBox());
+    icon->set_image("warning_icon.png");
+    box->add(icon, false, true);
+    
+    Box *vb = manage(new Box(false));
+    vb->set_padding(8);
+    Label *label = manage(new Label());
+    label->set_style(BoldStyle);
+    label->set_color("#dd0000");
+    label->set_text(_("Orphaned document changes detected"));
+    vb->add(label, true, true);
+    if (model > 0 && sqleditor == 0)
+      label = manage(new Label(_("Unsaved changes in a model from a previous run of Workbench were detected.\n"
+      "To restore them open the corresponding document and save it. Otherwise these changes will be lost.")));
+    else if (model == 0 && sqleditor > 0)
+      label = manage(new Label(_("Unsaved changes of an SQL Editor workspace were found.\n"
+      "To restore them, open the corresponding SQL connection and save the SQL data. Otherwise these changes will be lost.")));
+    else
+      label = manage(new Label(_("There are unsaved changes in SQL Editors and models from a previous run of Workbench.\n"
+      "To restore them, open the corresponding document and SQL connections and save the documents.")));
+    label->set_style(SmallStyle);
+    vb->add(label, false, true);
+    box->add(vb, true, true);
+    
+    Button *button = manage(new Button());
+    button->set_text(_("Dismiss"));
+    box->add_end(button, false, true);
+    box->set_back_color("#eeeef7");
+    add(box, false, true);
+    
+    button->signal_clicked().connect(sigc::bind(sigc::mem_fun(this, &Box::remove), box));
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void HomeScreen::set_callback(home_screen_action_callback callback, void* user_data)
 {
   _callback= callback;
@@ -1680,7 +1732,7 @@ void HomeScreen::add_list_entry(HomeScreenListType list, const std::string& titl
   if (list == HomeScreenStarters)
     _central_content->add_starter(icon_name, object);
   else
-    _workspace_content->add_list_entry(list, title, description, object);
+    _workspace_content->add_list_entry(list, title, description, object, icon_name);
 }
 
 //--------------------------------------------------------------------------------------------------
