@@ -140,7 +140,7 @@ namespace wb
   class ActionLink
   {
   public:
-    ActionLink(const std::string& icon_name, const std::string& title, const std::string& description,
+    ActionLink(cairo_surface_t *icon, const std::string& title, const std::string& description,
                const grt::ValueRef &object, HomeScreenAction action, bool enabled);
     ~ActionLink();
     
@@ -201,8 +201,11 @@ namespace wb
       int top_spacing, int bottom_spacing, int left_spacing, int right_spacing);
     ~ActionLinkBox();
     
-    void add_link(const std::string& icon_name, const std::string& title, const std::string& description,
+    void add_link(cairo_surface_t *icon, const std::string& title, const std::string& description,
                   const grt::ValueRef &object, HomeScreenAction action, bool enabled);
+    void add_link(const std::string &icon_name, const std::string& title, const std::string& description,
+                  const grt::ValueRef &object, HomeScreenAction action, bool enabled);
+
     void clear();
     void set_back_color(double red, double green, double blue);
     void set_back_color(int color);
@@ -280,7 +283,8 @@ namespace wb
     ~ActionList();
 
     void set_spacer_back_color(int color);
-    void add_entry(const std::string& title, const std::string& description, const grt::ValueRef &object);
+    void add_entry(const std::string& title, const std::string& description, const grt::ValueRef &object,
+                   cairo_surface_t *icon=0);
     void select(int index);
     ActionLink* selected();
     void clear();
@@ -292,7 +296,7 @@ namespace wb
     GradientBox* _left_spacer;
     GradientBox* _right_spacer;
     ActionLinkBox* _link_box;
-    std::string _icon_name; // The icon to be used for all entries.
+    cairo_surface_t *_icon; // The icon to be used for all entries.
     HomeScreenAction _action;      // One action for all.
   };
   
@@ -305,7 +309,8 @@ namespace wb
     Workspace(HomeScreen* owner);
     ~Workspace();
     
-    void add_list_entry(HomeScreenListType list, const std::string& title, const std::string& description, const grt::ValueRef &object);
+    void add_list_entry(HomeScreenListType list, const std::string& title, const std::string& description, 
+                        const grt::ValueRef &object, const std::string &icon_name);
     void select_list_entry(HomeScreenListType list, int index);
     void clear_list(HomeScreenListType list);
     
@@ -336,6 +341,8 @@ namespace wb
     WorkspaceGradient* _separator_gradient;
   };
   
+  class CommandUI;
+
   /**
    * This class implements the main (home) screen in MySQL Workbench, containing
    * workbench central and the workspace sections.
@@ -350,10 +357,18 @@ namespace wb
     home_screen_action_callback _callback;
     void* _user_data;
 
+    mforms::Box *_auto_save_warning;
   public:
-    HomeScreen(grt::GRT* grt);
+    HomeScreen(CommandUI *cmdui, grt::GRT *grt);
     ~HomeScreen();
     
+    virtual mforms::MenuBar *get_menubar() { return _menubar; }
+    
+    void set_wb_central_expanded(bool flag);
+    bool get_wb_central_expanded();
+    
+    void show_autosave_warning(int model, int sqleditor);
+    
     sigc::slot<void, HomeScreenListType, grt::ValueRef, std::string> handle_context_menu;
     
     void set_context_menu_items(HomeScreenListType list, const bec::MenuItemList &menu_items);
