@@ -1,20 +1,20 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 
@@ -28,7 +28,7 @@
 #include <vector>
 #endif
 
-#include "ui_form.h"
+#include "base/ui_form.h"
 #include "grt/grt_manager.h"
 #include "grt/plugin_manager.h"
 #include "grt/icon_manager.h"
@@ -49,8 +49,8 @@
 
 #define APP_MAJOR_NUMBER 5
 #define APP_MINOR_NUMBER 2
-#define APP_RELEASE_NUMBER 30
-#define APP_REVISION_NUMBER 6790
+#define APP_RELEASE_NUMBER 33
+#define APP_REVISION_NUMBER 7506
 
 
 
@@ -79,7 +79,6 @@ namespace wb {
     RefreshSchema,
     RefreshNewDiagram,
     RefreshSelection,
-    RefreshToolbar,
     RefreshCloseEditor, // argument: object-id (close all if "")
     RefreshFindOutput,
     
@@ -93,7 +92,6 @@ namespace wb {
     RefreshDocument,
     RefreshCloseDocument,
     RefreshZoom,
-    RefreshMenubar,
     RefreshTimer,
 
     RefreshFinishEdits // Force all ongoing edit operations (eg in TreeView cells) to be commited
@@ -136,8 +134,6 @@ namespace wb {
   #define WB_CONTEXT_QUERY "dbquery"
   
   // basic toolbar names
-  #define WB_TOOLBAR_MAIN    "main"
-  #define WB_TOOLBAR_TOOLS   "tools"
   #define WB_TOOLBAR_OPTIONS "options"
 
   // main view types
@@ -187,7 +183,7 @@ namespace wb {
     sigc::slot<void, std::string> perform_command;
 
     // Create a new diagram.
-    sigc::slot<mdc::CanvasView*, std::string, std::string> create_diagram;
+    sigc::slot<mdc::CanvasView*, const model_DiagramRef&> create_diagram;
     // Destroy a previously created canvas view
     sigc::slot<void, mdc::CanvasView*> destroy_view;
     // Signals the current view has been changed
@@ -205,10 +201,6 @@ namespace wb {
     sigc::slot<void, RefreshType, std::string, NativeHandle> refresh_gui;
     sigc::slot<void,bool> lock_gui;
 
-    // Show a Popup Menu on the canvas
-    // Args: items, x, y
-    sigc::slot<void, bec::MenuItemList*, int, int> popup_menu;
-
     // Closes the application
     sigc::slot<void> quit_application;
   };
@@ -451,11 +443,11 @@ namespace wb {
     std::string _filename;
     // only used for comparing pointers
     grt::UndoAction *_save_point;
-    grt::UndoAction *_auto_save_point;
-    mdc::Timestamp _last_auto_save_time;
-
+    
     TunnelManager *_tunnel_manager;
     
+    sigc::signal<void> _signal_app_closing;
+    
     void delete_canvas_item(mdc::CanvasItem *item);
 
     grt::ListRef<app_PaperType> get_paper_types(grt::GRT *grt);
@@ -477,10 +469,13 @@ namespace wb {
     void load_app_options(bool update);
     
     bool auto_save_document();
+    std::string get_auto_save_dir();
   public:
     void save_app_options();
     void add_recent_file(const std::string &file);
 
+    sigc::signal<void> &signal_app_closing() { return _signal_app_closing; }
+    
   protected:
     void load_app_state();
     void save_app_state();
@@ -515,6 +510,8 @@ namespace wb {
     std::string request_connection_password(const db_mgmt_ConnectionRef &conn, bool force_asking);
     bool show_error(const std::string& title, const std::string& message);
 
+    void determine_opengl_support(WBOptions* command_line_options);
+
   public: // front end callbacks
     sigc::slot<std::string,std::string,std::string,std::string> show_file_dialog;
 
@@ -524,7 +521,7 @@ namespace wb {
 
     sigc::slot<void,std::string,void*> show_gui_plugin;
 
-    sigc::slot<mdc::CanvasView*, std::string, std::string> create_diagram;
+    sigc::slot<mdc::CanvasView*, const model_DiagramRef&> create_diagram;
     sigc::slot<void, mdc::CanvasView*> destroy_view;
     sigc::slot<void, mdc::CanvasView*> switched_view;
 
@@ -536,8 +533,6 @@ namespace wb {
     sigc::slot<void, RefreshType, std::string, NativeHandle> refresh_gui;
     sigc::slot<void,bool> lock_gui;
 
-    sigc::slot<void, bec::MenuItemList*, int, int> popup_menu;
-
     sigc::slot<void, std::string> perform_command;
     
     sigc::slot<void> quit_application;
