@@ -42,7 +42,7 @@
 
 #include "mforms/utilities.h"
 #include "mforms/app.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace wb;
 using namespace bec;
@@ -109,6 +109,10 @@ bool WBContextUI::init(WBFrontendCallbacks *callbacks, WBOptions *options)
   // has to be called after WBContext is initialized
   add_backend_builtin_commands();
   
+  // look for auto-save files (must be done at startup to not confuse with autosaves created by ourself)
+  WBContextModel::detect_auto_save_files(_wb->get_auto_save_dir());
+  WBContextSQLIDE::detect_auto_save_files(_wb->get_auto_save_dir());
+  
   // Finally show initial home screen.
   show_home_screen();
   
@@ -177,18 +181,6 @@ void WBContextUI::perform_quit()
 }
 
 
-void WBContextUI::refresh_toolbar(const std::string &toolbar)
-{
-  _wb->request_refresh(RefreshToolbar, toolbar, 0);
-}
-
-
-void WBContextUI::refresh_menus()
-{
-  _wb->request_refresh(RefreshMenubar, "", 0);
-}
-
-
 void WBContextUI::reset()
 {
   if (!dynamic_cast<OverviewBE*>(_active_form))
@@ -217,10 +209,6 @@ void WBContextUI::history_changed()
     _wb->request_refresh(RefreshDocument, "", (NativeHandle)0);
 
   _last_unsaved_changes_state= _wb->has_unsaved_changes();
-
-  // Signal that the main menu and the main toolbar should be refreshed.
-  _wb->request_refresh(RefreshMenubar, "", (NativeHandle)0);
-  _wb->request_refresh(RefreshToolbar, "main", (NativeHandle)0);
 }
 
 void WBContextUI::update_current_diagram(bec::UIForm *form)
@@ -239,26 +227,11 @@ void WBContextUI::overview_selection_changed()
   if (get_active_main_form() == get_physical_overview())
   {
     _wb->request_refresh(RefreshSelection, "", (NativeHandle)get_physical_overview()->get_frontend_data());
-
-    _wb->request_refresh(RefreshMenubar, "");
+    _command_ui.signal_validate_edit_menu_items().emit();
   }
 }
 
 
-void WBContextUI::view_selection_changed(bool, mdc::CanvasItem *item, ModelDiagramForm *view)
-{
-  if (get_active_main_form() == view)
-  {
-///    if (_layer_tree)
-///      _layer_tree->refresh_selection_status();
-
-    _wb->request_refresh(RefreshMenubar, "");
-    _wb->request_refresh(RefreshSelection, "", 0);
-  }
-}
-
-
-
 void WBContextUI::load_app_options(bool update)
 {
   if (!update)
@@ -452,8 +425,6 @@ void WBContextUI::activate_figure(const grt::ValueRef &value)
 void WBContextUI::form_changed()
 {  
   _wb->request_refresh(RefreshZoom, "", (NativeHandle)0);
-  _wb->request_refresh(RefreshMenubar, "", (NativeHandle)0);
-  _wb->request_refresh(RefreshToolbar, "", (NativeHandle)0);
   
   if (_active_form != NULL && _active_form == _home_screen)
   {
@@ -525,7 +496,7 @@ bec::ValueInspectorBE *WBContextUI::create_inspector_for_selection(std::vector<s
   {
     if (selection.count()==1)
     {
-			GrtObjectRef obj= selection[0];
+      GrtObjectRef obj= selection[0];
       if (obj.is_valid() && obj->has_member(name_mem_name))
       {
         items.push_back(strfmt("%s: %s", obj.get_string_member(name_mem_name).c_str(), obj.get_metaclass()->get_attribute("caption").c_str()));
@@ -659,6 +630,7 @@ std::string WBContextUI::get_description_for_selection(bec::UIForm *form, grt::L
   activeObjList= grt::ListRef<model_Object>(selection.get_grt());
 
   std::string comment_mem_name("comment");
+  std::string descr_mem_name("description");
 
   if (selection.is_valid() && selection.count()>0)
   {
@@ -683,6 +655,19 @@ std::string WBContextUI::get_description_for_selection(bec::UIForm *form, grt::L
 
         first= false;
       }
+      else if (!dbobject.is_valid() && figure.is_valid() && figure.has_member(descr_mem_name))
+      {
+        activeObjList.insert(figure);
+        
+        items.push_back(strfmt("%s: %s", figure->name().c_str(), figure->get_metaclass()->get_attribute("caption").c_str()));
+        std::string comment= figure.get_string_member(descr_mem_name);
+        if (first)
+          res= comment;
+        else if (0 != res.compare(comment))
+          res= _("<Multiple Items>\nThat means not all selected items have same comment.\nBeware applying changes will override comments for all selected objects.");
+        
+        first= false;        
+      }
     }
     if (items.size() > 1)
       items.insert(items.begin(), _("Multiple Items"));
@@ -697,22 +682,33 @@ void WBContextUI::set_description_for_selection(const grt::ListRef<GrtObject> &o
   if (objList.is_valid() && objList.count()>0)
   {
     std::string comment_mem_name("comment");
-
+    std::string descr_mem_name("description");
+    
     grt::AutoUndo undo(_wb->get_grt());
     
     if (objList.count()==1)
     {
       GrtObjectRef obj(objList[0]);
-      if (obj.is_valid() && obj.has_member(comment_mem_name))
-        obj.set_member(comment_mem_name, grt::StringRef(val));
+      if (obj.is_valid())
+      {
+        if (obj.has_member(comment_mem_name))
+          obj.set_member(comment_mem_name, grt::StringRef(val));
+        else if (obj.has_member(descr_mem_name))
+          obj.set_member(descr_mem_name, grt::StringRef(val));
+      }
     }
     else
     {
       for (size_t c= objList.count(), i= 0; i < c; i++)
       {
         GrtObjectRef obj(objList[i]);
-        if (obj.is_valid() && obj.has_member(comment_mem_name))
-          obj.set_member(comment_mem_name, grt::StringRef(val));
+        if (obj.is_valid())
+        {
+          if (obj.has_member(comment_mem_name))
+            obj.set_member(comment_mem_name, grt::StringRef(val));
+          else if (obj.has_member(descr_mem_name))
+            obj.set_member(descr_mem_name, grt::StringRef(val));
+        }
       }
     }
 
@@ -723,7 +719,7 @@ void WBContextUI::set_description_for_selection(const grt::ListRef<GrtObject> &o
 
 DiagramOptionsBE *WBContextUI::create_diagram_options_be(mdc::CanvasView *view)
 {
-  model_DiagramRef model_diagram(get_wb()->get_model_context()->get_active_model_diagram(false));
+  model_DiagramRef model_diagram(get_wb()->get_model_context()->get_active_model_diagram(true));
   
   if (model_diagram.is_valid())
     return new DiagramOptionsBE(view, model_diagram, _wb);
@@ -970,14 +966,26 @@ void WBContextUI::set_wb_options_value(const std::string &model, const std::stri
   switch (type)
   {
   case grt::DoubleType:
-      options.set(key, grt::DoubleRef(atof(value.c_str())));
+    {
+      grt::DoubleRef v(atof(value.c_str()));
+      if (options.get_double(key) != *v)
+        options.set(key, v);
       break;
+    }
   case grt::IntegerType:
-      options.set(key, grt::IntegerRef(atoi(value.c_str())));
+    {
+      grt::IntegerRef v(atoi(value.c_str()));
+      if (options.get_int(key) != *v)
+        options.set(key, v);
       break;
+    }
   case grt::StringType:
-      options.set(key, grt::StringRef(value));
+    {
+      grt::StringRef v(value);
+      if (options.get_string(key) != *v)
+        options.set(key, v);
       break;
+    }
   default:
     throw std::runtime_error("No valid grt type specified when setting options value.");
   }
