@@ -74,16 +74,13 @@ namespace wb {
 
     bool request_quit();
     void perform_quit();
+    bool is_quitting() { return _quitting; };
     
     void reset();
 
     WBContext *get_wb() { return _wb; }
     CommandUI *get_command_ui() { return &_command_ui; }
 
-    // 
-    void refresh_menus();
-    void refresh_toolbar(const std::string &toolbar);
-    
     // form/panel backends
     PhysicalOverviewBE *get_physical_overview();
 
@@ -168,7 +165,7 @@ namespace wb {
 
     void overview_selection_changed();
     friend class WBContextModel; // for these callbacks, remove once everythign is moved there
-    void view_selection_changed(bool, mdc::CanvasItem*, ModelDiagramForm *view);
+
     static void *form_destroyed(void *data);
 
     void activate_find_result(const GrtObjectRef &location, const GrtObjectRef &object, const std::string &text);
@@ -206,7 +203,7 @@ namespace wb {
     OutputView *_output_view;
 
     CommandUI _command_ui;
-
+        
     // form management
     bec::UIForm *_active_form;
     bec::UIForm *_active_main_form;
