@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,8 +22,9 @@
 
 #include "wb_context_ui.h"
 
-#include "file_functions.h"
-#include "string_utilities.h"
+#include "base/file_functions.h"
+#include "base/file_utilities.h"
+#include "base/string_utilities.h"
 
 #include "server_instance_editor.h"
 #include "grtui/grtdb_connection_editor.h"
@@ -33,6 +34,10 @@
 #include "edit_table_data_wizard.h"
 
 
+#define MODEL_AUTOSAVED_ICON_FILE "db_model_warning.png"
+#define SQLEDITOR_AUTOSAVED_ICON_FILE "db_connection_warning.png"
+
+
 using namespace bec;
 using namespace wb;
 using namespace base;
@@ -50,11 +55,13 @@ std::string get_server_info(db_mgmt_ServerInstanceRef instance)
       system.c_str());
   else
   {
-    if (instance->serverInfo().get_int("windowsAdmin") || system == "Windows")
+    if (instance->serverInfo().get_int("windowsAdmin")/* || system == "Windows"*/)
     {
       std::string host = instance->loginInfo().get_string("wmi.hostName");
       if (host == "localhost" || host.empty() || host == "127.0.0.1")
+      {
         text = "Local  Type: Windows";
+      }
       else
         text = "Host: " + host + "  Type: Windows";
     }
@@ -64,7 +71,9 @@ std::string get_server_info(db_mgmt_ServerInstanceRef instance)
         instance->connection()->parameterValues().get_string("hostName") : "Invalid";
 
       if (host == "localhost" || host.empty() || host == "127.0.0.1")
+      {
         text = strfmt("Local  Type: %s", system.c_str());
+      }
       else
         text = strfmt("Host: %s  Type: DB Only", host.c_str());
     }
@@ -83,6 +92,15 @@ static bool on_home_closing(WBContextUI* owner)
 
 //--------------------------------------------------------------------------------------------------
 
+static void save_state(WBContext *wb, HomeScreen *home)
+{
+  if (!home->get_wb_central_expanded())
+    wb->save_state("WBCentralCollapsedInVersion", "home",
+                   strfmt("%i.%i.%i", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER));
+  else
+    wb->save_state("WBCentralCollapsedInVersion", "home", std::string(""));  
+}
+
 /**
  * Creates the main home screen (Workbench Central, Workspace) if not yet done and docks it to 
  * the main application window.
@@ -90,14 +108,21 @@ static bool on_home_closing(WBContextUI* owner)
 void WBContextUI::show_home_screen()
 {
   if (_home_screen == NULL)
-  {
-    _home_screen= mforms::manage(new HomeScreen(_wb->get_grt()));
+  {    
+    _home_screen= mforms::manage(new HomeScreen(&_command_ui, _wb->get_grt()));
     _home_screen->set_on_close(sigc::bind(sigc::ptr_fun(on_home_closing), this));
     _home_screen->set_callback((home_screen_action_callback)&WBContextUI::home_action_callback, this);
     _home_screen->handle_context_menu = sigc::mem_fun(this, &WBContextUI::handle_home_context_menu);
+    
+    _wb->signal_app_closing().connect(sigc::bind(sigc::ptr_fun(save_state), _wb, _home_screen));
+
     // setup context menus
     bec::MenuItemList items, pitems;
     bec::MenuItem item, sep;
+    
+    std::string value = _wb->read_state("WBCentralCollapsedInVersion", "home", std::string());
+    if (value == strfmt("%i.%i.%i", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER))
+      _home_screen->set_wb_central_expanded(false);
 
     sep.type = bec::MenuSeparator;
     item.name= "open_connection";
@@ -203,7 +228,7 @@ void WBContextUI::show_home_screen()
  * Called when the home screen is closed by the UI. We have to clear our reference then.
  */
 void WBContextUI::home_screen_closing()
-{
+{  
   _home_screen= NULL;
 }
 
@@ -780,6 +805,8 @@ void WBContextUI::refresh_home_connections(int selected)
 {
   if (!_home_screen) return;
   grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+
+  std::map<std::string, std::string> auto_save_files = WBContextSQLIDE::auto_save_sessions();
   
   _home_screen->clear_list(HomeScreenSQLDevList);
 
@@ -822,10 +849,17 @@ void WBContextUI::refresh_home_connections(int selected)
     std::string text = strfmt("User: %s  Host: %s %s  %s", dict.get_string("userName").c_str(), 
                               host_entry.c_str(), tunnel.c_str(), schema.c_str());
     
-    _home_screen->add_list_entry(HomeScreenSQLDevList, (*inst)->name(), text, *inst);
+    if (auto_save_files.find((*inst)->id()) != auto_save_files.end())
+      _home_screen->add_list_entry(HomeScreenSQLDevList, std::string((*inst)->name())+" (autosaved)", text, *inst,
+                                   SQLEDITOR_AUTOSAVED_ICON_FILE);
+    else
+      _home_screen->add_list_entry(HomeScreenSQLDevList, (*inst)->name(), text, *inst);
   }
   
   _home_screen->select_list_entry(HomeScreenSQLDevList, selected);
+  
+  _home_screen->show_autosave_warning((int)WBContextModel::auto_save_files().size(), 
+                                      (int)auto_save_files.size());
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -836,12 +870,78 @@ void WBContextUI::refresh_home_models(int selected)
     return;
   _home_screen->clear_list(HomeScreenModellingList);
   
-  for (grt::StringListRef::const_iterator end = _wb->get_root()->options()->recentFiles().end(),
-       f = _wb->get_root()->options()->recentFiles().begin(); f != end; ++f)
+  std::map<std::string, std::string> auto_save_models(WBContextModel::auto_save_files());
+
+  // find list of auto-saved models that are not in the list of recent files
+  for (std::map<std::string, std::string>::const_iterator iter = auto_save_models.begin();
+       iter != auto_save_models.end(); ++iter)
+  {
+    bool found = false;
+    for (grt::StringListRef::const_iterator end = _wb->get_root()->options()->recentFiles().end(),
+         f = _wb->get_root()->options()->recentFiles().begin(); f != end; ++f)
+    {
+      if (*f == iter->first || strcmp(g_basename((*f).c_str()), iter->first.c_str()) == 0)
+      {
+        found = true;
+        break;
+      }
+    }
+    if (!found)
+      _home_screen->add_list_entry(HomeScreenModellingList, iter->first, "Unsaved Model", grt::StringRef(iter->second),
+                                   MODEL_AUTOSAVED_ICON_FILE);
+  }
+  
+  // Create list entries for each recently opened file.
+  // First count how many there are so we can add examples if the list is empty.
+  int model_count = 0;
+  grt::StringListRef recentFiles(_wb->get_root()->options()->recentFiles());
+  for (grt::StringListRef::const_iterator end = recentFiles.end(), f = recentFiles.begin();
+    f != end; ++f)
+  {
+    if (g_str_has_suffix((*f).c_str(), ".mwb"))
+      model_count++;
+  }
+
+  // If there is no entry in the MRU list then scan the examples folder for the Sakila model
+  // file and add this as initial entry, so that a new user has an easy start.
+  if (model_count == 0)
+  {
+    std::list<std::string> examples_paths;
+#ifdef _WIN32
+    examples_paths.push_back(mforms::Utilities::get_special_folder(mforms::WinProgramFilesX86) +
+      "\\MySQL\\Samples and Examples 5.5");
+    examples_paths.push_back(mforms::Utilities::get_special_folder(mforms::WinProgramFiles) +
+      "\\MySQL\\Samples and Examples 5.5");
+#endif
+    for (std::list<std::string>::const_iterator path_iterator = examples_paths.begin();
+      path_iterator != examples_paths.end(); path_iterator++)
+    {
+      if (g_file_test(path_iterator->c_str(), G_FILE_TEST_IS_DIR))
+      {
+        std::string pattern = make_path(*path_iterator, "*.mwb");
+        std::list<std::string> sample_model_files = base::scan_for_files_matching(pattern, true);
+        for (std::list<std::string>::const_iterator iterator = sample_model_files.begin();
+          iterator != sample_model_files.end(); iterator++)
+          recentFiles->insert_checked(grt::StringRef(*iterator));
+      }
+    }
+  }
+
+  for (grt::StringListRef::const_iterator end = recentFiles.end(), f = recentFiles.begin();
+    f != end; ++f)
   {
     if (!g_str_has_suffix((*f).c_str(), ".mwb"))
       continue;
     
+    bool has_auto_save;
+    
+    // check if there's an autosave of the model
+    if (auto_save_models.find(*f) != auto_save_models.end()
+        || auto_save_models.find(g_basename((*f).c_str())) != auto_save_models.end())
+      has_auto_save = true;
+    else
+      has_auto_save = false;
+    
 #ifdef _WIN32
     struct _stat stbuf;
     
@@ -859,7 +959,8 @@ void WBContextUI::refresh_home_models(int selected)
       char *file = g_path_get_basename((*f).c_str());
       if (g_str_has_suffix(file, ".mwb"))
         file[strlen(file)-4]= 0;
-      _home_screen->add_list_entry(HomeScreenModellingList, file, text, *f);
+      _home_screen->add_list_entry(HomeScreenModellingList, file, text, *f,
+                                   has_auto_save ? MODEL_AUTOSAVED_ICON_FILE : "");
       g_free(file);
     }
 #else
@@ -875,12 +976,17 @@ void WBContextUI::refresh_home_models(int selected)
       char *file = g_path_get_basename((*f).c_str());
       if (g_str_has_suffix(file, ".mwb"))
         file[strlen(file)-4]= 0;
-      _home_screen->add_list_entry(HomeScreenModellingList, file, text, *f);
+      _home_screen->add_list_entry(HomeScreenModellingList, file, text, *f,
+                                   has_auto_save ? MODEL_AUTOSAVED_ICON_FILE : "");
       g_free(file);
     }
 #endif
   }
+
   _home_screen->select_list_entry(HomeScreenModellingList, selected);
+    
+  _home_screen->show_autosave_warning((int)WBContextModel::auto_save_files().size(), 
+                                      (int)WBContextSQLIDE::auto_save_sessions().size());
 }
 
 //--------------------------------------------------------------------------------------------------
