@@ -30,10 +30,11 @@
 #include <glib/gstdio.h>
 
 #include "grtpp.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
+#include "base/file_utilities.h"
 #include "mforms/utilities.h"
 
-#include "util_public_interface.h"
+#include "base/util_public_interface.h"
 
 #include "grt/common.h"
 #include "grt/grt_manager.h"
@@ -41,9 +42,6 @@
 #include "grts/structs.workbench.h"
 
 
-#define MAIN_DOCUMENT_NAME "document.mwb.xml"
-#define MAIN_DOCUMENT_AUTOSAVE_NAME "document-autosave.mwb.xml"
-
 #define DOCUMENT_FORMAT "MySQL Workbench Model"
 // version history:
 // switched to 1.1.6 in 5.0.20
@@ -52,7 +50,8 @@
 // switched from 1.2.0 to 1.3.0 in 5.1.7
 // updated to 1.4.0 in 5.2.0
 // updated to 1.4.1 in 5.2.19
-#define DOCUMENT_VERSION "1.4.1"
+// updated to 1.4.2 in 5.2.33
+#define DOCUMENT_VERSION "1.4.2"
 
 #define ZIP_FILE_FORMAT "1.0"
 
@@ -92,6 +91,7 @@ static std::string fmt_ziperror(zip *z, const char *msg)
   return strfmt("%s: %s", msg, buffer);
 }*/
 
+const std::string ModelFile::lock_filename("lock");
 
 void ModelFile::copy_file(const std::string &srcfile, const std::string &destfile)
 {
@@ -184,7 +184,17 @@ std::string ModelFile::create_document_dir(const std::string &dir, const std::st
 #endif
     }
     else
-      break;
+    {
+      try 
+      {
+        _temp_dir_lock = new base::LockFile(bec::make_path(path, lock_filename.c_str()));
+        break;
+      }
+      catch (const base::file_locked_error)
+      {
+        // continue
+      }
+    }
     sprintf(s, "d%i", ++i);
   }
 
@@ -201,7 +211,7 @@ struct AutoLock
 
 
 ModelFile::ModelFile(const std::string &tmpdir)
-: _dirty(false)
+: _temp_dir_lock(0), _dirty(false)
 {
   _temp_dir= tmpdir;
 
@@ -261,7 +271,6 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
   }
 
   gchar *basename= g_path_get_basename(path.c_str());
-  bool recover= false;
   
   // do some sanity checks
   if (strlen(basename) == 0)
@@ -269,11 +278,38 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
     g_free(basename);
     throw std::runtime_error("Invalid path "+path);
   }
-  std::string auto_save_dir = std::string(_temp_dir).append("/").append(basename).append("d");
-  if (g_file_test(auto_save_dir.c_str(), G_FILE_TEST_EXISTS))
+
+  std::string auto_save_dir = bec::make_path(_temp_dir, basename).append("d"); // default
+  std::list<std::string> possible_autosaves = base::scan_for_files_matching(auto_save_dir+"*");
+  for (std::list<std::string>::const_iterator d = possible_autosaves.begin(); d != possible_autosaves.end(); ++d)
+  {
+    gchar *path;
+    gsize length;
+
+    // check if this autosave is active/in use
+    if (base::LockFile::check(bec::make_path(*d, lock_filename.c_str())) != base::LockFile::NotLocked)
+      continue;
+
+    if (g_file_get_contents(bec::make_path(*d, "real_path").c_str(), &path, &length, NULL))
+    {
+      // ensure this autosave is for the model being opened
+      if (std::string(path, length) == path)
+      {
+        auto_save_dir = *d;
+        g_free(path);
+        break;
+      }
+      g_free(path);
+    }
+    else // if the autosave dir has no file, then maybe it's from a version that still didn't save the path info
+      break;
+  }
+
+  bool recover = false;
+  if (!auto_save_dir.empty() && g_file_test(auto_save_dir.c_str(), G_FILE_TEST_EXISTS))
   {
     time_t file_ts = get_file_timestamp(path);
-    time_t autosave_ts = get_file_timestamp(auto_save_dir+"/"+MAIN_DOCUMENT_NAME);
+    time_t autosave_ts = get_file_timestamp(bec::make_path(auto_save_dir, MAIN_DOCUMENT_NAME));
     if (autosave_ts == 0)
       autosave_ts = get_file_timestamp(auto_save_dir);
     
@@ -284,7 +320,7 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
     if (mforms::Utilities::show_warning(_("Document Recovery"),
                                         base::strfmt(_("The document %s was not properly closed in a previous session on %s.\n"
                                                        "The file you're about to open was last saved %s.\n"
-                                                       "Would you like to open its recovered contents or continue opening the selected file and clean up the autosave data?"),
+                                                       "Would you like to use the recovered model? Continuing without recovering will remove the auto-saved data."),
                                                      path.c_str(), bec::fmttime(autosave_ts, DATETIME_FMT).c_str(), bec::fmttime(file_ts, DATETIME_FMT).c_str()),
                                         _("Recover"), _("Continue"), "") == mforms::ResultOk)
     {
@@ -318,7 +354,7 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
         }
       }
     }
-    else
+    else // Cancel recovery
     {
       g_message("Cleaning up leftover auto-save directory %s", auto_save_dir.c_str());
       rmdir_recursively(auto_save_dir.c_str());
@@ -352,6 +388,9 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
   {
     g_free(basename);
     _dirty= true;
+
+    // re-lock it for ourselves
+    _temp_dir_lock = new base::LockFile(bec::make_path(_content_dir, lock_filename.c_str()));
   }
 }
 
@@ -381,6 +420,7 @@ workbench_DocumentRef ModelFile::retrieve_document(grt::GRT *grt)
 
   xmlDocPtr xmldoc= grt->load_xml(get_path_for(MAIN_DOCUMENT_NAME));
 
+retry:
   try
   {
     workbench_DocumentRef doc(unserialize_document(grt, xmldoc, get_path_for(MAIN_DOCUMENT_NAME)));
@@ -393,13 +433,20 @@ workbench_DocumentRef ModelFile::retrieve_document(grt::GRT *grt)
     
     return doc;
   }
-  catch (std::exception)
+  catch (grt::grt_runtime_error &exc)
+  {
+    if (strstr(exc.detail.c_str(), "Type mismatch: expected object of type"))
+      if (check_and_fix_duplicate_uuid_bug(grt, xmldoc))
+        goto retry;
+    throw;
+  }
+  catch (std::exception &exc)
   {
     if (xmldoc)
       xmlFreeDoc(xmldoc);
-
     throw;
   }
+  return workbench_DocumentRef();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -467,6 +514,10 @@ void ModelFile::unpack_zip(const std::string &zipfile, const std::string &destdi
     gchar *dirname= g_path_get_dirname(zname);
     gchar *basename= g_path_get_basename(zname);
 
+    //skip lock file as it is already locked and inaccessible
+    if (!strcmp(basename,lock_filename.c_str()))
+      continue;
+
     std::string outpath= destdir;
 
     if (dirname && *dirname)
@@ -660,7 +711,7 @@ workbench_DocumentRef ModelFile::unserialize_document(grt::GRT *grt, xmlDocPtr x
   {
     // first phase of document upgrade will upgrade it at XML level
     if (!attempt_xml_document_upgrade(xmldoc, version))
-      throw std::runtime_error("The document was created in an incompatible version of the appplication.");
+      throw std::runtime_error("The document was created in an incompatible version of the application.");
   }
 
   check_and_fix_inconsistencies(xmldoc, version);
@@ -755,6 +806,7 @@ bool ModelFile::save_to(const std::string &path)
 
   // saving the file for real can delete the autosave
   g_remove(get_path_for("document-autosave.mwb.xml").c_str());
+  g_remove(get_path_for("real_path").c_str());
 
   if (g_path_is_absolute(path.c_str()))
     pack_zip(path, _content_dir);
@@ -775,6 +827,9 @@ void ModelFile::cleanup()
 {
   AutoLock lock(_mutex);
 
+  delete _temp_dir_lock;
+  _temp_dir_lock = 0;
+
   rmdir_recursively(_content_dir.c_str());
 }
 
