@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
- * Boston, MA 02110-1301  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -27,15 +27,17 @@
 #include "grts/structs.db.mysql.h"
 #include "sqlide/table_inserts_loader_be.h"
 
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace wb;
 using namespace grt;
 using namespace base;
                             
 #ifdef _WIN32
+#ifndef strcasecmp
 #define strcasecmp strcmpi
 #endif
+#endif
 
 static void fixup_layer_tree(XMLTraverser &xml, xmlNodePtr layerNode, double xoffs, double yoffs)
 {
@@ -90,7 +92,7 @@ static bool fix_user_datatypes(xmlNodePtr parent, xmlNodePtr node)
 // Low-level upgrade by manipulation of XML document
 bool ModelFile::attempt_xml_document_upgrade(xmlDocPtr xmldoc, const std::string &version)
 {
-  std::vector<std::string> ver= bec::split_string(version, ".");
+  std::vector<std::string> ver= base::split(version, ".");
   int major, minor, revision;
 
   major= atoi(ver[0].c_str());
@@ -430,7 +432,7 @@ static db_IndexRef find_matching_fk_index(std::map<std::string, db_IndexRef> &in
 workbench_DocumentRef ModelFile::attempt_document_upgrade(const workbench_DocumentRef &doc,
       xmlDocPtr xmldoc, const std::string &version)
 {
-  std::vector<std::string> ver= bec::split_string(version, ".");
+  std::vector<std::string> ver= base::split(version, ".");
   int major, minor, revision;
 
   major= atoi(ver[0].c_str());
@@ -693,8 +695,8 @@ static void check_figure_layers(workbench_physical_DiagramRef view, std::list<st
       {
         load_warnings.push_back(strfmt("Table figure %s is not attached to any database table and was deleted.", figure->name().c_str()));
         orphan= true;
-  }
-}
+      }
+    }
     else if (figure.is_instance<workbench_physical_ViewFigure>())
     {
       if (!workbench_physical_ViewFigureRef::cast_from(figure)->view().is_valid())
@@ -785,6 +787,165 @@ static void fix_broken_foreign_keys(XMLTraverser &traverser, std::list<std::stri
 }
 
 
+
+
+static int fix_duplicate_uuid_bug(xmlNodePtr node, 
+                                   std::map<std::string, std::string> &object_types,
+                                  std::map<std::string, std::map<std::string, std::string> > &remapped_ids)
+{
+  xmlNodePtr n;
+  int fixes = 0;
+  
+  for (n = node->children; n; n = n->next)
+  {
+    if (n->type == XML_ELEMENT_NODE)
+    {
+      if (strcmp((char*)n->name, "value") == 0 && XMLTraverser::node_prop(n, "type") == "object")
+      {
+        std::string id = XMLTraverser::node_prop(n, "id");
+        std::string sname = XMLTraverser::node_prop(n, "struct-name");
+        std::map<std::string,std::string>::iterator it = object_types.find(id);
+        if (it != object_types.end())
+        {
+          std::string remapped = strfmt("%s%i", id.c_str(), (int)remapped_ids.size());
+          if (remapped_ids.find(id) != remapped_ids.end()
+              || remapped_ids[id].find(sname) != remapped_ids[id].end())
+            g_warning("Found more than 1 duplicate entry for same UUID, will be unable to correctly fix file (%s %s)",
+                      sname.c_str(), id.c_str());
+          if (remapped_ids[id].size() == 0)
+          {
+            // add the 1st found object
+            remapped_ids[id][object_types[id]] = id;
+          }
+          remapped_ids[id][sname] = remapped;
+          g_message("changed duplicate %s %s to %s", id.c_str(), sname.c_str(), remapped.c_str());
+          xmlSetProp(n, (const xmlChar*)"id", (const xmlChar*)remapped.c_str());
+          fixes++;
+        }
+        else
+        {
+          object_types[id] = sname;
+        }
+      }
+      fixes += fix_duplicate_uuid_bug(n, object_types, remapped_ids);
+    }
+  }
+  return fixes;
+}
+
+
+static void fix_duplicate_uuid_bug_references(grt::GRT *grt, xmlNodePtr node, 
+                                              std::map<std::string, std::map<std::string, std::string> > &remapped_ids)
+{
+  xmlNodePtr n;
+  
+  for (n = node->children; n; n = n->next)
+  {
+    if (n->type == XML_ELEMENT_NODE)
+    {
+      if (strcmp((char*)n->name, "link") == 0 && XMLTraverser::node_prop(n, "type") == "object")
+      {
+        xmlChar *content = xmlNodeGetContent(n);
+        std::string id = (char*)content?(char*)content:"";
+        xmlFree(content);
+        std::string sname = XMLTraverser::node_prop(n, "struct-name");
+        std::map<std::string, std::map<std::string, std::string> >::iterator it = remapped_ids.find(id);
+        if (it != remapped_ids.end())
+        {
+          if (sname.empty())
+          {
+            if (XMLTraverser::node_prop(node, "type") == "list" && XMLTraverser::node_prop(node, "content-type") == "object"
+                && !(sname = XMLTraverser::node_prop(node, "content-struct-name")).empty())
+            {
+              g_message("link %s is in list of %s", id.c_str(), sname.c_str());
+            }
+            else
+            {
+              g_message("link %s has no struct-name", id.c_str());
+              continue;
+            }
+          }
+        retry:
+          if (it->second.find(sname) == it->second.end())
+          {
+            bool ok = false;
+            std::map<std::string, std::string> structs = it->second;
+            
+            // handle refs to owner, which some times have a generic type like GrtObject so they're very ambiguous
+            if (XMLTraverser::node_prop(n, "key") == "owner")
+            {
+              if (sname == "GrtObject")
+              {
+                xmlNodePtr gparent = node->parent;
+                while (gparent)
+                {
+                  if (XMLTraverser::node_prop(gparent, "type") == "object")
+                  {
+                    g_message("%s %s -> %s (%s)", id.c_str(), sname.c_str(), XMLTraverser::node_prop(gparent, "struct-name").c_str(),
+                              XMLTraverser::node_prop(gparent, "id").c_str());
+                    sname = XMLTraverser::node_prop(gparent, "struct-name");
+                    break;
+                  }
+                  gparent = gparent->parent;
+                }                
+              }
+            }
+            
+            // go through the alternatives checking their superclasses
+            for (std::map<std::string, std::string>::const_iterator opt = structs.begin();
+                 opt != structs.end(); ++opt)
+            {
+              grt::MetaClass *mc = grt->get_metaclass(opt->first);
+              while (mc)
+              {
+                if (mc->name() == sname)
+                {
+                  g_message("found match %s for %s (%s -> %s)", opt->first.c_str(), sname.c_str(), id.c_str(), opt->second.c_str());
+                  xmlNodeSetContent(n, (xmlChar*)opt->second.c_str());
+                  ok = true;
+                  break;
+                }
+                mc = mc->parent();
+              }
+            }
+
+            if (!ok)
+              g_message("WARNING: Could not resolve link for %s %s (%i) key='%s'. Report to the developers to get a fix.",
+                        id.c_str(), sname.c_str(), (int)structs.size(),
+                        XMLTraverser::node_prop(n, "key").c_str());
+          }
+          else
+            xmlNodeSetContent(n, (xmlChar*)it->second[sname].c_str());
+        }
+      }
+      fix_duplicate_uuid_bug_references(grt, n, remapped_ids);
+    }
+  }
+}
+
+bool ModelFile::check_and_fix_duplicate_uuid_bug(grt::GRT *grt, xmlDocPtr xmldoc)
+{
+  if (XMLTraverser::node_prop(xmlDocGetRootElement(xmldoc), "version") == "1.4.1")
+  {
+    // in 5.2.32, the UUID generator was replaced with a boost implementation.
+    // for some reason, that caused duplicate UUIDs to be generated, which screws up with saving/loading
+    // of models. So if a model fails because of that, we go through the whole XML,
+    // find objects with duplicate ids and tweak these id's to be unique based on their id + class pair.
+    // the recovery won't be perfect because if 2 objects that are of the same class have the same id,
+    // it will not be possible to accurately match references to them.
+
+    std::map<std::string, std::string> object_types;
+    std::map<std::string, std::map<std::string, std::string> > remapped_ids;
+    if (fix_duplicate_uuid_bug(xmlDocGetRootElement(xmldoc), object_types, remapped_ids))
+    {
+      fix_duplicate_uuid_bug_references(grt, xmlDocGetRootElement(xmldoc), remapped_ids);
+      return true;
+    }
+  }
+  return false;
+}
+
+
 static void check_schema_objects(db_SchemaRef schema, std::list<std::string> &load_warnings)
 {
   // check for indexes that have a null referencedColumn value
@@ -828,7 +989,7 @@ static void check_schema_objects(db_SchemaRef schema, std::list<std::string> &lo
 
 void ModelFile::check_and_fix_inconsistencies(xmlDocPtr xmldoc, const std::string &version)
 {
-  std::vector<std::string> ver= bec::split_string(version, ".");
+  std::vector<std::string> ver= base::split(version, ".");
   int major, minor, revision;
   
   major= atoi(ver[0].c_str());
