@@ -1,24 +1,27 @@
 /* 
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
- * Boston, MA 02110-1301  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
 
+#include <sstream>
+
+#include "base/config_file.h"
 #include "wb_config.h"
 #include "wb_module.h"
 #include "wb_overview.h"
@@ -31,7 +34,7 @@
 
 #include "grt/common.h"
 
-#include "util_public_interface.h"
+#include "base/util_public_interface.h"
 
 #include "wbcanvas/model_figure_impl.h"
 
@@ -45,7 +48,8 @@
 
 #include "server_instance_editor.h"
 #include "grtui/grtdb_connection_editor.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
+#include "grtdb/db_helpers.h"
 
 using namespace wb;
 using namespace grt;
@@ -273,15 +277,15 @@ ListRef<app_Plugin> WorkbenchImpl::getPluginInfo()
   def_plugin("edit", "goToNextSelected", "internal", "Go to Next Selected", "Go to Next Selected Object");
   def_plugin("edit", "goToPreviousSelected", "internal", "Go to Previous Selected", "Go to Previous Selected Object");
 
-  def_plugin("view", "zoomIn", "normal", "Zoom In", "Zoom In Diagram");
-  def_plugin("view", "zoomOut", "normal", "Zoom Out", "Zoom Out Diagram");
-  def_plugin("view", "zoomDefault", "normal", "Zoom 100%", "Zoom Back Diagram to Default");
+  def_plugin("view", "zoomIn", "internal", "Zoom In", "Zoom In Diagram");
+  def_plugin("view", "zoomOut", "internal", "Zoom Out", "Zoom Out Diagram");
+  def_plugin("view", "zoomDefault", "internal", "Zoom 100%", "Zoom Back Diagram to Default");
 
-  def_arg_plugin("view", "goToMarker", "normal", "Go to Marker", "Go to Previously Set Diagram Marker");
-  def_arg_plugin("view", "setMarker", "normal", "Set a Marker", "Set a Diagram Marker to the Current Location");
+  def_arg_plugin("view", "goToMarker", "internal", "Go to Marker", "Go to Previously Set Diagram Marker");
+  def_arg_plugin("view", "setMarker", "internal", "Set a Marker", "Set a Diagram Marker to the Current Location");
 
-  def_model_arg_plugin("view", "setFigureNotation", "normal", "Set Objects Notation", "Set Object Notation");
-  def_model_arg_plugin("view", "setRelationshipNotation", "normal", "Set Relationships Notation", "Set Relationship Notation");
+  def_model_arg_plugin("view", "setFigureNotation", "internal", "Set Objects Notation", "Set Object Notation");
+  def_model_arg_plugin("view", "setRelationshipNotation", "internal", "Set Relationships Notation", "Set Relationship Notation");
 
   def_model_plugin("model", "newDiagram", "Add New Diagram", "Add a New Diagram to the Model");
 
@@ -1096,6 +1100,8 @@ int WorkbenchImpl::debugGrtStats()
   return 0;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 int WorkbenchImpl::debugShowInfo()
 {
   grt::GRT *grt= get_grt();
@@ -1123,6 +1129,7 @@ int WorkbenchImpl::debugShowInfo()
 
   grt->send_output(strfmt("MySQL Workbench "EDITION_NAME" for "PLATFORM_NAME" version %i.%i.%i\n", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER));
 
+  grt->send_output(strfmt("Configuration Directory: %s\n", _wb->get_grt_manager()->get_user_datadir().c_str()));
   grt->send_output(strfmt("Data Directory: %s\n", _wb->get_grt_manager()->get_basedir().c_str()));
   
   int cver= cairo_version();
@@ -1165,9 +1172,7 @@ int WorkbenchImpl::debugShowInfo()
   g_free(tmp);
 
 #ifdef _WIN32
-  tmp= get_video_driver_info();
-  grt->send_output(tmp);
-  g_free(tmp);
+  grt->send_output(getFullVideoAdapterInfo());
 
   int locale_buffer_size = GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLANGUAGE, NULL, 0);
   if (locale_buffer_size > 0)
@@ -1197,6 +1202,7 @@ int WorkbenchImpl::debugShowInfo()
   return 0;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 std::string WorkbenchImpl::request_input_m(const std::string &caption)
 {
@@ -1293,8 +1299,8 @@ int WorkbenchImpl::showDocumentProperties()
 
 int WorkbenchImpl::showModelOptions(const workbench_physical_ModelRef &model)
 {
-  PreferencesForm *prefs= new PreferencesForm(_wb->get_ui(), model.id());
-  prefs->show();
+  PreferencesForm prefs(_wb->get_ui(), model.id());
+  prefs.show();
   
   return 0;
 }
@@ -1302,8 +1308,8 @@ int WorkbenchImpl::showModelOptions(const workbench_physical_ModelRef &model)
 
 int WorkbenchImpl::showOptions()
 {
-  PreferencesForm *prefs= new PreferencesForm(_wb->get_ui());
-  prefs->show();
+  PreferencesForm prefs(_wb->get_ui());
+  prefs.show();
   
   return 0;
 }
@@ -1559,6 +1565,377 @@ int WorkbenchImpl::wmiStopMonitoring(int monitor_id)
   return 1;
 }
 
+#endif
+
 //--------------------------------------------------------------------------------------------------
 
+static const char *DEFAULT_RDBMS_ID= "com.mysql.rdbms.mysql";
+
+/**
+ * Creates a new connection ref and adds it to the stored connections collection.
+ * The new connection is also returned.
+ */
+db_mgmt_ConnectionRef WorkbenchImpl::create_connection(const std::string& host, const std::string& user,
+                                                       const std::string socket_or_pipe_name,
+                                                       bool can_use_networking,
+                                                       bool can_use_socket_or_pipe,
+                                                       int port,
+                                                       const std::string& name)
+{
+  db_mgmt_RdbmsRef rdbms = find_object_in_list(_wb->get_root()->rdbmsMgmt()->rdbms(), DEFAULT_RDBMS_ID);
+  grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+
+  db_mgmt_ConnectionRef connection = db_mgmt_ConnectionRef(_wb->get_grt());
+  db_mgmt_DriverRef driver;
+  if (can_use_networking)
+    driver = rdbms->defaultDriver();
+  else
+  {
+    // If networking is enabled but sockets/named pipes aren't enabled then the server
+    // is actually not accessible. We play nice though and use the default driver in that case.
+    if (can_use_socket_or_pipe)
+    {
+      driver = find_object_in_list(rdbms->drivers(), "com.mysql.rdbms.mysql.driver.native_socket");
+      if (!driver.is_valid())
+        driver = rdbms->defaultDriver();
+      else
+        connection->parameterValues().gset("socket", socket_or_pipe_name);
+    }
+    else
+      driver = rdbms->defaultDriver();
+  }
+
+  connection->driver(driver);
+  connection->name(name);
+
+  connection->parameterValues().gset("hostName", "localhost");
+  connection->parameterValues().gset("userName", "root");
+  connection->parameterValues().gset("port", port);
+
+  connection->hostIdentifier(bec::get_host_identifier_for_connection(connection));
+  connections.insert(connection);
+
+  return connection;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Returns a list of Dicts with data for each locally installed MySQL server.
+ */
+grt::DictListRef WorkbenchImpl::getLocalServerList()
+{
+  grt::DictListRef entries;
+  try
+  {
+#ifdef _WIN32
+    int session = wmiOpenSession("", "", "");
+    entries = wmiQuery(session,
+      "select * from Win32_Service where (Name like \"%mysql%\" or DisplayName like \"%mysql%\")");
+    wmiCloseSession(session);
+#else
+    // TODO: other OSes go here.
 #endif
+  }
+  catch (...)
+  {
+    // If for some reason (e.g. insufficient rights) the retrieval fails then we return an empty list.
+  }
+
+  return entries;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Creates a list of new connections to all local servers found.
+ */
+int WorkbenchImpl::createConnectionsFromLocalServers()
+{
+  grt::DictListRef servers = getLocalServerList();
+  if (!servers.is_valid())
+    return -1;
+
+  db_mgmt_RdbmsRef rdbms = find_object_in_list(_wb->get_root()->rdbmsMgmt()->rdbms(), DEFAULT_RDBMS_ID);
+  grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+  size_t count = servers->count();
+  for (size_t i = 0; i < count; i++)
+  {
+    grt::DictRef server(servers[i]);
+
+    std::string service_display_name = server.get_string("DisplayName", "invalid");
+    std::string path = server.get_string("PathName", "invalid");
+    std::string config_file = base::extract_option_from_command_line("--defaults-file", path);
+
+    if (g_file_test(config_file.c_str(), G_FILE_TEST_EXISTS))
+    {
+      ConfigurationFile key_file(config_file, base::AutoCreateSections | base::AutoCreateKeys);
+
+      // Early out if the default section is not there.
+      if (!key_file.has_section("mysqld"))
+        continue;
+
+      bool can_use_networking = true;
+      bool can_use_socket_or_pipe = false;
+      std::string socket_or_pipe_name;
+
+      int port = key_file.get_int("port", "mysqld");
+      if (port == INT_MIN)
+        port = 3306;
+      if (key_file.has_key("skip-networking", "mysqld"))
+        can_use_networking = false;
+      if (key_file.has_key("enable-named-pipe", "mysqld"))
+        can_use_socket_or_pipe = true;
+
+      if (can_use_socket_or_pipe)
+      {
+        socket_or_pipe_name = key_file.get_string("socket", "mysqld");
+        if (socket_or_pipe_name.size() == 0)
+          socket_or_pipe_name = "MySQL"; 
+      }
+
+      create_connection("localhost", "root", socket_or_pipe_name, can_use_networking,
+        can_use_socket_or_pipe, port, _("Local ") + service_display_name);
+    }
+  }
+  return 0;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Creates a list of server instance entries for all local servers found.
+ */
+int WorkbenchImpl::createInstancesFromLocalServers()
+{
+  extern std::string get_admin_script_for_os(const std::string &os);
+
+  try
+  {
+    grt::DictListRef servers = getLocalServerList();
+    if (!servers.is_valid())
+      return -1;
+
+    db_mgmt_RdbmsRef rdbms = find_object_in_list(_wb->get_root()->rdbmsMgmt()->rdbms(), DEFAULT_RDBMS_ID);
+    grt::ListRef<db_mgmt_ServerInstance> instances(_wb->get_root()->rdbmsMgmt()->storedInstances());
+    size_t count = servers->count();
+    for (size_t i = 0; i < count; i++)
+    {
+      grt::DictRef entry(servers[i]);
+
+      std::string service_display_name = entry.get_string("DisplayName", "invalid");
+      std::string service_name = entry.get_string("Name", "invalid");
+      std::string path = entry.get_string("PathName", "invalid");
+      std::string config_file = base::extract_option_from_command_line("--defaults-file", path);
+
+      if (g_file_test(config_file.c_str(), G_FILE_TEST_EXISTS))
+      {
+        ConfigurationFile key_file(config_file, base::AutoCreateSections | base::AutoCreateKeys);
+
+        // Early out if the default section is not there.
+        if (!key_file.has_section("mysqld"))
+          continue;
+
+        bool can_use_networking = true;
+        bool can_use_socket_or_pipe = false;
+        std::string socket_or_pipe_name;
+
+        int port = key_file.get_int("port", "mysqld");
+        if (port == INT_MIN)
+          port = 3306;
+        if (key_file.has_key("skip-networking", "mysqld"))
+          can_use_networking = false;
+        if (key_file.has_key("enable-named-pipe", "mysqld"))
+          can_use_socket_or_pipe = true;
+
+        if (can_use_socket_or_pipe)
+        {
+          socket_or_pipe_name = key_file.get_string("socket", "mysqld");
+          if (socket_or_pipe_name.size() == 0)
+            socket_or_pipe_name = "MySQL"; 
+        }
+
+        db_mgmt_ServerInstanceRef instance(_wb->get_grt());
+
+        instance->owner(_wb->get_root()->rdbmsMgmt());
+
+        instance->serverInfo().gset("sys.system", "Windows");
+        instance->serverInfo().gset("windowsAdmin", 1);
+        instance->loginInfo().gset("wmi.userName", ""); // Only used for remote connections.
+        instance->loginInfo().gset("wmi.hostName", ""); // Only used for remote connections.
+
+        instance->name("Local " + service_display_name);
+
+        instance->serverInfo().gset("sys.preset", "Custom");
+        instance->serverInfo().gset("sys.script", get_admin_script_for_os("Windows"));
+
+        instance->serverInfo().gset("sys.config.path", config_file);
+        instance->serverInfo().gset("sys.config.section", "mysqld");
+        instance->serverInfo().gset("sys.mysqld.service_name", service_name);
+
+        // Finally look up a connection or create one we can use for this instance.
+        db_mgmt_ConnectionRef connection;
+        grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+        for (grt::ListRef<db_mgmt_Connection>::const_iterator end = connections.end(),
+          iterator = connections.begin(); iterator != end; ++iterator)
+        {
+          grt::DictRef parameters((*iterator)->parameterValues());
+
+          // Must be a localhost connection.
+          std::string parameter = parameters.get_string("hostName");
+          if (!parameter.empty() && parameter != "localhost" && parameter != "127.0.0.1")
+            continue;
+
+          // For now we only consider connections with a root user. We might later add support
+          // for any user.
+          parameter = parameters.get_string("userName");
+          if (parameter != "root")
+            continue;
+
+          if (can_use_networking)
+          {
+            int other_port = parameters.get_int("port");
+            if (other_port != port)
+              continue;
+
+            // Only native tcp/ip network connections for now.
+            if ((*iterator)->driver()->id() != "com.mysql.rdbms.mysql.driver.native")
+              continue;
+          }
+          else
+            if (can_use_socket_or_pipe)
+            {
+              parameter = parameters.get_string("socket");
+              if (parameter.size() == 0)
+                parameter = "MySQL"; // Default pipe/socket name.
+
+              if (parameter != socket_or_pipe_name)
+                continue;
+
+              // Only native socket/pipe connections for now.
+              if ((*iterator)->driver()->id() != "com.mysql.rdbms.mysql.driver.native_socket")
+                continue;
+            }
+
+            // All parameters are ok. This is a connection we can use.
+            connection = *iterator;
+            break;
+        }
+
+        // If we did not find a connection for this instance then create a new one.
+        if (!connection.is_valid())
+          connection = create_connection("localhost", "root", socket_or_pipe_name, can_use_networking,
+          can_use_socket_or_pipe, port, _("Local instance ") + service_display_name);
+
+        instance->connection(connection);
+        instances.insert(instance);
+      }
+    }
+  }
+  catch (...)
+  {
+    // If for some reason (e.g. insufficient rights) the wmi session throws an exception
+    // we don't want to see it. We simply do not create the list in that case.
+  }
+
+  return 0;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Returns a short string describing the currently active video adapter, especially the used chipset.
+ */
+std::string WorkbenchImpl::getVideoAdapter()
+{
+  std::string result = _("Unknown");
+  try
+  {
+#ifdef _WIN32
+    int session = wmiOpenSession("", "", "");
+    grt::DictListRef entries = wmiQuery(session,
+      "select Availability, VideoProcessor from Win32_VideoController");
+    wmiCloseSession(session);
+
+    size_t count = entries->count();
+    for (size_t i = 0; i < count; i++)
+    {
+      grt::DictRef entry(entries[i]);
+
+      if (entry.get_string("Availability", "0") != "3")
+        continue;
+
+      // Return the first active adapter we find.
+      result = entry.get_string("VideoProcessor", "Unknown video processor");
+      break;
+    }
+#else
+    // TODO: other OSes go here.
+#endif
+  }
+  catch (...)
+  {
+    // If for some reason (e.g. insufficient rights) the retrieval fails then we return an empty list.
+  }
+  return result;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Returns all available info about the currently active video adapter in human readable format.
+ */
+std::string WorkbenchImpl::getFullVideoAdapterInfo()
+{
+  std::stringstream result;
+  try
+  {
+#ifdef _WIN32
+    int session = wmiOpenSession("", "", "");
+    grt::DictListRef entries = wmiQuery(session, "select * from Win32_VideoController");
+    wmiCloseSession(session);
+
+    size_t count = entries->count();
+    for (size_t i = 0; i < count; i++)
+    {
+      grt::DictRef entry(entries[i]);
+
+      if (entry.get_string("Availability", "0") != "3")
+        continue;
+
+      // Return the first active adapter we find.
+      result << "Active video adapter: " << entry.get_string("Caption", "") << '\n';
+      std::string value = entry.get_string("AdapterRAM", "");
+      if (value.size() > 0)
+      {
+        int size = atoi(value.c_str()) / 1024 / 1024;
+        result << "Installed video RAM: " << size << " MB\n";
+      }
+      else
+        result << "Installed video RAM: unknown\n";
+
+      result << "Current video mode: " << entry.get_string("VideoModeDescription", "unknown") << '\n';
+
+      value = entry.get_string("CurrentBitsPerPixel", "");
+      if (value.size() > 0)
+        result << "Used bit depth: " << value << '\n';
+      else
+        result << "Used bit depth: unknown\n";
+      
+      result << "Driver version: " << entry.get_string("DriverVersion", "unknown") << '\n';
+      result << "Installed display drivers: " << entry.get_string("InstalledDisplayDrivers", "unknown") << '\n';
+
+      break;
+    }
+#else
+    // TODO: other OSes go here.
+#endif
+  }
+  catch (...)
+  {
+    // If for some reason (e.g. insufficient rights) the retrieval fails then we return an empty list.
+  }
+  return result.str();
+}
+
+//--------------------------------------------------------------------------------------------------
