@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -32,7 +32,7 @@
 #include "workbench/wb_context.h"
 #include "workbench/wb_context_ui.h"
 #include "grt/validation_manager.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 /**
  * @file  wb_overview.cpp
@@ -500,8 +500,6 @@ void OverviewBE::unselect_all(const NodeId &node)
 
   if (container)
     ::unselect_all(*container);
-
-  _wb->request_refresh(RefreshMenubar, "");
 }
 
 
@@ -590,8 +588,6 @@ void OverviewBE::focus_node(const bec::NodeId &node)
 
   if (parent_id.is_valid())
     focus_node(parent_id);
-
-  _wb->request_refresh(RefreshMenubar, "");
 }
 
 
@@ -912,7 +908,10 @@ void OverviewBE::copy()
       }
     }
     if (count > 0)
+    {
       _wb->get_clipboard()->set_content_description(get_edit_target_name());
+      _wb->get_clipboard()->changed();
+    }
   }
 
   if (count > 0)
