@@ -23,7 +23,7 @@
 #include "wb_context.h"
 #include "mforms/mforms.h"
 
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #include <errno.h>
 #ifdef _WIN32
@@ -40,6 +40,9 @@
 #include <sstream>
 #include "boost/scoped_array.hpp"
 
+#include "base/log.h"
+ENABLE_LOG("wbprivate")
+
 using namespace wb;
 using namespace base;
 
@@ -240,6 +243,12 @@ void TunnelManager::close_tunnel(int port)
 
 sql::TunnelConnection* TunnelManager::create_tunnel(db_mgmt_ConnectionRef connectionProperties)
 {
+  if (!_tunnel)
+  {
+    log_info("Starting tunnel");
+    start();
+  }
+
   sql::TunnelConnection* tunnel = 0;
   grt::DictRef parameter_values= connectionProperties->parameterValues();
 
