@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
- * Boston, MA 02110-1301  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -22,7 +22,7 @@
 #include <iostream>
 #include <sstream>
 
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include "wmi.h"
 #include "grt/common.h"
 
@@ -159,23 +159,36 @@ WmiMonitor::WmiMonitor(IWbemServices* services, const std::string& parameter)
   CComPtr<IWbemConfigureRefresher> config;
   hr = _refresher->QueryInterface(IID_IWbemConfigureRefresher, (void**) &config);
   if (FAILED(hr))
-    throw std::runtime_error(_("WMI - Could not create monitor object.\n\n") + wmiResultToString(hr));
-
-  CComBSTR path;
-  if (parameter == "LoadPercentage")
   {
-    path = L"Win32_PerfFormattedData_PerfOS_Processor";
-    _propertyName = L"PercentProcessorTime";
-    _findTotal = true;
+    // _refersher is a smart pointer and is automatically freed.
+    throw std::runtime_error(_("WMI - Could not create monitor object.\n\n") + wmiResultToString(hr));
   }
 
+  const std::vector<std::string> args = base::split(parameter, ".");
+  
+  if (args.size() < 2)
+  {
+	throw std::runtime_error(_("Wrong monitor format. Got '") + parameter + _("', expected '<WMIClass.Name>'"));
+  }
+
+  CComBSTR path = args[0].c_str();
+  //if (parameter == "LoadPercentage")
+  //{
+  //  path = L"Win32_PerfFormattedData_PerfOS_Processor";
+  //  _propertyName = L"PercentProcessorTime";
+  //  _findTotal = true;
+  //}
+  _propertyName = args[1].c_str();
+
   if (path.Length() == 0)
     throw std::runtime_error(_("WMI - Invalid monitoring parameter specified."));
 
   // Add an enumerator to the refresher. This is what actually gets the value to monitor.
   hr = config->AddEnum(_services, path, 0, NULL, &_enumerator, &_enumeratorId);
   if (FAILED(hr))
-    throw std::runtime_error(_("WMI - Could not register monitoring enumerator.\n\n") + wmiResultToString(hr));
+  {
+	throw std::runtime_error(_("WMI - Could not register monitoring enumerator.\n\n") + wmiResultToString(hr));
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -314,11 +327,11 @@ WmiServices::WmiServices(const std::string& server, const std::string& user, con
     // Set a blanket to our service proxy to establish a security context.
     hr = CoSetProxyBlanket(
       _services,                   // Indicates the proxy to set
-      RPC_C_AUTHN_DEFAULT,           // RPC_C_AUTHN_xxx
-      RPC_C_AUTHZ_NONE,            // RPC_C_AUTHZ_xxx
-      NULL,                        // Server principal name
-      RPC_C_AUTHN_LEVEL_PKT,      // RPC_C_AUTHN_LEVEL_xxx
-      RPC_C_IMP_LEVEL_IMPERSONATE, // RPC_C_IMP_LEVEL_xxx
+      RPC_C_AUTHN_DEFAULT,         // Authentication service
+      RPC_C_AUTHZ_DEFAULT,         // Authorization service
+      COLE_DEFAULT_PRINCIPAL,      // Server principal name
+      RPC_C_AUTHN_LEVEL_DEFAULT,   // Authentication level
+      RPC_C_IMP_LEVEL_IMPERSONATE, // Impersonation level
       NULL,                        // client identity
       //&cID,
       EOAC_NONE                    // proxy capabilities
@@ -386,7 +399,7 @@ void WmiServices::deallocate_locator()
  * @return A list of dictionaries containing an entry for each object returned by the query, with
  *         name/value pairs of object properties.
  */
-grt::DictListRef WmiServices::query( grt::GRT* grt, const std::string& query )
+grt::DictListRef WmiServices::query(grt::GRT* grt, const std::string& query)
 {
   // Making this function explicitly thread-safe might be unnecessary as we don't have
   // any data which is allocated/deallocated concurrently. But since we know we will be called
@@ -408,10 +421,10 @@ grt::DictListRef WmiServices::query( grt::GRT* grt, const std::string& query )
       RPC_C_IMP_LEVEL_IMPERSONATE, NULL, EOAC_NONE);
 
     // Read the returned results and create our own response.
-    CComPtr<IWbemClassObject> wbemObject = NULL;
     ULONG count = 0;
     while (true)
     {
+      CComPtr<IWbemClassObject> wbemObject = NULL;
       hr = enumerator->Next(WBEM_INFINITE, 1L, &wbemObject, &count);
       if (FAILED(hr))
         throw std::runtime_error(_("WMI enumeration failed.\n\n") + wmiResultToString(hr));
@@ -420,7 +433,7 @@ grt::DictListRef WmiServices::query( grt::GRT* grt, const std::string& query )
         break;
 
       SAFEARRAY* names;
-      wbemObject->GetNames(NULL, 0, NULL, &names);
+      wbemObject->GetNames(NULL, WBEM_FLAG_NONSYSTEM_ONLY, NULL, &names);
 
       long lowBound, highBound;
       SafeArrayGetLBound(names, 1, &lowBound);
