@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 
@@ -27,8 +27,8 @@
 #include "grt/common.h"
 #include "glib/gstdio.h"
 
-#include "file_functions.h"
-#include "string_utilities.h"
+#include "base/file_functions.h"
+#include "base/string_utilities.h"
 #include "mforms/utilities.h"
 
 using namespace grt;
@@ -43,34 +43,6 @@ std::map<grt::GRT*,GRTManager*> GRTManager::_instances;
 
 GStaticMutex _instance_mutex = G_STATIC_MUTEX_INIT;
 
-int rmdir_recursively(const char *path)
-{
-  int res= 0;
-  GError *error= NULL;
-  GDir* dir;
-  const char *dir_entry;
-  gchar *entry_path;
-  
-  dir= g_dir_open(path, 0, &error);
-  if (!dir && error)
-    return error->code;
-
-  while ((dir_entry= g_dir_read_name(dir)))
-  {
-    entry_path= g_build_filename(path, dir_entry, NULL);
-    if (g_file_test(entry_path, G_FILE_TEST_IS_DIR))
-      (void) rmdir_recursively(entry_path);
-    else
-      (void) g_remove(entry_path);
-    g_free(entry_path);
-  }
-
-  (void) g_rmdir(path);
-
-  g_dir_close(dir);
-  return res;
-}
-
 
 static void init_all()
 {
@@ -575,7 +547,7 @@ void GRTManager::initialize(const std::string &loader_module_path)
   add_python_module_dir(_grt, _basedir + "\\python");
   add_python_module_dir(_grt, _basedir + "\\modules");
 #elif !defined(__APPLE__)
-  std::vector<std::string> path(split_string(_module_pathlist, G_SEARCHPATH_SEPARATOR_S));
+  std::vector<std::string> path(base::split(_module_pathlist, G_SEARCHPATH_SEPARATOR_S));
   for (std::vector<std::string>::const_iterator i= path.begin(); i != path.end(); ++i)
     add_python_module_dir(_grt, *i);
 #endif
@@ -612,9 +584,6 @@ void GRTManager::perform_idle_tasks()
       i->first->flush_pending_callbacks();
   }
 
-  // flush the async notification queue
-//QQQ  _grt->flush_notifications();
-
   if (_shell)
   {
     // flush the shell output buffer
@@ -632,13 +601,16 @@ void GRTManager::perform_idle_tasks()
     for (std::list<sigc::slot<bool> >::iterator iter= idle_slots.begin();
       iter != idle_slots.end(); ++iter)
     {
-      bool flag= (*iter)();
-
-      if (flag)
+      if (!iter->empty()) // skip if slot got disconnected
       {
-        g_mutex_lock(_idle_mutex);
-        _idle_slots.push_back(*iter);
-        g_mutex_unlock(_idle_mutex);
+        bool flag= (*iter)();
+        
+        if (flag)
+        {
+          g_mutex_lock(_idle_mutex);
+          _idle_slots.push_back(*iter);
+          g_mutex_unlock(_idle_mutex);
+        }
       }
     }
   }
@@ -1067,15 +1039,23 @@ std::string GRTManager::get_tmp_dir()
 
 std::string GRTManager::get_unique_tmp_subdir()
 {
-  std::string unique_name= get_guid();
+  for (;;)
+  {
+    std::string unique_name= get_guid(); 
+    // get_guid returns upper-lower case combined string (base64), which could potentially lead
+    // to duplicate dirnames in case-insensitive filesystems
   
-  return get_tmp_dir().append(unique_name).append("/");
+    std::string path = get_tmp_dir().append(unique_name).append("/");
+    if (!g_file_test(path.c_str(), G_FILE_TEST_EXISTS))
+      return path;
+  }
+  return "";
 }
 
 
 void GRTManager::cleanup_tmp_dir()
 {
-  (void) rmdir_recursively(get_tmp_dir().c_str());
+  (void) base_rmdir_recursively(get_tmp_dir().c_str());
 }
 
 
