@@ -25,7 +25,7 @@
 
 #include "grt_manager.h"
 #include "grt_shell.h"
-#include "file_functions.h"
+#include "base/file_functions.h"
 
 
 using namespace grt;
