@@ -22,7 +22,7 @@
 #include "plugin_manager.h"
 #include "editor_base.h"
 #include "grt_manager.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #include "interfaces/plugin.h"
 
