@@ -23,7 +23,7 @@
 #include <string>
 #include <map>
 
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include "charset_utils.h"
 
 #define ARR_CAPACITY(arr) (sizeof(arr)/sizeof(arr[0]))
