@@ -20,7 +20,7 @@
 
 #include "grt/grt_dispatcher.h"
 #include "editor_routinegroup.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 /**
  * @file  editor_routinegroup.cpp
