@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -23,7 +23,7 @@
 #include "db_object_helpers.h"
 #include <grtpp_util.h>
 #include "grt/validation_manager.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include "mforms/utilities.h"
 
 using namespace bec;
@@ -448,6 +448,8 @@ bool TableColumnsListBE::set_column_type_from_string(db_ColumnRef &col, const st
        */
     }
   }
+  else
+    g_message("%s is not a valid column type", type.c_str());
   return flag;
 }
 
@@ -606,21 +608,28 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, const std::st
     }
     return true;
   case Type:
-    if (value != old)
     {
-      AutoUndoEdit undo(_owner);
-
-      if (set_column_type_from_string(col, value))
+      bool result = true;
+      if (value != old)
       {
-        _owner->update_change_date();
-        undo.end(strfmt(_("Set Type of '%s.%s'"), _owner->get_name().c_str(), col->name().c_str()));
+        AutoUndoEdit undo(_owner);
+
+        if (set_column_type_from_string(col, value))
+        {
+          _owner->update_change_date();
+          undo.end(strfmt(_("Set Type of '%s.%s'"), _owner->get_name().c_str(), col->name().c_str()));
+        }
+        else
+        {
+          undo.cancel();
+          result = false;
+        }
+
+        _owner->do_partial_ui_refresh(::bec::TableEditorBE::RefreshColumnCollation);
       }
-      else
-        undo.cancel();
-      
-      _owner->do_partial_ui_refresh(::bec::TableEditorBE::RefreshColumnCollation);
+      return result;
     }
-    return true;
+
   case IsPK:
     return false;
   case IsNotNull:
@@ -747,7 +756,13 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, int value)
   case IsNotNull:
     {
       AutoUndoEdit undo(_owner);
-      col->isNotNull(value!=0);
+      col->isNotNull(value != 0);
+
+      // When setting the not-null flag then having a default value of NULL is meaningless.
+      // Remove that if it is set.
+      if (col->defaultValueIsNull())
+        bec::ColumnHelper::set_default_value(col, "");
+      
       TableHelper::update_foreign_keys_from_column_notnull(_owner->get_table(), col);
       _owner->update_change_date();
       _owner->get_table()->signal_refreshDisplay().emit("column");
@@ -1915,7 +1930,7 @@ bool FKConstraintColumnsListBE::set_column_is_fk(const NodeId &node, bool flag)
           if (srccolumn != columns[i])
           {
             //if (ColumnHelper::format_column_type(columns[i], 0, true) == type)
-            if (ColumnHelper::compare_column_types(columns[i], srccolumn) == COLUMNS_TYPES_EQUAL)
+            if (_owner->get_owner()->check_column_referenceable_by_fk(columns[i], srccolumn))
             {
               if (table->isPrimaryKeyColumn(columns[i]))
               {
@@ -2353,6 +2368,9 @@ bool FKConstraintListBE::set_field(const NodeId &node, int column, const std::st
           schema= _owner->get_schema_with_name(parts[0]);
         }
 
+        if (!schema.is_valid())
+          return false;
+
         db_TableRef dbtable= grt::find_named_object_in_list(schema->tables(), table, "name");
 
         if (dbtable.is_valid() && dbtable != fk->referencedTable())
@@ -2531,6 +2549,8 @@ _table(table),
 _fk_list(this)
 {
   if (table.class_name() == "db.Table") throw std::logic_error("table object is abstract");
+  
+  table->signal_refreshDisplay().connect(sigc::hide(sigc::mem_fun(this, &BaseEditor::on_object_changed)));
 }
 #ifdef _MSC_VER
 #pragma warning(pop)
