@@ -22,7 +22,7 @@
 #include <glib.h>
 #include "file_charset_dialog.h"
 #include "grt/common.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace mforms;
 using namespace base;
@@ -69,8 +69,10 @@ std::string FileCharsetDialog::run()
 
 
 bool FileCharsetDialog::ensure_filedata_utf8(const char *data, size_t length,
+                                             const std::string &encoding,                                             
                                              const std::string &filename,
-                                             std::string &output_str)
+                                             std::string &output_str,
+                                             std::string *original_encoding)
 {
   // Byte order marks.
   const char *utf16le_bom= "\xff\xfe";
@@ -80,7 +82,7 @@ bool FileCharsetDialog::ensure_filedata_utf8(const char *data, size_t length,
   const char *utf8_bom= "\xef\xbb\xbf";
 
   const gchar *end;
-  
+  bool retrying = false;
 retry:
   
   if (!g_utf8_validate(data, length, &end))
@@ -106,22 +108,29 @@ retry:
       }
     }
 
-    FileCharsetDialog dlg(_("Unknown File Encoding"), 
-                          strfmt("The file '%s'\n"
-                                 "has an unknown character set encoding.\n"
-                                 "Please select the encoding of the file and press OK for Workbench to convert and open it.\n"
-                                 "Note that as Workbench works with UTF-8 text, if you save back to the original file,\n"
-                                 "its contents will be replaced with the converted data.", filename.c_str()),
-                          default_encoding);
-
     std::string charset;
     char *converted;
     gsize bytes_read, bytes_written;
     GError *error = NULL;
-    
-    charset = dlg.run();
-    if (charset.empty())
+
+    if (encoding.empty() || retrying)
+    {
+      FileCharsetDialog dlg(_("Unknown File Encoding"), 
+                            strfmt("The file '%s'\n"
+                                   "has an unknown character set encoding.\n"
+                                   "Please select the encoding of the file and press OK for Workbench to convert and open it.\n"
+                                   "Note that as Workbench works with UTF-8 text, if you save back to the original file,\n"
+                                   "its contents will be replaced with the converted data.", filename.c_str()),
+                            default_encoding);    
+      charset = dlg.run();
+      if (charset.empty())
         return false;
+    }
+    else
+    {
+      charset = encoding;
+      retrying = true; // in case we fail..
+    }
 
     converted = g_convert(data, length, "UTF-8", charset.c_str(),
                           &bytes_read, &bytes_written,
@@ -175,8 +184,10 @@ retry:
       data = converted; // We still need to go through the following BOM check.
       length = bytes_written;
     }
+    if (original_encoding)
+      *original_encoding = charset;
   }
-  
+    
   // Check (again) for a byte-order-mark and skip it if there is one.
   // We only have UTF-8 now, so only the BOM for this encoding is checked.
   if ((length >= 3) && (strncmp(data, utf8_bom, 3) == 0))
