@@ -21,7 +21,7 @@
 
 #include "stdafx.h"
 
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include <mforms/mforms.h>
 #include <algorithm>
 
