@@ -28,7 +28,7 @@
 #include "grt/common.h"
 
 #include "wbpublic_public_interface.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 namespace bec {
   class GRTManager;
