@@ -19,7 +19,7 @@
 
 #include "stdafx.h"
 #include "grtdb_connect_dialog.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace grtui;
 
