@@ -1,22 +1,24 @@
 /* 
- * (c) 2007-2008 MySQL AB, 2008-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
+
+
 #include "stdafx.h"
 
 #include "grtdb_connect_panel.h"
@@ -25,7 +27,7 @@
 #include "grtdb/db_helpers.h"
 
 #include <grt/common.h>
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace grtui;
 using namespace mforms;
@@ -51,6 +53,7 @@ DbConnectPanel::DbConnectPanel(bool show_connection_combo)
   _initialized= false;
   _updating= false;
   
+  _skip_schema_name= false;
   _delete_connection_be= false;
   
   set_spacing(4);
@@ -130,6 +133,12 @@ DbConnectPanel::~DbConnectPanel()
 }
 
 
+void DbConnectPanel::set_skip_schema_name(bool flag)
+{
+  _skip_schema_name= flag;
+}
+
+
 void DbConnectPanel::suspend_view_layout(bool flag)
 {
   if (flag)
@@ -181,7 +190,7 @@ void DbConnectPanel::init(const db_mgmt_ManagementRef &mgmt, const db_mgmt_Conne
 
   DbConnection *conn= new DbConnection();
   
-  conn->init(mgmt);
+  conn->init(mgmt, _skip_schema_name);
 
   init(conn, default_conn);
   _delete_connection_be= true;
@@ -519,7 +528,7 @@ void DbConnectPanel::set_keychain_password(DbDriverParam *param, bool clear)
   std::string storage_key;
   std::string username;
   grt::DictRef paramValues(get_connection()->parameterValues());
-  std::vector<std::string> tokens = bec::split_string(param->get_value_repr(), "::");
+  std::vector<std::string> tokens = base::split(param->get_value_repr(), "::");
   
   if (tokens.size() == 2)
   {
@@ -749,7 +758,8 @@ void DbConnectPanel::create_control(::DbDriverParam *driver_param, const ::Contr
       if (value.is_valid())
         initial_value= *value;
       
-      ctrl->initialize(initial_value, mforms::OpenFile, "", "...", sigc::bind(sigc::mem_fun(this, &DbConnectPanel::param_value_changed), ctrl));
+      ctrl->initialize(initial_value, mforms::OpenFile, "", "...", true,
+        sigc::bind(sigc::mem_fun(this, &DbConnectPanel::param_value_changed), ctrl));
       box->add(mforms::manage(ctrl), true, true);
       _views.push_back(ctrl);
       
