@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,7 @@
 #include "grtdb_connection_editor.h"
 
 #include "cppdbc.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include <grt/common.h>
 
 //------------------------------------------------------------------------------
@@ -32,7 +32,7 @@ grtui::DbConnectionEditor::DbConnectionEditor(const db_mgmt_ManagementRef &mgmt)
   , _top_vbox(false)
   , _top_hbox(true)
   , _conn_list_buttons_hbox(true)
-  , _stored_connection_list(::mforms::TreeShowHeader)
+  , _stored_connection_list(mforms::TreeDefault)
   , _bottom_hbox(true)
 {
   set_title(_("Manage DB Connections"));
