@@ -23,7 +23,7 @@
 #include "grtpp_util.h"
 #include "grt/common.h"
 #include "wizard_view_text_page.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace grtui;
 using namespace base;
