@@ -28,8 +28,8 @@
 #include "recordset_export_interface.h"
 #include "grtui/binary_data_editor.h"
 #include "mforms/utilities.h"
-#include "string_utilities.h"
-#include "boost_smart_ptr_helpers.h"
+#include "base/string_utilities.h"
+#include "base/boost_smart_ptr_helpers.h"
 #include "sqlite/command.hpp"
 #include <boost/foreach.hpp>
 #include <fstream>
