@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 
@@ -24,7 +24,7 @@
 #include "recordset_be.h"
 #include "sqlide_generics_private.h"
 #include "grtsqlparser/sql_facade.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include <sqlite/query.hpp>
 #include <boost/cstdint.hpp>
 #include <boost/foreach.hpp>
@@ -76,7 +76,7 @@ public:
     {
       const size_t BUFF_SIZE= 4096;
       std::list<std::vector<char> > chunks;
-      size_t blob_size= 0;
+      std::streamsize blob_size= 0;
       while (!is->eof())
       {
         chunks.resize(chunks.size()+1);
@@ -93,7 +93,7 @@ public:
         memcpy(&(*blob)[n * BUFF_SIZE], &chunk[0], BUFF_SIZE);
         ++n;
       }
-      blob->resize(blob_size);
+      blob->resize((size_t) blob_size); // TODO: make this compatible to streamsize.
     }
     else
     {
@@ -126,7 +126,8 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
   Recordset::Column_names &column_names= get_column_names(recordset);
   Recordset::Column_types &column_types= get_column_types(recordset);
   Recordset::Column_types &real_column_types= get_real_column_types(recordset);
-
+  Recordset::Column_quoting &column_quoting= get_column_quoting(recordset);
+  
   boost::shared_ptr<sql::Statement> stmt;
   boost::shared_ptr<sql::ResultSet> rs;
   if (_dbc_resultset)
@@ -251,6 +252,7 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
     type_name= type_name.substr(0, tne);
     column_types.push_back(known_types[type_name]);
     real_column_types.push_back(known_real_types[type_name]);
+    column_quoting.push_back(!rs_meta->isNumField(n+1) && !rs_meta->isDecimal(n+1));
   }
 
   // column names
