@@ -23,8 +23,8 @@
 #include "recordset_sql_storage.h"
 #include "recordset_be.h"
 #include "grtsqlparser/sql_facade.h"
-#include "string_utilities.h"
-#include "boost_smart_ptr_helpers.h"
+#include "base/string_utilities.h"
+#include "base/boost_smart_ptr_helpers.h"
 #include <sqlite/query.hpp>
 #include <boost/foreach.hpp>
 #include <algorithm>
@@ -588,7 +588,7 @@ void Recordset_sql_storage::generate_inserts(const Recordset *recordset, sqlite:
   const Recordset::Column_quoting &column_quoted= get_column_quoting(recordset);
   const size_t partition_count= recordset->data_swap_db_partition_count();
 
-  RowId min_new_rowid= recordset->min_new_rowid();
+  //RowId min_new_rowid= recordset->min_new_rowid();
   ColumnId editable_col_count= recordset->get_column_count();
   sqlide::QuoteVar qv;
   init_variant_quoter(qv);
