@@ -24,7 +24,7 @@
 #include "recordset_be.h"
 #include "sqlide_generics_private.h"
 #include "grtsqlparser/sql_facade.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include <sqlite/execute.hpp>
 #include <sqlite/query.hpp>
 #include <boost/foreach.hpp>
