@@ -22,7 +22,7 @@
 
 #include "recordset_table_inserts_storage.h"
 #include "recordset_be.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #include <sqlite/execute.hpp>
 #include <sqlite/query.hpp>
