@@ -23,8 +23,8 @@
 #include "sql_editor_be.h"
 #include "grtsqlparser/sql_facade.h"
 #include "sqlide.h"
-#include "string_utilities.h"
-#include "boost_smart_ptr_helpers.h"
+#include "base/string_utilities.h"
+#include "base/boost_smart_ptr_helpers.h"
 #include <boost/foreach.hpp>
 
 #include "objimpl/db.query/db_query_QueryBuffer.h"
