@@ -24,14 +24,7 @@
 #include "table_figure.h"
 #include "mdc_box_side_magnet.h"
 
-#ifdef _WIN32
-// stupid VC++ workaround
-#undef max
-#undef min
-#define max _cpp_max
-#define min _cpp_min
-#endif
-
+using namespace std;
 using namespace wbfig;
 
 #define SPLIT_LINE_LENGTH 30
@@ -172,7 +165,7 @@ void ConnectionLineLayouter::set_type(ConnectionLineLayouter::Type type)
 
 
 Connection::Connection(mdc::Layer *layer, FigureEventHub *hub, model_Object *represented_object)
-: mdc::Line(layer), _hub(hub), _represented_object(represented_object)
+: mdc::Line(layer), _represented_object(represented_object), _hub(hub)
 {
   set_cache_toplevel_contents(false);
   set_accepts_selection(true);
@@ -532,9 +525,9 @@ mdc::Point Connection::get_middle_caption_pos(const mdc::Size &size, CaptionPos
         if (fabs(p1.y - p2.y) < (size.height+2))
         {
           if (pos == Above)
-            midpoint.y= std::min(p1.y, p2.y) - 10 - size.height;
+            midpoint.y= min(p1.y, p2.y) - 10 - size.height;
           else if (pos == Below)
-            midpoint.y= std::max(p1.y, p2.y) + 10;
+            midpoint.y= max(p1.y, p2.y) + 10;
 
           midpoint.x-= size.width/2;
         }
@@ -548,14 +541,14 @@ mdc::Point Connection::get_middle_caption_pos(const mdc::Size &size, CaptionPos
           midpoint.y-= size.height/2;
         }
       }
-      else // horizontalish line
+      else // horizontal-ish line
       {
         if (fabs(p1.x - p2.x) < (size.width+2) && !_center_captions)
         {
           if (pos == Above)
-            midpoint.x= std::min(p1.x, p2.x) - 10 - size.width;
+            midpoint.x= min(p1.x, p2.x) - 10 - size.width;
           else if (pos == Below)
-            midpoint.x= std::max(p1.x, p2.x) + 10;
+            midpoint.x= max(p1.x, p2.x) + 10;
 
           midpoint.y-= size.height/2;
         }
@@ -940,6 +933,17 @@ bool Connection::on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc:
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+bool Connection::on_double_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button,
+                          mdc::EventState state)
+{
+  if (!_hub->figure_double_click(_represented_object, target, point, button, state))
+    return super::on_double_click(target, point, button, state);
+  return false;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 bool Connection::on_enter(mdc::CanvasItem *target, const mdc::Point &point)
 {
