@@ -20,7 +20,7 @@
 #include "stdafx.h"
 
 #include "figure_common.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 
 mdc::FontSpec parse_font_spec(const std::string &font)
