@@ -31,7 +31,7 @@
 #include "badge_figure.h"
 
 #include "grt/grt_manager.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace grt;
 
@@ -564,6 +564,8 @@ void model_Diagram::ImplData::select_object(const model_ObjectRef &object)
     throw std::runtime_error("trying to select invalid object");
 
   _updating_selection--;
+  
+  _selection_changed_signal.emit(model_DiagramRef(_self));
 }
 
 
