@@ -27,7 +27,7 @@
 #include "grt/common.h"
 #include "grt/grt_manager.h"
 #include <grtpp_undo_manager.h>
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace base;
 
