@@ -31,19 +31,12 @@
 
 #include <grtpp_util.h>
 #include <grtpp_undo_manager.h>
-#include "string_utilities.h"
-
-#ifdef _WIN32
-// stupid VC++ workaround
-#undef max
-#undef min
-#define max _cpp_max
-#define min _cpp_min
-#endif
+#include "base/string_utilities.h"
 
 #define LAYER_ALPHA 1
 #define LAYER_BORDER_COLOR mdc::Color(0.8, 0.8, 0.8)
 
+using namespace std;
 
 model_Layer::ImplData::ImplData(model_Layer *self)
   : model_Object::ImplData(self), _area_group(0)
@@ -65,11 +58,17 @@ void model_Layer::ImplData::unrealize()
     if (is_canvas_view_valid())
       get_canvas_view()->lock();
 
+    for (size_t c= self()->_figures.count(), i= 0; i < c; i++)
+    {
+      model_Figure::ImplData *fig= self()->_figures[i]->get_data();
+      if (fig)
+        fig->unrealize();
+    }
+    
     _area_group->set_visible(false);
 
     _area_group->get_view()->remove_item(_area_group);
 
-
     if (_area_group != _area_group->get_layer()->get_root_area_group())
     {
       delete _area_group;
@@ -269,7 +268,7 @@ bool model_Layer::ImplData::realize()
 
     dynamic_cast<wbfig::LayerAreaGroup*>(_area_group)->set_title(*self()->_name);
 
-    _area_group->set_position(mdc::Point(std::max(0.0, *self()->_left), std::max(0.0, *self()->_top)));
+    _area_group->set_position(mdc::Point(max(0.0, *self()->_left), max(0.0, *self()->_top)));
     _area_group->resize_to(mdc::Size(*self()->_width, *self()->_height));
 
     layer->add_item(_area_group);
@@ -289,11 +288,12 @@ bool model_Layer::ImplData::realize()
         mdc::Point pos(item->get_root_position());
         pos.x-= *self()->_left;
         pos.y-= *self()->_top;
-        item->set_parent(0); // unparent 1st
         _area_group->add(item);
         item->move_to(pos);
         _area_group->raise_item(item);
       }
+      else if (fig)
+        fig->realize();
     }
 
     self()->owner()->get_data()->stack_layer(model_LayerRef(self()), _area_group);
