@@ -1,20 +1,20 @@
 /* 
- * (c) 2007-2008 MySQL AB, 2008-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 #include "stdafx.h"
 
@@ -30,7 +30,8 @@
 #include "grt/grt_manager.h"
 #include "grtdb/db_object_helpers.h"
 
-#include "string_utilities.h"
+#include "base/wb_iterators.h"
+#include "base/string_utilities.h"
 
 using namespace base;
 
@@ -39,6 +40,7 @@ workbench_physical_TableFigure::ImplData::ImplData(workbench_physical_TableFigur
 {
   _pending_columns_sync= false;
   _pending_index_sync= false;
+  _pending_trigger_sync= false;
 
   self->signal_changed().connect(sigc::mem_fun(this, &ImplData::member_changed));
 }
@@ -123,6 +125,7 @@ void workbench_physical_TableFigure::ImplData::set_table(const db_TableRef &tabl
       _figure->get_title()->set_title(*self()->_table->name());
       run_later(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::sync_columns));
       run_later(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::sync_indexes));
+      run_later(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::sync_triggers));
     }
     else
     {
@@ -188,6 +191,12 @@ void workbench_physical_TableFigure::ImplData::content_changed(const std::string
     _pending_index_sync= true;
     run_later(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::sync_indexes));
   }
+
+  if (where == "trigger" && _figure && !_pending_trigger_sync)
+  {
+    _pending_trigger_sync= true;
+    run_later(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::sync_triggers));
+  }  
 }
 
 
@@ -338,6 +347,57 @@ void workbench_physical_TableFigure::ImplData::sync_indexes()
   }
 }
 
+static bool compare_trigger(const std::pair<std::string, std::string> &a, const std::pair<std::string, std::string> &b)
+{
+  return a.second.substr(5) > b.second.substr(5);
+}
+
+void workbench_physical_TableFigure::ImplData::sync_triggers()
+{
+  if (_figure)
+  {
+    grt::ListRef<db_Trigger> triggers(self()->_table->triggers());
+    std::vector<std::pair<std::string, std::string> > items;
+    
+    for (size_t c= triggers.count(), i= 0; i < c; i++)
+    {
+      db_TriggerRef trigger(triggers.get(i));
+      std::string text;
+      
+      if (g_strcasecmp(trigger->timing().c_str(), "AFTER") == 0)
+        text = "AFT";
+      else
+        text = "BEF";
+      
+      std::string event = trigger->event();
+      if (g_strcasecmp(event.c_str(), "INSERT") == 0)
+        text.append(" INSERT ");
+      else if (g_strcasecmp(event.c_str(), "UPDATE") == 0)
+        text.append(" UPDATE ");
+      else if (g_strcasecmp(event.c_str(), "DELETE") == 0)
+        text.append(" DELETE ");
+      
+      text.append(trigger->name());
+      
+      items.push_back(std::make_pair(trigger.id(), text));
+    }
+    
+    std::sort(items.begin(), items.end(), compare_trigger);
+    
+    wbfig::Table::ItemList::iterator iter= _figure->begin_triggers_sync();
+    for (base::const_range<std::vector<std::pair<std::string, std::string> > > trig(items); trig; ++trig)
+      iter= _figure->sync_next_trigger(iter,
+                                       trig->first,
+                                       trig->second);
+    _figure->end_triggers_sync(iter);
+    
+    if (_figure->get_trigger_title() && !_figure->triggers_hidden())
+      _figure->get_trigger_title()->set_visible(triggers.count() > 0);
+    
+    _pending_trigger_sync= false;
+  }
+}
+
 
 bool workbench_physical_TableFigure::ImplData::is_realizable()
 {
@@ -421,11 +481,14 @@ bool workbench_physical_TableFigure::ImplData::realize()
 
     _figure->toggle(*self()->_expanded!=0);
     _figure->toggle_indexes(*self()->_indicesExpanded!=0);
+    _figure->toggle_triggers(*self()->_triggersExpanded!=0);
     
     if (self()->_table->columns().count() > 0)
       sync_columns();
     if (self()->_table->indices().count() > 0)
       sync_indexes();
+    if (self()->_table->triggers().count() > 0)
+      sync_triggers();
 
     finish_realize();
     
