@@ -33,9 +33,13 @@ namespace MySQL {
   
 static String^ CppStringToNative(const std::string &str)
 {
-  String^ result = gcnew String(str.c_str(), 0, (int) str.size(), Encoding::UTF8);
-  
-  return result->Replace("\n", "\r\n");
+  if (str.find('\r') == std::string::npos)
+  {
+    // convert from \n to \r\n (but don't double-convert \r\n to \r\r\n)
+    String^ result = gcnew String(str.c_str(), 0, (int) str.size(), Encoding::UTF8);
+    return result->Replace("\n", "\r\n");
+  }
+  return gcnew String(str.c_str(), 0, (int) str.size(), Encoding::UTF8);
 }
 
 
