@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,58 +33,24 @@ namespace MySQL {
     public ref class GrtManager 
     {
     public:
-      delegate void VoidStringStringDelegate(String^ str1, String^ str2);
-      delegate void VoidIntStringStringDelegate(int i, String^ str1, String^ str2);
-      delegate void VoidStringDelegate(String^ str);
       delegate bool BoolStringStringFloatDelegate(String^ str1, String^ str2, float f);
 
     private:
-      delegate void VoidStringStringWrapperDelegate(const std::string& str1, const std::string& str2);
-      typedef void (*GrtManager::VOID_STRING_STRING_HANDLER_CB)(const std::string& str1, const std::string& str2);
-
-      delegate void VoidIntStringStringWrapperDelegate(int i, const std::string& str1, const std::string& str2);
-      typedef void (*GrtManager::VOID_INT_STRING_STRING_HANDLER_CB)(int i, const std::string& str1, const std::string& str2);
-
-      delegate void VoidStringWrapperDelegate(const std::string& str);
-      typedef void (*GrtManager::VOID_STRING_HANDLER_CB)(const std::string& str);
-
+      [UnmanagedFunctionPointerAttribute(CallingConvention::Cdecl)]
       delegate bool BoolStringStringFloatWrapperDelegate(const std::string& str1, const std::string& str2, float f);
       typedef bool (*GrtManager::BOOL_STRING_STRING_FLOAT_HANDLER_CB)(const std::string& str1, const std::string& str2, float f);
 
       ::bec::GRTManager *inner;
       GrtShell^ managed_shell;
       GRT^ managed_grt;
-
-      VoidStringStringDelegate^ error_callback_delegate;
-      VoidStringStringWrapperDelegate^ error_callback_wrapper_delegate;
-
-      void error_callback_wrapper(const std::string& str1, const std::string& str2)
-      { error_callback_delegate(CppStringToNative(str1), CppStringToNative(str2)); }
-
-      VoidIntStringStringDelegate^ message_callback_delegate;
-      VoidIntStringStringWrapperDelegate^ message_callback_wrapper_delegate;
-
-      void message_callback_wrapper(int i, const std::string& str1, const std::string& str2)
-      { message_callback_delegate(i, CppStringToNative(str1), CppStringToNative(str2)); }
-
-      VoidStringDelegate^ output_callback_delegate;
-      VoidStringWrapperDelegate^ output_callback_wrapper_delegate;
-
-      void output_callback_wrapper(const std::string& str)
-      { output_callback_delegate(CppStringToNative(str)); }
-
-      VoidStringDelegate^ shell_output_callback_delegate;
-      VoidStringWrapperDelegate^ shell_output_callback_wrapper_delegate;
-
-      void shell_output_callback_wrapper(const std::string& str)
-      { shell_output_callback_delegate(CppStringToNative(str)); }
-
       BoolStringStringFloatDelegate^ progress_callback_delegate;
       BoolStringStringFloatWrapperDelegate^ progress_callback_wrapper_delegate;
-
+      
       bool progress_callback_wrapper(const std::string& str1, const std::string& str2, float f)
-      { return progress_callback_delegate(CppStringToNative(str1), CppStringToNative(str2), f); }
-
+      {
+        return progress_callback_delegate(CppStringToNative(str1), CppStringToNative(str2), f);
+      }
+      
     public:
       GrtManager(bool threaded, bool verbose)
         : inner(new ::bec::GRTManager(threaded, verbose)), managed_shell(nullptr), managed_grt(nullptr)
@@ -110,7 +76,7 @@ namespace MySQL {
       Font^ get_font_option(String^ option_name)
       {
         String^ font_string= CppStringToNative(inner->get_app_option_string(NativeToCppString(option_name)));
-        return MySQL::Utilities::ControlUtilities::getFont(font_string);
+        return MySQL::Utilities::ControlUtilities::GetFont(font_string);
       }
 
       void set_module_extensions(List<String^>^ extensions)
