@@ -46,10 +46,10 @@ namespace MySQL {
     private:
       static GrtIconManager^ _grtIconManager = nullptr;
 
-      static ImageList^ imageList16;
-      static ImageList^ imageList24;
-      static ImageList^ imageList32;
-      static ImageList^ imageList48;
+      ImageList^ imageList16;
+      ImageList^ imageList24;
+      ImageList^ imageList32;
+      ImageList^ imageList48;
 
       Dictionary<int, Bitmap^>^ icon_cache;
 
@@ -82,7 +82,7 @@ namespace MySQL {
       inline ::bec::IconManager *get_unmanaged_object()
       { return inner; }
 
-      // Singelton
+      // Singleton class.
       static GrtIconManager^ get_instance()
       {
         if (_grtIconManager == nullptr)
@@ -192,22 +192,22 @@ namespace MySQL {
 
       property static ImageList^ ImageList16
       {
-        ImageList^ get() { return imageList16; }
+        ImageList^ get() { return get_instance()->imageList16; }
       }
 
       property static ImageList^ ImageList24
       {
-        ImageList^ get() { return imageList24; }
+        ImageList^ get() { return get_instance()->imageList24; }
       }
 
       property static ImageList^ ImageList32
       {
-        ImageList^ get() { return imageList32; }
+        ImageList^ get() { return get_instance()->imageList32; }
       }
 
       property static ImageList^ ImageList48
       {
-        ImageList^ get() { return imageList48; }
+        ImageList^ get() { return get_instance()->imageList48; }
       }
     };
   } // namespace Grt
