@@ -22,7 +22,7 @@
 #include "edit_table_data_wizard.h"
 #include "grt/grt_manager.h"
 #include "grtpp_util.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include "sqlide/wb_sql_editor_form.h"
 
 using namespace base;
@@ -165,7 +165,8 @@ void TableSelectionPage::schema_changed()
 
 void TableSelectionPage::table_changed()
 {
-  validate();
+  //validate(); // Commented out to fix bug #57127
+                // The validate check is done in allow_next
 }
 
 //--------------------------------------------------------------------------------------------------
