@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,9 +19,9 @@
 
 #include "stdafx.h"
 #include "grt_shell_window.h"
-#include "file_functions.h"
-#include "string_utilities.h"
-#include "wb_iterators.h"
+#include "base/file_functions.h"
+#include "base/string_utilities.h"
+#include "base/wb_iterators.h"
 #include <errno.h>
 
 #include "grt/common.h"
@@ -43,7 +43,7 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
   _content(false), _padding_box(false), _hsplitter(true), _side_tab(mforms::TabViewPalette),
   _main_tab(mforms::TabViewDocumentClosable),
   _global_box1(false), _global_box2(false), _global_splitter(false), _global_combo(mforms::SelectorCombobox),
-  _global_tree((mforms::TreeOptions)0), _global_list(true), 
+  _global_tree(mforms::TreeDefault), _global_list(mforms::TreeDefault), 
 #ifdef _WIN32
   _side_header_panel(mforms::FilledHeaderPanel),
   _lower_tab(mforms::TabViewPalette),
@@ -51,8 +51,9 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
 #else
   _lower_tab(mforms::TabViewDocument),
 #endif
-  _classes_box(false), _classes_splitter(false), _classes_tree(false), _classes_text(mforms::VerticalScrollBar),
-  _modules_splitter(false), _modules_tree(false), _modules_text(mforms::VerticalScrollBar),
+  _classes_box(false), _classes_splitter(false), _classes_tree(mforms::TreeNoBorder),
+  _classes_text(mforms::VerticalScrollBar),
+  _modules_splitter(false), _modules_tree(mforms::TreeNoBorder), _modules_text(mforms::VerticalScrollBar),
   _right_splitter(false), 
   _shell_box(false), _shell_text(mforms::VerticalScrollBar), _shell_hbox(true),
   _output_text(mforms::VerticalScrollBar),
@@ -163,7 +164,7 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
 #ifdef _WIN32
   _files_tree = mforms::manage(new mforms::TreeView(mforms::TreeNoBorder));
 #else
-  _files_tree = mforms::manage(new mforms::TreeView(mforms::TreeNone));
+  _files_tree = mforms::manage(new mforms::TreeView(mforms::TreeDefault));
 #endif
 
   _files_menu.add_item("Add New File", "file-from-template");
@@ -276,9 +277,9 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
  
   // setup shell
 #ifdef _WIN32
-  _snippet_list = mforms::manage(new TreeView((mforms::TreeOptions)(mforms::TreeShowHeader | mforms::TreeNoBorder)));
+  _snippet_list = mforms::manage(new TreeView(mforms::TreeNoBorder));
 #else
-  _snippet_list = mforms::manage(new TreeView(mforms::TreeShowHeader));
+  _snippet_list = mforms::manage(new TreeView(mforms::TreeDefault));
 #endif
 
   _shell_box.add(&_shell_text, true, true);
