@@ -24,7 +24,7 @@
 #include "grtpp_util.h"
 #include "grt/common.h"
 #include "mforms/utilities.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace mforms;
 using namespace base;
