@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,7 +19,7 @@
 
 #include "stdafx.h"
 
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include "new_server_instance_wizard.h"
 #include "grt/grt_manager.h"
 #include "grtui/grtdb_connection_editor.h"
@@ -556,7 +556,6 @@ HostAndRemoteTypePage::HostAndRemoteTypePage(WizardForm* host)
   _os_box.set_spacing(10);
   _os_panel.set_padding(8);
   
-  _os_panel.show(false);
   add(&_os_panel, false, true);
 }
 
@@ -692,6 +691,7 @@ void HostAndRemoteTypePage::toggle_remote_admin()
     {
       refresh_profiles = true;
       _os_panel.show(true);
+      _os_panel.relayout();
       if (detected_os_type.empty() && wizard()->is_local())
       {
 #ifdef _WIN32
@@ -747,7 +747,7 @@ bool HostAndRemoteTypePage::advance()
     {
       values().remove("template_path");
       values().remove("template");
-      // dont remove the options otherwise the profile wont work in older WB versions
+      // Don't remove the options otherwise the profile wont work in older WB versions.
       values().gset("remoteAdmin", 0);
       values().gset("windowsAdmin", 0);
     }
@@ -998,15 +998,18 @@ WindowsManagementPage::WindowsManagementPage(WizardForm* host, wb::WBContext* co
 
 void WindowsManagementPage::refresh_config_path()
 {
-  _config_path.set_value(_config_paths[_service_selector.get_selected_index()]);
+  if (_service_selector.get_selected_index() >= 0 && _service_selector.get_selected_index() < (int) _config_paths.size())
+    _config_path.set_value(_config_paths[_service_selector.get_selected_index()]);
+  else
+    _config_path.set_value("");
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void WindowsManagementPage::leave(bool advancing)
 {
-  // if we're going back, reset the progress label. We can't set it right before
-  // performing the slow operation, because it blocks the UI
+  // If we're going back, reset the progress label. We can't set it right before
+  // performing the slow operation, because it blocks the UI.
   if (!advancing)
   {
      _progress_label.set_text(_("Initializing WMI, please wait..."));
@@ -1043,7 +1046,8 @@ void WindowsManagementPage::enter(bool advancing)
 
       user = values().get_string("wmi_user_name", "");
 
-      std::string title = strfmt(_("Remote Windows Login (%s)"), host.c_str());
+      std::string title = strfmt(_("Remote Windows Login (%s)"), host.c_str()) + "|";
+      title += _("Please enter a Windows user login and password for the remote server with rights to WMI");
       if (!Utilities::credentials_for_service(title, "wmi@" + host, user, false, password))
       {
         _progress_label.set_text(_("Need valid user credentials to connect to server."));
@@ -1095,31 +1099,40 @@ void WindowsManagementPage::enter(bool advancing)
       grt::DictListRef entries = grt::DictListRef::cast_from(wmi_result);
 
       size_t count = entries->count();
-      for (size_t i = 0; i < count; i++)
+      if (count > 0)
       {
-        grt::DictRef entry(entries[i]);
-
-        std::string service_name = entry.get_string("Name", "invalid");
-        std::string service_display_name = entry.get_string("DisplayName", "invalid");
-        std::string path = entry.get_string("PathName", "invalid");
-        std::string state = entry.get_string("State", "unknown");
-        std::string start_mode = entry.get_string("StartMode", "unknown");
-
-        std::string config_file = base::extract_option_from_command_line("--defaults-file", path);
-        if (!local_connection)
+        for (size_t i = 0; i < count; i++)
         {
-          if (config_file.empty())
-            config_file = "C$\\";
-          config_file = "\\\\" + host + "\\" + config_file;
-          base::replace(config_file, ":", "$");
-        }
-        _config_paths.push_back(config_file);
-        _service_names.push_back(service_name);
-        std::string selector_text = service_display_name + " (" + state + ", Start mode: " + start_mode + ")";
-        _service_selector.add_item(selector_text);
-      }
+          grt::DictRef entry(entries[i]);
 
-      _progress_label.set_text("");
+          std::string service_name = entry.get_string("Name", "invalid");
+          std::string service_display_name = entry.get_string("DisplayName", "invalid");
+          std::string path = entry.get_string("PathName", "invalid");
+          std::string state = entry.get_string("State", "unknown");
+          std::string start_mode = entry.get_string("StartMode", "unknown");
+
+          std::string config_file = base::extract_option_from_command_line("--defaults-file", path);
+          if (!local_connection)
+          {
+            if (config_file.empty())
+              config_file = "C$\\";
+            config_file = "\\\\" + host + "\\" + config_file;
+            base::replace(config_file, ":", "$");
+          }
+          _config_paths.push_back(config_file);
+          _service_names.push_back(service_name);
+          std::string selector_text = service_display_name + " (" + state + ", Start mode: " + start_mode + ")";
+          _service_selector.add_item(selector_text);
+        }
+        _progress_label.set_text("");
+      }
+      else
+      {
+        _progress_label.set_text("No MySQL service found.");
+        wizard()->set_problem(_("In order to manage a MySQL server it must be installed as a service. "
+          "The wizard could not find any MySQL service on the target machine, hence the server instance "
+          "cannot be created."));
+      }
 
       refresh_config_path();
     }
@@ -1129,7 +1142,7 @@ void WindowsManagementPage::enter(bool advancing)
       wizard()->set_problem(e.what());
 
       // In case this error was caused by wrong credentials (we cannot be sure from the error message)
-      // we remove the already stored password to make it possible to enter the current one.
+      // we remove the already stored password to make it possible to (re) enter the current one.
       Utilities::forget_password("wmi@" + host, user);
       values().gset("wmi_user_name", "");
     }
@@ -1140,6 +1153,9 @@ void WindowsManagementPage::enter(bool advancing)
 
 bool WindowsManagementPage::advance()
 {
+  if (_service_names.size() == 0 || _service_selector.get_selected_index() < 0)
+    return false;
+
   values().gset("ini_path", _config_path.get_string_value());
   values().gset("ini_section", "mysqld");
   values().gset("service_name", _service_names[_service_selector.get_selected_index()]); 
@@ -1271,9 +1287,10 @@ void TestHostMachineSettingsPage::leave(bool advancing)
     bool require_review = false;
     if (values().get_int("host_tests_succeeded") == 1)
     {
-      require_review = Utilities::show_message(_("Review settings"), _("The tests returned without error, so it appears the connection "
-        "settings are correct. Do you still want to review them for changes that may be required?"), _("Review"), _("Continue without review"))
-        == mforms::ResultOk;
+      require_review = Utilities::show_message(_("Review settings"),
+        _("Checks succeeded for Connection and Configuration Settings for this new Server Instance."),
+        _("Continue"), "", _("I'd like to review the settings again"))
+        == mforms::ResultOther;
     }
     else
       require_review = true;
@@ -1560,7 +1577,7 @@ void PathsPage::test_path()
   bool success;
   try
   {
-    if (wizard()->is_local())
+    if (values().get_int("windowsAdmin", 0) != 0 || wizard()->is_local())
       success= wizard()->test_setting("check_config_path/local", detail);
     else
       success= wizard()->test_setting("check_config_path", detail);
@@ -1592,7 +1609,7 @@ void PathsPage::test_section()
   bool success;
   try
   {
-    if (wizard()->is_local())
+    if (values().get_int("windowsAdmin", 0) != 0 || wizard()->is_local())
       success= wizard()->test_setting("check_config_section/local", detail);
     else
       success= wizard()->test_setting("check_config_section", detail);
