@@ -24,6 +24,7 @@
 #include "mforms/mforms.h"
 #include "mforms/grttreeview.h"
 #include "mforms/splitter.h"
+#include "mforms/menu.h"
 
 class OutputView : public mforms::AppView
 {
@@ -39,9 +40,11 @@ private:
   mforms::Splitter _splitter;
   mforms::GRTTreeView _message_list;
   mforms::TextBox _output_text;
+  mforms::Menu _context_menu;
   
+  void handle_command(const std::string& command);
+
   void row_added();
-  
   bool will_close();
 };
 
