@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,17 +22,91 @@
 #include "mforms/mforms.h"
 #include "mforms/widgets.h"
 #include "mforms/sectionbox.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #if defined(_WIN32) || defined(__APPLE__)
 #define HAVE_BUNDLED_MYSQLDUMP
 #endif
 
 using namespace base;
+using namespace mforms;
+
+
+static mforms::Label *new_label(const std::string &text, bool right_align=false, bool help=false)
+{
+  mforms::Label *label= mforms::manage(new mforms::Label());
+  label->set_text(text);
+  if (right_align)
+    label->set_text_align(mforms::MiddleRight);
+  if (help)
+    label->set_style(mforms::SmallHelpTextStyle);
+  return label;
+}
+
+
+
+class OptionTable : public mforms::Panel
+{
+  PreferencesForm *_owner;
+  mforms::Table _table;
+  int _rows;
+  bool _help_column;
+  
+public:
+  OptionTable(PreferencesForm *owner, const std::string &title, bool help_column)
+  : mforms::Panel(mforms::TitledBoxPanel), _owner(owner), _rows(0), _help_column(help_column)
+  {
+    set_title(title);
+    
+    add(&_table);
+    
+    _table.set_padding(8);
+    _table.set_row_spacing(12);
+    _table.set_column_spacing(8);
+    
+    _table.set_column_count(_help_column ? 3 : 2);    
+  }
+  
+  void add_option(mforms::View *control, const std::string &caption, const std::string &help)
+  {
+    _table.set_row_count(++_rows);
+   
+    _table.add(new_label(caption, true), 0, 1, _rows-1, _rows, mforms::HFillFlag|mforms::HExpandFlag);
+    _table.add(control, 1, 2, _rows-1, _rows, mforms::HFillFlag);
+
+    mforms::Label *l;
+    _table.add(l = new_label(help, false, true), 2, 3, _rows-1, _rows, mforms::HFillFlag);
+  }
+  
+
+  void add_entry_option(const std::string &option, const std::string &caption, const std::string &tooltip)
+  {
+    _table.set_row_count(++_rows);
+
+    mforms::TextEntry *entry = _owner->new_entry_option(option, false);
+    entry->set_tooltip(tooltip);
+    
+    _table.add(new_label(caption, true), 0, 1, _rows-1, _rows, mforms::HFillFlag|mforms::HExpandFlag);
+    _table.add(entry, 1, 2, _rows-1, _rows, mforms::HFillFlag);
+  }
+  
+  void add_checkbox_option(const std::string &option, const std::string &caption, const std::string &tooltip)
+  {
+    _table.set_row_count(++_rows);
+    
+    mforms::CheckBox *cb = _owner->new_checkbox_option(option);
+    cb->set_text(caption);
+    cb->set_tooltip(tooltip);
+    
+    _table.add(cb, 1, 2, _rows-1, _rows, mforms::HFillFlag);
+  }
+};
+
+
 
 PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const std::string &model_id)
   : Form(NULL, mforms::FormResizable), _top_box(false), _bottom_box(true), _button_box(true), 
-  _font_list(mforms::TreeShowHeader), _tabview(mforms::TabViewSystemStandard)
+  _font_list(mforms::TreeDefault), _tabview(mforms::TabViewDocument)
 {
   _wbui= wbui;
   _model_id= model_id;
@@ -42,8 +116,12 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const std::string &model
   else
     set_title(_("Model Options"));
 
-  set_size(700, 600);
+  set_size(700, 680);
   set_content(&_top_box);
+#ifdef _WIN32
+  set_back_color("#283752");
+#endif
+
   _top_box.set_padding(4);
   _top_box.set_spacing(4);
 
@@ -70,6 +148,9 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const std::string &model
   if (!model_id.empty())
   {
     _use_global.set_text(_("Use Global Settings"));
+#ifdef _WIN32
+    _use_global.set_front_color("#FFFFFF");
+#endif
     _bottom_box.add(&_use_global, true, true);
     _use_global.signal_clicked().connect(sigc::mem_fun(this, &PreferencesForm::toggle_use_global));
   }
@@ -310,12 +391,12 @@ mforms::Selector *PreferencesForm::new_selector_option(const std::string &option
   if (choices_string.empty())
     _wbui->get_wb_options_value(_model_id, "@"+option_name+"/Items", choices_string);
 
-  std::vector<std::string> choices, parts= bec::split_string(choices_string, ",");
+  std::vector<std::string> choices, parts= base::split(choices_string, ",");
 
   for (std::vector<std::string>::const_iterator iter= parts.begin();
        iter != parts.end(); ++iter)
   {
-    std::vector<std::string> tmp= bec::split_string(*iter, ":", 1);
+    std::vector<std::string> tmp= base::split(*iter, ":", 1);
     if (tmp.size() == 1)
     {
       selector->add_item(*iter);
@@ -345,38 +426,23 @@ PreferencesForm::~PreferencesForm()
 }
 
 
-mforms::Label *PreferencesForm::new_label(const std::string &text, bool right_align, bool help)
-{
-  mforms::Label *label= mforms::manage(new mforms::Label());
-  label->set_text(text);
-  if (right_align)
-    label->set_text_align(mforms::MiddleRight);
-  if (help)
-    label->set_style(mforms::SmallHelpTextStyle);
-  return label;
-}
+//--------------------------------------------------------------------------------------------------
 
 void PreferencesForm::ok_clicked()
 {
   update_values();
 
   mforms::Form::show(false);
-  _handle_close();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void PreferencesForm::cancel_clicked()
 {
   mforms::Form::show(false);
-  _handle_close();
-}
-
-
-void PreferencesForm::set_closed_handler(const sigc::slot<void> &slot)
-{
-  _handle_close= slot;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void PreferencesForm::create_admin_page()
 {
@@ -535,7 +601,7 @@ void PreferencesForm::create_sqlide_page()
       check->set_text(_("Forbid UPDATE and DELETE statements without a WHERE clause (safe updates). Requires a reconnection."));
       check->set_tooltip(_(
                            "Enables the SQL_SAFE_UPDATES option for the session, "
-                           "preventing SELECT and DELETE statements to be executed if a WHERE "
+                           "preventing UPDATE and DELETE statements to be executed if a WHERE "
                            "clause is not present. That will avoid some dangerous commands that "
                            "would update or delete all rows in a table by accident.\n"
                            "Changing this option requires a reconnection (Query -> Reconnect to Server)"));
@@ -670,85 +736,72 @@ void PreferencesForm::create_sqlide_page()
 
 void PreferencesForm::create_general_page()
 {
-  mforms::Table *table= mforms::manage(new mforms::Table());
+  mforms::Box *top_box = mforms::manage(new mforms::Box(false));
+  top_box->set_padding(12);
+  top_box->set_spacing(8);
 
-  table->set_padding(18);
-  table->set_row_spacing(8);
-  table->set_column_spacing(4);
-  table->set_row_count(6);
-  table->set_column_count(3);
+  OptionTable *table;
 
+  table = mforms::manage(new OptionTable(this, _("EER Modeler"), true));
+  top_box->add(table, false, true);
   {
-    mforms::CheckBox *check= new_checkbox_option("workbench.AutoReopenLastModel");
-    check->set_text(_("Automatically reopen previous model at start"));
-    table->add(check, 1, 3, 0, 1, mforms::HFillFlag);
-  }
-  
-  {
-    mforms::CheckBox *check= new_checkbox_option("Sidebar:RightAligned");
-    check->set_text(_("Place sidebar on the right side"));
-    check->set_tooltip(_(
-      "Whether to place sidebar on the left or right side in module windows.\n"
-      "Clear this checbox to place sidebar on the left side."));
-    table->add(check, 1, 3, 1, 2, mforms::HFillFlag);
-  }
-
+    table->add_checkbox_option("workbench.AutoReopenLastModel", _("Automatically reopen previous model at start"), "");
+    
 #ifndef __APPLE__
-  {
-    mforms::CheckBox *check= new_checkbox_option("workbench:ForceSWRendering");
-    check->set_text(_("Force use of software based rendering for EER diagrams"));
-    check->set_tooltip(_("Enable this option if you have drawing problems in Workbench modeling.\nYou must restart Workbench for the option to take effect."));
-    table->add(check, 1, 3, 2, 3, mforms::HFillFlag);
-  }
+    table->add_checkbox_option("workbench:ForceSWRendering", _("Force use of software based rendering for EER diagrams"), 
+                               _("Enable this option if you have drawing problems in Workbench modeling.\nYou must restart Workbench for the option to take effect."));
 #endif
-  {
-    table->add(new_label(_("Undo history size:"), true), 0, 1, 3, 4, mforms::HFillFlag);
-
-    mforms::TextEntry *entry= new_numeric_entry_option("workbench:UndoEntries", 1, 500);
-    entry->set_max_length(5);
-    entry->set_size(50, -1);
-    table->add(entry, 1, 2, 3, 4, mforms::HFillFlag);
-
-    mforms::Label *descr= mforms::manage(new mforms::Label());
-    descr->set_size(300, -1);
-    descr->set_wrap_text(true);
-    descr->set_text(_("Allowed values are from 1 up.\nNote: using high values (> 100) will increase memory usage and slow down operation."));
-    descr->set_style(mforms::SmallHelpTextStyle);
-    table->add(descr, 2, 3, 3, 4, mforms::HFillFlag | mforms::HExpandFlag);
-  }
-  
-  {
-    table->add(new_label(_("Auto-save model interval:"), true), 0, 1, 4, 5, mforms::HFillFlag);
     
-    static const char *auto_save_intervals= "disable:0,30 seconds:30,1 minute:60,5 minutes:300,10 minutes:600,20 minutes:1200";    
-    mforms::Selector *sel = new_selector_option("workbench:AutoSaveModelInterval", auto_save_intervals);
-    table->add(sel, 1, 2, 4, 5, mforms::HFillFlag);
+    {    
+      mforms::TextEntry *entry= new_numeric_entry_option("workbench:UndoEntries", 1, 500);
+      entry->set_max_length(5);
+      entry->set_size(100, -1);
+      
+      table->add_option(entry, _("Model undo history size:"), 
+                        _("Allowed values are from 1 up.\nNote: using high values (> 100) will increase memory usage\nand slow down operation."));
+    }
     
-    mforms::Label *descr= mforms::manage(new mforms::Label());
-    descr->set_size(300, -1);
-    descr->set_wrap_text(true);
-    descr->set_text(_("Interval to perform auto-saving of the open model. The model will be restored from the last auto-saved version if Workbench unexpectedly quits."));
-    descr->set_style(mforms::SmallHelpTextStyle);
-    table->add(descr, 2, 3, 4, 5, mforms::HFillFlag | mforms::HExpandFlag);
+    {
+      static const char *auto_save_intervals= "disable:0,10 seconds:10,15 seconds:15,30 seconds:30,1 minute:60,5 minutes:300,10 minutes:600,20 minutes:1200";    
+      mforms::Selector *sel = new_selector_option("workbench:AutoSaveModelInterval", auto_save_intervals, true);
+      
+      table->add_option(sel, _("Auto-save model interval:"), 
+                        _("Interval to perform auto-saving of the open model.\nThe model will be restored from the last auto-saved version\nif Workbench unexpectedly quits."));
+    }
   }
-  
-  {
-    table->add(new_label(_("Interactive GRT Shell language:"), true), 0, 1, 5, 6, mforms::HFillFlag);
 
+  table = mforms::manage(new OptionTable(this, _("SQL Editor"), true));
+  top_box->add(table, false, true);
+  {
+    table->add_checkbox_option("workbench:SaveSQLWorkspaceOnClose",
+                               _("Save snapshot of open editors on close"),
+                               _("A snapshot of all open scripts is saved when the SQL Editor is closed. Next time it is opened to the same connection that state is restored. Unsaved files will remain unsaved, but their contents will be preserved."));
+
+    {
+      static const char *auto_save_intervals= "disable:0,10 seconds:10,15 seconds:15,30 seconds:30,1 minute:60,5 minutes:300,10 minutes:600,20 minutes:1200";    
+      mforms::Selector *sel = new_selector_option("workbench:AutoSaveScriptsInterval", auto_save_intervals, true);
+      
+      table->add_option(sel, _("Auto-save scripts interval:"), 
+                        _("Interval to perform auto-saving of all open script tabs.\nThe scripts will be restored from the last auto-saved version\nif Workbench unexpectedly quits."));
+    }    
+  }
+
+  
+  table = mforms::manage(new OptionTable(this, _("Others"), true));
+  top_box->add(table, false, true);
+  {
+    table->add_checkbox_option("Sidebar:RightAligned", _("Place sidebar on the right side"), 
+                               _("Whether to place sidebar on the left or right side in module windows.\n"
+                                 "Clear this checbox to place sidebar on the left side."));
+    
     mforms::Selector *combo= new_selector_option("grtshell:ShellLanguage");
-    table->add(combo, 1, 2, 5, 6, mforms::HFillFlag);
-
-    mforms::Label *descr= mforms::manage(new mforms::Label());
-    descr->set_style(mforms::SmallHelpTextStyle);
-    descr->set_wrap_text(true);
-    descr->set_size(300, -1);
-    descr->set_text(_("Select the language to use in the interactive GRT shell. "
-      "Scripts, modules and plugins will work regardless of this setting. "
-      "This option requires a restart."));
-    table->add(descr, 2, 3, 5, 6, mforms::HFillFlag | mforms::HExpandFlag);
+    table->add_option(combo, _("Interactive GRT Shell language:"), 
+                      _("Select the language to use in the interactive GRT shell.\n"
+                        "Scripts, modules and plugins will work regardless of this setting.\n"
+                        "This option requires a restart."));
   }
 
-  _tabview.add_page(table, _("General"));
+  _tabview.add_page(top_box, _("General"));
 }
 
 
@@ -1159,7 +1212,52 @@ void PreferencesForm::create_appearance_page()
  */
 void PreferencesForm::create_test_page()
 {
-  mforms::Box* content= mforms::manage(new mforms::Box(false));
+  Box* content= manage(new Box(false));
+
+  TextEntry* file_name = manage(new TextEntry());
+  file_name->set_value("dummy entry");
+  file_name->set_size(300, -1);
+  Selector* section = manage(new Selector());
+  section->set_enabled(false);
+  section->set_size(150, -1);
+  
+  Box* bottom_box = manage(new Box(true));
+  content->add(bottom_box, false, false);
+  
+  Button* accept_btn = manage(new Button());
+  accept_btn->set_text("Apply ...");
+  accept_btn->set_size(80, -1);
+  
+  Button* discard_btn = manage(new Button());
+  discard_btn->set_text("Discard");
+  discard_btn->set_size(80, -1);
+  
+  Label* search_label = manage(new Label("Locate option:"));
+  TextEntry* option_lookup_entry = manage(new TextEntry());
+  Button* search_btn = manage(new Button());
+  search_btn->set_text("Find ...");
+  
+  Box* search_box = manage(new Box(true));
+  search_box->set_padding(2);
+  search_box->set_spacing(4);
+  search_box->set_size(300, -1);
+  search_box->add(search_label, false, false);
+  search_box->add(option_lookup_entry, true, false);
+  search_box->add(search_btn, false, false);
+  Panel* search_panel = manage(new Panel(mforms::BorderedPanel));
+  search_panel->set_size(300, -1);
+  //search_panel->add(search_box);
+
+  bottom_box->add(manage(new Label("Configuration File:")), false, false);
+  bottom_box->add(file_name, true, false);
+  bottom_box->add(section, false, false);
+  bottom_box->add(search_box, false, false);
+  
+  Utilities::add_end_ok_cancel_buttons(bottom_box, accept_btn, discard_btn);
+  
+  bottom_box->set_spacing(8);
+  bottom_box->set_padding(12);
+  bottom_box->set_back_color("#FFFFFF");
 
   _tabview.add_page(content, _("Test page"));
 }
@@ -1206,7 +1304,7 @@ void PreferencesForm::show_colors_and_fonts()
         try
         {
           std::string part= iter->substr(pos + 1);
-          std::string figure= bec::split_string(iter->substr(0, pos), ".")[2];
+          std::string figure= base::split(iter->substr(0, pos), ".")[2];
           std::string caption;
 
           part= part.substr(0, part.length() - 4);
