@@ -1,20 +1,20 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
- * Boston, MA 02110-1301  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -24,7 +24,7 @@
 
 #include "grtpp_util.h"
 #include <grt/common.h>
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #define SYSTEM_STAT_HEADER "# Customize this script to suit your system if necessary\n"\
 "# Shell function names and output format must be maintained\n\n"
@@ -86,6 +86,18 @@ std::string get_admin_script_for_os(const std::string &os)
 }
 
 
+#define WINDOWS_CONFIG_HINT_TEXT "\nWindows Hint:\n\n"\
+"When Server Instance is configured for\n"\
+"\"Windows Remote Management\", the \"Configuration File\"\n"\
+"must be specified using a mapped drive,\n"\
+"network share or administrative share.\n\n"\
+"Examples of these are\n"\
+"M:\\<path to file>\\my.ini\n"\
+"\\\\<server>\\<share>\\<path to file>\\my.ini\n"\
+"\\\\<server>\\C$\\Program Fiels\\MySQL\\MySQL Server 5.5\\my.ini"
+
+
+
 static const char *filter_help = 
 "Workbench expects status scripts to return 0 on success (running) and 1 otherwise.\n"
 "In case of OS where execution status is not always correctly detected\n"
@@ -169,7 +181,7 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
 , _top_hbox(true)
 , _content_box(false)
 , _inst_list_buttons_hbox(true)
-, _stored_instance_list(TreeShowHeader)
+, _stored_instance_list(TreeDefault)
 , _tabview(mforms::TabViewSystemStandard)
 , _conn_box(false)
 , _remote_param_box(false)
@@ -297,9 +309,7 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
     _ssh_remote_admin.signal_toggled().connect(sigc::mem_fun(this, &ServerInstanceEditor::toggle_administration));
 
     remote_admin_box->add(&_no_remote_admin, false, true);
-#ifdef NOT_READY
     remote_admin_box->add(&_win_remote_admin, false, true);
-#endif
     remote_admin_box->add(&_ssh_remote_admin, false, true);
 
     _remote_param_box.set_spacing(12);
@@ -394,7 +404,7 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
     
     _sys_box.add(label, false, true);
 
-    Table *table = manage(NewTable(5, 2));
+    Table *table = manage(NewTable(6, 2));
     _sys_box.add(table, false, true);
 
     {
@@ -435,6 +445,8 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
     _sys_win_service_name.signal_changed().connect(sigc::bind(sigc::mem_fun(this, &ServerInstanceEditor::entry_changed), 
                                                   &_sys_win_service_name));
 
+    _sys_win_hint_label.set_text(WINDOWS_CONFIG_HINT_TEXT);
+    table->add(&_sys_win_hint_label, 1, 2, 5, 6, HFillFlag|HExpandFlag);
 
     _details_description.set_text(_("\nThe following options specify how Workbench should perform certain management actions. "
       "Commands given here are the same as if they would be run in a system shell. Commands which return "
@@ -610,7 +622,11 @@ void ServerInstanceEditor::set_password(bool clear)
   
   std::string storage_key;
   if (_ssh_remote_admin.get_active())
-    storage_key = strfmt("ssh@%s:%s", _remote_host.get_string_value().c_str(), port.empty() ? "22" : port.c_str());
+  {
+    // WBA stores password with key ssh@host, without port
+    //storage_key = strfmt("ssh@%s:%s", _remote_host.get_string_value().c_str(), port.empty() ? "22" : port.c_str());
+    storage_key = strfmt("ssh@%s", _remote_host.get_string_value().c_str());
+  }
   else
     storage_key = "wmi@" + _remote_host.get_string_value();
   std::string username = _remote_user.get_string_value();
@@ -785,7 +801,7 @@ void ServerInstanceEditor::run_filechooser_wrapper(mforms::TextEntry* entry) //
 
 void ServerInstanceEditor::run_filechooser(mforms::TextEntry* entry)
 {
-  mforms::FileChooser fc(mforms::OpenFile);
+  mforms::FileChooser fc(mforms::OpenFile, true);
   //TODO: Add set directory
   if (fc.run_modal())
   {
@@ -945,7 +961,6 @@ void ServerInstanceEditor::toggle_administration()
   // enable the system commands section.
   bool can_administer= local_connection || ssh_administration || win_administration;
   _sys_box.set_enabled(can_administer);
-
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1102,7 +1117,7 @@ void ServerInstanceEditor::reorder_instance(bool up)
 
 void ServerInstanceEditor::browse_file()
 {
-  FileChooser fsel(mforms::OpenFile);
+  FileChooser fsel(mforms::OpenFile, true);
 
   fsel.set_title(_("Pick SSH Private Key"));
 
@@ -1525,17 +1540,21 @@ void ServerInstanceEditor::show_instance()
 
   std::string storage_key;
   std::string port = _ssh_port.get_string_value();
-  std::string username = _remote_user.get_string_value();
+  std::string username;
   if (_ssh_remote_admin.get_active())
   {
     _remote_host.set_value(loginInfo.get_string("ssh.hostName"));
     _remote_user.set_value(loginInfo.get_string("ssh.userName"));
-    storage_key = strfmt("ssh@%s:%s", _remote_host.get_string_value().c_str(), port.empty() ? "22" : port.c_str());
+    username = _remote_user.get_string_value();
+    // WBA stores password key as "ssh@<host>"
+    //storage_key = strfmt("ssh@%s:%s", _remote_host.get_string_value().c_str(), port.empty() ? "22" : port.c_str());
+    storage_key = strfmt("ssh@%s", _remote_host.get_string_value().c_str());
   }
   else
   {
     _remote_host.set_value(loginInfo.get_string("wmi.hostName"));
     _remote_user.set_value(loginInfo.get_string("wmi.userName"));
+    username = _remote_user.get_string_value();
     storage_key = "wmi@" + _remote_host.get_string_value();
   }
   _ssh_port.set_value(loginInfo.get_string("ssh.tunnelPort"));
@@ -1556,6 +1575,7 @@ void ServerInstanceEditor::show_instance()
     _details_tabview.show(false);
     _sys_win_service_name.show(true);
     _sys_win_service_name_label->show(true);
+    _sys_win_hint_label.show(true);
   }
   else
   {
@@ -1563,9 +1583,10 @@ void ServerInstanceEditor::show_instance()
     _details_tabview.show(true);
     _sys_win_service_name.show(false);
     _sys_win_service_name_label->show(false);
+    _sys_win_hint_label.show(false);
   }
 
-    // If the MySQL connection is to a local server then remote administration makes no sense.
+  // If the MySQL connection is to a local server then remote administration makes no sense.
   // Disable it in this case.
   if (local_connection)
   {
