@@ -3,7 +3,7 @@
 
 #include <gtkmm/notebook.h>
 #include <gtkmm/widget.h>
-#include "ui_form.h"
+#include "base/ui_form.h"
 
 class PluginEditorBase;
 
