@@ -33,6 +33,15 @@ Gtk::TreeModel::Path node2path(const ::bec::NodeId& node)
 //------------------------------------------------------------------------------
 ColumnsModel::~ColumnsModel()
 {
+  reset(true); // true means clean up only self, and do not touch tree view
+}
+
+//------------------------------------------------------------------------------
+void ColumnsModel::reset(const bool cleanup_only_self)
+{
+  if (!cleanup_only_self)
+    _treeview->remove_all_columns();
+
   std::list<Gtk::TreeModelColumnBase*>::iterator         it = _columns.begin();
   std::list<Gtk::TreeModelColumnBase*>::const_iterator last = _columns.end();
 
@@ -42,13 +51,6 @@ ColumnsModel::~ColumnsModel()
   _columns.clear();
 }
 
-//------------------------------------------------------------------------------
-void ColumnsModel::reset()
-{
-  _treeview->remove_all_columns();
-  _columns.clear();
-}
-
 //------------------------------------------------------------------------------
 void ColumnsModel::add_bec_index_mapping(const int bec_tm_index)
 {
@@ -197,6 +199,40 @@ ColumnsModel::append_string_column(const int bec_tm_idx, const std::string& name
   return *col;
 }
 
+//------------------------------------------------------------------------------
+const StringColumn&
+ColumnsModel::append_markup_column(const int bec_tm_idx, const std::string& name, const Iconic have_icon)
+{
+  Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > *icon= 0;
+
+  Gtk::TreeViewColumn *column= Gtk::manage(new Gtk::TreeViewColumn(bec::replace_string(name, "_", "__")));
+
+  if ( have_icon == WITH_ICON )
+  {
+    icon = new Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> >;
+    add(*icon);
+    add_bec_index_mapping(bec_tm_idx);
+    column->pack_start(*icon, false);
+
+    _columns.push_back(icon);
+  }
+
+  Gtk::TreeModelColumn<Glib::ustring> *col = new Gtk::TreeModelColumn<Glib::ustring>;
+  Gtk::CellRendererText *cell = Gtk::manage(new Gtk::CellRendererText());
+  add(*col);
+  add_bec_index_mapping(bec_tm_idx);
+  column->pack_start(*cell);
+  column->add_attribute(cell->property_markup(), *col);
+
+  _columns.push_back(col);
+
+  int nr_of_cols= _treeview->append_column(*column);
+  _treeview->get_column(nr_of_cols-1)->set_resizable(true);
+
+  return *col;
+}
+
+
 //------------------------------------------------------------------------------
 const StringColumn&
 ColumnsModel::append_combo_column(const int bec_tm_idx
@@ -320,6 +356,7 @@ ListModelWrapper::ListModelWrapper(bec::ListModel* tm, Gtk::TreeView *treeview,
                     , _name(name)
 {
   tm->tree_changed_signal().connect(sigc::mem_fun(*this, &::ListModelWrapper::model_changed));
+  tm->add_destroy_notify_callback(this, &ListModelWrapper::on_bec_model_destroyed);
   #ifndef NO_MENU_MANAGER
   if (_treeview)
     _treeview->signal_event().connect(sigc::mem_fun(this, &ListModelWrapper::handle_popup_event));
@@ -328,6 +365,18 @@ ListModelWrapper::ListModelWrapper(bec::ListModel* tm, Gtk::TreeView *treeview,
 
 ListModelWrapper::~ListModelWrapper()
 {
+  if (_tm)
+    _tm->remove_destroy_notify_callback(this);
+}
+
+void* ListModelWrapper::on_bec_model_destroyed(void* data)
+{
+  ListModelWrapper* self = (ListModelWrapper*)(data);
+  if (self)
+  {
+    self->_tm = 0;
+  }
+  return 0;
 }
 
 void ListModelWrapper::set_iconview(Gtk::IconView* iv)
@@ -363,6 +412,11 @@ ListModelWrapper::NodeIdArray ListModelWrapper::get_selection() const
   return nodes;
 }
 
+void ListModelWrapper::set_be_model(bec::ListModel* tm)
+{
+  _tm = tm;
+}
+
 #ifndef NO_MENU_MANAGER
 void ListModelWrapper::handle_popup(const int x, const int y, const int time)
 {
@@ -469,10 +523,15 @@ bec::NodeId ListModelWrapper::node_for_iter(const iterator &iter) const throw()
 
   if ( tm && tm == _tm && uid )
   {
-    if (it->stamp == _stamp)
-      return bec::NodeId(tm->nodeuid_to_path(uid));
-    else
-      g_warning("reference to invalid iter in grt list/tree model");
+    try
+    {
+      if (it->stamp == _stamp)
+        return bec::NodeId(tm->nodeuid_to_path(uid));
+      else
+        g_warning("reference to invalid iter in grt list/tree model");
+    }
+    catch (...)
+    {}
   }
   return bec::NodeId();
 }
