@@ -3,7 +3,7 @@
 #include "sql_editor_fe.h"
 #include "linux_utilities/gtk_helpers.h"
 #include "mforms/mforms.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
  //------------------------------------------------------------------------------
 SqlEditorFE::SqlEditorFE()
@@ -110,6 +110,9 @@ void SqlEditorFE::notify(SCNotification *event)
           sigc::mem_fun(this, &SqlEditorFE::on_background_action_timer),
           2000); //! get value from wb options
         _text_changed_signal.emit();
+
+        if (_be)
+          _be->sql(get_text());
       }
       if (mod_type & SC_MOD_CHANGEFOLD)
         fold_changed(event->line, event->foldLevelNow, event->foldLevelPrev);
@@ -129,8 +132,18 @@ void SqlEditorFE::notify(SCNotification *event)
         int end = send_editor(SCI_GETSELECTIONEND);
         
         if (start != _old_selection_start || end != _old_selection_end)
+        {
+          if (_be)
+            _be->set_selected_range(start, end);
           _selection_changed_signal.emit();
-        
+        }
+
+        if (_be)
+        {
+          int pos= send_editor(SCI_GETCURRENTPOS);
+          _be->set_cursor_pos(pos);
+        }
+
         _old_selection_start= start;
         _old_selection_end= end;
       }
@@ -189,7 +202,9 @@ std::string SqlEditorFE::get_text()
     }
   }
 
-  return buffer ? buffer : "";
+  std::string ret = buffer ? buffer : "";
+  delete[] buffer;
+  return ret;
 }
 
 //------------------------------------------------------------------------------
@@ -207,9 +222,12 @@ void SqlEditorFE::set_font(const std::string &font)
   std::string name;
   int size;
   bool bold, italic;
-  if (!font.empty() && Sql_editor::parse_font(font, name, size, bold, italic))
+  if (!font.empty() && base::parse_font_description(font, name, size, bold, italic))
   {
-    name+= "-iso10646-1,*";
+    // scintilla requires the ! in front of the font name to interpret it as a pango/fontconfig font
+    // the non-pango version is totally unusable
+    if (!name.empty() && name[0] != '!')
+      name = "!"+name;
     for (int i = 0; i < 32; i++)
     {
       send_editor(SCI_STYLESETFONT, i, (sptr_t)name.c_str());
@@ -248,12 +266,25 @@ void SqlEditorFE::be(Sql_editor::Ref be)
     _be->sql_parser_err_cb(sigc::mem_fun(this, &SqlEditorFE::process_sql_error));//!
     _be->insert_text_slot= sigc::bind_return(sigc::mem_fun(this, &SqlEditorFE::insert_text), 0);
     _be->replace_selected_text_slot= sigc::bind_return(sigc::mem_fun(this, &SqlEditorFE::replace_selected_text), 0);
+    _be->change_selected_range_slot= sigc::mem_fun(this, &SqlEditorFE::change_selected_range);
+    _be->change_cursor_pos_slot= sigc::mem_fun(this, &SqlEditorFE::change_cursor_pos);
 
     set_font(grt::StringRef::cast_from(be->grtm()->get_app_option("workbench.general.Editor:Font")));
   }
   return;
 }
 
+void SqlEditorFE::change_selected_range(int start, int end)
+{
+  send_editor(SCI_SETSELECTIONSTART, start);
+  send_editor(SCI_SETSELECTIONEND, end);
+}
+
+void SqlEditorFE::change_cursor_pos(int pos)
+{
+  send_editor(SCI_GOTOPOS, pos);
+}
+
 bool SqlEditorFE::on_background_action_timer()
 {
   _background_action_cb();
@@ -600,15 +631,6 @@ void SqlEditorFE::replace_selected_text(const std::string &text)
 
 //------------------------------------------------------------------------------
 
-void SqlEditorFE::set_selected_range()
-{
-  int start= send_editor(SCI_GETSELECTIONSTART);
-  int end= send_editor(SCI_GETSELECTIONEND);
-  _be->set_selected_range(start, end);
-}
-
-//------------------------------------------------------------------------------
-
 void SqlEditorFE::set_dirty(bool flag)
 {
   _dirty= flag;
@@ -706,7 +728,7 @@ void SqlEditorFE::redo()
 
 bool SqlEditorFE::on_button_press_event(GdkEventButton *event)
 {
-  if (event->button == 3) // right button
+  if (event->button == 3 && _be) // right button
   {
     bec::MenuItemList items= _be->get_context_menu();
     if (!items.empty())
