@@ -32,7 +32,9 @@
 #include "linux_utilities/toolbar_manager.h"
 #include "sqlide/sql_editor_fe.h"
 
-#include "string_utilities.h"
+#include "base/string_utilities.h"
+#include "mforms/../gtk/lf_menubar.h"
+#include "mforms/../gtk/lf_toolbar.h"
 
 using base::strfmt;
 
@@ -71,21 +73,29 @@ MainForm::MainForm(wb::WBContextUI* ctx)
   _ui= new MGGladeXML(grtm->get_data_file_path("wb.glade").c_str(), "", "");
   _toolbar_manager = new ToolbarManager(ctx->get_command_ui());
 
+  _accel_group = get_mainwindow()->get_accel_group();
+  if (!_accel_group)
+  {
+    _accel_group= Gtk::AccelGroup::create();
+    get_mainwindow()->add_accel_group(_accel_group);
+  }
+  mforms::gtk::set_accel_group(_accel_group);
+
   get_mainwindow()->signal_delete_event().connect(sigc::mem_fun(this, &MainForm::close_window));
   get_mainwindow()->signal_set_focus().connect(sigc::mem_fun(this, &MainForm::on_focus_widget));
 
   get_mainwindow()->set_title("MySQL Workbench");
   
-  _accel_group= Gtk::AccelGroup::create();
-  _menu_manager= new MenuManager(ctx->get_command_ui(), _accel_group);
-  get_mainwindow()->add_accel_group(_accel_group);
+  //_accel_group= Gtk::AccelGroup::create();
+  _menu_manager = new MenuManager(ctx->get_command_ui(), _accel_group);
+  //get_mainwindow()->add_accel_group(_accel_group);
 
   _model_panel= 0;
 
   _find_panel= 0;
 
   get_upper_note()->signal_switch_page().connect(sigc::mem_fun(*this, &MainForm::switch_page));
-
+  
   Gtk::Statusbar *status = 0;
   _ui->get("statusbar1", &status);
   if (status)
@@ -163,9 +173,11 @@ void MainForm::on_focus_widget(Gtk::Widget *w)
       _wbui_context->set_active_form(form);
   }
 
+  _wbui_context->get_command_ui()->signal_validate_edit_menu_items().emit();
+
   // for updating the Edit menu
-  Gtk::Alignment* menu_holder;
-  _menu_manager->rebuild(_ui->get("main_menu_box", &menu_holder));
+ //!!! Gtk::Alignment* menu_holder;
+ //!!! _menu_manager->rebuild(_ui->get("main_menu_box", &menu_holder));
 }
 
 //------------------------------------------------------------------------------
@@ -174,14 +186,8 @@ void MainForm::setup_ui()
 {  
   set_window_icons(get_mainwindow());
 
-
   register_commands();
   
-  Gtk::Alignment* menu_holder;
-  _menu_manager->rebuild(_ui->get("main_menu_box", &menu_holder));
-
-  refresh_toolbar("");
-
   reset_layout();
 
   switch_page(0, 0);
@@ -366,7 +372,8 @@ void MainForm::call_find()
   }
 
   Gtk::Box *box;
-  _toolbar_manager->focus_searchbox(_ui->get("main_toolbar", &box));
+ // _toolbar_manager->focus_searchbox(_ui->get("main_toolbar", &box));
+#warning missing here
 }
 
 
@@ -880,7 +887,7 @@ void MainForm::perform_command_becb(const std::string& command)
 //------------------------------------------------------------------------------
 mdc::CanvasView* MainForm::create_view_becb(const model_DiagramRef &diagram)
 {
-  ModelDiagramPanel *model_panel= Gtk::manage(ModelDiagramPanel::create(_wbui_context, _toolbar_manager));
+  ModelDiagramPanel *model_panel= Gtk::manage(ModelDiagramPanel::create(_wbui_context));
 
   model_panel->set_close_editor_callback(sigc::bind(sigc::ptr_fun(close_plugin), _wbui_context->get_wb()));
 
@@ -889,8 +896,8 @@ mdc::CanvasView* MainForm::create_view_becb(const model_DiagramRef &diagram)
   model_panel->init(diagram.id());
 
   add_form_pane(model_panel, !model_panel->get_diagram_form()->is_closed());
-  model_panel->set_data("canvas", model_panel->get_canvas());
-  model_panel->set_data("model_panel", model_panel);
+ // model_panel->set_data("model_panel", model_panel);
+
 
   return model_panel->get_canvas();
 }
@@ -900,13 +907,15 @@ mdc::CanvasView* MainForm::create_view_becb(const model_DiagramRef &diagram)
 void MainForm::destroy_view_becb(mdc::CanvasView* view)
 {
   Gtk::Notebook *note= get_upper_note();
-  
+
   if (!_wbui_context->get_wb()->get_grt_manager()->in_main_thread())
     G_BREAKPOINT();
   
   for (int c= note->get_n_pages(), i= 1; i < c; i++)
   {
-    ModelDiagramPanel *model_panel = reinterpret_cast<ModelDiagramPanel*>(note->get_nth_page(i)->get_data("model_panel"));
+    FormViewBase *form = reinterpret_cast<FormViewBase*>(note->get_nth_page(i)->get_data("FormViewBase"));
+    ModelDiagramPanel *model_panel = dynamic_cast<ModelDiagramPanel*>(form);
+    //ModelDiagramPanel *model_panel = reinterpret_cast<ModelDiagramPanel*>(note->get_nth_page(i)->get_data("model_panel"));
 
     if (model_panel && model_panel->get_canvas() == view)
     {
@@ -964,9 +973,6 @@ void MainForm::tool_changed_becb(mdc::CanvasView* view)
   ModelDiagramPanel *panel= get_panel_for_view(view);
   if (panel)
   {
-    refresh_toolbar(WB_TOOLBAR_TOOLS);
-    refresh_toolbar(WB_TOOLBAR_OPTIONS);
-
     // update cursor in canvas
     panel->update_tool_cursor();
   }
@@ -979,7 +985,6 @@ const char* RefreshTypeStr[] = {
     "RefreshSchema",
     "RefreshNewDiagram",
     "RefreshSelection",
-    "RefreshToolbar",
     "RefreshMessages",
     "RefreshCloseEditor",
     "RefreshNewModel",
@@ -990,7 +995,6 @@ const char* RefreshTypeStr[] = {
     "RefreshDocument",
     "RefreshCloseDocument",
     "RefreshZoom",
-    "RefreshMenubar",
     "RefreshTimer",
     "RefreshFinishEdit"
 };
@@ -1102,10 +1106,6 @@ void MainForm::refresh_gui_becb(wb::RefreshType type, const std::string& arg_id,
     break;
   }
   
-  case wb::RefreshToolbar:
-    refresh_toolbar(arg_id);
-    break;
-
   case wb::RefreshViewName:
     {
       // Find tab with view and rename it
@@ -1113,19 +1113,13 @@ void MainForm::refresh_gui_becb(wb::RefreshType type, const std::string& arg_id,
       Gtk::Notebook *note= get_upper_note();
       for (int c= note->get_n_pages(), i= 1; i < c; i++)
       {
-        ModelDiagramPanel *model_panel = reinterpret_cast<ModelDiagramPanel*>(note->get_nth_page(i)->get_data("model_panel"));
+        ModelDiagramPanel *model_panel = dynamic_cast<ModelDiagramPanel*>(reinterpret_cast<FormViewBase*>(note->get_nth_page(i)->get_data("FormViewBase")));
 
-        if (dynamic_cast<FormViewBase*>(model_panel) == form)
+        if (model_panel && model_panel == form)
           model_panel->set_title(arg_id);
       }
       break;
     }
-  case wb::RefreshMenubar:
-    {
-      Gtk::Alignment* menu_holder;
-      _menu_manager->rebuild(_ui->get("main_menu_box", &menu_holder));
-      break;
-    }
   case wb::RefreshDocument:
     {
       get_mainwindow()->set_title(_wbui_context->get_title());
@@ -1240,52 +1234,19 @@ void MainForm::lock_gui_becb(bool lock)
   _gui_locked= lock;
 }
 
-//------------------------------------------------------------------------------
-void MainForm::popup_menu_becb(bec::MenuItemList* items, int x, int y)
-{
-  run_popup_menu(*items, GDK_CURRENT_TIME,
-                 sigc::mem_fun(_wbui_context->get_command_ui(), 
-                               (void(wb::CommandUI::*)(const std::string&))&wb::CommandUI::activate_command));
-}
-
-//------------------------------------------------------------------------------
-void MainForm::refresh_toolbar(const std::string &toolbar)
-{
-  Gtk::Box *box;
-  if (toolbar.empty() || toolbar == WB_TOOLBAR_MAIN)
-    _toolbar_manager->rebuild_toolbar(_ui->get("main_toolbar", &box), WB_TOOLBAR_MAIN);
-
-  if (toolbar.empty() || (toolbar == WB_TOOLBAR_OPTIONS || toolbar == WB_TOOLBAR_TOOLS))
-  {
-    bec::UIForm *form= _wbui_context->get_active_main_form();
-    if (form)
-    {
-      ModelDiagramPanel *model= dynamic_cast<ModelDiagramPanel*>(reinterpret_cast<FormViewBase*>(form->get_frontend_data()));
-      if (model)
-      {
-        model->update_toolbar(toolbar);
-      }
-    }
-  }
-}
-
 //------------------------------------------------------------------------------
 void MainForm::switch_page(GtkNotebookPage*, guint pagenum)
 {
   Gtk::Notebook *note= get_upper_note();
-
   Gtk::Widget *page = note->get_nth_page(pagenum);
-  page->show();
-  note->set_current_page(pagenum);
+  if (page)
+  {
+    page->show();
 
-  bec::UIForm* uiform = static_cast<bec::UIForm*>(page->get_data("uiform"));
-  _wbui_context->set_active_form(uiform);
-  
-  // force an immediate refresh so that UI updates quicker
-  //  refresh_toolbar("");
-
-  Gtk::Alignment* menu_holder;
-  _menu_manager->rebuild(_ui->get("main_menu_box", &menu_holder));
+    bec::UIForm* uiform = static_cast<bec::UIForm*>(page->get_data("uiform"));
+    _wbui_context->set_active_form(uiform);
+  }
+  _wbui_context->get_command_ui()->signal_validate_edit_menu_items().emit();
 }
 
 
@@ -1399,6 +1360,10 @@ void MainForm::close_active_tab()
     {
       Gtk::Widget *page = upper->get_nth_page(curpagenum);
       FormViewBase *panel = reinterpret_cast<FormViewBase*>(page->get_data("FormViewBase"));
+      bec::UIForm *form = reinterpret_cast<bec::UIForm*>(page->get_data("uiform"));
+      if (form && form->get_form_context_name() == "home")
+        return;
+      
       if (panel)
       {
         if (!panel->close_focused_tab())
@@ -1413,16 +1378,16 @@ void MainForm::close_active_tab()
 
 void MainForm::close_tab(Gtk::Notebook *note, Gtk::Widget *page, const sigc::slot<bool> &on_close)
 {
-  // check if the page has a sidebar associated and remove it if so
-  FormViewBase *form = reinterpret_cast<FormViewBase*>(page->get_data("FormViewBase"));
-  
   // on_close should return true if the form should be closed/removed
   if (on_close)
   {
     if (on_close())
     {
+      bool is_appview = page->get_data("AppView") != 0;
       page->hide();
       note->remove_page(*page);
+      if (!is_appview)
+        page->unreference();
     }
     else
       return;
@@ -1592,11 +1557,11 @@ void MainForm::show_about()
   Gtk::Fixed fixed;
   dialog.add(fixed);
 
-  Gtk::Label label("Version " PACKAGE_VERSION);
+  Gtk::Label label(base::strfmt("Version %s\nrev %i", PACKAGE_VERSION, APP_REVISION_NUMBER));
   label.set_alignment(1.0, 0.0);
   label.set_justify(Gtk::JUSTIFY_RIGHT);
-  fixed.put(label, 0, 150);
-  label.set_size_request(525, 50);
+  fixed.put(label, 10, 140);
+  label.set_size_request(540, 50);
   label.modify_fg(Gtk::STATE_NORMAL, label.get_style()->get_black());
 
   dialog.set_modal(true);
@@ -1703,7 +1668,7 @@ static std::list<std::pair<int,int> > event_loop_exit_codes;
 static int begin_event_loop(mforms::App*,float timeout)
 {
   Gtk::Main::run();
-  if (event_loop_exit_codes.empty() || event_loop_exit_codes.back().first != Gtk::Main::level())
+  if (event_loop_exit_codes.empty() || event_loop_exit_codes.back().first != (int)Gtk::Main::level())
   {
     // means something other than end_event_loop() called quit(), could mean the app is being quit.. 
     Gtk::Main::quit();
@@ -1744,6 +1709,42 @@ void MainForm::setup_mforms_app()
 }
 
 
+
+Gtk::Widget *MainForm::decorate_widget(Gtk::Widget *panel, bec::UIForm *form)
+{
+  mforms::MenuBar *menu = form->get_menubar();
+  mforms::ToolBar *toolbar = form->get_toolbar();
+
+  Gtk::VBox *top_box = Gtk::manage(new Gtk::VBox(false, 0));
+  if (menu)
+  {
+    Gtk::Widget *w = mforms::gtk::widget_for_menubar(menu);
+    top_box->pack_start(*w, false, true);
+    w->show();
+  }
+
+  if (toolbar)
+  {
+    Gtk::Widget *w = mforms::gtk::widget_for_toolbar(toolbar);
+    top_box->pack_start(*w, false, true);
+    w->show();
+  }
+  top_box->pack_start(*panel, true, true);
+  panel->show();
+
+  top_box->show();
+
+
+  return top_box;
+}
+
+
+static gpointer delete_appview(mforms::AppView *appview)
+{
+  delete appview;
+  return 0;
+}
+
 void MainForm::dock_view(mforms::App *app, mforms::AppView *view, const std::string &position)
 {
   MainForm *self= reinterpret_cast<MainForm*>(app->get_data_ptr());
@@ -1753,14 +1754,52 @@ void MainForm::dock_view(mforms::App *app, mforms::AppView *view, const std::str
 
   if (position == "maintab")
   {
-    Gtk::Label *label= 0;
+    Gtk::Notebook *note = self->get_upper_note();
+    // Make a lookup of existing page with the same content
+    bool already_have_page = false;
+    for (int i = note->get_n_pages() - 1; i >= 0; --i)
+    {
+      Gtk::Widget *page;
+      if ((page = note->get_nth_page(i)) && page->get_data("uiform")  == dynamic_cast<bec::UIForm*>(view))
+      {
+        already_have_page = true;
+        break;
+      }
+    }
+    if (already_have_page)
+      return;
+    
+    if (!view->get_menubar())
+    {
+      view->set_menubar(self->_wbui_context->get_command_ui()->create_menubar_for_context(""));
+      view->get_menubar()->set_managed();
+      view->get_menubar()->release();
+    }
 
-    self->append_tab_page(self->get_upper_note(), w, "", true,
-                          sigc::mem_fun(view, &mforms::AppView::on_close),
-                          &label);
-    w->set_data("uiform", dynamic_cast<bec::UIForm*>(view));
-    w->set_data("tablabel", label);
-    w->set_data("containerNote", self->get_upper_note());
+    Gtk::Widget *decorated = self->decorate_widget(w, view);
+
+    decorated->set_data("AppView", view);
+    decorated->add_destroy_notify_callback(view, (gpointer(*)(gpointer))delete_appview);
+
+    if (dynamic_cast<bec::UIForm*>(view)->get_form_context_name() == "home")
+    {
+      Gtk::Widget *tab = Gtk::manage(new Gtk::Image(bec::IconManager::get_instance()->get_icon_path("WB_Home.png")));
+      Gtk::Alignment *align = Gtk::manage(new Gtk::Alignment());
+      align->add(*tab);
+      align->show_all();
+      align->set_padding(0, 0, 10, 10);
+      note->append_page(*decorated, *align);
+    }
+    else
+    {
+      Gtk::Label *label= 0;
+      self->append_tab_page(self->get_upper_note(), decorated, "", true,
+                            sigc::mem_fun(view, &mforms::AppView::on_close),
+                            &label);    
+      w->set_data("tablabel", label);
+    }
+    decorated->set_data("uiform", dynamic_cast<bec::UIForm*>(view));
+    w->set_data("containerNote", note);
   }
   else if (position == "editortab")
   {
