@@ -75,7 +75,6 @@ public:
   void tool_changed_becb(mdc::CanvasView* view);
   void refresh_gui_becb(wb::RefreshType type, const std::string& arg_id, NativeHandle arg_ptr);
   void lock_gui_becb(bool lock);
-  void popup_menu_becb(bec::MenuItemList* items, int x, int y);
   void create_main_form_view_becb(const std::string &name, boost::shared_ptr<bec::UIForm> form);
   void destroy_main_form_view_becb(bec::UIForm* form);
 
@@ -107,9 +106,7 @@ private:
                        bool auto_switch, 
                        const sigc::slot<bool> &on_close= sigc::slot<bool>(),
                        Gtk::Label **title_label_ret= 0);
-  
-  void refresh_toolbar(const std::string &toolbar);
-    
+
   void show_about();
   void show_find();
   void show_output();
@@ -169,6 +166,8 @@ private:
 
   bool find_callback(const std::string &search, const std::string &replace, 
                      mforms::SearchFlags flags, SqlEditorFE *editor);
+  
+  Gtk::Widget *decorate_widget(Gtk::Widget *panel, bec::UIForm *form);
 private:
   std::map<std::string, FormViewFactory> _form_view_factories;
 
