@@ -1,11 +1,12 @@
 #include <gtk/gtkhpaned.h>
- 
+
 #include "model_diagram_panel.h"
 #include "image_cache.h"
 #include "grtdb/db_object_helpers.h"
 #include "workbench/wb_context_ui.h"
-#include "linux_utilities/toolbar_manager.h"
 #include "linux_utilities/MGGladeXML.h"
+#include "mforms/../gtk/lf_toolbar.h"
+#include "mforms/../gtk/lf_menubar.h"
 
 #include "user_types_box.h"
 #include "catalog_tree.h"
@@ -14,7 +15,7 @@
 #include "navigator_box.h"
 #include "history_tree.h"
 #include "documentation_box.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 
 #include "gtk_helpers.h"
@@ -23,7 +24,8 @@
 #include <gdkmm/cursor.h>
 #include <gdk/gdkkeysyms.h>
 
-
+#include "base/log.h"
+ENABLE_LOG("fe")
 
 class CanvasViewer : public mdc::GtkCanvas
 {
@@ -212,14 +214,14 @@ void ModelDiagramPanel::InlineEditor::set_multiline(bool flag)
 
 //--------------------------------------------------------------------------------
 
-ModelDiagramPanel *ModelDiagramPanel::create(wb::WBContextUI *wb, ToolbarManager *toolbar_manager)
+ModelDiagramPanel *ModelDiagramPanel::create(wb::WBContextUI *wb)
 {
   MGGladeXML *xml= new MGGladeXML(wb->get_wb()->get_grt_manager()->get_data_file_path("diagram_view.glade").c_str(), "", "");
   GtkHPaned *paned;
 
   paned = GTK_HPANED(xml->get_unwrapped("diagram_pane"));
 
-  ModelDiagramPanel *panel = new ModelDiagramPanel(xml, paned, wb, toolbar_manager);
+  ModelDiagramPanel *panel = new ModelDiagramPanel(xml, paned, wb);
 
   delete xml;
   
@@ -234,16 +236,29 @@ static void fix_size(Gtk::Paned *paned)
 }
 
 
-ModelDiagramPanel::ModelDiagramPanel(MGGladeXML *xml, GtkHPaned *paned, wb::WBContextUI *wb, ToolbarManager *toolbar_manager)
-  : Gtk::HPaned(paned), _wb(wb), _be(0), _canvas(0), _cursor(0), _inline_editor(this), _toolbar_manager(toolbar_manager)
+ModelDiagramPanel::ModelDiagramPanel(MGGladeXML *xml, GtkHPaned *paned, wb::WBContextUI *wb)
+  : Gtk::HPaned(paned)
+  , _top_box(false, 0)
+  , _diagram_hbox(xml->get("diagram_hbox", &_diagram_hbox))
+  , _wb(wb)
+  , _be(0)
+  , _canvas(0)
+  , _cursor(0)
+  , _inline_editor(this)
 {
+  //_top_box = new Gtk::VBox(false, 0);
+  _top_box.show();
+
+  _top_box.pack_end(*this, true, true);
+  show();
+
   if (_wb->get_wb()->get_wb_options().get_int("Sidebar:RightAligned", 0))
   {
     swap_panned_children(this, true);
     signal_realize().connect(sigc::bind(sigc::ptr_fun(fix_size), this));
   }
 
-  xml->get("tools_toolbar", &_tools_toolbar);
+  //xml->get("tools_toolbar", &_tools_toolbar);
   xml->get("diagram_box", &_vbox);
   
   xml->get("editor_note", &_editor_note);
@@ -293,8 +308,6 @@ ModelDiagramPanel::ModelDiagramPanel(MGGladeXML *xml, GtkHPaned *paned, wb::WBCo
   label = Gtk::manage(new Gtk::Label(_("<small>History</small>")));
   note->append_page(*_history_tree, *label);
   label->set_use_markup(true);
-
-  _options_toolbar.set_name("options_toolbar");
 }
 
 
@@ -312,15 +325,55 @@ void ModelDiagramPanel::init(const std::string &view_id)
 {
   _be= _wb->get_wb()->get_model_context()->get_diagram_form_for_diagram_id(view_id);
   _be->set_frontend_data(dynamic_cast<FormViewBase*>(this));
+  _canvas= Gtk::manage(new CanvasViewer(_be, false));//_wb->get_wb()->using_opengl()));
+
+  {
+    mforms::MenuBar *menubar = _be->get_menubar();
+    if (menubar)
+    {
+      Gtk::Widget *w = mforms::gtk::widget_for_menubar(menubar);
+      _top_box.pack_start(*w, false, true);
+      w->show();
+    }
+    mforms::ToolBar *toolbar = _be->get_toolbar();
+    if (toolbar)
+    {
+      Gtk::Widget *w = mforms::gtk::widget_for_toolbar(toolbar);
+      _top_box.pack_start(*w, false, true);
+      w->show();
+      _top_box.pack_start(*(w=Gtk::manage(new Gtk::HSeparator)), false, true);
+      w->show();
+    }
+
+    mforms::ToolBar *tools_toolbar = _be->get_tools_toolbar();
+    log_debug3("ModelDiagramPanel::init got tools toolbar %p", tools_toolbar);
+    if (tools_toolbar)
+    {
+      _tools_toolbar = dynamic_cast<Gtk::VBox*>(mforms::gtk::widget_for_toolbar(tools_toolbar));
+      log_debug3("ModelDiagramPanel::init cast _tools_toolbar = %p", _tools_toolbar);
+      if (_tools_toolbar)
+      {
+        _diagram_hbox->pack_start(*_tools_toolbar, false, true);
+        _diagram_hbox->reorder_child(*_tools_toolbar, 0);
+        _tools_toolbar->show();
+      }
+    }
+
+    mforms::ToolBar *options_toolbar = _be->get_options_toolbar();
+    if (toolbar)
+    {
+      Gtk::Widget *w = mforms::gtk::widget_for_toolbar(options_toolbar);
+      _vbox->pack_start(*w, false, true);
+      w->show();
+    }
+  }
 
   _be->set_inline_editor_context(&_inline_editor);
-    
-  _vbox->pack_start(_options_toolbar, false, false);
+
   _vbox->pack_start(_scroller, true, true);
 
-  _canvas= Gtk::manage(new CanvasViewer(_be, false));//_wb->get_wb()->using_opengl()));
-  _scroller.add(*_canvas);  
-  
+  _scroller.add(*_canvas);
+
   std::vector<Gtk::TargetEntry> targets;
   std::vector<std::string> types= _be->get_accepted_drop_types();
   for (std::vector<std::string>::const_iterator iter= types.begin(); iter != types.end(); ++iter)
@@ -341,7 +394,6 @@ void ModelDiagramPanel::init(const std::string &view_id)
   signal_show().connect(sigc::bind(sigc::mem_fun(_be, &wb::ModelDiagramForm::set_closed), false));
   signal_hide().connect(sigc::bind(sigc::mem_fun(_be, &wb::ModelDiagramForm::set_closed), true));
 
-  _options_toolbar.hide();
   
   _catalog_tree->refresh();
   _layer_tree->refresh();
@@ -482,6 +534,8 @@ void ModelDiagramPanel::update_tool_cursor()
     std::string cursor= _be->get_cursor();
     std::string path= bec::IconManager::get_instance()->get_icon_path(cursor+".cur");
 
+    _be->get_options_toolbar(); // refresh options toolbar
+
     if (path.empty())
     {
       _canvas->get_window()->set_cursor();
@@ -506,30 +560,11 @@ void ModelDiagramPanel::set_title(const std::string &title)
 
 bool ModelDiagramPanel::will_close()
 {
-  hide();
-  
+  get_parent()->hide(); // hide the container
+
   return false; // don't close
 }
 
-
-void ModelDiagramPanel::update_toolbar(const std::string &toolbar)
-{
-  if (toolbar == WB_TOOLBAR_TOOLS || toolbar.empty())
-  {
-    _toolbar_manager->rebuild_toolbar(_tools_toolbar, WB_TOOLBAR_TOOLS);
-  }
-  
-  if (toolbar == WB_TOOLBAR_OPTIONS || toolbar.empty())
-  {
-    _toolbar_manager->rebuild_toolbar(&_options_toolbar, WB_TOOLBAR_OPTIONS);
-    if (_options_toolbar.children().empty())
-      _options_toolbar.hide();
-    else
-      _options_toolbar.show();
-  }
-}
-
-
 void ModelDiagramPanel::refresh_catalog()
 {
   _catalog_tree->refresh();
@@ -561,4 +596,3 @@ void ModelDiagramPanel::find_text(const std::string &text)
   _be->search_and_focus_object(text);
 }
 
-  
