@@ -5,6 +5,7 @@
 #include <map>
 #include <gtkmm/paned.h>
 #include <gtkmm/notebook.h>
+#include <gtkmm/box.h>
 #include "linux_utilities/form_view_base.h"
 
 #include "workbench/wb_overview.h"
@@ -24,7 +25,7 @@ public:
   
   virtual bool will_close();
   virtual std::string get_title();
-  virtual Gtk::Widget *get_panel() { return this; }
+  virtual Gtk::Widget *get_panel() { return _top_box; }
   virtual bec::UIForm *get_form() const;
   virtual void toggle_sidebar();
 
@@ -38,6 +39,7 @@ public:
 private:
   wb::WBContextUI *_wb;
   OverviewPanel *_overview;
+  Gtk::VBox *_top_box;
   Gtk::Paned *_editor_paned;
   Gtk::Widget *_sidebar;
   bec::NodeId                  _last_found_node;
