@@ -12,7 +12,7 @@
 #include <glib/gstdio.h>
 #include <gtkmm.h>
 #include "gtk_helpers.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using base::strfmt;
 
@@ -980,6 +980,9 @@ void OverviewPanel::reset()
   _container->foreach(sigc::mem_fun(_container, &Gtk::Container::remove));
   
   _group_containers_by_id.clear();
+  for (std::map<std::string, OverviewItemContainer*>::iterator iter = _item_containers_by_id.begin();
+        iter != _item_containers_by_id.end(); ++iter)
+    iter->second->unset_models();
   _item_containers_by_id.clear();
   
   Gtk::DrawingArea *filler= Gtk::manage(new Gtk::DrawingArea());
