@@ -22,7 +22,7 @@
 
 #include "sqlide_main.h"
 
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using base::strfmt;
 
@@ -45,6 +45,7 @@ Program::Program(wb::WBOptions wboptions)
 
   _grt_manager = _wb_context->get_grt_manager();
 
+#ifdef ENBLE_DEBUG
   if ( !getenv("MWB_DATA_DIR") )
   {
     const char *path = "../share/mysql-workbench";
@@ -72,7 +73,7 @@ Program::Program(wb::WBOptions wboptions)
     g_message("MWB_PLUGIN_DIR is unset! Setting MWB_PLUGIN_DIR to predifined value '%s'", path);
     setenv("MWB_PLUGIN_DIR", path, 1);
   }
-
+#endif
   if ( !getenv("MWB_DATA_DIR") || (!getenv("MWB_MODULE_DIR")))
   {
     g_print("Please start Workbench through mysql-workbench instead of calling mysql-workbench-bin directly\n");
@@ -81,8 +82,6 @@ Program::Program(wb::WBOptions wboptions)
 
   _grt_manager->set_datadir(getenv("MWB_DATA_DIR"));
 
-  Gtk::RC::add_default_file(_grt_manager->get_data_file_path("/workbench.rc"));
-
   // Main form holds UI code, Glade wrapper, etc ...
   _main_form = new MainForm(_wb_context_ui);
 
@@ -106,7 +105,6 @@ Program::Program(wb::WBOptions wboptions)
   wbcallbacks.tool_changed= sigc::mem_fun(_main_form, &MainForm::tool_changed_becb);
   wbcallbacks.refresh_gui= sigc::mem_fun(_main_form, &MainForm::refresh_gui_becb);
   wbcallbacks.lock_gui= sigc::mem_fun(_main_form, &MainForm::lock_gui_becb);
-  wbcallbacks.popup_menu= sigc::mem_fun(_main_form, &MainForm::popup_menu_becb);
   wbcallbacks.quit_application= sigc::ptr_fun(&Gtk::Main::quit);
 
   wboptions.basedir = getenv("MWB_DATA_DIR");
@@ -114,7 +112,7 @@ Program::Program(wb::WBOptions wboptions)
   wboptions.struct_search_path = wboptions.basedir + "/grt";
   wboptions.module_search_path = getenv("MWB_MODULE_DIR");
   wboptions.library_search_path = getenv("MWB_LIBRARY_DIR");
-  wboptions.cdbc_driver_search_path = getenv("DBC_DRIVER_PATH");
+  wboptions.cdbc_driver_search_path = getenv("DBC_DRIVER_PATH")?:"";
   if (wboptions.cdbc_driver_search_path.empty())
     wboptions.cdbc_driver_search_path= wboptions.library_search_path;
   wboptions.user_data_dir = std::string(g_get_home_dir()).append("/.mysql/workbench");
