@@ -1,28 +1,25 @@
 #include "sqlide_form.h"
 #include "sqlide/utils_fe.h"
 #include "linux_utilities/gtk_helpers.h"
+#include "sql_snippets_view.h"
 #include "workbench/menu_manager.h"
 #include "mforms/utilities.h"
 #include <boost/foreach.hpp>
 #include <fstream>
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include "linux_utilities/toolbar_manager.h"
+#include "mforms/../gtk/lf_view.h"
+#include "mforms/../gtk/lf_menubar.h"
+#include "mforms/../gtk/lf_toolbar.h"
 
 using base::strfmt;
 
-
 void DbSqlEditorView::refresh_ui(const int what)
 {
   switch (what)
   {
   case Db_sql_editor::RefreshSidebar:
-    _sidebar.refresh(true);
-    break;
-  case Db_sql_editor::RefreshActiveSchemaSelector:
-    _sidebar.refresh_active_schema_selector(false);
-    break;
-  case Db_sql_editor::RefreshSchemaTree:
-    _sidebar.refresh_schema_tree();
+    //_sidebar.refresh(true);
     break;
   case Db_sql_editor::RefreshEditor:
     active_editor()->set_text(_be->sql_editor()->sql());
@@ -37,7 +34,7 @@ void DbSqlEditorView::refresh_ui(const int what)
     recordset_caption_changed();
     break;
   case Db_sql_editor::RefreshSnippets:
-    _sidebar.sql_snippets_view()->refresh();
+    _sql_snippets_view->refresh();
     break;
   case Db_sql_editor::RunCurrentScript:
     execute_sql_script(false);
@@ -46,30 +43,55 @@ void DbSqlEditorView::refresh_ui(const int what)
     execute_sql_script(true);
     break;
   case Db_sql_editor::RefreshOverview:
-    _live_physical_overview->refresh_active_group_node_children();
+    if (_live_physical_overview)
+      _live_physical_overview->refresh_active_group_node_children();
     break;
   }
 }
 
-DbSqlEditorView::DbSqlEditorView(Db_sql_editor::Ref editor_be)
+
+DbSqlEditorView *DbSqlEditorView::create(Db_sql_editor::Ref editor_be)
+{
+  MGGladeXML *xml = new MGGladeXML(editor_be->grt_manager()->get_data_file_path("db_sql_editor_view.glade"));
+  GtkVBox *vbox = GTK_VBOX(xml->get_unwrapped("db_sql_editor_view"));
+  g_object_ref_sink(vbox);
+  gtk_widget_unparent(GTK_WIDGET(vbox));
+  
+  DbSqlEditorView *v = Gtk::manage(new DbSqlEditorView(editor_be, vbox, xml));
+  
+  return v;
+}
+
+
+DbSqlEditorView::DbSqlEditorView(Db_sql_editor::Ref editor_be, GtkVBox *vbox, MGGladeXML *xml)
 :
-Glib::ObjectBase(typeid(DbSqlEditorView)),
+Gtk::VBox(vbox),
 FormViewBase(),
-_xml(editor_be->grt_manager()->get_data_file_path("db_sql_editor_view.glade")),
-_sidebar(editor_be, &_xml)
+_xml(xml),
+_live_physical_overview(0),
+_closing(false)
 {
   _be= editor_be;
 
   if (editor_be->wbsql()->get_wbui()->get_wb()->get_wb_options().get_int("Sidebar:RightAligned", 0))
   {
     Gtk::Paned *paned;
-    _xml.get("hpaned1", &paned);
+    _xml->get("hpaned1", &paned);
     swap_panned_children(paned, true);
   }
 
-  _be->exec_sql_task->finish_cb(sigc::mem_fun(this, &DbSqlEditorView::after_exec_sql_finished));
+  {
+    Gtk::Container *placeholder;
+    _xml->get("sql_snippets_placeholder", &placeholder);
+
+    wb::WBContextUI *wbui= editor_be->wbsql()->get_wbui();
+    _sql_snippets_view= Gtk::manage(new SqlSnippetsView(wbui, _xml));
+    placeholder->add(*_sql_snippets_view);
+    _sql_snippets_view->show_all();
+  }
+
   _be->exec_sql_task->progress_cb(sigc::mem_fun(this, &DbSqlEditorView::on_exec_sql_progress));
-  _be->close_recordset_ui.connect(sigc::bind(sigc::mem_fun(this, &DbSqlEditorView::close_recordset), false));
+  _be->recordset_list_changed.connect(sigc::mem_fun(this, &DbSqlEditorView::recordset_list_changed));
   _be->output_text_slot= sigc::mem_fun(this, &DbSqlEditorView::output_text);
   
   _be->set_partial_refresh_ui_slot(sigc::mem_fun(this, &DbSqlEditorView::refresh_ui));
@@ -77,12 +99,48 @@ _sidebar(editor_be, &_xml)
   _be->sql_editor_text_insert_signal.connect(sigc::mem_fun(this, &DbSqlEditorView::on_sql_editor_text_insert));
 
   _be->set_frontend_data(dynamic_cast<FormViewBase*>(this));
+
+  mforms::View *sbview = _be->get_sidebar();
+  Gtk::Widget *sidebar = mforms::gtk::ViewImpl::get_widget_for_view(sbview);
+  Gtk::Container *place;
+  _xml->get("sidebar_placeholder", &place);
+  place->add(*sidebar);
+  sidebar->show();
+  
+  _dispatch_rset_update.connect(sigc::mem_fun(this, &DbSqlEditorView::update_resultsets));
+  
+  // setup toolbar and menubar
+  {
+    Gtk::VBox *vbox;
+    _xml->get("db_sql_editor_view", &vbox);
+    
+    Gtk::Widget *w;
+    mforms::ToolBar *tbar = _be->get_toolbar();
+    w = mforms::gtk::widget_for_toolbar(tbar);
+    if (w)
+    {
+      vbox->pack_start(*w, false, true);
+      vbox->reorder_child(*w, 0);
+    }
+    
+    mforms::MenuBar *menu = _be->get_menubar();
+    w = mforms::gtk::widget_for_menubar(menu);
+    if (w)
+    {
+      vbox->pack_start(*w, false, true);
+      vbox->reorder_child(*w, 0);
+    }
+  }
 }
 
 
 DbSqlEditorView::~DbSqlEditorView()
 {
+  if (_live_physical_overview)
+  _live_physical_overview->reset();
+  delete _live_physical_overview;
   _be->close();
+  delete _xml;
 }
 
 //------------------------------------------------------------------------------
@@ -95,11 +153,8 @@ bool DbSqlEditorView::toolbar_action(const std::string& item)
 //------------------------------------------------------------------------------
 void DbSqlEditorView::init()
 {
-  _xml.get("editor_note", &_editor_note);
+  _xml->get("editor_note", &_editor_note);
   _editor_note->signal_switch_page().connect(sigc::mem_fun(this, &DbSqlEditorView::editor_page_switched));
-
-  _xml.get("db_sql_editor_view", &_view);
-  _view->reparent(*this);
     
   _be->sql_editor_new_ui.connect(sigc::bind_return(sigc::hide(sigc::mem_fun(this, &DbSqlEditorView::add_editor_tab)), 0));
 
@@ -107,19 +162,19 @@ void DbSqlEditorView::init()
         sigc::mem_fun(this, &DbSqlEditorView::explain_sql),
         sigc::mem_fun(this, &DbSqlEditorView::validate_explain_sql));
 
-  _xml.get("output_text", &_output_text);
+  _xml->get("output_text", &_output_text);
   {
     Pango::FontDescription font(_output_text->get_style()->get_font());
     font.set_family("Bitstream Vera Sans Mono");
     _output_text->modify_font(font);
   }
   Gtk::Button *clear_button;
-  _xml.get("output_clear_button", &clear_button);
+  _xml->get("output_clear_button", &clear_button);
   clear_button->signal_clicked().connect(sigc::mem_fun(this, &DbSqlEditorView::clear_output));
 
   // tab control
-  _xml.get("db_sql_editor_tab_pages", &_tab_pages);
-  _xml.get("log_page", &_log_page);
+  _xml->get("db_sql_editor_tab_pages", &_tab_pages);
+  _xml->get("log_page", &_log_page);
 
   // this is the lower tabview
   _tab_pages->signal_switch_page().connect(sigc::mem_fun(this, &DbSqlEditorView::lower_page_switched));
@@ -131,14 +186,14 @@ void DbSqlEditorView::init()
     Glib::RefPtr<Gtk::AccelGroup> _accel_group= Gtk::AccelGroup::create();//!
     _live_physical_overview= new OverviewPanel(_be->wbsql()->get_wbui(), _be->live_physical_overview(), true);
 
-    Gtk::HBox *live_physical_overview_toolbar = _xml.get("live_physical_overview_toolbar", &live_physical_overview_toolbar);
+    Gtk::HBox *live_physical_overview_toolbar = _xml->get("live_physical_overview_toolbar", &live_physical_overview_toolbar);
     ToolbarManager::rebuild_toolbar(live_physical_overview_toolbar,
                                     _be->live_physical_overview()->get_toolbar_items(bec::NodeId(0)),
                                     sigc::slot<Gtk::Widget*,bec::ToolbarItem>(),
                                     sigc::mem_fun(this, &DbSqlEditorView::toolbar_action)
                                    );
 
-    Gtk::Viewport *live_physical_overview_placeholder = _xml.get("live_physical_overview_placeholder", &live_physical_overview_placeholder);
+    Gtk::Viewport *live_physical_overview_placeholder = _xml->get("live_physical_overview_placeholder", &live_physical_overview_placeholder);
     live_physical_overview_placeholder->add(*_live_physical_overview);
     _live_physical_overview->set_data("FormViewBase", _live_physical_overview);
     _live_physical_overview->set_data("uiform", dynamic_cast<bec::UIForm*>(get_form())); //!_live_physical_overview->get_form()
@@ -149,7 +204,7 @@ void DbSqlEditorView::init()
     if (!_be->is_physical_overview_enabled())
     {
       Gtk::Container *live_physical_overview_page;
-      _xml.get("live_physical_overview_page", &live_physical_overview_page);
+      _xml->get("live_physical_overview_page", &live_physical_overview_page);
       int tab_page_index= _tab_pages->page_num(*live_physical_overview_page);
       _tab_pages->remove_page(tab_page_index);
     }
@@ -157,7 +212,7 @@ void DbSqlEditorView::init()
 
   // log panel
   //WIDGET_DEF (_xml, Container, log_placeholder) // auto-replaced
-  Gtk::Container *log_placeholder = _xml.get("log_placeholder", &log_placeholder);
+  Gtk::Container *log_placeholder = _xml->get("log_placeholder", &log_placeholder);
   _log_view= GridView::create(_be->log());
   log_placeholder->add(*_log_view);
   _log_view->row_numbers_visible(false);
@@ -165,33 +220,33 @@ void DbSqlEditorView::init()
     sigc::bind<bool>(sigc::mem_fun(_log_view, &GridView::refresh), false);
   _log_view->refresh(true);
 
-  //Gtk::HBox *output_toolbar = _xml.get("output_toolbar", &output_toolbar);
+  //Gtk::HBox *output_toolbar = _xml->get("output_toolbar", &output_toolbar);
   //ToolbarManager::rebuild_toolbar(output_toolbar,
   //                                _be->log()->get_toolbar_items(bec::NodeId(0)),
   //                                sigc::mem_fun(this, &DbSqlEditorView::output_toolbar_action)
   //                               );
 
 
-  _xml.get("log_event_detail_action_text_view", &_log_event_detail_action_text_view);
-  _xml.get("log_event_detail_message_text_view", &_log_event_detail_message_text_view);
+  _xml->get("log_event_detail_action_text_view", &_log_event_detail_action_text_view);
+  _xml->get("log_event_detail_message_text_view", &_log_event_detail_message_text_view);
 
-  _xml.get("output_note", &_output_note);
+  _xml->get("output_note", &_output_note);
   
   Gtk::RadioButton *radio;
-  _xml.get("output_radio1", &radio);
+  _xml->get("output_radio1", &radio);
   radio->signal_toggled().connect(sigc::bind(sigc::mem_fun(_output_note, &Gtk::Notebook::set_current_page), 0));
-  _xml.get("output_radio2", &radio);
+  _xml->get("output_radio2", &radio);
   _output_text_radio= radio;
   radio->signal_toggled().connect(sigc::bind(sigc::mem_fun(_output_note, &Gtk::Notebook::set_current_page), 1));
-  _xml.get("output_radio3", &radio);
+  _xml->get("output_radio3", &radio);
   radio->signal_toggled().connect(sigc::bind(sigc::mem_fun(_output_note, &Gtk::Notebook::set_current_page), 2));
   
   _log_view->get_selection()->signal_changed().connect(sigc::mem_fun(this, &DbSqlEditorView::on_log_view_selection_changed));
   on_log_view_selection_changed();
 
   // history panels
-  Gtk::Container *history_entries_placeholder = _xml.get("history_entries_placeholder", &history_entries_placeholder);
-  Gtk::Container *history_details_placeholder = _xml.get("history_details_placeholder", &history_details_placeholder);
+  Gtk::Container *history_entries_placeholder = _xml->get("history_entries_placeholder", &history_entries_placeholder);
+  Gtk::Container *history_details_placeholder = _xml->get("history_details_placeholder", &history_details_placeholder);
   _history_entries_view= GridView::create(_be->history()->entries_model());
   _history_details_view= GridView::create(_be->history()->details_model());
   history_entries_placeholder->add(*_history_entries_view);
@@ -203,7 +258,7 @@ void DbSqlEditorView::init()
   _history_entries_view->refresh(true);
   _history_details_view->refresh(true);
 
-  //Gtk::HBox *history_toolbar = _xml.get("history_toolbar", &history_toolbar);
+  //Gtk::HBox *history_toolbar = _xml->get("history_toolbar", &history_toolbar);
   //ToolbarManager::rebuild_toolbar(history_toolbar,
   //                                _be->history()->get_toolbar_items(bec::NodeId(0)),
   //                                sigc::mem_fun(this, &DbSqlEditorView::output_toolbar_action)
@@ -215,40 +270,85 @@ void DbSqlEditorView::init()
   _on_history_entries_selection_changed_cb= sigc::mem_fun(this, &DbSqlEditorView::on_history_entries_selection_changed);
   _history_entries_view->get_selection()->signal_changed().connect(_on_history_entries_selection_changed_cb);
   _history_details_view->signal_event().connect(sigc::mem_fun(this, &DbSqlEditorView::on_history_details_view_event));
-  _xml.get("history_details_popup_menu", &_history_details_popup_menu);
+  _xml->get("history_details_popup_menu", &_history_details_popup_menu);
   {
-    Gtk::MenuItem *w = _xml.get("append_to_sql_script_menuitem", &w);
+    Gtk::MenuItem *w = _xml->get("append_to_sql_script_menuitem", &w);
     w->signal_activate().connect(sigc::mem_fun(this, &DbSqlEditorView::on_append_to_sql_script_menuitem_activate));
   }
   {
-    Gtk::MenuItem *w = _xml.get("replace_sql_script_menuitem", &w);
+    Gtk::MenuItem *w = _xml->get("replace_sql_script_menuitem", &w);
     w->signal_activate().connect(sigc::mem_fun(this, &DbSqlEditorView::on_replace_sql_script_menuitem_activate));
   }
   
-  
-  add_editor_tab();
+  // realize pre-existing editors
+  for (int i = 0; i < _be->sql_editor_count(); i++)
+  {
+    _be->active_sql_editor_index(i);
+    add_editor_tab();
+  }
 }
 
 bool DbSqlEditorView::will_close()
 {
-  return _be->can_close();
+  if (_be->can_close())
+  {
+    _closing = true;
+    return true;
+  }
+  return false;
 }
 
 void DbSqlEditorView::editor_page_switched(GtkNotebookPage *page, guint index)
 {
   _be->active_sql_editor_index(index);
   
-  // not sure the following line is needed --alfredo
-  _be->request_refresh_of_menu_and_toolbar();
+  long long rskey = _be->active_recordset_for_sql_editor(index);
+  if (rskey >= 0)
+  {
+    // activate the recordset for this editor
+    for (int i= 0, rcount = _tab_pages->get_n_pages(); i < rcount; i++)
+    {
+      RecordsetView *rsview = dynamic_cast<RecordsetView*>(_tab_pages->get_nth_page(i));
+      if (rsview)
+      {
+        if (rsview->model()->key() == rskey)
+        {
+          _tab_pages->set_current_page(i);
+          break;
+        }
+      }
+    }
+  }
+
+ // not sure the following line is needed --alfredo
+ //!!! _be->request_refresh_of_menu_and_toolbar();
 }
 
 void DbSqlEditorView::lower_page_switched(GtkNotebookPage *page, guint index)
 {
+  if (_closing) return;
   RecordsetView *rs = active_recordset();
   if (rs)
+  {
     _be->active_recordset(rs->model());
+
+    int editor = _be->sql_editor_index_for_recordset(rs->model()->key());
+    if (editor >= 0)
+    {
+      for (std::vector<boost::shared_ptr<SqlEditorFE> >::iterator iter = _sql_editors.begin();
+        iter != _sql_editors.end(); ++iter)
+      {
+        if ((*iter)->be() == _be->sql_editor(editor))
+        {
+          _editor_note->set_current_page(_editor_note->page_num((*iter)->widget()));
+          break;
+        }
+      }
+    }
+  }
   else
     _be->active_recordset(Recordset::Ref());
+
 }
 
 void DbSqlEditorView::close_editor_tab(boost::shared_ptr<SqlEditorFE> editor)
@@ -287,6 +387,8 @@ void DbSqlEditorView::add_editor_tab()
 
   editor->set_font(_be->wbsql()->get_wbui()->get_wb()->get_wb_options().get_string("workbench.general.Editor:Font"));
   
+  editor->set_text(editor->be()->sql());
+
   Gtk::Label *label;
   _editor_note->append_page(editor->widget(), *create_closeable_tab(_be->sql_editor_caption(_be->active_sql_editor_index()),
                                                                     sigc::bind(sigc::mem_fun(this, &DbSqlEditorView::close_editor_tab),
@@ -304,6 +406,8 @@ void DbSqlEditorView::add_editor_tab()
   editor->signal_selection_changed().connect(sigc::mem_fun(this, &DbSqlEditorView::on_sql_editor_selection_change));
 
   _sql_editors.push_back(editor);
+
+  update_resultsets();
 }
 
 
@@ -345,6 +449,7 @@ void DbSqlEditorView::execute_sql_script(bool current_statement_only)
   if (sql.empty())
     return;
 
+ /*
   // close unpinned resultsets
   if (!_be->recordsets_are_pinned_by_default())
   {
@@ -363,40 +468,77 @@ void DbSqlEditorView::execute_sql_script(bool current_statement_only)
         recordsets_to_close.push_back(i.first->key());
     BOOST_FOREACH (long long &key, recordsets_to_close)
       close_recordset(key);
-  }
+  }*/
 
   _tab_pages->set_current_page(_tab_pages->page_num(*_log_page));
 
-  _be->exec_sql(sql, false, current_statement_only);
+  _be->exec_sql(sql, active_editor()->be(), false, current_statement_only);
 }
 
-int DbSqlEditorView::after_exec_sql_finished()
+
+void DbSqlEditorView::update_resultsets()
 {
-  int new_tab_pages_count= 0;
+  int non_rs_tabs = 0;
+  std::list<Gtk::Widget*> leftover_tabs;
+  for (int i = 0; i < _tab_pages->get_n_pages(); i++)
+  {
+    RecordsetView *rsview = dynamic_cast<RecordsetView*>(_tab_pages->get_nth_page(i));
+    if (rsview && _view2recordset.find(rsview) != _view2recordset.end())
+      leftover_tabs.push_back(rsview);
+    else
+      non_rs_tabs++;
+  }
+
   for (int n= 0, rs_count= _be->recordset_count(); n < rs_count; ++n)
   {
     Recordset::Ref rs= _be->recordset(n);
-    if (_recordset2view.end() != _recordset2view.find(rs))
-      continue;
 
-    ++new_tab_pages_count;
-    int tab_page_index= _tab_pages->get_n_pages();
-    Gtk::Label *label;
-    Gtk::Widget *tab = create_closeable_tab(rs->caption(),
+    if (_recordset2view.end() == _recordset2view.find(rs))
+    {
+      Gtk::Label *label;
+      Gtk::Widget *tab = create_closeable_tab(rs->caption(),
                                             sigc::hide_return(sigc::bind(sigc::mem_fun(this, &DbSqlEditorView::close_recordset), rs->key(), true)),
                                             &label);
-    RecordsetView *rs_view= RecordsetView::create(rs, NULL);
-    _tab_pages->insert_page(*rs_view, tab_page_index);
-    _tab_pages->set_tab_label(*rs_view, *tab);
+      RecordsetView *rs_view= RecordsetView::create(rs, NULL);
+      _tab_pages->insert_page(*rs_view, non_rs_tabs + n);
+      _tab_pages->set_tab_label(*rs_view, *tab);
 
-    rs_view->set_data("tab_label", label);
+      rs_view->set_data("tab_label", label);
 
-    _recordset2view[rs]= rs_view;
-    _view2recordset[rs_view]= rs;
+      _recordset2view[rs]= rs_view;
+      _view2recordset[rs_view]= rs;
 
-    rs_view->refresh();
+      rs_view->refresh();
+
+      _tab_pages->set_current_page(non_rs_tabs+n);
+    }
+    else
+    {
+      std::list<Gtk::Widget*>::iterator iter = std::find(leftover_tabs.begin(), leftover_tabs.end(), _recordset2view[rs]);
+      if (iter != leftover_tabs.end())
+        leftover_tabs.erase(iter);
+    }
   }
-  _tab_pages->set_current_page((new_tab_pages_count) ? _tab_pages->get_n_pages()-1 : _tab_pages->page_num(*_log_page));
+
+  // remove tabs that are gone
+  for (std::list<Gtk::Widget*>::iterator iter = leftover_tabs.begin(); iter != leftover_tabs.end(); ++iter)
+  {
+    View2Recordset::iterator rsiter = _view2recordset.find(dynamic_cast<RecordsetView*>(*iter));
+    if (rsiter != _view2recordset.end())
+    {
+      _recordset2view.erase(_recordset2view.find(rsiter->second));
+      _view2recordset.erase(rsiter);
+    }
+    _tab_pages->remove_page(**iter);
+  }
+}
+
+
+int DbSqlEditorView::after_exec_sql_finished()
+{
+  update_resultsets();
+  //XXX select log tab if there are errors
+ // _tab_pages->set_current_page((new_tab_pages_count) ? _tab_pages->get_n_pages()-1 : _tab_pages->page_num(*_log_page));
 
   refresh_log_view();
   on_history_entries_refresh(); //!_history_entries_view->refresh(false);
@@ -425,6 +567,11 @@ void DbSqlEditorView::refresh_log_view()
   }
 }
 
+void DbSqlEditorView::recordset_list_changed(Recordset::Ref rset, bool added)
+{
+  _dispatch_rset_update.emit();
+}
+
 int DbSqlEditorView::close_recordset(long long key, bool confirm)
 {
   Recordset2View::iterator i;
@@ -570,7 +717,7 @@ bool DbSqlEditorView::on_sql_editor_key_press_event(GdkEventKey *event)
 
 void DbSqlEditorView::on_sql_editor_selection_change()
 {
-  _be->request_refresh_of_menu_and_toolbar();
+  _be->wbsql()->get_wbui()->get_command_ui()->signal_validate_edit_menu_items().emit();
 }
 
 int DbSqlEditorView::on_sql_editor_text_insert(const std::string &text)
@@ -613,7 +760,7 @@ void DbSqlEditorView::toggle_sidebar()
 {
   Gtk::Widget *sidebar;
   
-  _xml.get("vbox1", &sidebar);
+  _xml->get("sidebar_placeholder", &sidebar);
   
   if (sidebar->is_visible())
     sidebar->hide();
