@@ -50,7 +50,7 @@
 @implementation MFGRTTreeViewImpl
 
 
-- (id)initWithObject:(mforms::GRTTreeView*)object
+- (id)initWithObject:(mforms::GRTTreeView*)object options:(mforms::TreeOptions)options
 {
   NSRect rect= NSMakeRect(0, 0, 50, 50);
   self= [super initWithFrame:rect];
@@ -78,6 +78,21 @@
                                      borderType:NSBezelBorder];
     [self setDocumentView: mOutline];
     [self setHasVerticalScroller: YES];    
+    
+    if (options & mforms::TreeNoBorder)
+      [self setBorderType: NSNoBorder];
+    else
+      [self setBorderType: NSBezelBorder];
+    
+    if (options & mforms::TreeNoHeader)
+      [mOutline setHeaderView:nil];
+    
+    if (options & mforms::TreeSidebar)
+    {
+      mSmallFont = YES;
+      [mOutline setFocusRingType: NSFocusRingTypeNone];
+      [self setAutohidesScrollers: YES];
+    }
   }
   return self;
 }
@@ -164,6 +179,9 @@
     if ([[mOutline tableColumns] count] == 1)
       [mOutline setOutlineTableColumn: column];
     [column setResizingMask: NSTableColumnAutoresizingMask|NSTableColumnUserResizingMask];
+    
+    if (mSmallFont)
+      [[column dataCell] setFont: [NSFont systemFontOfSize: [NSFont smallSystemFontSize]]];
 
     return [[mOutline tableColumns] count] - 1;
   }
@@ -196,10 +214,16 @@
 }
 
 
-
-static bool grttreeview_create(mforms::GRTTreeView *self, bool border)
+- (void) setBackgroundColor:(NSColor *)color
 {
-  [[[[MFGRTTreeViewImpl alloc] initWithObject: self] autorelease] setBorderType: border ? NSBezelBorder : NSNoBorder];
+  [super setBackgroundColor: color];
+  [mOutline setBackgroundColor: color];
+}
+
+
+static bool grttreeview_create(mforms::GRTTreeView *self, mforms::TreeOptions options)
+{
+  [[[MFGRTTreeViewImpl alloc] initWithObject: self options:options] autorelease];
   return true;
 }
 
@@ -269,12 +293,23 @@ static int grttreeview_get_selection(mforms::GRTTreeView *self, std::vector<bec:
   return nodes.size();
 }
 
-static void grttreeview_row_count_changed(mforms::GRTTreeView *self, int old_count)
+static void grttreeview_row_count_changed(mforms::GRTTreeView *self, const bec::NodeId &parent, int old_count)
 {
   MFGRTTreeViewImpl* tree = self->get_data();
   [tree->mOutline noteNumberOfRowsChanged];
+  [tree->mOutline reloadData];
+  if (old_count < 0 && !parent.is_valid())
+    [tree->mOutline collapseItem: nil collapseChildren: YES];
 }
 
+static void grttreeview_set_expanded(mforms::GRTTreeView *self, const bec::NodeId& node, bool expanded)
+{
+  MFGRTTreeViewImpl* tree = self->get_data();
+  if (expanded)
+    [tree->mOutline expandItem: [tree->mDataSource itemForNodeId: node]];
+  else
+    [tree->mOutline collapseItem: [tree->mDataSource itemForNodeId: node]];
+}
 
 void cf_grttreeview_init()
 {
@@ -290,6 +325,7 @@ void cf_grttreeview_init()
   f->_grttreeview_impl.get_selected_node= &grttreeview_get_selected_node;
   f->_grttreeview_impl.get_selection= &grttreeview_get_selection;
   f->_grttreeview_impl.row_count_changed= &grttreeview_row_count_changed;
+  f->_grttreeview_impl.set_expanded= &grttreeview_set_expanded;
 }
 
 @end
