@@ -24,7 +24,6 @@
 
 @interface WBBasePanel : NSObject 
 {
-  
 }
 
 - (NSView*)topView;
@@ -34,6 +33,9 @@
 - (bec::UIForm*)formBE;
 - (NSSize)minimumSize;
 
+- (NSMenu*)menuBar;
+
+- (NSView*)decoratedTopView;
 
 - (BOOL)willClose;
 - (void)didOpen;
