@@ -71,8 +71,11 @@ static NSImage *descendingSortIndicator= nil;
 
 - (void)dealloc
 {
+  (*mData)->refresh_ui_cb.disconnect();
+  
   std::for_each(mSigConns.begin(), mSigConns.end(), sigc::mem_fun(&sigc::connection::disconnect));
   delete mData;
+  [mView release];
   [super dealloc];
 }
 
