@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,15 +28,13 @@
 #import "WBSplitViewUnbrokenizerDelegate.h"
 #import "WBOverviewPanel.h"
 #import "MScintillaView.h"
-#import "WBToolbarManager.h"////
 #import "WBMiniToolbar.h"
 #import "GRTListDataSource.h"
 
 #import "MSQLEditorController.h"
 
-#import "WBQuerySidebarController.h"
-
 #include "sqlide/wb_sql_editor_snippets.h"
+#include "mforms/../cocoa/MFView.h"
 
 @interface ColorBallTextCell : NSTextFieldCell
 {
@@ -66,6 +64,38 @@
 @end
 
 
+@interface ResultTabViewItem : NSTabViewItem
+{
+  MResultsetViewer *viewer;
+}
+
+- (void)setRSViewer: (MResultsetViewer*)viewer;
+- (MResultsetViewer*)RSViewer;
+
+@end
+
+
+@implementation ResultTabViewItem
+
+- (void)setRSViewer: (MResultsetViewer*)aViewer
+{
+  [viewer autorelease];
+  viewer = [aViewer retain];
+}
+
+- (MResultsetViewer*)RSViewer
+{
+  return viewer;
+}
+
+- (void) dealloc
+{
+  [viewer release];
+  [super dealloc];
+}
+
+@end
+
 
 
 
@@ -264,56 +294,70 @@ objectValueForTableColumn: (NSTableColumn*) aTableColumn
 }
 
 
-- (void)displayResultsets
+- (void)updateResultsets
 {
-  int n, rs_count;
-
-  for (n = 0, rs_count = mBackEnd->recordset_count(); n < rs_count; ++n)
+  int n, rs_count = mBackEnd->recordset_count();
+  NSMutableArray *leftoverResults = [[[mLowerTabView tabViewItems] mutableCopy] autorelease];
+  int non_rs_tabs = 0;
+  
+  for (NSTabViewItem *item in leftoverResults)
   {
-    Recordset::Ref rset(mBackEnd->recordset(n));
-    MResultsetViewer *rsview = [[[MResultsetViewer alloc] initWithRecordset: rset] autorelease];
-    [mResultsetViews addObject: rsview];
-    
-    NSTabViewItem *tabViewItem= 
-        [[[NSTabViewItem alloc] initWithIdentifier: [NSString stringWithFormat:@"rset%li", rset->key()]] autorelease];
-    
-    [tabViewItem setLabel: [NSString stringWithCPPString: rset->caption()]];
-    [tabViewItem setView: [rsview view]];
-
-    //[mLowerTabView insertTabViewItem: tabViewItem atIndex: n];
-    [mLowerTabView addTabViewItem: tabViewItem];
-    [rsview refresh];
+    if ([item isKindOfClass: [ResultTabViewItem class]])
+      break;
+    non_rs_tabs++;
   }
   
-//  if ([mResultsetViews count] > 0)
-//    [mLowerTabView selectFirstTabViewItem:nil];
-    
-  if (rs_count > 0)
-    [mLowerTabView selectLastTabViewItem:nil];
-}
-
-- (void)clearResultsets
-{
-  for (id tabView in [[mLowerTabView tabViewItems] reverseObjectEnumerator])
+  for (n = 0; n < rs_count; ++n)
   {
-    if ([[tabView identifier] compare: @"rset" options:0 range:NSMakeRange(0, 4)] == NSOrderedSame)
-      [mLowerTabView removeTabViewItem: tabView];
+    Recordset::Ref rset(mBackEnd->recordset(n));
+    
+    NSInteger index = [mLowerTabView indexOfTabViewItemWithIdentifier: [NSString stringWithFormat: @"rset%li", rset->key()]];
+    if (index == NSNotFound)
+    {
+      MResultsetViewer *rsview = [[[MResultsetViewer alloc] initWithRecordset: rset] autorelease];
+
+      ResultTabViewItem *tabViewItem= 
+        [[[ResultTabViewItem alloc] initWithIdentifier: [NSString stringWithFormat:@"rset%li", rset->key()]] autorelease];
+
+      [tabViewItem setRSViewer: rsview];
+      [tabViewItem setLabel: [NSString stringWithCPPString: rset->caption()]];
+      [tabViewItem setView: [rsview view]];
+      
+      [mLowerTabView insertTabViewItem: tabViewItem atIndex: n+non_rs_tabs];
+      [rsview refresh];
+    }
+    else
+    {
+      [leftoverResults removeObject: [mLowerTabView tabViewItemAtIndex: index]];
+    }
   }
-  [mResultsetViews removeAllObjects];
+
+  // remove tabs that are not used anymore
+  for (id tab in leftoverResults)
+  {
+    if ([tab isKindOfClass: [ResultTabViewItem class]])
+      [mLowerTabView removeTabViewItem: tab];
+  }
+  
+//  [mLowerTabView selectFirstTabViewItem:nil];
 }
 
 
 static int processTaskFinish(WBSQLQueryPanel *self)
 {
-  //XXX[self enableCancelButton:NO];
   [self->mMessagesTable reloadData];
-  [self displayResultsets];
+  [self updateResultsets];
   
   if (self->mBackEnd->exec_sql_error_count() > 0)
   {
     [self->mOutputTabView selectTabViewItemWithIdentifier: @"actions"];
     [self->mOutputSegmented selectSegmentWithTag: 0];
   }
+  else
+  {
+    // re-select the top editor to force re-selection of its resultset tab
+    [self->mUpperTabView selectTabViewItemWithIdentifier: [[self->mUpperTabView selectedTabViewItem] identifier]];
+  }
   
   return 0;
 }
@@ -321,6 +365,7 @@ static int processTaskFinish(WBSQLQueryPanel *self)
 static int processTaskProgress(float progress, const std::string &message, WBSQLQueryPanel *self)
 {
   [self->mMessagesTable reloadData];
+  [self->mMessagesTable scrollRowToVisible: [self->mMessagesTable numberOfRows]-1];
   
   return 0;
 }
@@ -330,31 +375,42 @@ static int reloadTable(NSTableView *table)
   [table reloadData];
   [[table delegate] tableViewSelectionDidChange:[NSNotification notificationWithName:NSTableViewSelectionDidChangeNotification
                                                                               object: table]];
+  [table scrollRowToVisible: [table numberOfRows]-1];
                                                  
   return 0;
 }
 
-static int closeRecordset(long key, WBSQLQueryPanel *self)
+- (void)removeRecordsetWithIdentifier:(id)identifier
 {
-  MResultsetViewer *viewer = nil;
-  for (id rsview in self->mResultsetViews)
+  NSInteger index = [mLowerTabView indexOfTabViewItemWithIdentifier: identifier];
+  if (index != NSNotFound)
   {
-    if ([rsview recordset]->key() == key)
-    {
-      viewer = rsview;
-      break;
-    }
-  }
-
-  id item = [self->mLowerTabView tabViewItemAtIndex: [self->mLowerTabView indexOfTabViewItemWithIdentifier: [NSString stringWithFormat:@"rset%li", key]]];
-  
-  [self->mLowerTabView removeTabViewItem: item];
-  [self->mResultsetViews removeObject: viewer]; 
-  
-  return 0;
+    id item = [mLowerTabView tabViewItemAtIndex: index];
+    if (item)
+      [mLowerTabView removeTabViewItem: item];
+  }  
 }
 
 
+static void recordsetListChanged(Recordset::Ref rs, bool added, WBSQLQueryPanel *self)
+{
+  if (!added)
+  {
+    if (![NSThread isMainThread])
+    {
+      NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
+      [self performSelectorOnMainThread: @selector(removeRecordsetWithIdentifier:) 
+                             withObject: [NSString stringWithFormat:@"rset%li", rs->key()]
+                          waitUntilDone: NO];
+      [pool release];
+    }
+    else
+      [self removeRecordsetWithIdentifier: [NSString stringWithFormat:@"rset%li", rs->key()]];
+  }
+  else
+    [self performSelectorOnMainThread:@selector(updateResultsets) withObject:nil waitUntilDone:NO];
+}
+
 static void addTextToOutput(const std::string &text, bool bring_to_front, WBSQLQueryPanel *self)
 {
   [self->mTextOutputLock lock];
@@ -396,13 +452,9 @@ static void addTextToOutput(const std::string &text, bool bring_to_front, WBSQLQ
   if (text.empty())
     return;
   
-  [self clearResultsets];
-  
   [mLowerTabView selectTabViewItemWithIdentifier: @"messages"];
   
-  //XXX[self enableCancelButton:YES];
-  
-  mBackEnd->exec_sql(text, false, currentStatementOnly);  
+  mBackEnd->exec_sql(text, [[self activeEditor] backEnd], false, currentStatementOnly);
 }
 
 
@@ -466,7 +518,11 @@ draggedHandleAtOffset: (NSPoint) offset
   [mEditors addObject: editor];
   
   [mUpperTabView addTabViewItem:item];
-  [mUpperTabView selectLastTabViewItem:nil];
+  
+  // update resultsets before selecting, so that the matching rset gets auto-selected
+  [self updateResultsets];
+  
+  [mUpperTabView selectTabViewItemWithIdentifier: [item identifier]];
   
   [[mView window] makeFirstResponder: [[editor view] content]];
 }
@@ -638,13 +694,9 @@ static void refreshUIPartial(const int what, WBSQLQueryPanel *panel)
 {
   switch (what)
   {
-    case Db_sql_editor::RefreshSidebar:
-      [panel->mSidebar refresh];
+    case Db_sql_editor::RefreshSidebar: // possibly deprecated
       [panel->mOverview rebuildAll];
       break;
-    case Db_sql_editor::RefreshSchemaTree:
-      [panel->mSidebar refreshSchemaTree];
-      break;
     case Db_sql_editor::RefreshEditor:
       [[panel activeEditor] setString: [NSString stringWithCPPString: panel->mBackEnd->sql_editor()->sql()]];
       break;
@@ -663,9 +715,6 @@ static void refreshUIPartial(const int what, WBSQLQueryPanel *panel)
     case Db_sql_editor::RunCurrentStatement:
       [panel executeQuery: panel currentStatementOnly: true];
       break;
-    case Db_sql_editor::RefreshActiveSchemaSelector:
-      [panel->mSidebar refreshActiveSchema];
-      break;
     case Db_sql_editor::RefreshRecordsetTitle:
       [panel updateActiveRecordsetTitle];
       break;
@@ -684,6 +733,8 @@ static void refreshUIPartial(const int what, WBSQLQueryPanel *panel)
           [panel->mOverview refreshNodeChildren: bec::NodeId(node).append(j)];
         }
       }
+      else
+        [panel->mOverview rebuildAll];
       break;
     }
   }
@@ -717,8 +768,6 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
     [NSBundle loadNibNamed: @"WBSQLQueryPanel"
                      owner: self];
     
-    mResultsetViews= [[NSMutableArray alloc] init];
-    
     mBackEnd= be;
     mBackEnd->log()->refresh_ui_cb= sigc::bind(sigc::ptr_fun(reloadTable), mMessagesTable);
     mBackEnd->history()->entries_model()->refresh_ui_cb= sigc::bind(sigc::ptr_fun(reloadTable), mHistoryTable);
@@ -731,12 +780,21 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
     
     mBackEnd->exec_sql_task->progress_cb(sigc::bind(sigc::ptr_fun(processTaskProgress), self));
     mBackEnd->exec_sql_task->finish_cb(sigc::bind(sigc::ptr_fun(processTaskFinish), self));
-    mBackEnd->close_recordset_ui.connect(sigc::bind(sigc::ptr_fun(closeRecordset), self));
+    mBackEnd->recordset_list_changed.connect(sigc::bind(sigc::ptr_fun(recordsetListChanged), self));
     
     mBackEnd->sql_editor_new_ui.connect(sigc::bind(sigc::ptr_fun(editorCreated), self));
     
     mBackEnd->set_frontend_data(self);
     
+    // dock the backend provided sidebar
+    {
+      mforms::View *sidebar = mBackEnd->get_sidebar();
+      mSidebarView = nsviewForView(sidebar);
+      [mView addSubview: mSidebarView positioned: NSWindowBelow relativeTo: [[mView subviews] lastObject]];
+      [mView setPosition: 200 ofDividerAtIndex: 0];
+    }
+    
+    
     mTextOutputLock= [[NSLock alloc] init];
     
     [mTextOutput setFont: [NSFont fontWithName: @"Andale Mono" 
@@ -751,7 +809,7 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
     [(id)mLowerTabView createDragger];
     
     [mWorkView setDividerThickness: 0];
-    [mView setDividerThickness: 3];
+    [mView setDividerThickness: 2];
     
     [(MContainerView*)[mLowerTabView superview] setMinContentSize: NSMakeSize(40, RESULTS_AREA_COLLAPSED_MIN_HEIGHT+20)];
     
@@ -760,6 +818,11 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
     [mHistoryDetailsTable setTarget: self];
     [mHistoryDetailsTable setDoubleAction: @selector(activateHistoryDetailEntry:)];
 
+    for (id item in [mLowerTabView tabViewItems])
+    {
+      [(id)mLowerTabView setIcon:nil forTabViewItem: [item identifier]];
+    }
+    
     if (mBackEnd->is_physical_overview_enabled())
     {
       NSTabViewItem *item = [mLowerTabView tabViewItemAtIndex: 0];
@@ -793,8 +856,6 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
       [mLowerTabView removeTabViewItem: [mLowerTabView tabViewItemAtIndex:0]];
     }
 
-    [mSidebar setup];
-
     //XXX[self enableCancelButton:NO];
 
     mEditors = [[NSMutableArray alloc] init];
@@ -854,6 +915,9 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
     {
       [self addEditorTabWithBackEndIndex: [mUpperTabView numberOfTabViewItems]];
     }
+    
+    if (mBackEnd->is_physical_overview_enabled())
+      [mLowerTabView selectTabViewItemWithIdentifier: @"overview"];
   }
   return self;
 }
@@ -895,11 +959,8 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
 {
   NSTabViewItem *selectedItem= [mLowerTabView selectedTabViewItem];
 
-  for (id rsview in mResultsetViews)
-  {
-    if ([selectedItem view] == [rsview view])
-      return rsview;
-  }
+  if ([selectedItem isKindOfClass: [ResultTabViewItem class]])
+    return [(ResultTabViewItem*)selectedItem RSViewer];
 
   return nil;
 }
@@ -930,13 +991,14 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
   if (tabView == mUpperTabView)
   {
     MSQLEditorController *editor = [tabViewItem identifier];
-    Sql_editor::Ref ed = [editor backEnd];
+    Sql_editor::Ref ed([editor backEnd]);
 
     for (int i= 0; i < mBackEnd->sql_editor_count(); i++)
     {
       if (mBackEnd->sql_editor(i) == ed)
       {
         mBackEnd->active_sql_editor_index(i);
+        [mLowerTabView selectTabViewItemWithIdentifier: [NSString stringWithFormat:@"rset%li", mBackEnd->active_recordset_for_sql_editor(i)]];
         break;
       }
     }
@@ -945,7 +1007,25 @@ static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
   {
     MResultsetViewer *rset= [self selectedResultset];
     if (rset)
+    {
       mBackEnd->active_recordset([rset recordset]);
+      int ed(mBackEnd->active_sql_editor_index());
+      if (ed >= 0)
+      {
+        Sql_editor::Ref editor(mBackEnd->sql_editor(ed));
+
+        for (NSTabViewItem *item in [mUpperTabView tabViewItems])
+        {
+          if ([[item identifier] isKindOfClass: [MSQLEditorController class]]
+              && [(MSQLEditorController*)[item identifier] backEnd] == editor)
+          {
+            if ([mUpperTabView selectedTabViewItem] != item)
+              [mUpperTabView selectTabViewItemWithIdentifier: [item identifier]];
+            break;
+          }
+        }
+      }
+    }
     else
       mBackEnd->active_recordset(Recordset::Ref());
   }
@@ -995,19 +1075,9 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
     if ([tabViewItem view] == mOverview)
       return [mOverview willClose];
     
-    MResultsetViewer *viewer = nil;
-    for (id rsview in mResultsetViews)
+    if ([tabViewItem isKindOfClass: [ResultTabViewItem class]])
     {
-      if ([tabViewItem view] == [rsview view])
-      {
-        viewer = rsview;
-        break;
-      }
-    }
-    
-    if (viewer)
-    {
-      [viewer close];
+      [[(ResultTabViewItem*)tabViewItem RSViewer] close];
       return NO;
     }
     return YES;
@@ -1026,9 +1096,11 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
   [mTextOutputLock release];
   
   [mEditors release];
-  [mResultsetViews release];
   [mOverview release];
   
+  [mSplitterDelegate release];
+  [mView release];
+  [snippetDS release];
   [super dealloc]; 
 }
 
@@ -1059,9 +1131,6 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
         [mView setPosition: NSWidth([mView frame])-lastSidebarWidth ofDividerAtIndex: 0];
       else
         [mView setPosition: lastSidebarWidth ofDividerAtIndex: 0];
-      NSRect frame = [mSchemaPop frame];
-      frame.size.width = lastSidebarWidth - frame.origin.x - 7;
-      [mSchemaPop setFrame: frame];
     }
     else
     {
