@@ -22,12 +22,13 @@
 {
   bec::TreeModel *_tree;
   NSMutableDictionary *_nodeCache;
+  NSFont *_normalFont;
+  NSFont *_boldFont;
   BOOL _hideRootNode;
   id _dragDelegate;
 }
 
 - (id)initWithTreeModel:(bec::TreeModel*)model;
-
 - (void)setTreeModel:(bec::TreeModel*)model;
 - (void)setHidesRootNode:(BOOL)flag;
 
