@@ -35,11 +35,28 @@
 {
   _owner->set_frontend_data(0);
   _owner->release();
+  if (_defaultMenuBar)
+    _defaultMenuBar->release();
   
   [super dealloc];
 }
 
 
+- (void)setDefaultMenuBar:(mforms::MenuBar*)menu
+{
+  _defaultMenuBar = menu;
+}
+
+
+- (NSMenu*)menuBar
+{
+  NSMenu *menu = [super menuBar];
+  if (!menu && _defaultMenuBar)
+    return _defaultMenuBar->get_data();
+  return menu;
+}
+
+
 - (NSView*)topView
 {
   return nsviewForView(_owner);
