@@ -1,12 +1,20 @@
-//
-//  WBMainController.mm
-//  MySQLWorkbench
-//
-//  Created by Alfredo Kojima on 1/Oct/08.
-//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
-//
+/* 
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * 
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
-#import "string_utilities.h"
+#include "base/string_utilities.h"
 #import "WBMainController.h"
 #import "WBMainWindow.h"
 #import "MCppUtilities.h"
@@ -31,7 +39,6 @@
 
 #import "WBSQLQueryUI.h"
 
-#import "WBToolbarManager.h"
 #import "WBMenuManager.h"
 #import "WBDiagramSizeController.h"
 #import "WBModelDiagramPanel.h"
@@ -365,8 +372,6 @@ static void windowToolChanged(mdc::CanvasView *canvas, WBMainWindow *window)
   if ([[window selectedMainPanel] isKindOfClass: [WBModelDiagramPanel class]])
   {
     [(WBModelDiagramPanel*)[window selectedMainPanel] canvasToolChanged:canvas];
-    
-    [[window toolbarManager] refreshToolsToolbar: [(WBModelDiagramPanel*)[window selectedMainPanel] toolsToolbar]];
   }
 }
 
@@ -383,12 +388,6 @@ static void windowLockGui(bool lock, WBMainWindow *window)
 }
 
 
-static void windowPopupMenu(bec::MenuItemList *items, int x, int y, WBMainWindow *window)
-{
-  [window popupDiagramMenu:items atPoint: NSMakePoint(x, y)];
-}
-
-
 static void quitApplication()
 {
   [NSApp terminate:nil];
@@ -525,6 +524,9 @@ static void call_closetab(WBMainWindow *mainWindow)
   if ([[mainWindow window] isKeyWindow])
   {
     id activePanel = [mainWindow activePanel];
+    bec::UIForm *form = [activePanel formBE];
+    if (form && form->get_form_context_name() == "home")
+      return;
     
     if (![activePanel respondsToSelector: @selector(closeActiveEditorTab)]
         || ![activePanel closeActiveEditorTab])
@@ -536,8 +538,13 @@ static void call_closetab(WBMainWindow *mainWindow)
 
 static bool validate_closetab(WBMainWindow *mainWindow)
 {  
+  id activePanel = [mainWindow activePanel];
+  bec::UIForm *form = [activePanel formBE];
+  if (form && form->get_form_context_name() == "home")
+    return false;
+  
   // find where this belongs to
-  return [mainWindow activePanel] != nil;
+  return activePanel != nil && [[mainWindow window] isKeyWindow];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -640,7 +647,8 @@ static void call_find(WBMainWindow *mainWindow)
 
 static bool validate_find(WBMainWindow *mainWindow)
 {
-  if ([[mainWindow toolbarManager] searchField])
+  bec::UIForm *form = [mainWindow context]->get_active_main_form();
+  if (form && form->get_toolbar() && form->get_toolbar()->find_item("searchbox"))
     return true;
   return validate_find_replace([mainWindow context]);
 }
@@ -702,7 +710,8 @@ static bool validate_redo(WBMainWindow *mainWindow)
        [notification object] == [firstResponder superview]))
   { 
     // refresh edit menu
-    [[mainWindow menuManager] updateEditMenu];
+    wb::WBContextUI *wbui = [mainWindow context];
+    wbui->get_command_ui()->signal_validate_edit_menu_items().emit();
   }
 }
 
@@ -818,14 +827,18 @@ static void flush_main_thread()
                                              object: nil];
 }
 
+#include "mforms/../cocoa/MFMenuBar.h"
 
 - (void)windowDidBecomeKey:(NSNotification*)notification
 {
   if ([notification object] != [mainWindow window])
-    [[mainWindow menuManager] rebuildMainMenuForAuxiliaryWindow];
+  {
+    cf_swap_edit_menu();
+  }
   else
   {
-    [[mainWindow menuManager] rebuildMainMenu];
+    cf_unswap_edit_menu();
+    _wb->get_ui()->get_command_ui()->signal_validate_edit_menu_items().emit();
   }
 }
 
@@ -878,7 +891,6 @@ static NSString *applicationSupportFolder()
     wbcallbacks.tool_changed= sigc::bind(sigc::ptr_fun(windowToolChanged), mainWindow);
     wbcallbacks.refresh_gui= sigc::bind(sigc::ptr_fun(windowRefreshGui), mainWindow);
     wbcallbacks.lock_gui= sigc::bind(sigc::ptr_fun(windowLockGui), mainWindow);
-    wbcallbacks.popup_menu= sigc::bind(sigc::ptr_fun(windowPopupMenu), mainWindow);
     wbcallbacks.quit_application= sigc::ptr_fun(quitApplication);
       
     _options= new wb::WBOptions();
@@ -1139,6 +1151,29 @@ static void init_mforms()
 }
 
 
+- (IBAction)menuItemClicked:(id)sender
+{
+  // identifiers from the App menu. These are set in the nib
+#define APP_MENU_ABOUT 100
+#define APP_MENU_PREFERENCES 101
+#define APP_MENU_QUIT 102
+
+  // special handling for some Application menu items
+  switch ([sender tag])
+  {
+    case APP_MENU_ABOUT:
+      _wbui->get_command_ui()->activate_command("builtin:show_about");
+      break;
+      
+    case APP_MENU_PREFERENCES:
+      _wbui->get_command_ui()->activate_command("plugin:wb.form.showOptions");
+      break;
+      
+    case APP_MENU_QUIT:
+      _wbui->get_command_ui()->activate_command("plugin:wb.file.exit");
+      break;
+  }
+}
 
 
 - (IBAction)showAbout:(id)sender
