@@ -1,5 +1,5 @@
 /* 
- * Copyright © 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
 
 #import "WBMainWindow.h"
 #import "WBMainController.h"
-#import "WBMenuManager.h"
-#import "WBToolbarManager.h"
 #import "WBOverviewPanel.h"
 #import "WBModelOverviewPanel.h"
 #import "WBModelDiagramPanel.h"
@@ -39,7 +37,7 @@
 #include "model/wb_overview_physical.h"
 
 #import "WBMainWindow_Model.h"
-#import "string_utilities.h"
+#include "base/string_utilities.h"
 
 
 @interface MyAutoreleasePool : NSAutoreleasePool
@@ -56,9 +54,24 @@
 @end
 
 
-
 @implementation WBWindow
 
+- (id)initWithContentRect:(NSRect)contentRect styleMask:(NSUInteger)aStyle backing:(NSBackingStoreType)bufferingType defer:(BOOL)flag
+{
+  self = [super initWithContentRect: contentRect styleMask: aStyle backing: bufferingType defer: flag];
+  if (self)
+  {
+    // setup pointer to main window in mforms
+    NSInteger rc = [self retainCount];
+    mforms::Form::main_form()->set_data(self);
+    if ([self retainCount] > rc) // don't let the mforms placeholder retain us
+      [self release];
+    
+    //[self setBackgroundColor: [NSColor whiteColor]];
+  }
+  return self;
+}
+
 /** Override to catch changes to firstResponders
  *
  * This method is overriden so that we can notify the backend when the 
@@ -93,29 +106,14 @@ void setup_mforms_app(WBMainWindow *mwin);
   return _mainController;
 }
 
-- (WBToolbarManager*)toolbarManager
-{
-  return _toolbarManager;
-}
-
-- (WBMenuManager*)menuManager
-{
-  return _menuManager;
-}
-
-
 - (void)setWBContext:(wb::WBContextUI*)wbui
 {
   _wbui= wbui;
   
   setup_mforms_app(self);
     
-  _menuManager= [[WBMenuManager alloc] initWithMenuBar:[NSApp mainMenu] mainWindow:[self window] WBContext:_wbui];
-  
 //  [(WBToolbarTabView*)topTabView setAllowsTabReordering:NO];
-  
-  [_menuManager rebuildMainMenu];
-  
+    
 //  _wbui->get_wb()->get_grt()->get_undo_manager()->signal_changed().connect(sigc::bind(sigc::ptr_fun(history_changed), self));
 }
 
@@ -131,30 +129,29 @@ void setup_mforms_app(WBMainWindow *mwin);
  * relayouting.
  */
 - (void)setupReady
-{
-  _toolbarManager= [[WBToolbarManager alloc] initWithToolbar:[[self window] toolbar] 
-                                                   WBContext:_wbui];
-  
-  [_toolbarManager refreshMainToolbar];
+{  
   [[self window] setTitle:[NSString stringWithUTF8String:_wbui->get_title().c_str()]];
 
   [self setupModel];
 }
 
+
 - (NSTabViewItem*)addTopPanel:(WBBasePanel*)panel
 {
   id tabItem= [[[NSTabViewItem alloc] initWithIdentifier:[panel identifier]] autorelease];
     
-  [tabItem setView:[panel topView]];
+  [tabItem setView:[panel decoratedTopView]];
   [tabItem setLabel:[panel title]];
+  if ([panel respondsToSelector: @selector(initialFirstResponder)])
+    [tabItem setInitialFirstResponder: [(id)panel initialFirstResponder]];
   
   [_panels setObject:panel forKey:[panel identifier]];
   
   [topTabView addTabViewItem:tabItem];
 
-  NSImage *icon= [panel tabIcon];
-  if (icon)
-    [(id)topTabView setIcon:icon forTabViewItem:[panel identifier]];
+//  NSImage *icon= [panel tabIcon];
+//  if (icon)
+//    [(id)topTabView setIcon:icon forTabViewItem:[panel identifier]];
     
   return tabItem;
 }
@@ -241,7 +238,6 @@ void setup_mforms_app(WBMainWindow *mwin);
     "RefreshSchema",
     "RefreshNewView",
     "RefreshSelection",
-    "RefreshToolbar",
     "RefreshMessages",
     "RefreshCloseEditor", // argument: object-id (close all if "")
     "RefreshFindOutput",
@@ -253,7 +249,6 @@ void setup_mforms_app(WBMainWindow *mwin);
     "RefreshDocument",
     "RefreshCloseDocument",
     "RefreshZoom",
-    "RefreshMenubar",
     "RefreshTimer",
     "RefreshFinishEdits"
   };
@@ -344,28 +339,6 @@ void setup_mforms_app(WBMainWindow *mwin);
       }
       break;
     }
-    case wb::RefreshMenubar:
-      [_menuManager rebuildMainMenu];
-      break;      
-      
-    case wb::RefreshToolbar:
-      if (arg1.empty() || arg1 == "main")
-        [_toolbarManager refreshMainToolbar];
-      if (arg1.empty() || arg1 == "tools")
-      {
-        if ([[self selectedMainPanel] respondsToSelector: @selector(toolsToolbar)])
-          [_toolbarManager refreshToolsToolbar: [(id)[self selectedMainPanel] toolsToolbar]];
-      }
-      if (arg1.empty() || arg1 == "options")
-      {
-        if ([[self selectedMainPanel] respondsToSelector: @selector(optionsToolbar)])
-        {
-          BOOL flag;
-          flag= [_toolbarManager refreshToolOptionsToolbar: [(id)[self selectedMainPanel] optionsToolbar]];
-          [(id)[self selectedMainPanel] showOptionsToolbar:flag];
-        }
-      }      
-      break;
       
     case wb::RefreshSchemaList:
     case wb::RefreshSchema:
@@ -434,8 +407,6 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (void)blockGUI:(BOOL)lock
 {
-  //[_menuManager setBlocked: lock];
-  //[_toolbarManager setBlocked: lock];
 }
 
 
@@ -486,9 +457,28 @@ void setup_mforms_app(WBMainWindow *mwin);
 }
 
 
+- (void) tabViewDidChangeNumberOfTabViewItems:(NSTabView *)TabView
+{
+  if (TabView == topTabView)
+    [tabSwitcher setNeedsDisplay:YES];
+}
+
+
 - (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
 {
   id panel = [_panels objectForKey: [tabViewItem identifier]];
+  
+  if (tabView == topTabView)
+  {
+    NSMenu *menu = [panel menuBar];
+    if (menu)
+      [NSApp setMainMenu: menu];
+    else
+      [NSApp setMainMenu: _defaultMainMenu];
+
+    [tabSwitcher setNeedsDisplay:YES];
+  }
+  
   if ([panel respondsToSelector: @selector(didActivate)])
     [panel didActivate];
   else
@@ -558,27 +548,42 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 //--------------------------------------------------------------------------------------------------
 
+- (BOOL)tabView:(NSTabView*)tabView itemIsClosable:(NSTabViewItem*)tabViewItem
+{
+  if ([tabView indexOfTabViewItem: tabViewItem] == 0) // Home is not closable
+    return NO;
+  return YES;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 - (BOOL)closePanel:(WBBasePanel*)panel
 {
-  id container= [[panel topView] nextResponder];
+  id identifier = [[[panel identifier] retain] autorelease];
   
   if ([panel respondsToSelector:@selector(willClose)] && ![panel willClose])
     return NO;
-
-  // one more nextResponder, because of the hack with custom NSTabViews
-  if ([container isKindOfClass:[NSTabView class]])
-    container= [container nextResponder];
-
-  if (container == topTabView)
-  {
-    [self closeTopPanelWithIdentifier: [panel identifier]];
-  }  
+  if ([topTabView indexOfTabViewItemWithIdentifier: identifier] != NSNotFound)
+    [self closeTopPanelWithIdentifier: identifier];
+  else
+    [self closeBottomPanelWithIdentifier: identifier];
   return YES;
 }
 
 //--------------------------------------------------------------------------------------------------
 
 - (void)closeTopPanelWithIdentifier:(id)identifier
+{
+  BOOL hideOnly = NO;
+  id form= [_panels objectForKey:identifier];
+  if ([form isKindOfClass: [WBModelDiagramPanel class]] ||
+      [form isKindOfClass: [WBModelOverviewPanel class]])
+    hideOnly = YES;
+  [self closeTopPanelWithIdentifier: identifier hideOnly: hideOnly];
+}
+
+
+- (void)closeTopPanelWithIdentifier:(id)identifier hideOnly:(BOOL)hideOnly
 {
   NSUInteger index= [topTabView indexOfTabViewItemWithIdentifier: identifier];
   if (index != NSNotFound)
@@ -588,12 +593,11 @@ void setup_mforms_app(WBMainWindow *mwin);
     
     [form retain];
     
-    if ([form isKindOfClass: [WBModelDiagramPanel class]] ||
-        [form isKindOfClass: [WBModelOverviewPanel class]])
+    if (hideOnly)
       [_closedTopTabs addObject: item];
     else
       [_panels removeObjectForKey: [item identifier]];
-      
+    
     [topTabView removeTabViewItem: item];
     
     // need to release now instead of autoreleasing because plugins should be released synchronously
@@ -602,6 +606,7 @@ void setup_mforms_app(WBMainWindow *mwin);
   }
 }
 
+
 //--------------------------------------------------------------------------------------------------
 
 - (void)closeBottomPanelWithIdentifier:(id)identifier
@@ -628,10 +633,6 @@ void setup_mforms_app(WBMainWindow *mwin);
     NSAssert(![panel isKindOfClass:[WBModelDiagramPanel class]] || [(WBModelDiagramPanel*)panel canvas], @"diagram panel has no canvas at moment of tab switch");
 
     _wbui->set_active_form([panel formBE]);
-    
-    // not needed in theory, but we refresh here anyway to make the update more immediate
-    if ([panel respondsToSelector: @selector(toolsToolbar)])
-      [_toolbarManager refreshToolsToolbar: [(id)panel toolsToolbar]];    
   }
 }
 
@@ -755,43 +756,42 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (void)focusSearchField:(id)sender
 {
-  id field = [_toolbarManager searchField];
-  if (field)
-    [[self window] makeFirstResponder:field];
-  else
-    NSBeep();
+  bec::UIForm *main_form = _wbui->get_active_main_form();
+  if (main_form && main_form->get_toolbar())
+  {
+    mforms::ToolBarItem *item = main_form->get_toolbar()->find_item("searchbox");
+    if (item)
+    {
+      id view = item->get_data();
+      if (view && [view isKindOfClass: [NSTextField class]])
+      {
+        [[self window] makeFirstResponder: view];
+        return;
+      }
+    }
+  }
+  NSBeep();
 }
 
 //--------------------------------------------------------------------------------------------------
 
 - (void)performSearchObject:(id)sender
 {
-  NSString *search= [_toolbarManager lastSearchString];
-  if (search && [search length] > 0)
+  bec::UIForm *main_form= _wbui->get_active_main_form();
+  if (main_form && main_form->get_frontend_data() && main_form->get_toolbar())
   {
-    bec::UIForm *main_form= _wbui->get_active_main_form();
-    if (main_form && main_form->get_frontend_data())
+    mforms::ToolBarItem *item = main_form->get_toolbar()->find_item("searchbox");
+    if (item)
     {
       id form= reinterpret_cast<id>(main_form->get_frontend_data());
       if ([form respondsToSelector:@selector(searchString:)])
-        [form searchString: search];
+        [form searchString: [NSString stringWithCPPString: item->get_text()]];
     }
   }
 }
 
 //--------------------------------------------------------------------------------------------------
 
-- (void)popupDiagramMenu:(bec::MenuItemList*)items atPoint:(NSPoint)pos
-{
-  NSView *diagram= [[_panels objectForKey: [[topTabView selectedTabViewItem] identifier]] canvasViewer];
-  
-  [_menuManager refreshMenu:contextMenu withItems:*items];
-  
-  [NSMenu popUpContextMenu:contextMenu withEvent:[NSApp currentEvent] forView:diagram];
-}
-
-//--------------------------------------------------------------------------------------------------
-
 - (void)resetWindowLayout
 {
   [self showMySQLOverview:nil];
@@ -808,6 +808,7 @@ void setup_mforms_app(WBMainWindow *mwin);
       id form= [_panels objectForKey: [item identifier]];
       if (!form)
         NSLog(@"Item in toplevel tabview is not a known form", [item label]);
+
       return form;
     }
   }
@@ -850,24 +851,43 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 //--------------------------------------------------------------------------------------------------
 
-- (WBBasePanel*)panelForResponder: (NSResponder*)responder
+- (WBBasePanel*)panelForResponder: (NSResponder*)aResponder
 {
+  NSResponder *responder = aResponder;
   // look up the responder chain until we reach one of the known toplevel containers
   NSResponder *previous= nil;
   NSResponder *previousPrevious= nil;
+  
   while (responder && (responder != topTabView))
   {
-    // because of the bizarre way our custom tabviews are implemented, there will be a dummy NSTabView between the
-    // actual tabview and the content view we want.
-    previousPrevious= previous;
     previous= responder;
-    responder= [responder nextResponder];
+    responder= [responder nextResponder];    
+  }
+  previousPrevious= previous;
+
+  if (responder && [responder isKindOfClass: [NSTabView class]])
+  {
+    WBBasePanel *panel = [self findPanelForView: (NSView*)previousPrevious inTabView: (NSTabView*)responder];
+    if (panel && previousPrevious && [panel isKindOfClass: [WBSplitPanel class]])
+    {
+      // check if the responder is in a panel inside the found panel (like an editor in the model diagram panel)
+      responder = aResponder;
+      while (responder && responder != previousPrevious)
+      {
+        WBBasePanel *subpanel;
+        
+        subpanel = [(WBSplitPanel*)panel findPanelForView: (NSView*)responder];
+        if (subpanel)
+          return subpanel;
+        
+        previous = responder;
+        responder = [responder nextResponder];
+      }
+    }
+    return panel;
   }
   
-  if (responder && [responder isKindOfClass: [NSTabView class]])
-    return [self findPanelForView: (NSView*)previousPrevious inTabView: (NSTabView*)responder];
-  
-  return 0;  
+  return nil;
 }
 
 
@@ -892,18 +912,21 @@ void setup_mforms_app(WBMainWindow *mwin);
 - (void)firstResponderChanged: (NSResponder*)responder
 {
   BOOL changedActivePanel = NO;
-  
+
   WBBasePanel *panel= [self panelForResponder:responder];
-  if (panel != [self activePanel])
+
+  if ([panel formBE] != _wbui->get_active_form())
   {
     changedActivePanel = YES;
     [self activatePanel: panel];
   }
   // replace Edit menu with the standard one so that copy/paste works without intervention
   if ([responder isKindOfClass: [NSTextView class]])
-    [_menuManager rebuildMainMenuForAuxiliaryWindow];    
+  {
+    NSLog(@"restore edit menu");
+  }
   else if (!changedActivePanel)
-    [_menuManager updateEditMenu];
+    _wbui->get_command_ui()->signal_validate_edit_menu_items().emit();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -944,6 +967,13 @@ void setup_mforms_app(WBMainWindow *mwin);
     _panels= [[NSMutableDictionary alloc] init];
     _closedTopTabs= [[NSMutableArray alloc] init];
     _closedBottomTabs= [[NSMutableArray alloc] init];
+    
+    _defaultMainMenu = [[NSApp mainMenu] retain];
+    
+    [[NSNotificationCenter defaultCenter] addObserver: self
+                                             selector: @selector(applicationWillTerminate:)
+                                                 name: NSApplicationWillTerminateNotification
+                                               object: nil];    
   }
   return self;
 }
@@ -965,13 +995,21 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 //--------------------------------------------------------------------------------------------------
 
+- (void)applicationWillTerminate:(NSNotification*)notification
+{
+  id panelsCopy = [_panels copy];
+  for (id panel in [panelsCopy objectEnumerator])
+    [self closePanel: panel];
+  [panelsCopy release];
+}
+
+//--------------------------------------------------------------------------------------------------
+
 - (void)dealloc
 {
   [_closedTopTabs release];
   [_closedBottomTabs release];
   
-  [_menuManager release];
-  [_toolbarManager release];
   [_panels release];
   [_physicalOverview release];
 	
@@ -1015,6 +1053,9 @@ static void dock_view(mforms::App *app, mforms::AppView *view, const std::string
     else
     {
       panel = [[WBMFormsPluginPanel alloc] initWithAppView: view];
+      if (!view->get_menubar())
+        [panel setDefaultMenuBar: [self context]->get_command_ui()->create_menubar_for_context("")];
+        
       [self addTopPanelAndSwitch: [panel autorelease]];
     }
   }
@@ -1138,6 +1179,49 @@ static void free_ar_pool()
   [[[NSThread currentThread] threadDictionary] removeObjectForKey:@"MFormsAutoReleasePool"];
 }
 
+//--------------------------------------------------------------------------------------------------
+
+static int enter_event_loop(mforms::App *app, float timeout)
+{
+  WBMainWindow *mwin = app->get_data();
+  if (mwin)
+  {
+    NSDate *later = timeout > 0.0 ? [NSDate dateWithTimeIntervalSinceNow: timeout] : [NSDate distantFuture];
+    mwin->_eventLoopRetCode = -0xdead1009;
+
+    while (mwin->_eventLoopRetCode == -0xdead1009)
+    {
+      NSEvent *e = [NSApp nextEventMatchingMask: NSAnyEventMask
+                                      untilDate: later
+                                         inMode: NSDefaultRunLoopMode
+                                        dequeue: YES];
+      
+      if (e != nil)
+        [NSApp sendEvent: e];
+      else
+        if ([[NSDate date] earlierDate: later] == later)
+        {
+          mwin->_eventLoopRetCode = -1;
+          break;
+        }
+    }
+    
+    return mwin->_eventLoopRetCode;
+  }
+  return -1;
+}
+
+
+static void exit_event_loop(mforms::App *app, int retcode)
+{
+  WBMainWindow *mwin = app->get_data();
+  if (mwin)
+  {
+    mwin->_eventLoopRetCode = retcode;
+  }
+}
+
+
 void setup_mforms_app(WBMainWindow *mwin)
 {
   mforms::ControlFactory *cf = mforms::ControlFactory::get_instance();
@@ -1154,6 +1238,8 @@ void setup_mforms_app(WBMainWindow *mwin)
   cf->_app_impl.get_bounds= get_main_window_bounds;
   cf->_app_impl.begin_thread_loop= alloc_ar_pool;
   cf->_app_impl.end_thread_loop= free_ar_pool;
+  cf->_app_impl.enter_event_loop= enter_event_loop;
+  cf->_app_impl.exit_event_loop= exit_event_loop;
 }
 
 //--------------------------------------------------------------------------------------------------
