@@ -23,11 +23,9 @@
 #import "MCanvasScrollView.h"
 #import "GRTIconCache.h"
 #import "GRTTreeDataSource.h"
-#import "WBToolbarManager.h"
-#import "WBMiniToolbar.h"
 #import "WBObjectDescriptionController.h"
 #import "WBModelSidebarController.h"
-
+#import "MCPPUtilities.h"
 
 static int zoom_levels[]= {
 200,
@@ -64,7 +62,6 @@ sharedSidebarController:(WBModelSidebarController*) controller
     _formBE->set_frontend_data(self);
 
     [NSBundle loadNibNamed:@"WBModelDiagram" owner:self];
-        
     _identifier= [oid retain];
     _viewer= [[[MCanvasViewer alloc] initWithFrame:NSMakeRect(0, 0, 300, 300)] autorelease];
     
@@ -86,7 +83,6 @@ sharedSidebarController:(WBModelSidebarController*) controller
     // setup navigator
     for (int i= 0; i < sizeof(zoom_levels)/sizeof(int); i++)
       [zoomCombo addItemWithObjectValue:[NSNumber numberWithFloat:zoom_levels[i]]];
-    
     [navigatorViewer setupQuartz];
     [navigatorViewer setPostsFrameChangedNotifications:YES];
     
@@ -94,7 +90,6 @@ sharedSidebarController:(WBModelSidebarController*) controller
                                              selector: @selector(navigatorFrameChanged:)
                                                  name: NSViewFrameDidChangeNotification
                                                object: navigatorViewer];    
-    
     [_viewer setupQuartz];
     [_viewer setDelegate:self];
     [scrollView setContentCanvas:_viewer];
@@ -112,6 +107,27 @@ sharedSidebarController:(WBModelSidebarController*) controller
     sidebarAtRight = NO;
     [self setRightSidebar: be->get_wb()->get_wb_options().get_int("Sidebar:RightAligned", 0)];
     lastSidebarWidth = [sidebar frame].size.width;
+    
+    
+    // setup tools toolbar
+    mforms::ToolBar *tbar = _formBE->get_tools_toolbar();
+    if (tbar)
+    {
+      NSView *view = tbar->get_data();
+      [toolbar addSubview: view];
+      [view setAutoresizingMask: NSViewHeightSizable|NSViewMinXMargin|NSViewMaxYMargin];
+      [view setFrame: [toolbar bounds]];
+    }
+
+    // setup options toolbar
+    tbar = _formBE->get_options_toolbar();
+    if (tbar)
+    {
+      NSView *view = tbar->get_data();
+      [optionsToolbar addSubview: view];
+      [view setAutoresizingMask: NSViewWidthSizable|NSViewMinXMargin|NSViewMaxYMargin];
+      [view setFrame: [optionsToolbar bounds]];
+    }
   }
   return self;
 }
@@ -119,7 +135,18 @@ sharedSidebarController:(WBModelSidebarController*) controller
 
 - (void)dealloc
 {
+  [_identifier release];
+  [[NSNotificationCenter defaultCenter] removeObserver: self];
+  [sidebarController invalidate];
+  
   [_viewer setDelegate: nil];
+  [topView release];
+  [sidebarController release];
+  [descriptionController release];
+  [mPropertiesController release];
+  [mainSplitViewDelegate release];
+  [layerDS release];
+  
   [super dealloc];
 }
 
@@ -130,18 +157,6 @@ sharedSidebarController:(WBModelSidebarController*) controller
 }
 
 
-- (NSView*) toolsToolbar
-{
-  return toolbar;
-}
-
-
-- (id)optionsToolbar
-{
-  return optionsToolbar;
-}
-
-
 - (void)showOptionsToolbar:(BOOL)flag
 {
   if ([optionsToolbar isHidden] != !flag)
@@ -162,6 +177,8 @@ sharedSidebarController:(WBModelSidebarController*) controller
     [scrollView setFrame: rect];
 */
   }
+  else
+    [optionsToolbar setNeedsDisplay: YES];
 }
 
 
@@ -254,6 +271,12 @@ static NSPoint loadCursorHotspot(const std::string &path)
   return _formBE;
 }
 
+
+- (NSView*)initialFirstResponder
+{
+  return _viewer;
+}
+
 - (BOOL)isClosed
 {
   return _formBE->is_closed();
@@ -404,9 +427,9 @@ static void refresh_layer_tree(const bec::NodeId &node, int old_count, WBModelDi
 
 - (void)canvasToolChanged:(mdc::CanvasView*)canvas
 {
-  BOOL flag;
-  flag= [[WBToolbarManager defaultManager] refreshToolOptionsToolbar: optionsToolbar];
-  [self showOptionsToolbar:flag];
+  mforms::ToolBar *tb = _formBE->get_options_toolbar();
+  
+  [self showOptionsToolbar: tb && !tb->get_items().empty()];
   
   [self updateCursor];
 }
@@ -575,4 +598,5 @@ static void refresh_layer_tree(const bec::NodeId &node, int old_count, WBModelDi
 
 //--------------------------------------------------------------------------------------------------
 
+
 @end
