@@ -29,6 +29,7 @@ using System.Windows.Forms;
 
 using MySQL.Utilities;
 using MySQL.Utilities.SysUtils;
+using System.Drawing.Text;
 
 namespace MySQL.Controls
 {
@@ -45,6 +46,8 @@ namespace MySQL.Controls
       TopNormal,      // Tabs standing on the content above the top line. Focus color is a constant color.
       TopHanging,     // Tabs hanging down above the content. Focus color is a gradient.
       BottomNormal,   // Tabs hanging down below the content, selected color is Tab.BackColor.
+      TopTransparent, // Tabs standing on the content, like TopNormal. Different colors, though.
+                      // Background of space not covered by a tab is transparent. Slightly different outline too.
     }
 
     public enum CloseButtonVisiblity
@@ -151,6 +154,7 @@ namespace MySQL.Controls
       // Both buttons really should have the same size.
       darkCloseButton = ((System.Drawing.Bitmap)(resources.GetObject("tab_close_dark")));
       lightCloseButton = ((System.Drawing.Bitmap)(resources.GetObject("tab_close_light")));
+      
       buttonSize = new Size(darkCloseButton.Width, darkCloseButton.Height);
     }
 
@@ -202,7 +206,8 @@ namespace MySQL.Controls
           // The "larger" value indicates the direction of the computation.
           // A 1 means from display rectangle to window rectangle. A 0 the other way around.
           int topOffset = 0;
-          if (tabStyle == TabStyleType.TopNormal || tabStyle == TabStyleType.TopHanging)
+          if (tabStyle == TabStyleType.TopNormal || tabStyle == TabStyleType.TopHanging || 
+            tabStyle == TabStyleType.TopTransparent)
             topOffset = ItemSize.Height;
           int bottomOffset = 0;
           if (tabStyle == TabStyleType.BottomNormal)
@@ -365,6 +370,12 @@ namespace MySQL.Controls
 
     #region Drawing the control and its parts.
 
+    protected override void OnPaintBackground(PaintEventArgs e)
+    {
+      // Do not erase the background. We either use a transparent background or completely
+      // draw it in the DrawControl function.
+    }
+
     protected override void OnPaint(PaintEventArgs e)
 	  {
 	    base.OnPaint(e); 
@@ -377,11 +388,23 @@ namespace MySQL.Controls
 			if (!Visible)
 				return;
 
+      g.SmoothingMode = SmoothingMode.HighQuality;
+
       bool drawFocused = showFocusState && ControlUtilities.IsHierarchyFocused(this);
+      RectangleF clientArea = ClientRectangle;
+      clientArea.Offset(-0.5f, -0.5f);
 
 			// Fill client area with the control's background color
       // (everything outside the inner rectangle, e.g. including margin).
-      Rectangle clientArea = ClientRectangle;
+      // Exclude the transparent part if TopTransparent is set as tab style (though only if
+      // we are running on Aero).
+      if (tabStyle == TabStyleType.TopTransparent && ControlUtilities.IsCompositionEnabled())
+      {
+        int topSpace = ItemSize.Height + Margin.Top;
+        clientArea.Y += topSpace;
+        clientArea.Height -= topSpace;
+      }
+
       using (Brush brush = new SolidBrush(BackgroundColor))
         g.FillRectangle(brush, clientArea);
 
@@ -404,34 +427,34 @@ namespace MySQL.Controls
           // with the inner color but keep the top padding area the same color as that of the tab.
           if (SelectedTab != null)
           {
-            Rectangle innerArea = clientArea;
+            RectangleF innerArea = clientArea;
             innerArea.X += Margin.Left;
             innerArea.Width -= Margin.Horizontal;
             innerArea.Y += Margin.Top + ItemSize.Height + ContentPadding.Top;
             innerArea.Height -= Margin.Vertical + ItemSize.Height + ContentPadding.Top;
 
-            SolidBrush brush = new SolidBrush(topNormalContentBackgroundColor);
-            g.FillRectangle(brush, innerArea);
-            brush.Dispose();
+            using (SolidBrush brush = new SolidBrush(topNormalContentBackgroundColor))
+              g.FillRectangle(brush, innerArea);
 
             innerArea.Y -= ContentPadding.Top;
             innerArea.Height = ContentPadding.Top;
+            Color color;
             if (drawFocused)
-              brush = new SolidBrush(topNormalFocusedTabGradientStart);
+              color = topNormalFocusedTabGradientStart;
             else
-              brush = new SolidBrush(topNormalContentBackgroundColor);
-            g.FillRectangle(brush, innerArea);
-            brush.Dispose();
+              color = topNormalContentBackgroundColor;
+            using (SolidBrush brush = new SolidBrush(color))
+              g.FillRectangle(brush, innerArea);
           }
 
           break;
 
         case TabStyleType.BottomNormal:
-          // Fill the inner rectangle (tab control area within margins but not covered by content)
+          // Fill the inner rectangle (tab control area within margins but not covered by content or tabs)
           // with the inner color but keep the top padding area the same color as that of the tab.
           if (SelectedTab != null)
           {
-            Rectangle innerArea = clientArea;
+            RectangleF innerArea = clientArea;
             innerArea.X += Margin.Left;
             innerArea.Width -= Margin.Horizontal;
             innerArea.Y += Margin.Top;
@@ -442,8 +465,16 @@ namespace MySQL.Controls
           }
 
           break;
+
+        case TabStyleType.TopTransparent:
+          {
+          }
+          break;
       }
 
+      if (TabPages.Count == 0)
+        return;
+
       // Draw only tabs which lie at least partially in the visible area.
       Rectangle clipRect = new Rectangle(Margin.Left, Margin.Top, ClientSize.Width - Margin.Horizontal,
         ItemSize.Height);
@@ -452,112 +483,212 @@ namespace MySQL.Controls
       g.SetClip(clipRect);
 
       int leftBorder = Margin.Left - scrollOffset;
-      int rightBorder = clientArea.Width - Margin.Horizontal;
+      float rightBorder = clientArea.Width - Margin.Horizontal;
       for (int i = 0; i < TabCount; i++)
       {
         ValidateTab(i);
         if (layoutInfo[i].tabArea.Right > leftBorder || layoutInfo[i].tabArea.Left < rightBorder)
           DrawTab(g, i, drawFocused);
       }
+
+      // The last thing we need is a separator line in certain styles.
+      if (tabStyle == TabStyleType.TopTransparent)
+      {
+        // Most of the separator is gray, so draw this first and then "remove" the piece
+        // below the active tab. The color of this line is that of the tab outline.
+        using (Pen grayPen = new Pen(Color.FromArgb(0xff, 0x92, 0x9b, 0xa2)))
+          g.DrawLine(grayPen, Margin.Left - 0.5f, Margin.Top + ItemSize.Height - 1, rightBorder, Margin.Top + ItemSize.Height - 1);
+
+        // Fade out the gray line on the left hand side of the active tab (if it is not the first one).
+        RectangleF area = GetTabRect(SelectedIndex);
+        area.X += 0.5f;
+        if (SelectedIndex > 0)
+        {
+          using (LinearGradientBrush gradientBrush = new LinearGradientBrush(
+            new RectangleF(area.Left - 2, 0, 4, 1),
+            Color.FromArgb(0xff, 0x92, 0x9b, 0xa2), Color.FromArgb(0xff, 0xd6, 0xdc, 0xe5),
+            LinearGradientMode.Horizontal))
+          {
+            using (Pen pen = new Pen(gradientBrush))
+              g.DrawLine(pen, area.Left - 2, area.Bottom - 1, area.Left + 3, area.Bottom - 1);
+          }
+        }
+
+
+        // The color to erase the separator line is what is the final color of the tab gradient.
+        using (Pen lightPen = new Pen(Color.FromArgb(0xff, 0xd6, 0xdc, 0xe5)))
+        {
+          g.DrawLine(lightPen, area.Left, Margin.Top + ItemSize.Height - 1, area.Right - 2,
+            Margin.Top + ItemSize.Height - 1);
+        }
+
+        // Fade out the gray line on the right hand side of the active tab.
+        using (LinearGradientBrush gradientBrush = new LinearGradientBrush(
+          new RectangleF(area.Right - 2, 0, 3, 1),
+          Color.FromArgb(0xff, 0xd6, 0xdc, 0xe5), Color.FromArgb(0xff, 0x92, 0x9b, 0xa2), 
+          LinearGradientMode.Horizontal))
+        {
+          using (Pen pen = new Pen(gradientBrush))
+            g.DrawLine(pen, area.Right - 2, area.Bottom - 1, area.Right + 1, area.Bottom - 1);
+        }
+      }
 		}
 
-		internal void DrawTab(Graphics g, int index, bool drawFocused)
-		{
-			RectangleF bounds = GetTabRect(index);
+    internal void DrawTab(Graphics g, int index, bool drawFocused)
+    {
+	    RectangleF bounds = GetTabRect(index);
 
       // For high quality drawing with anti aliased lines we need to specify line and gradient coordinates
       // which lie between two pixels (not exactly *on* one, so adjust by a half pixel offset for these cases.
       bounds.X -= 0.5f;
-      bounds.Y -= 0.5f;
+      if (tabStyle != TabStyleType.TopTransparent)
+        bounds.Y -= 0.5f;
+      
+      TabPage page = TabPages[index];
 
       bool isSelected = (SelectedIndex == index);
-      if (isSelected)
+      if (isSelected || tabStyle == TabStyleType.TopTransparent)
       {
-        // The active tab gets a special background.
-        g.SmoothingMode = SmoothingMode.HighQuality;
+        GraphicsPath outline = GetTabOutline(ref bounds, index == 0);
 
-        // Define form of the tab.
-        using (GraphicsPath path = GetTabOutline(ref bounds))
+        // Draw interior first.
+        GraphicsPath interior = (GraphicsPath) outline.Clone();
+
+        // In order to avoid artifacts result from overlapping filled interior and the border
+        // move the interior down a bit.
+        float border = 0; // Area not covered by gradient.
+        if (tabStyle == TabStyleType.TopTransparent)
         {
+          Matrix matrix = new Matrix();
+          matrix.Translate(0, 0.5f);
+          interior.Transform(matrix);
+          border = 1.5f;
+        }
 
-          // Define gradients based on the tab style.
-          ColorBlend gradientTupels = GetTabGradient(index, drawFocused);
+        ColorBlend gradientTupels = GetTabGradient(index, drawFocused);
 
+        using (LinearGradientBrush gradientBrush = new LinearGradientBrush(
+          new RectangleF(Margin.Left, Margin.Top - 0.5f, 1, ItemSize.Height - border),
+          Color.Black, Color.White,
+          LinearGradientMode.Vertical))
+        {
+          gradientBrush.InterpolationColors = gradientTupels;
+          gradientBrush.ScaleTransform(-1, -1);
+
+          g.FillPath(gradientBrush, interior);
+        }
+
+        // Next draw the outline if the current tab style has one.
+        if (tabStyle == TabStyleType.TopTransparent)
+        {
+          // Draw the outline of the tab. For some reasons the outline must be specified
+          // "on-pixel", instead of intermediate positions. Go figure...
+          // Undo the previous move of 1 pixel on the way.
+          Matrix matrix = new Matrix();
+          matrix.Translate(0.5f, 0);
+          outline.Transform(matrix);
+
+          int alpha = 192;
+          if (isSelected)
+            alpha = 255;
           using (LinearGradientBrush gradientBrush = new LinearGradientBrush(
-            new RectangleF(Margin.Left, Margin.Top - 0.5f, 1, ItemSize.Height),
-            Color.Black, Color.White,
+            new RectangleF(0, Margin.Top, 1, ItemSize.Height - Margin.Top + 0.5f),
+            Color.FromArgb(alpha, 0x89, 0x89, 0x8a), Color.FromArgb(alpha, 0x92, 0x9b, 0xa2),
             LinearGradientMode.Vertical))
           {
-            gradientBrush.InterpolationColors = gradientTupels;
-            gradientBrush.ScaleTransform(-1, -1);
-
-            g.FillPath(gradientBrush, path);
+            using (Pen pen = new Pen(gradientBrush))
+              g.DrawPath(pen, outline);
           }
+
+          // The first tab is a bit special as its left hand side goes straight down, so we
+          // don't have curve there. Which in turn means there is a little gap. Fill that too.
+          using (Pen pen = new Pen(Color.FromArgb(0xff, 0x92, 0x9b, 0xa2)))
+            g.DrawLine(pen, Margin.Left, Margin.Top + ItemSize.Height - 2, Margin.Left, Margin.Top + ItemSize.Height + 1);
         }
-        g.SmoothingMode = SmoothingMode.Default;
       }
 
-			// Tab icon.
-      TabPage page = TabPages[index];
-			if ((page.ImageIndex >= 0) && (ImageList != null) && (ImageList.Images[page.ImageIndex] != null))
-			{
-				int nLeftMargin = 8;
-				int nRightMargin = 2;
+      bounds.Offset(itemPadding.Left, 0);
+      
+      // Tab icon.
+      if ((page.ImageIndex >= 0) && (ImageList != null) && (ImageList.Images[page.ImageIndex] != null))
+      {
+        int iconSpacing = 4;
 
-				Image img = ImageList.Images[page.ImageIndex];
+        Point position = new Point((int) bounds.X, (int) bounds.Y);
 				
-				RectangleF rimage = new RectangleF(bounds.X + nLeftMargin, bounds.Y + 1, img.Width, img.Height);
-				
-				// adjust rectangles
-				float nAdj = (float)(nLeftMargin + img.Width + nRightMargin);
+        int adjustment = ImageList.ImageSize.Width + iconSpacing;
 
-				rimage.Y += ((float) bounds.Height - img.Height) / 2;
-				bounds.X += nAdj;
-				bounds.Width -= nAdj;
-
-				g.DrawImage(img, rimage);
-			}
+        position.Y += 1 + ((int)bounds.Height - ImageList.ImageSize.Height) / 2;
+        bounds.X += adjustment;
+        bounds.Width -= adjustment;
+        
+        ImageList.Draw(g, position, page.ImageIndex);
+      }
 
 			// Tab text.
-      using (StringFormat stringFormat = new StringFormat())
+      if (page.Text.Length > 0)
       {
-        stringFormat.Alignment = StringAlignment.Near;
-        stringFormat.LineAlignment = StringAlignment.Center;
-        stringFormat.Trimming = StringTrimming.EllipsisCharacter;
-        stringFormat.FormatFlags = StringFormatFlags.NoWrap;
-
-        Brush brush;
-        if (isSelected)
+        using (StringFormat stringFormat = new StringFormat())
         {
-          if (tabStyle == TabStyleType.TopHanging || tabStyle == TabStyleType.TopNormal)
-          {
-            if (drawFocused)
-              brush = new SolidBrush(Color.Black);
+          stringFormat.Alignment = StringAlignment.Near;
+          stringFormat.Trimming = StringTrimming.EllipsisCharacter;
+          stringFormat.FormatFlags = StringFormatFlags.NoWrap;
+          stringFormat.LineAlignment = StringAlignment.Center;
+
+          Brush brush;
+          if (tabStyle == TabStyleType.TopTransparent)
+            brush = new SolidBrush(Color.Black);
+          else
+            if (isSelected)
+            {
+              if (tabStyle == TabStyleType.TopHanging || tabStyle == TabStyleType.TopNormal)
+              {
+                if (drawFocused)
+                  brush = new SolidBrush(Color.Black);
+                else
+                  brush = new SolidBrush(topStyleFocusedTextColor);
+              }
+              else
+                brush = new SolidBrush(Color.Black);
+            }
             else
-              brush = new SolidBrush(topStyleFocusedTextColor);
+              brush = new SolidBrush(Color.White);
+
+          int buttonPart = 0;
+          if (IsCloseButtonVisible(index))
+            buttonPart = buttonSpacing + buttonSize.Width;
+          bounds.Width -= itemPadding.Horizontal + buttonPart;
+          bounds.Height -= itemPadding.Vertical;
+
+          // For writing text on a (semi) transparent surface (e.g. when using Aero)
+          // normal text output routines will mess up the ClearType/anti aliasing extra pixels
+          // or the text color (sometimes both together). Hence we use a path to draw the text.
+          if (tabStyle != TabStyleType.TopTransparent || isSelected)
+          {
+            // Since GraphicsPath.AddString renders text significantly different we use it only
+            // if we really need to.
+            g.DrawString(page.Text, Font, brush, bounds, stringFormat);
           }
           else
-            brush = new SolidBrush(Color.Black);
+          {
+            using (GraphicsPath textPath = new GraphicsPath())
+            {
+              float emSize = g.DpiY * Font.SizeInPoints / 72;
+              if (tabStyle == TabStyleType.TopTransparent)
+                bounds.Y--; // Account for outline border.
+              textPath.AddString(page.Text, Font.FontFamily, (int)Font.Style, emSize, bounds, stringFormat);
+              g.FillPath(brush, textPath);
+            }
+          }
+          brush.Dispose();
         }
-        else
-          brush = new SolidBrush(Color.White);
-
-        int buttonPart = 0;
-        if (IsCloseButtonVisible(index))
-          buttonPart = buttonSpacing + buttonSize.Width;
-        bounds.Offset(itemPadding.Left, itemPadding.Top);
-        bounds.Width -= itemPadding.Horizontal + buttonPart;
-        bounds.Height -= itemPadding.Vertical;
-        g.DrawString(page.Text, Font, brush, bounds, stringFormat);
-
-        brush.Dispose();
       }
 
-      // Finally the close button if this tab is active.
-      if (isSelected && IsCloseButtonVisible(index))
+      // Finally the close button if this tab is active (or we use the TopTransparent style).
+      if ((isSelected || tabStyle == TabStyleType.TopTransparent) && IsCloseButtonVisible(index))
       {
         Rectangle buttonRect = CloseButtonRect(index);
-        if (drawFocused)
+        if (drawFocused || tabStyle == TabStyleType.TopTransparent)
           g.DrawImage(darkCloseButton, buttonRect);
         else
           g.DrawImage(lightCloseButton, buttonRect);
@@ -569,10 +700,10 @@ namespace MySQL.Controls
     /// </summary>
     /// <param name="bounds">The bounding rectangle for the tab.</param>
     /// <returns>The outline of the tab as path, ready to be filled.</returns>
-    private GraphicsPath GetTabOutline(ref RectangleF recBounds)
+    private GraphicsPath GetTabOutline(ref RectangleF recBounds, bool isFirst)
     {
       GraphicsPath path = new GraphicsPath();
-      float cornerSize = 6;
+      float cornerSize = 6.5f;
 
       switch (tabStyle)
       {
@@ -596,13 +727,27 @@ namespace MySQL.Controls
 
         case TabStyleType.BottomNormal:
           path.AddLine(recBounds.Left, recBounds.Top, recBounds.Right, recBounds.Top);
-          path.AddLine(recBounds.Right, recBounds.Top, recBounds.Right, recBounds.Bottom - cornerSize);
           path.AddArc(recBounds.Right - cornerSize, recBounds.Bottom - cornerSize, cornerSize, cornerSize, 0, 90);
-          path.AddLine(recBounds.Right - cornerSize, recBounds.Bottom, recBounds.Left + cornerSize, recBounds.Bottom);
           path.AddArc(recBounds.Left, recBounds.Bottom - cornerSize, cornerSize, cornerSize, 90, 90);
           path.AddLine(recBounds.Left, recBounds.Bottom - cornerSize, recBounds.Left, recBounds.Top);
           break;
 
+        case TabStyleType.TopTransparent:
+          if (isFirst)
+          {
+            path.AddLine(recBounds.Left, recBounds.Bottom - 2, recBounds.Left, recBounds.Top + cornerSize);
+            path.AddArc(recBounds.Left, recBounds.Top, cornerSize, cornerSize, 180, 90);
+          }
+          else
+          {
+            path.AddArc(recBounds.Left - cornerSize + 2, recBounds.Bottom - 2 - cornerSize,
+              cornerSize, cornerSize, 90, -90);
+            path.AddArc(recBounds.Left + 2, recBounds.Top, cornerSize, cornerSize, 180, 90);
+          }
+          path.AddArc(recBounds.Right - cornerSize - 2, recBounds.Top, cornerSize, cornerSize, -90, 90);
+          path.AddArc(recBounds.Right - 2, recBounds.Bottom - 2 - cornerSize,
+            cornerSize, cornerSize, 180, -90);
+          break;
       }
 
       return path;
@@ -649,6 +794,7 @@ namespace MySQL.Controls
               };
           }
           break;
+
         case TabStyleType.TopNormal:
           if (drawFocused)
           {
@@ -670,11 +816,10 @@ namespace MySQL.Controls
                 topNormalContentBackgroundColor
               };
 
-            result.Positions = new float[] {
-                0, 1
-              };
+            result.Positions = new float[] { 0, 1 };
           }
           break;
+      
         case TabStyleType.BottomNormal:
           result.Colors = new Color[] {
               TabPages[index].BackColor,
@@ -685,6 +830,24 @@ namespace MySQL.Controls
               0, 1
             };
           break;
+
+        case TabStyleType.TopTransparent:
+          {
+            int alpha = 64;
+            if (index == SelectedIndex)
+              alpha = 255;
+            result.Colors = new Color[] {
+                Color.FromArgb(alpha, 0xd6, 0xdc, 0xe5),
+                Color.FromArgb(alpha, 0xe5, 0xe9, 0xee),
+                Color.FromArgb(alpha, 0xee, 0xf2, 0xf6),
+                Color.FromArgb(alpha, 0xed, 0xf0, 0xf5)
+              };
+
+            result.Positions = new float[] {
+                0, 0.4f, 0.49f, 1
+              };
+          }
+          break;
       }
 
       return result;
@@ -807,7 +970,7 @@ namespace MySQL.Controls
         if (s == "msctls_updown32")
         {
           scroller = new SubClass(pWnd, true);
-          scroller.SubClassedWndProc += new SubClass.SubClassWndProcEventHandler(ScrollerWndProc);
+          scroller.SubClassedWndProc += new SubClass.SubClassWndProcEventHandler(ScrollerWndProc); //LLL not deleted anywhere
           break;
         }
 
@@ -1140,12 +1303,55 @@ namespace MySQL.Controls
         while (++currentIndex <= index)
         {
           TabInfo info = layoutInfo[currentIndex];
-          SizeF textSize = g.MeasureString(info.page.Text, Font);
+          int textWidth = 0;
+
+          if (info.page.Text.Length > 0)
+          {
+            // In order to determine the space needed for text we do a double measurement.
+            // The reason is that we need to render text in two different ways depend on tab style
+            // and selection state of the tab. Main reason is that certain styles are semi-transparent
+            // which does not fit at all to ClearType-rendered text.
+            SizeF measuredTextSize = g.MeasureString(info.page.Text, Font);
+            using (GraphicsPath textPath = new GraphicsPath())
+            {
+              // This path only for text on semi-transparent tabs.
+              StringFormat stringFormat = new StringFormat();
+
+              stringFormat.Alignment = StringAlignment.Near;
+              stringFormat.FormatFlags = StringFormatFlags.NoWrap;
+
+              float emSize = g.DpiY * Font.Size / 72;
+              textPath.AddString(info.page.Text, Font.FontFamily, (int)Font.Style, emSize,
+                new Point(0, 0), stringFormat);
+              RectangleF textSize = textPath.GetBounds();
+
+              // The following computation does some additional magic. Even though for string
+              // measurement and drawing (in DrawTab) we use the same calls the GraphicsPath.AddString
+              // method sees its own width computation as too small (thus adding ellipsis where not needed).
+              // By trial-and-error I found a difference of 6 pixels, which partially can be compensated
+              // by the left offset. The rest is just the remaining rest.
+              textWidth = (int)Math.Ceiling(textSize.Left + textSize.Width + 3);
+            }
+
+            // Finally use the wider of both widths.
+            if (measuredTextSize.Width > textWidth)
+              textWidth = (int)Math.Ceiling(measuredTextSize.Width);
+          }
 
           int buttonPart = 0;
           if (IsCloseButtonVisible(currentIndex))
             buttonPart = buttonSpacing + buttonSize.Width;
-          int tabWidth = itemPadding.Horizontal + (int)Math.Ceiling(textSize.Width) + buttonPart;
+          int tabWidth = itemPadding.Horizontal + textWidth + buttonPart;
+          if ((info.page.ImageIndex >= 0) && (ImageList != null) &&
+            (ImageList.Images[info.page.ImageIndex] != null))
+          {
+            tabWidth += ImageList.ImageSize.Width;
+            
+            // Add 4 px spacing if there is something after the icon.
+            if (IsCloseButtonVisible(currentIndex) || textWidth > 0)
+              tabWidth += 4;
+          }
+
           if (ItemSize.Width > tabWidth)
             tabWidth = ItemSize.Width;
           if (tabWidth > maxTabSize)
@@ -1275,6 +1481,27 @@ namespace MySQL.Controls
     // tab to make this work. Otherwise use the tab pages directly and handle search etc. yourself.
     // See also ITabDocument and TabDocument and the Documents property below.
 
+    /// <summary>
+    /// Scans the given control and all its children to find a docked ITabDocument.
+    /// Due to various docking structures the document can be anywhere in the hierarchy.
+    /// </summary>
+    /// <param name="control"></param>
+    /// <returns></returns>
+    private ITabDocument DocumentFromHierarchy(Control control)
+    {
+      if (control is ITabDocument)
+        return control as ITabDocument;
+
+      foreach (Control child in control.Controls)
+      {
+        ITabDocument result = DocumentFromHierarchy(child);
+        if (result != null)
+          return result;
+      }
+
+      return null;
+    }
+
     /// <summary>
     /// Returns the document (i.e. the first control on a tab page) whose page has the given title.
     /// </summary>
@@ -1284,10 +1511,11 @@ namespace MySQL.Controls
     {
       foreach (TabPage page in TabPages)
         if (page.Text == text)
-          if (page.Controls.Count > 0 && page.Controls[0] is ITabDocument && page.Text == text)
-            return page.Controls[0] as ITabDocument;
-          else
-            return null;
+        {
+          ITabDocument result = DocumentFromHierarchy(page);
+          if (result != null)
+            return result;
+        }
 
       return null;
     }
@@ -1300,15 +1528,24 @@ namespace MySQL.Controls
       return DocumentFromPage(TabPages[index]);
     }
 
+    /// <summary>
+    /// Returns the index for the given document or -1 if not found.
+    /// </summary>
+    public int IndexFromDocument(ITabDocument document)
+    {
+      foreach (TabPage page in TabPages)
+        if (DocumentFromHierarchy(page) == document)
+          return page.TabIndex;
+
+      return -1;
+    }
+
     /// <summary>
     /// Returns the document stored on the given tab page or null if there is none.
     /// </summary>
     public ITabDocument DocumentFromPage(TabPage page)
     {
-      if (page.Controls.Count > 0 && page.Controls[0] is ITabDocument)
-        return page.Controls[0] as ITabDocument;
-      else
-        return null;
+      return DocumentFromHierarchy(page);
     }
 
     /// <summary>
@@ -1318,7 +1555,7 @@ namespace MySQL.Controls
     public bool HasDocument(ITabDocument document)
     {
       foreach (TabPage page in TabPages)
-        if (page.Controls.Count > 0 && page.Controls[0] == document)
+        if (DocumentFromHierarchy(page) == document)
           return true;
 
       return false;
@@ -1352,7 +1589,7 @@ namespace MySQL.Controls
     public void RemoveDocument(ITabDocument document)
     {
       foreach (TabPage page in TabPages)
-        if (page.Controls.Count > 0 && page.Controls[0] == document)
+        if (DocumentFromHierarchy(page) == document)
         {
           page.Controls.Clear();
           TabPages.Remove(page);
@@ -1369,7 +1606,7 @@ namespace MySQL.Controls
     public bool CloseDocument(ITabDocument document)
     {
       foreach (TabPage page in TabPages)
-        if (page.Controls.Count > 0 && page.Controls[0] == document)
+        if (DocumentFromHierarchy(page) == document)
           return CloseTabPage(page);
 
       return false;
@@ -1400,15 +1637,15 @@ namespace MySQL.Controls
     {
       get
       {
-        if (SelectedTab != null && SelectedTab.Controls.Count > 0 && SelectedTab.Controls[0] is ITabDocument)
-          return SelectedTab.Controls[0] as ITabDocument;
+        if (SelectedTab != null)
+          return DocumentFromHierarchy(SelectedTab);
         return null;
       }
     }
 
     /// <summary>
-    /// Helper enumeration that returns the first control on each tab page, which is often the only
-    /// control forming an own "document" with many other content.
+    /// Helper enumeration that returns the control which represents an ITabDocument on each page
+    /// or null if there is none on the specific page.
     /// </summary>
     /// <returns></returns>
     public IEnumerable<ITabDocument> Documents
@@ -1416,10 +1653,7 @@ namespace MySQL.Controls
       get
       {
         foreach (TabPage page in TabPages)
-          if (page.Controls.Count > 0 && page.Controls[0] is ITabDocument)
-            yield return page.Controls[0] as ITabDocument;
-          else
-            yield return null;
+          yield return DocumentFromHierarchy(page);
       }
     }
 
@@ -1546,7 +1780,10 @@ namespace MySQL.Controls
       }
     }
 
-    [Editor(typeof(TabpageExCollectionEditor), typeof(UITypeEditor))]
+    // With the switch to .NET 4.0 client profile we have to separate runtime
+    // and design time code.
+//    [Editor(typeof(FlatTabControlCollectionEditor), typeof(UITypeEditor))]
+    [Editor("MySQL.Controls.FlatTabControlCollectionEditor", "UITypeEditor")]
     public new TabPageCollection TabPages
     {
       get
@@ -1556,22 +1793,6 @@ namespace MySQL.Controls
     }
 
     #endregion
-
-		#region TabpageExCollectionEditor
-
-		internal class TabpageExCollectionEditor : CollectionEditor
-		{
-			public TabpageExCollectionEditor(System.Type type): base(type)
-			{
-			}
-            
-			protected override Type CreateCollectionItemType()
-			{
-				return typeof(TabPage);
-			}
-		}
-        
-		#endregion
 	}
 
   //------------------------------------------------------------------------------------------------
@@ -1585,6 +1806,8 @@ namespace MySQL.Controls
     void Close();
 
     String TabText { get; set; }
+    Control Content { get; }
+    int ToolbarHeight { get; }
   }
 
   public class TabDocument : Form, ITabDocument
@@ -1663,6 +1886,26 @@ namespace MySQL.Controls
       }
     }
 
+    public virtual Control Content
+    {
+      get { return this; }
+    }
+
+    public int ToolbarHeight
+    {
+      get {
+        if (Controls.Count > 0 && Controls[0] is Panel)
+        {
+          // If this tab document has a menu and/or toolbar then it is placed on a panel
+          // in the root of this document.
+          Panel container = Controls[0] as Panel;
+          if (container.Controls.Count > 0 && (container.Controls[0] is ToolStrip))
+            return container.Height;
+        }
+        return 0;
+      }
+    }
+
     public virtual String ToolTipText
     {
       get { return toolTipText; }
