@@ -1,3 +1,22 @@
+/**
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ *  
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *  
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 using System;
 using System.Collections.Generic;
 using System.Drawing;
@@ -6,8 +25,8 @@ using System.Reflection;
 using System.Threading;
 using System.Windows.Forms;
 
-using ScintillaNet;
 using Aga.Controls.Tree;
+using ScintillaNet;
 
 using MySQL.Controls;
 using MySQL.Forms;
@@ -17,6 +36,7 @@ using MySQL.GUI.Workbench.Plugins;
 using MySQL.GUI.Workbench.Properties;
 using MySQL.Utilities;
 using MySQL.Workbench;
+using MySQL.Utilities.SysUtils;
 
 namespace MySQL.GUI.Workbench
 {
@@ -32,6 +52,7 @@ namespace MySQL.GUI.Workbench
       "images/grt/structs",
       "images/cursors",
       "images/home",
+      "images/sql",
     };
 
     // The Workbench context
@@ -44,13 +65,15 @@ namespace MySQL.GUI.Workbench
     
     // Special Forms
     private ModelOverviewForm workbenchPhysicalOverviewForm = null;
-    private WorkbenchMenuManager workbenchMenuManager = null;
+
+    // TODO: this should go once the entire menu/toolbar handling is in the backend.
     private WorkbenchToolbarManager workbenchToolbarManager = null;
 
     private Form findForm = null;
 
     private bool shuttingDown = false;
-    
+    private ImageList tabImageList = null;
+
     #endregion
 
     #region Constructors
@@ -73,8 +96,13 @@ namespace MySQL.GUI.Workbench
       wbContext = WbContext;
       grtManager = wbContext.get_grt_manager();
 
-      workbenchMenuManager = new WorkbenchMenuManager(wbContext);
-      workbenchToolbarManager = new WorkbenchToolbarManager(wbContext, toolBarStrip);
+      workbenchToolbarManager = new WorkbenchToolbarManager(wbContext);
+
+      tabImageList = new ImageList();
+      tabImageList.ColorDepth = ColorDepth.Depth32Bit;
+      tabImageList.ImageSize = new Size(16, 16);
+      ImageListHelper.Add(ApplicationCommand(AppCommand.AppGetResourcePath, null, "WB_Home.png"), tabImageList);
+      contentTabControl.ImageList = tabImageList;
 
 
       // Create a timer to be triggered when the backend needs
@@ -109,8 +137,6 @@ namespace MySQL.GUI.Workbench
       host = new ToolStripControlHost(statusStripImg);
       host.Alignment = ToolStripItemAlignment.Right;
       mainStatusStrip.Items.Add(host);
-
-      toolBarStrip.Renderer = new FlatMainToolStripRenderer();
     }
 
     #endregion
@@ -284,7 +310,11 @@ namespace MySQL.GUI.Workbench
           UndockDocument(view.GetHost());
           break;
         case AppCommand.AppSetViewTitle:
-          view.GetHost().TabText = str;
+          {
+            ITabDocument document = view.GetHost();
+            if (contentTabControl.IndexFromDocument(document) > 0)
+              view.GetHost().TabText = str;
+          }
           break;
         case AppCommand.AppQuit: // Not yet used.
           Close();
@@ -401,17 +431,6 @@ namespace MySQL.GUI.Workbench
           ForwardRefreshToActivDocument(refresh, str, ptr);
           break;
 
-        case RefreshType.RefreshToolbar:
-          if (str == "main" || str == "")
-            workbenchToolbarManager.UpdateMainToolbar();
-          if (str == "tools" || str == "")
-            RefreshToolsToolbar();
-          break;
-
-        case RefreshType.RefreshMenubar:
-          workbenchMenuManager.ConstructMainMenu(mainMenuStrip);
-          break;
-
         case RefreshType.RefreshTimer:
           UpdateTimer();
           break;
@@ -474,12 +493,9 @@ namespace MySQL.GUI.Workbench
 
     public void ToolChanged(BaseWindowsCanvasView canvasView)
     {
-      workbenchToolbarManager.UpdateMainToolbar();
-      RefreshToolsToolbar();
-
       ModelDiagramForm modelDiagramForm = canvasView.GetOwnerForm() as ModelDiagramForm;
       if (modelDiagramForm != null)
-        modelDiagramForm.UpdateCursor();
+        modelDiagramForm.OnToolChanged();
     }
 
     public IntPtr OpenPlugin(GrtManager GrtManager, GrtModule GrtModule, string AssemblyName,
@@ -597,8 +613,7 @@ namespace MySQL.GUI.Workbench
         {
           case "dbquery":
             DbSqlEditor db_sql_editor = new DbSqlEditor(wbContext, viewBE);
-            form = db_sql_editor;
-            DockDocument(form, true, true);
+            DockDocument(db_sql_editor, true, true);
             break;
         }
         if (null != form)
@@ -649,42 +664,37 @@ namespace MySQL.GUI.Workbench
       }
     }
 
+    #region Edit Menu Handlers
 
-    private void RefreshToolsToolbar()
-    {
-      ModelDiagramForm model= contentTabControl.ActiveDocument as ModelDiagramForm;
-
-      if (model != null)
-        workbenchToolbarManager.UpdateToolsToolbar(model.ToolStrip);
-    }
-
-    public void PopupCanvasMenu(List<MySQL.Grt.MenuItem> items, int x, int y)
-    {
-      ModelDiagramForm model= contentTabControl.ActiveDocument as ModelDiagramForm;
-
-      if (model != null)
-        workbenchMenuManager.ShowContextMenu(model.CanvasViewer, items, x, y);
-    }
-
-#region Edit Menu Handlers
+    // Note: the EditCan* functions are called to determine the enabled state of the edit menu
+    //       items. However this is always interpreted as edit actions for WB items.
+    //       Hence, if a normal edit control is focused currently we have to return false.
+    //       This will make the menu entries disabled unfortunately, but allows us to do clipboard
+    //       operations in the edit controls.
+    //       We, nonetheless, handle the actions triggered by the shortcuts (which are processed
+    //       by WB).
+    // TODO: edit menu update must be reworked to also take normal platform specific actions into account.
     private void EditUndo()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
       if (activeControl is TextBoxBase)
         (activeControl as TextBoxBase).Undo();
       else
-        if (activeControl is Scintilla)
-          (activeControl as Scintilla).UndoRedo.Undo();
+        if (activeControl is ComboBox)
+          Win32Api.Undo(activeControl as ComboBox);
         else
-          if (wbContext.edit_can_undo())
-            wbContext.edit_undo();
+          if (activeControl is Scintilla)
+            (activeControl as Scintilla).UndoRedo.Undo();
+          else
+            if (wbContext.edit_can_undo())
+              wbContext.edit_undo();
     }
 
     private bool EditCanUndo()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is TextBoxBase)
-        return (activeControl as TextBoxBase).CanUndo;
+      if (activeControl is TextBoxBase || activeControl is ComboBox)
+        return false;
       else
         if (activeControl is Scintilla)
           return (activeControl as Scintilla).UndoRedo.CanUndo;
@@ -695,17 +705,20 @@ namespace MySQL.GUI.Workbench
     private void EditRedo()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is Scintilla)
-        (activeControl as Scintilla).UndoRedo.Redo();
+      if (activeControl is TextBoxBase || activeControl is ComboBox)
+        return;
       else
-        if (wbContext.edit_can_redo())
-          wbContext.edit_redo();
+        if (activeControl is Scintilla)
+          (activeControl as Scintilla).UndoRedo.Redo();
+        else
+          if (wbContext.edit_can_redo())
+            wbContext.edit_redo();
     }
 
     private bool EditCanRedo()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is TextBoxBase)
+      if (activeControl is TextBoxBase || activeControl is ComboBox)
         return false;
       else
         if (activeControl is Scintilla)
@@ -721,26 +734,26 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase)
         (activeControl as TextBoxBase).Copy();
       else
-        if (activeControl is Scintilla)
-          (activeControl as Scintilla).Clipboard.Copy();
+        if (activeControl is ComboBox)
+          Win32Api.Copy(activeControl as ComboBox);
         else
-          if (wbContext.edit_can_copy())
-            wbContext.edit_copy();
+          if (activeControl is Scintilla)
+            (activeControl as Scintilla).Clipboard.Copy();
+          else
+            if (wbContext.edit_can_copy())
+              wbContext.edit_copy();
     }
 
     private bool EditCanCopy()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is TextBoxBase)
-        return (activeControl as TextBoxBase).SelectionLength > 0;
+      if (activeControl is TextBoxBase || activeControl is ComboBox)
+        return false;
       else
         if (activeControl is Scintilla)
           return (activeControl as Scintilla).Selection.Length > 0;
         else
-          if ((activeControl is TreeViewAdv) || (activeControl != null && activeControl.Parent is TreeViewAdv))
-            return false; // Returning false tells the backend not to take over the operation.
-          else
-            return wbContext.edit_can_copy();
+          return wbContext.edit_can_copy();
     }
 
     private void EditCut()
@@ -749,18 +762,21 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase)
         (activeControl as TextBoxBase).Cut();
       else
-        if (activeControl is Scintilla)
-          (activeControl as Scintilla).Clipboard.Cut();
+        if (activeControl is ComboBox)
+          Win32Api.Cut(activeControl as ComboBox);
         else
-          if (wbContext.edit_can_copy())
-            wbContext.edit_cut();
+          if (activeControl is Scintilla)
+              (activeControl as Scintilla).Clipboard.Cut();
+            else
+              if (wbContext.edit_can_copy())
+                wbContext.edit_cut();
     }
 
     private bool EditCanCut()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is TextBoxBase)
-        return (activeControl as TextBoxBase).SelectionLength > 0;
+      if (activeControl is TextBoxBase || activeControl is ComboBox)
+        return false;
       else
         if (activeControl is Scintilla)
           return (activeControl as Scintilla).Selection.Length > 0;
@@ -777,18 +793,21 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase)
         (activeControl as TextBoxBase).Paste();
       else
-        if (activeControl is Scintilla)
-          (activeControl as Scintilla).Clipboard.Paste();
+        if (activeControl is ComboBox)
+          Win32Api.Paste(activeControl as ComboBox);
         else
-          if (wbContext.edit_can_paste())
-            wbContext.edit_paste();
+          if (activeControl is Scintilla)
+            (activeControl as Scintilla).Clipboard.Paste();
+          else
+            if (wbContext.edit_can_paste())
+              wbContext.edit_paste();
     }
 
     private bool EditCanPaste()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is TextBoxBase)
-        return !(activeControl as TextBoxBase).ReadOnly;
+      if (activeControl is TextBoxBase || activeControl is ComboBox)
+        return false;
       else
         if (activeControl is Scintilla)
           return (activeControl as Scintilla).Clipboard.CanPaste;
@@ -812,8 +831,8 @@ namespace MySQL.GUI.Workbench
     private bool EditCanSelectAll()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is TextBoxBase)
-        return true;
+      if (activeControl is TextBoxBase || activeControl is ComboBox)
+        return false;
       else
         if (activeControl is Scintilla)
           return true;
@@ -837,8 +856,8 @@ namespace MySQL.GUI.Workbench
     private bool EditCanDelete()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is TextBoxBase)
-        return (activeControl as TextBoxBase).SelectionLength > 0;
+      if (activeControl is TextBoxBase || activeControl is ComboBox)
+        return false;
       else
         if (activeControl is Scintilla)
           return (activeControl as Scintilla).Selection.Length > 0;
@@ -848,13 +867,13 @@ namespace MySQL.GUI.Workbench
 
     private void EditFind()
     {
-      if (workbenchToolbarManager.searchBox != null)
-        workbenchToolbarManager.searchBox.Focus();
+      // TODO: Remove once the simple search code moved to the individual pages.
     }
 
     private bool EditCanFind()
     {
-      return (workbenchToolbarManager.searchBox != null);
+      //return (workbenchToolbarManager.searchBox != null);
+      return false; // TODO: The simple search code should now go to the individual pages.
     }
 
     private void EditFindReplace()
@@ -878,6 +897,8 @@ namespace MySQL.GUI.Workbench
 
     private void SearchActiveTab()
     {
+      // TODO: remove this once the code moved to the individual pages.
+      /*
       String text= workbenchToolbarManager.lastSearchString;
       if (text != "")
       {
@@ -891,6 +912,7 @@ namespace MySQL.GUI.Workbench
               (contentTabControl.ActiveDocument as DbSqlEditor).SearchText(text);
         }
       }
+      */
     }
 #endregion
 
@@ -1122,10 +1144,6 @@ namespace MySQL.GUI.Workbench
     public void SetupUI()
     {
       RegisterCommands();
-
-      workbenchMenuManager.ConstructMainMenu(mainMenuStrip);
-      workbenchToolbarManager.UpdateMainToolbar();
-      RefreshToolsToolbar();
     }
 
     public void LockGUI(bool value)
@@ -1176,6 +1194,10 @@ namespace MySQL.GUI.Workbench
       // Reset status bar to clean up any previously done action message.
       StatusBarText = "Ready";
 
+      // For the glass effect behind the tabs (and the proper content drawing) we need the
+      // height of the tabs as well as menu and toolbar of the active tab.
+      int topAreaHeight = 0;
+
       // Suspend dock layout to prevent flicker.
       SuspendLayout();
       contentTabControl.SuspendLayout();
@@ -1185,6 +1207,7 @@ namespace MySQL.GUI.Workbench
         if (tabControl.ActiveDocument != null)
         {
           ITabDocument activeDocument = tabControl.ActiveDocument;
+          topAreaHeight += activeDocument.ToolbarHeight;
 
           if (activeDocument is IWorkbenchDocument)
           {
@@ -1231,6 +1254,9 @@ namespace MySQL.GUI.Workbench
         contentTabControl.ResumeLayout(true);
         ResumeLayout();
       }
+
+      // Different pages might have a different height of the menu/toolbar area.
+      AdjustGlassFrame(topAreaHeight);
     }
 
     /// <summary>
@@ -1288,19 +1314,42 @@ namespace MySQL.GUI.Workbench
     {
       if (main)
       {
-         if (!contentTabControl.HasDocument(document))
-          contentTabControl.AddDocument(document);
+        int index = -1;
+        if (!contentTabControl.HasDocument(document))
+        {
+          IWorkbenchDocument wbDocument = document as IWorkbenchDocument;
+          SetupDockLayout(document, (wbDocument != null) ? wbDocument.BackendClass : null);
+          index = contentTabControl.AddDocument(document);
+
+          InstallFocusChangeHandlers(document as Control);
+        }
+
+        if (index == 0) // The main tab.
+        {
+          contentTabControl.TabPages[index].ImageIndex = 0;
+          contentTabControl.SetCloseButtonVisibility(0, FlatTabControl.CloseButtonVisiblity.HideButton);
+        }
+        
         if (activate)
+        {
           document.Activate();
+          AdjustGlassFrame(document.ToolbarHeight);
+        }
       }
       else
       {
         ITabDocument host = contentTabControl.ActiveDocument;
         if (host is ModelDiagramForm)
-          (host as ModelDiagramForm).DockDocument(document, activate);
+        {
+          ModelDiagramForm form = host as ModelDiagramForm;
+          form.DockDocument(document, activate);
+        }
         else
           if (host is ModelOverviewForm)
-            (host as ModelOverviewForm).DockDocument(document, activate);
+          {
+            ModelOverviewForm form = host as ModelOverviewForm;
+            form.DockDocument(document, activate);
+          }
       }
     }
 
@@ -1442,7 +1491,7 @@ namespace MySQL.GUI.Workbench
       wbContext.perform_quit();
     }
 
-    private void loadFormState()
+    private void LoadFormState()
     {
       int x = wbContext.read_state("left", "mainform", 100);
       int y = wbContext.read_state("top", "mainform", 100);
@@ -1514,7 +1563,8 @@ namespace MySQL.GUI.Workbench
       Application.ThreadException += // Windows Forms
          new System.Threading.ThreadExceptionEventHandler(OnGuiUnhandledException);
 
-      loadFormState();
+      LoadFormState();
+      InstallFocusChangeHandlers(this);
 
       SplashScreen.CloseSplash();
 
@@ -1552,7 +1602,7 @@ namespace MySQL.GUI.Workbench
     {
       // force a refresh of the menu so that Edit -> Copy/Paste/Cut/Select All can get updated accordingly
       if (!shuttingDown)
-        workbenchMenuManager.RefreshMainEditMenu(mainMenuStrip);
+        wbContext.validate_edit_menu();
     }
 
     private EventHandler FocusChangeHandler= null;
@@ -1590,6 +1640,7 @@ namespace MySQL.GUI.Workbench
       e.canClose = true;
 
       ITabDocument document = (sender as FlatTabControl).DocumentFromPage(e.page);
+      RemoveFocusChangeHandlers(document as Control);
       if (document is DbSqlEditor)
         e.canClose = (document as DbSqlEditor).Backend.can_close();
       if (document is Plugins.DockablePlugin)
@@ -1643,5 +1694,109 @@ namespace MySQL.GUI.Workbench
       Activate();
     }
 
+    /// <summary>
+    /// Make the upper part of the window extend the glass frame (if we are on Aero).
+    /// </summary>
+    /// <param name="toolbarHeight"></param>
+    private void AdjustGlassFrame(int toolbarHeight)
+    {
+      if (ControlUtilities.IsCompositionEnabled())
+      {
+        int topAreaHeight = contentTabControl.ItemSize.Height + contentTabControl.Margin.Top +
+          toolbarHeight + Margin.Top;
+
+        Win32.MARGINS margins = new Win32.MARGINS(2, topAreaHeight, 2, 0);
+        Win32.DwmExtendFrameIntoClientArea(Handle, margins);
+      }
+    }
+
+    /// <summary>
+    /// Prepares the given document to conform to the overall layout. This requires to draw an
+    /// intermediate set of controls (panels) and add menu + toolbar.
+    /// </summary>
+    /// <param name="document">The document to adjust, which comes with the actual content.</param>
+    /// <param name="backend">The backend UI form used to get toolbar and menu.</param>
+    private void SetupDockLayout(ITabDocument document, UIForm backend)
+    {
+      // Setting up the layout requires some additional nesting of controls.
+      // A tab document should have only one container as top level control, which we re-host.
+      Control root = document.Content;
+      Control content = (root.Controls.Count > 0) ? root.Controls[0] : null;
+
+      root.Controls.Clear();
+
+      // Content area.
+      DrawablePanel contentPanel = new DrawablePanel();
+      contentPanel.BackColor = Color.FromArgb(255, 40, 55, 82);
+      contentPanel.CustomBackground = false;
+      contentPanel.Dock = DockStyle.Fill;
+      contentPanel.Padding = new Padding(7);
+      root.Controls.Add(contentPanel);
+      if (content != null)
+      {
+        contentPanel.Controls.Add(content);
+        content.Dock = DockStyle.Fill;
+      }
+
+      // Menu + toolbar area. We have two types of tab documents:
+      // - appview based (and hence backend-managed)
+      // - native Form (with interface via WBContext).
+      // Appview based documents have own methods to return their menu and toolbar because
+      // retrieval via managed code requires a managed UIForm. However appview classes are
+      // already managed via mforms and adding another reference management via UIForm
+      // will lead to unpredictable results.
+      ToolStrip toolbar;
+      MenuStrip menu;
+      if (backend != null)
+      {
+        // Retrieval via WBContext will return a default toolbar if no other is defined.
+        toolbar = wbContext.toolbar_for_form(backend);
+        menu = wbContext.menu_for_form(backend);
+      }
+      else
+      {
+        AppViewDockContent dockContent = document as AppViewDockContent;
+        toolbar = dockContent.GetAppView().GetToolBar();
+        if (toolbar == null)
+          toolbar = wbContext.toolbar_for_form(null);
+        menu = dockContent.GetAppView().GetMenuBar();
+        if (menu == null)
+          menu = wbContext.menu_for_form(null);
+      }
+      DrawablePanel menuPanel = new DrawablePanel();
+      if (toolbar != null || menu != null)
+      {
+        // For menu + toolbar we have a special gradient background, which we implement by
+        // placing them on another panel.
+        menuPanel.BackColor = Color.Transparent;
+        menuPanel.CustomBackground = true;
+        menuPanel.Dock = DockStyle.Top;
+        menuPanel.Padding = new Padding(5, 0, 5, 0);
+        menuPanel.AutoSize = true;
+        menuPanel.PaintBackground += new EventHandler<PaintEventArgs>(menuPanel_PaintBackground);
+
+        root.Controls.Add(menuPanel);
+        if (toolbar != null)
+        {
+          menuPanel.Controls.Add(toolbar);
+          toolbar.Dock = DockStyle.Top;
+          toolbar.AutoSize = true;
+          toolbar.Padding = new Padding(0, 0, 0, 3);
+        }
+
+        if (menu != null)
+        {
+          menuPanel.Controls.Add(menu);
+          menu.Dock = DockStyle.Top;
+          menu.AutoSize = true;
+        }
+      }
+    }
+    
+    void menuPanel_PaintBackground(object sender, PaintEventArgs e)
+    {
+      ToolStripHelper.MenuPanelPaint(e.Graphics, (sender as Control).ClientRectangle);
+    }
+
   }
 }
