@@ -28,6 +28,7 @@ using MySQL.Controls;
 using MySQL.Grt;
 using MySQL.GUI.Mdc;
 using MySQL.GUI.Workbench.Plugins;
+using MySQL.Forms;
 using MySQL.Workbench;
 using MySQL.Utilities.SysUtils;
 
@@ -51,6 +52,8 @@ namespace MySQL.GUI.Workbench
     private UndoHistoryForm historyForm;
 
     private bool splitterMovePending = false;
+    private ToolStrip optionsToolStrip = null;
+    private ToolStrip toolsToolStrip = null;
 
     #endregion
 
@@ -85,6 +88,20 @@ namespace MySQL.GUI.Workbench
       modelObjectDescriptionForm = new ModelObjectDescriptionForm(wbContext);
 
       SetupSideBar();
+
+      toolsToolStrip = formBE.get_tools_toolbar();
+      toolsToolStrip.Dock = DockStyle.Left;
+      diagramPanel.Controls.Add(toolsToolStrip);
+
+      optionsToolStrip = formBE.get_options_toolbar();
+      optionsToolStrip.Padding = new Padding(2);
+      optionsToolStrip.Dock = DockStyle.None;
+      optionsToolStrip.AutoSize = false;
+      diagramPanel.Controls.Add(optionsToolStrip);
+      diagramPanel.Controls.SetChildIndex(optionsToolStrip, 0);
+      optionsToolStrip.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right;
+      optionsToolStrip.Hide();
+
       FocusCanvasControl();
     }
 
@@ -214,17 +231,7 @@ namespace MySQL.GUI.Workbench
       get { return (BaseWindowsCanvasView)openGlCanvasViewer.Canvas; }
     }
 
-    public MySQL.Utilities.WindowsCanvasViewer CanvasViewer
-    {
-      get { return openGlCanvasViewer; }
-    }
-
-    public System.Windows.Forms.ToolStrip ToolStrip
-    {
-      get { return workToolStrip; }
-    }
-
-    public double zoom
+    public double Zoom
     {
       get { return formBE.get_zoom(); }
       set { formBE.set_zoom(value); }
@@ -309,7 +316,7 @@ namespace MySQL.GUI.Workbench
     {
       // Explicitly close all editors to ensure plugins are unregistered from the backend.
       foreach (ITabDocument content in bottomTabControl.Documents)
-        content.Close();
+        CloseTabDocument(content);
 
       openGlCanvasViewer.FinalizeCanvas();
       
@@ -376,6 +383,11 @@ namespace MySQL.GUI.Workbench
         contentSplitContainer.Panel2Collapsed = true;
 
       ITabDocument document = (sender as FlatTabControl).DocumentFromPage(e.page);
+      CloseTabDocument(document);
+    }
+
+    private void CloseTabDocument(ITabDocument document)
+    {
       if (document is Plugins.DockablePlugin)
       {
         Plugins.DockablePlugin plugin = document as Plugins.DockablePlugin;
@@ -392,7 +404,6 @@ namespace MySQL.GUI.Workbench
             // TODO: adjust code to support OnClosing/OnClose duality also in AppView.
           }
     }
-
     private void ModelDiagramForm_Shown(object sender, EventArgs e)
     {
       LoadFormState();
@@ -402,7 +413,7 @@ namespace MySQL.GUI.Workbench
 
     #region Other Implementation
 
-    public void SetupSideBar()
+    private void SetupSideBar()
     {
       mainSplitContainer.SuspendLayout();
       sideSplitContainer.SuspendLayout();
@@ -533,8 +544,12 @@ namespace MySQL.GUI.Workbench
       formBE = null;
     }
 
-    public void UpdateCursor()
+    public void OnToolChanged()
     {
+      formBE.update_options_toolbar();
+      optionsToolStrip.Bounds = new Rectangle(toolsToolStrip.Width, diagramPanel.Padding.Top,
+        openGlCanvasViewer.ClientSize.Width - toolsToolStrip.Width, 25);
+      optionsToolStrip.Visible = optionsToolStrip.Items.Count > 0;
       SetCursor(formBE.get_tool_cursor());
     }
 
