@@ -29,6 +29,7 @@ using MySQL.GUI.Workbench.Plugins;
 using MySQL.GUI.Workbench.Properties;
 using MySQL.Utilities;
 using MySQL.Utilities.SysUtils;
+using MySQL.Forms;
 using MySQL.Workbench;
 using System.IO;
 
@@ -911,8 +912,12 @@ namespace MySQL.GUI.Workbench
       sectionListview.BorderStyle = BorderStyle.None;
       sectionListview.AllowColumnReorder = true;
       sectionListview.ShowItemToolTips = true;
+
+      // Undocumented feature: enabling AutoSize will indeed make the control automatically resize
+      // (the docs say it wouldn't) though without considering long captions. For them to work 
+      // additionally Scrollable can be set to true to show a scrollbar, but only in this special case.
       sectionListview.AutoSize = true;
-      sectionListview.Scrollable = false;
+      sectionListview.Scrollable = true;
       sectionListview.Visible = true;
       sectionListview.Font = overviewPanel.Font;
       sectionListview.Tag = CreateIdentifier(controlNodeId);
@@ -1594,7 +1599,7 @@ namespace MySQL.GUI.Workbench
 
     #region Other Implementation
 
-    public void SetupSideBar()
+    private void SetupSideBar()
     {
       mainSplitContainer.SuspendLayout();
       sideSplitContainer.SuspendLayout();
