@@ -1,30 +1,49 @@
+/**
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ *  
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *  
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 using System;
-using System.Collections.Generic;
-using System.Windows.Forms;
-using System.Runtime.InteropServices;
 using System.Reflection;
+using System.Threading;
+using System.Windows.Forms;
 
 using MySQL.Grt;
-using MySQL.Workbench;
 using MySQL.Utilities;
-using MySQL.Forms;
-using System.Threading;
 using MySQL.Utilities.SysUtils;
+using MySQL.Workbench;
+using MySQL.Forms;
+
+using SingleInstanceApplication;
 
 namespace MySQL.GUI.Workbench
 {
-	static class Program
+  static class Program
   {
     #region Static Variables and Enums
 
     // The types of application metadata information
     public enum ApplicationMetaInfo { Company, Copyright, Version, Revision, Configuration };
 
-		// The Workbench Context
-		private static WbContext wbContext = null;
+    // The Workbench Context
+    private static WbContext wbContext = null;
 
-		// The GRT Manager
-		private static GrtManager grtManager = null;
+    // The GRT Manager
+    private static GrtManager grtManager = null;
 
     // Timer to guarantee idle handler is called periodically when nothing happens
     private static System.Windows.Forms.Timer timer = null;
@@ -42,12 +61,17 @@ namespace MySQL.GUI.Workbench
     #endregion
     
     /// <summary>
-		/// The main entry point for the application.
-		/// </summary>
+    /// The main entry point for the application.
+    /// </summary>
     
     [STAThread]
-	  static void Main(string[] Args)
+    static void Main(string[] Args)
     {
+      // First check if this is the first instance of Workbench.
+      if (!ApplicationInstanceManager.CreateSingleInstance(
+        Assembly.GetExecutingAssembly().GetName().Name,  Args, SingleInstanceCallback))
+        return;
+
       // Give the main thread a proper name, so we can later check for it when needed.
       Thread.CurrentThread.Name = "mainthread";
 
@@ -73,6 +97,10 @@ namespace MySQL.GUI.Workbench
       }
       Environment.SetEnvironmentVariable("PATH", systemdir + syspath);
 
+      // Clear PYTHONPATH environment variable, as we do not need it but our python impl
+      // seriously gets confused with it.
+      Environment.SetEnvironmentVariable("PYTHONPATH", "");
+
       // Initialize forms stuff.
       MySQL.Forms.Manager formsManager = new MySQL.Forms.Manager();
 
@@ -173,19 +201,19 @@ namespace MySQL.GUI.Workbench
       #region Initialize GRT
 
       // Try to instantiate the Workbench context and the GRT Manager and catch exceptions
-		  try
-		  {
-			  // Create Workbench Context
-			  wbContext = new WbContext((CommandLine["verbose"] != null || CommandLine["v"] != null));
+      try
+      {
+        // Create Workbench Context
+        wbContext = new WbContext((CommandLine["verbose"] != null || CommandLine["v"] != null));
 
-			  if (wbContext != null)
-			  {
-				  // Create the GRT Manager instance
-				  grtManager = wbContext.get_grt_manager();
-			  }
-		  }
-		  catch (Exception ex)
-		  {
+        if (wbContext != null)
+        {
+          // Create the GRT Manager instance
+          grtManager = wbContext.get_grt_manager();
+        }
+      }
+      catch (Exception ex)
+      {
         MainForm.HandleException(ex);
       }
 
@@ -194,33 +222,30 @@ namespace MySQL.GUI.Workbench
       #region Initialize Callbacks and Mainform
 
       // If the Workbench Context and GRT Manager were successfully created, 
-		  // initialize the application
-		  if (wbContext != null && grtManager != null)
-		  {
-			  mainForm = new MainForm(wbContext);
+      // initialize the application
+      if (wbContext != null && grtManager != null)
+      {
+        mainForm = new MainForm(wbContext);
 
-			  // Initialize the Workbench context
-			  WbFrontendCallbacks wbFrontendCallback = new WbFrontendCallbacks(
+        // Initialize the Workbench context
+        AppImpl formsApplication = new AppImpl(
           formsManager,
-          //new WbFrontendCallbacks.IntStrStrStrStrStrDelegate(ShowConfirmDialog),
-          new WbFrontendCallbacks.StrStrStrStrDelegate(mainForm.ShowFileDialog),
-				  new WbFrontendCallbacks.VoidStrDelegate(mainForm.ShowStatusText),
-				  new WbFrontendCallbacks.BoolStrStrFloatDelegate(mainForm.ShowProgress),
-          new WbFrontendCallbacks.BoolStrIntStrPtrDelegate(mainForm.RequestInput),
-				  new WbFrontendCallbacks.CanvasViewStringStringDelegate(mainForm.CreateNewDiagram),
-				  new WbFrontendCallbacks.VoidCanvasViewDelegate(mainForm.DestroyView),
-				  new WbFrontendCallbacks.VoidCanvasViewDelegate(mainForm.SwitchedView),
-				  new WbFrontendCallbacks.VoidCanvasViewDelegate(mainForm.ToolChanged),
-          new WbFrontendCallbacks.VoidStrUIFormDelegate(mainForm.CreateMainFormView),
-          new WbFrontendCallbacks.IntPtrGRTManagerModuleStrStrGrtListFlagsDelegate(mainForm.OpenPlugin),
-          new WbFrontendCallbacks.VoidIntPtrDelegate(mainForm.ShowPlugin),
-          new WbFrontendCallbacks.VoidIntPtrDelegate(mainForm.HidePlugin),
-				  new WbFrontendCallbacks.VoidRefreshTypeStringIntPtrDelegate(mainForm.RefreshGUI),
-          new WbFrontendCallbacks.VoidBoolDelegate(mainForm.LockGUI),
-          new WbFrontendCallbacks.VoidMenuItemListIntIntDelegate(mainForm.PopupCanvasMenu),
-          new WbFrontendCallbacks.VoidStrDelegate(mainForm.PerformCommand),
-          new WbFrontendCallbacks.VoidDelegate(mainForm.QuitApplication),
-          new WbFrontendCallbacks.StringAppCommandAppViewStringDelegate(mainForm.ApplicationCommand));
+          new AppImpl.StrStrStrStrDelegate(mainForm.ShowFileDialog),
+          new AppImpl.VoidStrDelegate(mainForm.ShowStatusText),
+          new AppImpl.BoolStrStrFloatDelegate(mainForm.ShowProgress),
+          new AppImpl.BoolStrIntStrPtrDelegate(mainForm.RequestInput),
+          new AppImpl.CanvasViewStringStringIntPtrDelegate(mainForm.CreateNewDiagram),
+          new AppImpl.VoidCanvasViewDelegate(mainForm.DestroyView),
+          new AppImpl.VoidCanvasViewDelegate(mainForm.SwitchedView),
+          new AppImpl.VoidCanvasViewDelegate(mainForm.ToolChanged),
+          new AppImpl.IntPtrGRTManagerModuleStrStrGrtListFlagsDelegate(mainForm.OpenPlugin),
+          new AppImpl.VoidIntPtrDelegate(mainForm.ShowPlugin),
+          new AppImpl.VoidIntPtrDelegate(mainForm.HidePlugin),
+          new AppImpl.VoidRefreshTypeStringIntPtrDelegate(mainForm.RefreshGUI),
+          new AppImpl.VoidBoolDelegate(mainForm.LockGUI),
+          new AppImpl.VoidStrDelegate(mainForm.PerformCommand),
+          new AppImpl.VoidDelegate(mainForm.QuitApplication),
+          new AppImpl.StringAppCommandAppViewStringDelegate(mainForm.ApplicationCommand));
 
 
         /// mainForm.PrintOutputText
@@ -285,11 +310,11 @@ namespace MySQL.GUI.Workbench
         WbOptions wbOptions = new WbOptions(baseDir, // basedir
           baseDir, // plugin
           "",       // struct
-				  baseDir + "/modules", // modules
-				  baseDir, // library
-				  System.IO.Path.Combine(System.IO.Path.Combine(
-						  Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),
-						  "MySQL"), "Workbench"),
+          baseDir + "/modules", // modules
+          baseDir, // library
+          System.IO.Path.Combine(System.IO.Path.Combine(
+              Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),
+              "MySQL"), "Workbench"),
           openAtStartup,
           openAtStartupType,
           runScript, runScriptLanguage,
@@ -298,10 +323,12 @@ namespace MySQL.GUI.Workbench
           (CommandLine["log"] != null),
           (CommandLine["quit-when-done"] != null || CommandLine["quitwhendone"] != null));
 
-			  wbContext.init(wbFrontendCallback, wbOptions);
+        wbContext.init(formsApplication, wbOptions,
+          new WbContext.VoidStrUIFormDelegate(mainForm.CreateMainFormView)
+        );
 
-			  // Set the Application.Idle event handler
-			  Application.Idle += new EventHandler(OnApplicationIdle);
+        // Set the Application.Idle event handler
+        Application.Idle += new EventHandler(OnApplicationIdle);
 
         // Don't call the idle handler too often.
         timer = new System.Windows.Forms.Timer();
@@ -309,27 +336,30 @@ namespace MySQL.GUI.Workbench
         timer.Tick += new EventHandler(timer_Tick);
         timer.Start();
 
-			  // Trigger GRT idle tasks
-			  grtManager.perform_idle_tasks();
+        // Trigger GRT idle tasks
+        grtManager.perform_idle_tasks();
         
         // Setup Menus
         mainForm.SetupUI();
         mainForm.Show();
 
-			  // Tell the backend our main UI is ready. This will also load a model if it was given via command line
+        // Tell the backend our main UI is ready. This will also load a model if it was given via command line
         // and opens the overview form for it.
         wbContext.finished_loading(wbOptions);
 
-        // Start the Application
-        try
+        // Start the Application if we are not already shutting down.
+        if (!wbContext.is_quitting())
         {
-          Application.Run(new MySQLApplicationContext(mainForm));
+          try
+          {
+            Application.Run(new MySQLApplicationContext(mainForm));
+          }
+          catch (Exception e)
+          {
+            MainForm.HandleException(e);
+          }
         }
-        catch (Exception e)
-        {
-          MainForm.HandleException(e);
-        }
-  			
+        
         // shutdown wb context
         if (wbContext != null)
         {
@@ -338,6 +368,7 @@ namespace MySQL.GUI.Workbench
 
           wbContext.finalize();
         }
+        formsApplication.Dispose();
       }
       
       #endregion
@@ -347,7 +378,7 @@ namespace MySQL.GUI.Workbench
      
     //----------------------------------------------------------------------------------------------
 
-    #region Application Idle Handler
+    #region Application handlers
 
     static void timer_Tick(object sender, EventArgs e)
     {
@@ -372,7 +403,7 @@ namespace MySQL.GUI.Workbench
     }
 
     private static void OnApplicationIdle(object sender, EventArgs e)
-		{
+    {
       if (gcRequested)
       {
         gcRequested = false;
@@ -380,6 +411,77 @@ namespace MySQL.GUI.Workbench
       }
     }
 
+    /// <summary>
+    /// Handler that gets called if this another instance of the application was started. It passes
+    /// the command line arguments from the other instance on so we can act accordingly here.
+    /// </summary>
+    /// <param name="sender">The sender.</param>
+    /// <param name="args">The instance containing the event data.</param>
+    private static void SingleInstanceCallback(object sender, InstanceCallbackEventArgs args)
+    {
+      if (args == null || mainForm == null)
+        return;
+
+      Action<bool> d = (bool x) =>
+      {
+        mainForm.Activate(x);
+
+        // Parse command line and extract the model file name if there is one.
+        Arguments CommandLine = new Arguments();
+
+        CommandLine.AddOptionName("-verbose", true);
+        CommandLine.AddOptionName("-v", true);
+        CommandLine.AddOptionName("-help", true);
+        CommandLine.AddOptionName("-h", true);
+        CommandLine.AddOptionName("-open", false);
+        CommandLine.AddOptionName("-nologo", true);
+        CommandLine.AddOptionName("-version", true);
+        CommandLine.AddOptionName("-grtversion", true);
+        CommandLine.AddOptionName("-swrendering", false);
+        CommandLine.AddOptionName("-log", true);
+        CommandLine.AddOptionName("-query", false);
+        CommandLine.AddOptionName("-model", false);
+        CommandLine.AddOptionName("-admin", false);
+        CommandLine.AddOptionName("-script", false);
+        CommandLine.AddOptionName("-run", false);
+        CommandLine.AddOptionName("-run-python", false);
+        CommandLine.AddOptionName("-run-script", false);
+        CommandLine.AddOptionName("-quit-when-done", true);
+
+        CommandLine.Parse(args.CommandLineArgs);
+
+        string fileType = "";
+        string fileToOpen = null;
+
+        if (CommandLine["open"] != null)
+        {
+          fileToOpen = CommandLine["open"];
+          fileType = "";
+        }
+        else if (CommandLine["model"] != null)
+        {
+          fileToOpen = CommandLine["model"];
+          fileType = "model";
+        }
+
+        if (fileToOpen == null)
+        {
+          fileToOpen = "";
+          if (CommandLine.Leftovers.Count > 0)
+            fileToOpen = CommandLine.Leftovers[CommandLine.Leftovers.Count - 1];
+        }
+        if (fileToOpen != null && fileToOpen != "" && (fileType == "" || fileType == "model"))
+        {
+          if (!System.IO.Path.IsPathRooted(fileToOpen))
+            fileToOpen = System.IO.Path.Combine(System.IO.Directory.GetCurrentDirectory(), fileToOpen);
+        }
+
+        if (!wbContext.get_filename().Equals(fileToOpen))
+          wbContext.open_document(fileToOpen);
+      };
+      mainForm.Invoke(d, true);
+    }
+
     #endregion
 
     #region Message Callbacks
@@ -502,7 +604,7 @@ namespace MySQL.GUI.Workbench
     {
       try
       {
-				System.Diagnostics.Process.Start("http://wb.mysql.com/workbench/version-check.php?config="
+        System.Diagnostics.Process.Start("http://wb.mysql.com/workbench/version-check.php?config="
           + GetApplicationMetaInfo(ApplicationMetaInfo.Configuration).Replace(' ', '_')
           + "&version="
           + GetApplicationMetaInfo(ApplicationMetaInfo.Version)
