@@ -11,6 +11,7 @@ using MySQL.Utilities;
 using MySQL.Grt;
 using MySQL.Grt.Db;
 using MySQL.Grt.Db.Sql;
+using MySQL.Forms;
 using MySQL.Workbench;
 using MySQL.Controls;
 using ScintillaNet;
@@ -52,8 +53,11 @@ namespace MySQL.GUI.Workbench
       editorTabControl.TabClosed += new EventHandler<MySQL.Controls.TabClosedEventArgs>(editorTabControl_TabClosed);
       editorTabControl.SelectedIndexChanged += new EventHandler(editorTabControl_SelectedIndexChanged);
 
-      SqlEditor sqlEditor = AddSqlEditor("", dbSqlEditorBE.active_sql_editor_index());
-      ActiveControl = sqlEditor;
+      for (int i = 0; i < dbSqlEditorBE.sql_editor_count(); i++)
+      {
+        SqlEditor sqlEditor = AddSqlEditor(i);
+        ActiveControl = sqlEditor;
+      }
       dbSqlEditorBE.sql_editor_new_ui_cb(OnSqlEditorNew);
 
       logView = new GridView(dbSqlEditorBE.log());
@@ -107,7 +111,7 @@ namespace MySQL.GUI.Workbench
 
       dbSqlEditorBE.exec_sql_task.finish_cb(AfterExecSql);
       dbSqlEditorBE.exec_sql_task.progress_cb(OnExecSqlProgress);
-      dbSqlEditorBE.close_recordset_ui_cb(CloseRecordset);
+      dbSqlEditorBE.recordset_list_changed_cb(RecordsetListChanged);
       dbSqlEditorBE.output_text_ui_cb(RecordsetTextOutput);
       resultSetTextBox.Font = grtManager.get_font_option("workbench.general.Editor:Font");
 
@@ -127,6 +131,16 @@ namespace MySQL.GUI.Workbench
       sideBar.Dock = DockStyle.Fill;
       sideBar.Show();
 
+      ToolStrip menu = wbContext.toolbar_for_form(dbSqlEditorBE);
+      Controls.Add(menu);
+      menu.Dock = DockStyle.Top;
+      menu.AutoSize = true;
+
+      menu = wbContext.menu_for_form(dbSqlEditorBE);
+      Controls.Add(menu);
+      menu.Dock = DockStyle.Top;
+      menu.AutoSize = true;
+
       outputPageToolStrip.Renderer = new FlatSubToolStripRenderer();
       toolStrip4.Renderer = new FlatSubToolStripRenderer();
 
@@ -146,7 +160,7 @@ namespace MySQL.GUI.Workbench
       foreach (KeyValuePair<long, RecordsetView> pair in recordset2placeholder)
         recordsetKeyList.Add(pair.Key);
       foreach (long key in recordsetKeyList)
-        CloseRecordset(key);
+        RecordsetListChanged(key, false);
 
       foreach (SqlEditor sqlEditor in sqlEditors)
         sqlEditor.Dispose();
@@ -181,7 +195,6 @@ namespace MySQL.GUI.Workbench
 
     protected void OnRefresh()
     {
-      UpdateTooltip();
       livePhysicalOverview.RebuildModelContents();
       sideBar.RefreshEx(this, true);
     }
@@ -192,13 +205,6 @@ namespace MySQL.GUI.Workbench
       switch (refresh_type)
       {
         case Db_sql_editor.PartialRefreshType.RefreshSidebar:
-          UpdateTooltip();
-          sideBar.Refresh(this);
-          break;
-        case Db_sql_editor.PartialRefreshType.RefreshActiveSchemaSelector:
-          sideBar.RefreshActiveSchemaSelector(false);
-          break;
-        case Db_sql_editor.PartialRefreshType.RefreshSchemaTree:
           sideBar.Refresh(this);
           break;
         case Db_sql_editor.PartialRefreshType.RefreshEditor:
@@ -251,8 +257,7 @@ namespace MySQL.GUI.Workbench
         return 0;
 
       outdatedLogEventCount = logView.RowCount;
-      resultTabControl.SelectedTab = outputPage;
-      dbSqlEditorBE.exec_sql(sql, currentStatementOnly);
+      dbSqlEditorBE.exec_sql(sql, true, currentStatementOnly);
 
       return 0;
     }
@@ -264,56 +269,70 @@ namespace MySQL.GUI.Workbench
       return 0;
     }
 
+    private void UpdateRecordsets()
+    {
+      List<long> leftoverResults = new List<long>();
+      foreach (long key in recordset2page.Keys)
+      {
+        leftoverResults.Add(key);
+      }
+
+      // count how many non-recordset tabs there are
+      int extraTabCount = 0;
+      foreach (TabPage page in resultTabControl.TabPages)
+      {
+        if (page.Tag != null)
+          break;
+        extraTabCount++;
+      }
+
+      // go through list of recordsets and add new ones that arent there yet
+      for (int n = 0, rsCount = dbSqlEditorBE.recordset_count(); n < rsCount; ++n)
+      {
+        MySQL.Grt.Db.Recordset rs = dbSqlEditorBE.recordset(n);
+        if (recordset2page.ContainsKey(rs.key()))
+        {
+          leftoverResults.Remove(rs.key());
+          continue;
+        }
+
+        TabPage page = new TabPage(rs.caption());
+        page.BackColor = SystemColors.ButtonFace;
+
+        RecordsetView recordsetView = new RecordsetView();
+        page.Tag = recordsetView;
+        components.Add(recordsetView);
+        pendingRelayouts.Add(recordsetView);
+
+        recordsetView.Embed(page, rs);
+        resultTabControl.TabPages.Insert(extraTabCount+n, page);
+        resultTabControl.SetCloseButtonVisibility(extraTabCount+n, FlatTabControl.CloseButtonVisiblity.ShowButton);
+        recordset2page.Add(rs.key(), page);
+        recordset2placeholder.Add(rs.key(), recordsetView);
+        page2recordset.Add(page, rs);
+        recordsetView.ProcessModelChange();
+      }
+
+      // close recordsets that have gone
+      foreach (long key in leftoverResults)
+      {
+        CloseRecordset(key);
+      }
+    }
+
     private int AfterExecSql()
     {
       {
-        TabPage firstNewPage = null;
-        resultTabControl.SelectedTab = outputPage;
-        resultTabControl.Update();
+        UpdateRecordsets();
 
-        // close recordsets that have gone
+        if (dbSqlEditorBE.exec_sql_error_count() > 0)
         {
-          Dictionary<long, long> recordsets = new Dictionary<long, long>();
-          for (int n = 0, rsCount = dbSqlEditorBE.recordset_count(); n < rsCount; ++n)
-          {
-            MySQL.Grt.Db.Recordset rs = dbSqlEditorBE.recordset(n);
-            recordsets.Add(rs.key(), 1);
-          }
-          List<long> recordsetsToClose = new List<long>();
-          foreach (KeyValuePair<long, RecordsetView> pair in recordset2placeholder)
-            if (!recordsets.ContainsKey(pair.Key))
-              recordsetsToClose.Add(pair.Key);
-          foreach (long key in recordsetsToClose)
-            CloseRecordset(key);
+          // Show output page if something went wrong.
+          resultTabControl.SelectedTab = outputPage;
+          outputSelector.SelectedIndex = 0; // The actions page on the output page.
         }
-
-        // add new recordsets
-        for (int n = 0, rsCount = dbSqlEditorBE.recordset_count(); n < rsCount; ++n)
-        {
-          MySQL.Grt.Db.Recordset rs = dbSqlEditorBE.recordset(n);
-          if (recordset2page.ContainsKey(rs.key()))
-            continue;
-
-          TabPage page = new TabPage(rs.caption());
-          page.BackColor = SystemColors.ButtonFace;
-          if (null == firstNewPage)
-            firstNewPage = page;
-
-          RecordsetView recordsetView = new RecordsetView();
-          components.Add(recordsetView);
-          pendingRelayouts.Add(recordsetView);
-
-          recordsetView.Embed(page, rs);
-          resultTabControl.TabPages.Add(page);
-          resultTabControl.SetCloseButtonVisibility(resultTabControl.TabCount - 1,
-            FlatTabControl.CloseButtonVisiblity.ShowButton);
-          recordset2page.Add(rs.key(), page);
-          recordset2placeholder.Add(rs.key(), recordsetView);
-          page2recordset.Add(page, rs);
-          recordsetView.ProcessModelChange();
-        }
-
-        resultTabControl.SelectedTab = (null == firstNewPage) ? outputPage : firstNewPage;
+        else
+          editorTabControl_SelectedIndexChanged(null, null);
       }
 
       logView.ProcessModelRowsChange();
@@ -343,9 +362,6 @@ namespace MySQL.GUI.Workbench
         view.GridView.Dock = DockStyle.Fill;
       }
       pendingRelayouts.Clear();
-
-      // Workaround for the anchored clear button. Position is completely wrong if we don't adjust it (at least once).
-      clearButton.Location = new Point(outputPage.Width - clearButton.Width - 6, 2);
     }
     #endregion
 
@@ -360,7 +376,7 @@ namespace MySQL.GUI.Workbench
 
     private int OnSqlEditorNew(int index)
     {
-      AddSqlEditor("", index);
+      AddSqlEditor(index);
       return 0;
     }
 
@@ -369,18 +385,20 @@ namespace MySQL.GUI.Workbench
       editorTabControl.TabPages[dbSqlEditorBE.active_sql_editor_index()].Text = dbSqlEditorBE.sql_editor_caption(dbSqlEditorBE.active_sql_editor_index());
     }
 
-    private SqlEditor AddSqlEditor(String filepath, int index)
+    private SqlEditor AddSqlEditor(int index)
     {
       if (-1 == index)
         index = dbSqlEditorBE.add_sql_editor();
 
       SqlEditor sqlEditor = new SqlEditor(grtManager);
       sqlEditor.BE = dbSqlEditorBE.sql_editor(index);
+      sqlEditor.Text = sqlEditor.BE.sql();
       String caption = dbSqlEditorBE.sql_editor_caption(index);
       TabPage page = new TabPage(caption);
+      page.Tag = sqlEditor;
+
       page.Controls.Add(sqlEditor);
       editorTabControl.TabPages.Add(page);
-      editorTabControl.SelectedIndex = index;
 
       sqlEditor.Selection.Start = sqlEditor.Text.Length;
       sqlEditor.Selection.End = sqlEditor.Text.Length;
@@ -388,6 +406,11 @@ namespace MySQL.GUI.Workbench
 
       sqlEditors.Add(sqlEditor);
 
+      UpdateRecordsets();
+
+      // Select tab last so that callbacks will work on attached recordset
+      editorTabControl.SelectedIndex = index;
+
       return sqlEditor;
     }
 
@@ -397,6 +420,7 @@ namespace MySQL.GUI.Workbench
         index = dbSqlEditorBE.active_sql_editor_index();
       dbSqlEditorBE.remove_sql_editor(index);
       SqlEditor sqlEditor = sqlEditors[index];
+      sqlEditor.Tag = null;
       sqlEditors.RemoveAt(index);
       sqlEditor.Dispose();
     }
@@ -426,6 +450,14 @@ namespace MySQL.GUI.Workbench
     void editorTabControl_SelectedIndexChanged(object sender, EventArgs e)
     {
       dbSqlEditorBE.active_sql_editor_index(editorTabControl.SelectedIndex);
+
+      long rset = dbSqlEditorBE.active_recordset_for_sql_editor(dbSqlEditorBE.active_sql_editor_index());
+      if (rset >= 0 && recordset2page.ContainsKey(rset))
+        resultTabControl.SelectedTab = recordset2page[rset];
+
+      SqlEditor sqlEditor = SqlEditor();
+      if (sqlEditor != null && sqlEditor.CanFocus)
+        sqlEditor.Focus();
     }
 
     void editorTabControl_TabClosing(object sender, MySQL.Controls.TabClosingEventArgs e)
@@ -449,7 +481,7 @@ namespace MySQL.GUI.Workbench
         dbSqlEditorBE.active_sql_editor_index(editorTabControl.SelectedIndex);
       else
       {
-        SqlEditor sqlEditor = AddSqlEditor("", -1);
+        SqlEditor sqlEditor = AddSqlEditor(-1);
         dbSqlEditorBE.active_sql_editor_index(0);
         ActiveControl = sqlEditor;
       }
@@ -503,7 +535,25 @@ namespace MySQL.GUI.Workbench
       page.Text = rs.caption();
     }
 
-    private int CloseRecordset(long key)
+    delegate void DelegateFunc();
+    private void RecordsetListChanged(long key, bool added)
+    {
+      if (!added)
+      {
+        if (InvokeRequired)
+        {
+          DelegateFunc deleg = delegate 
+          {
+            CloseRecordset(key);
+          };
+          BeginInvoke(deleg);
+        }
+        else
+          CloseRecordset(key);
+      }
+    }
+
+    private void CloseRecordset(long key)
     {
       if (recordset2page.ContainsKey(key))
       {
@@ -513,6 +563,7 @@ namespace MySQL.GUI.Workbench
         recordset2page.Remove(key);
 
         RecordsetView recordsetView = recordset2placeholder[key];
+        page.Tag = null;
         recordset2placeholder.Remove(key);
         components.Remove(recordsetView);
         recordsetView.Dispose();
@@ -523,7 +574,6 @@ namespace MySQL.GUI.Workbench
         if (IsHandleCreated)
           BeginInvoke(new CollectGarbageDelegate(CollectGarbage));
       }
-      return 0;
     }
 
     private delegate void RecordsetTextOutputDelegate(String text, bool bringToFront);
@@ -548,7 +598,35 @@ namespace MySQL.GUI.Workbench
 
     private void resultTabControl_SelectedIndexChanged(object sender, EventArgs e)
     {
-      dbSqlEditorBE.active_recordset(ActiveRecordset);
+      Grt.Db.Recordset rset = ActiveRecordset;
+
+      if (rset != null)
+      {
+        dbSqlEditorBE.active_recordset(rset);
+
+        SqlEditor editor = SqlEditor();
+        if (editor != null)
+        {
+          foreach (TabPage page in editorTabControl.TabPages)
+          {
+            SqlEditor page_editor = page.Tag as SqlEditor;
+            if (page_editor == editor)
+            {
+              editorTabControl.SelectedTab = page;
+
+              if (resultTabControl.SelectedTab != null)
+              {
+                Control ctl = resultTabControl.SelectedTab;
+                if (ctl != null && ctl.CanFocus)
+                  ctl.Focus();
+              }
+              break;
+            }
+          }
+        }
+      }
+      else
+        dbSqlEditorBE.active_recordset(null);
     }
 
     private void resultTabControl_TabClosing(object sender, TabClosingEventArgs e)
@@ -564,7 +642,11 @@ namespace MySQL.GUI.Workbench
       TabPage page = e.page;
       MySQL.Grt.Db.Recordset rs = page2recordset.ContainsKey(e.page) ? page2recordset[e.page] : null;
       if (null != rs)
+      {
         rs.close();
+
+        UpdateRecordsets();
+      }
     }
 
     private void renamePage_ToolStripMenuItem_Click(object sender, EventArgs e)
@@ -749,7 +831,7 @@ namespace MySQL.GUI.Workbench
           return dbSqlEditorBE.live_physical_overview().activate_toolbar_item(new NodeId(0), name);
         };
 
-        new WorkbenchToolbarManager(wbContext, toolStrip4).UpdateToolbar(toolStrip4,
+        new WorkbenchToolbarManager(wbContext).UpdateToolbar(toolStrip4,
           dbSqlEditorBE.live_physical_overview().get_toolbar_items(new NodeId(0)),
           cb);
 
@@ -814,14 +896,6 @@ namespace MySQL.GUI.Workbench
       wbContext.save_state("sidebar_width", "query_editor", sidebarWidth);
     }
 
-    /// <summary>
-    /// Updates the editor's tab tooltip to contain the current connection info.
-    /// </summary>
-    public void UpdateTooltip()
-    {
-      ToolTipText = dbSqlEditorBE.connection_info();
-    }
-
     void outputPaneIndexChanged(object sender, System.EventArgs e)
     {
       ToolStripComboBox selector = sender as ToolStripComboBox;
