@@ -6,7 +6,7 @@ using System.Windows.Forms;
 using MySQL.Controls;
 using MySQL.Grt;
 using MySQL.Grt.Db;
-using MySQL.Workbench;
+using MySQL.Forms;
 
 namespace MySQL.GUI.Workbench.Plugins
 {
@@ -310,8 +310,7 @@ namespace MySQL.GUI.Workbench.Plugins
             liveObjectControlsPanel.Dock = DockStyle.Fill;
             {
               closeLiveEditorButton = new Button();
-              closeLiveEditorButton.FlatStyle = FlatStyle.Flat;
-              closeLiveEditorButton.ForeColor = Color.White;
+              closeLiveEditorButton.UseVisualStyleBackColor = true;
               closeLiveEditorButton.Parent = liveObjectControlsPanel;
               closeLiveEditorButton.Text = "Close";
               closeLiveEditorButton.Location = new Point(
@@ -321,8 +320,7 @@ namespace MySQL.GUI.Workbench.Plugins
             }
             {
               revertLiveObjectButton = new Button();
-              revertLiveObjectButton.FlatStyle = FlatStyle.Flat;
-              revertLiveObjectButton.ForeColor = Color.White;
+              revertLiveObjectButton.UseVisualStyleBackColor = true;
               revertLiveObjectButton.Parent = liveObjectControlsPanel;
               revertLiveObjectButton.Text = "Revert";
               revertLiveObjectButton.Location = new Point(
@@ -332,8 +330,7 @@ namespace MySQL.GUI.Workbench.Plugins
             }
             {
               applyLiveObjectButton = new Button();
-              applyLiveObjectButton.FlatStyle = FlatStyle.Flat;
-              applyLiveObjectButton.ForeColor = Color.White;
+              applyLiveObjectButton.UseVisualStyleBackColor = true;
               applyLiveObjectButton.Parent = liveObjectControlsPanel;
               applyLiveObjectButton.Text = "Apply";
               applyLiveObjectButton.Location = new Point(
