@@ -39,8 +39,6 @@ namespace MySQL.GUI.Workbench
     public String lastSearchString = "";
     public ToolStripTextBox searchBox = null;
 
-    private ToolStrip mainToolStrip;
-
     private static WorkbenchToolbarManager instance = null;
 
     #endregion
@@ -52,31 +50,16 @@ namespace MySQL.GUI.Workbench
 
     #region Constructors
 
-    public WorkbenchToolbarManager(WbContext WbContext, ToolStrip MainToolStrip)
-	{
+    public WorkbenchToolbarManager(WbContext WbContext)
+    {
       instance = this;
-
-	  wbContext = WbContext;
-      mainToolStrip = MainToolStrip;
+      wbContext = WbContext;
     }
 
     #endregion
 
     public delegate bool ToolbarCallback(String command);
 
-    public void UpdateMainToolbar()
-    {
-      mainToolStrip.Items.Clear();
-      mainToolStrip.Items.AddRange(buildToolbar(wbContext.get_items_for_toolbar("main")));
-      mainToolStrip.Items.AddRange(buildToolbar(wbContext.get_items_for_toolbar("options")));
-    }
-
-    public void UpdateToolsToolbar(ToolStrip WorkToolStrip)
-    {
-      WorkToolStrip.Items.Clear();
-      WorkToolStrip.Items.AddRange(buildToolbar(wbContext.get_items_for_toolbar("tools"), true));
-    }
-
     public void UpdateToolbar(ToolStrip toolstrip, List<ToolbarItem> toolbarItems,
           ToolbarCallback callback)
     {
@@ -85,20 +68,6 @@ namespace MySQL.GUI.Workbench
       toolstrip.Items.AddRange(buildToolbar(toolbarItems));
     }
 
-    /// <summary>
-    /// Search for a specific ToolStripItem by the command name
-    /// </summary>
-    /// <param name="cmd">command name string</param>
-    /// <returns>the ToolStripItem if found</returns>
-    public ToolStripItem GetToolStripItem(String cmd)
-    {
-      foreach (ToolStripItem item in mainToolStrip.Items)
-        if (item.Tag != null && item.Tag.ToString().Equals(cmd))
-          return item;
-
-      return null;
-    }
-
     ToolStripItem[] buildToolbar(List<ToolbarItem> toolbarItems)
     {
       return buildToolbar(toolbarItems, false);
@@ -193,114 +162,9 @@ namespace MySQL.GUI.Workbench
             }
 
           case ToolbarItemType.ToolbarCheck:
-            {
-              CheckBox cb = new CheckBox();
-              cb.Name = subitem.get_name();
-              cb.Text = subitem.get_caption();
-              cb.Tag = subitem.get_command();
-              cb.Enabled = subitem.get_enabled();
-              cb.Checked = subitem.get_checked();
-              cb.Font = new System.Drawing.Font(cb.Font.FontFamily, 7);
-              cb.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
-              cb.Click += new System.EventHandler(checkbox_Click);
-              cb.AutoSize = true;
-              ToolStripControlHost chost = new ToolStripControlHost(cb);
-              chost.ToolTipText = subitem.get_tooltip();
-              itemlist[i++] = chost;
-
-              if (chost.Name.EndsWith("__right"))
-                chost.Alignment = ToolStripItemAlignment.Right;
-              break;
-            }
-
           case ToolbarItemType.ToolbarDropDown:
-            {
-              String selected_item;
-              List<String> items = wbContext.get_dropdown_items(subitem.get_name(), subitem.get_command(), out selected_item);
+            throw new Exception("SQL IDE: Invalid toolbar item used.");
 
-              if (items.Count > 0 && items[0][0] == '#')
-              {
-                ToolStripDropDownButton smitem;
-                ToolStripDropDown dropdown= new ToolStripDropDown();
-                System.Drawing.Bitmap selected= null;
-                System.Drawing.SolidBrush brush= new System.Drawing.SolidBrush(System.Drawing.Color.White);
-								System.Drawing.Pen pen = new System.Drawing.Pen(System.Drawing.Color.LightGray);
-                List<ToolStripButton> buttons = new List<ToolStripButton>();
-
-                smitem = new ToolStripDropDownButton();
-                smitem.Name = subitem.get_name();
-                smitem.Tag = subitem.get_command();
-                smitem.Enabled = subitem.get_enabled();
-                smitem.ToolTipText = subitem.get_tooltip();
-                smitem.DropDown= dropdown;
-                smitem.ShowDropDownArrow= true;
-                smitem.ImageScaling= ToolStripItemImageScaling.None;
-                smitem.DropDownDirection= ToolStripDropDownDirection.BelowRight;
-                smitem.AutoSize = true;
-
-                foreach (String item in items)
-                {
-                  try
-                  {
-                    ToolStripButton button = new ToolStripButton();
-                    System.Drawing.Color color = System.Drawing.ColorTranslator.FromHtml(item);
-                    System.Drawing.Bitmap bmp = new System.Drawing.Bitmap(20, 15);
-                    System.Drawing.Graphics gfx = System.Drawing.Graphics.FromImage(bmp);
-                    brush.Color = color;
-                    gfx.FillRectangle(brush, new System.Drawing.Rectangle(1, 0, 18, 14));
-                    gfx.DrawRectangle(pen, new System.Drawing.Rectangle(1, 0, 18, 14));
-
-                    button.Name = item;
-                    button.Image = bmp;
-                    button.ImageScaling = ToolStripItemImageScaling.None;
-                    button.Tag = smitem;
-                    button.Click += new EventHandler(colorDropDownItem_Click);
-
-                    if (item == selected_item || selected == null)
-                      selected = bmp;
-
-                    buttons.Add(button);
-                  }
-                  catch
-                  {
-                  }
-                }
-                dropdown.Items.AddRange(buttons.ToArray());
-
-                smitem.Image = selected;
-
-                itemlist[i++] = smitem;
-              }
-              else
-              {
-                ToolStripComboBox smitem;
-                String[] strings= new String[items.Count];
-                String selected= null;
-                int j = 0;
-
-                smitem = new ToolStripComboBox();
-                smitem.Name = subitem.get_name();
-                smitem.Tag = subitem.get_command();
-                smitem.Enabled = subitem.get_enabled();
-                smitem.ToolTipText = subitem.get_tooltip();
-                smitem.DropDownHeight = 250;
-                smitem.SelectedIndexChanged += new EventHandler(textDropDownItem_Changed);
-
-                foreach (String item in items)
-                {
-                  strings[j++]= item;
-
-                  if (item == selected_item || selected == null)
-                    selected = item;
-                }
-                smitem.Items.AddRange(strings);
-
-                smitem.Text = selected;
-
-                itemlist[i++] = smitem;
-              }
-              break;
-            }
           case ToolbarItemType.ToolbarSearch:
             {
               // Instead of creating the same search box again and again we share one
@@ -380,34 +244,8 @@ namespace MySQL.GUI.Workbench
       }
     }
 
-    void checkbox_Click(object sender, EventArgs e)
-    {
-      CheckBox item = sender as CheckBox;
-
-      if (item != null)
-      {
-        wbContext.toggle_checkbox_item(item.Name, (String)item.Tag, item.Checked);
-      }
-    }
-
-    void colorDropDownItem_Click(object sender, EventArgs e)
-    {
-      ToolStripButton button = sender as ToolStripButton;
-      ToolStripDropDownButton smitem= button.Tag as ToolStripDropDownButton;
-
-      smitem.Image = button.Image;
-
-      wbContext.select_dropdown_item(smitem.Name, (String)smitem.Tag, button.Name);
-    }
-
-    void textDropDownItem_Changed(object sender, EventArgs e)
-    {
-      ToolStripComboBox combo = sender as ToolStripComboBox;
-
-      wbContext.select_dropdown_item(combo.Name, (String)combo.Tag, (String)combo.SelectedItem);
-    }
   }
-
+  
 
   public class SqlIdeMenuManager
   {
