@@ -1,5 +1,5 @@
 ﻿/* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010,2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -181,7 +181,7 @@ namespace MySQL.Utilities
     /// </summary>
     /// <param name="fontDescription"></param>
     /// <returns></returns>
-    public static Font getFont(String fontDescription)
+    public static Font GetFont(String fontDescription)
     {
       // When parsing the font description keep in mind font family names can contain space characters
       // too, so parse from the end to filter out size and style first.
@@ -225,7 +225,7 @@ namespace MySQL.Utilities
       for (int i = 1; i <= index; i++)
         familyName += " " + parts[i];
 
-      return getFont(familyName, fontSize, style);
+      return GetFont(familyName, fontSize, style);
     }
 
     /// <summary>
@@ -236,12 +236,12 @@ namespace MySQL.Utilities
     /// <param name="size">Size in points of the new font.</param>
     /// <param name="style">Font styles like bold, italic.</param>
     /// <returns>A new font object. This is guaranteed to be a valid font.</returns>
-    public static Font getFont(String familyName, float size)
+    public static Font GetFont(String familyName, float size)
     {
-      return getFont(familyName, size, FontStyle.Regular);
+      return GetFont(familyName, size, FontStyle.Regular);
     }
 
-    public static Font getFont(String familyName, float size, FontStyle style)
+    public static Font GetFont(String familyName, float size, FontStyle style)
     {
       Font result = null;
       FontFamily family = null;
@@ -281,5 +281,30 @@ namespace MySQL.Utilities
       return principal.IsInRole(WindowsBuiltInRole.Administrator);
     }
 
+    /// <summary>
+    /// Determines if we are running on Vista or a new OS.
+    /// </summary>
+    /// <returns>True if this is Vista, Windows 7 or above, otherwise false.</returns>
+    static public bool IsVistaOrAbove()
+    {
+      OperatingSystem system = Environment.OSVersion;
+      if (system.Platform != PlatformID.Win32NT)
+        return false;
+
+      return (system.Version.Major > 5) || (system.Version.Major == 5 && system.Version.Minor > 1);
+    }
+
+    /// <summary>
+    /// Tells the caller if we are running on an Aero-enabled desktop.
+    /// </summary>
+    /// <returns>True, if Aero is available and enabled, otherwise false.</returns>
+    static public bool IsCompositionEnabled()
+    {
+      if (!IsVistaOrAbove())
+        return false;
+
+      return Win32.DwmIsCompositionEnabled();
+    }
+
   } // ControlUtilities
 } // MySQL.Utilities
