@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -41,9 +41,13 @@
 
 using namespace mdc;
 
+
+int mdc_live_item_count = 0;
+
 CanvasItem::CanvasItem(Layer *layer)
   : _layer(layer), _parent(0)
 {
+  mdc_live_item_count++;
   _pos.x= 0.0f;
   _pos.y= 0.0f;
   _focused= 0;
@@ -88,6 +92,7 @@ CanvasItem::CanvasItem(Layer *layer)
 
 CanvasItem::~CanvasItem()
 {
+  --mdc_live_item_count;
   delete _highlight_color;
   
   if (_parent)
@@ -1318,7 +1323,7 @@ bool CanvasItem::on_drag_handle(ItemHandle *handle, const Point &pos, bool dragg
   return true;
 }
 
-
+//--------------------------------------------------------------------------------------------------
 
 bool CanvasItem::on_click(CanvasItem *target, const Point &point, MouseButton button, EventState state)
 {
@@ -1350,7 +1355,7 @@ bool CanvasItem::on_click(CanvasItem *target, const Point &point, MouseButton bu
     }
     else // a child item
     {
-      // if the parent is focused and we're focuseable, then focus ourselves
+      // if the parent is focused and we're focusable, then focus ourselves
       if (accepts_focus())
       {
         CanvasItem *parent= get_parent();
@@ -1369,6 +1374,14 @@ bool CanvasItem::on_click(CanvasItem *target, const Point &point, MouseButton bu
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+bool CanvasItem::on_double_click(CanvasItem *target, const Point &point, MouseButton button, EventState state)
+{
+  return false;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 bool CanvasItem::on_button_press(CanvasItem *target, const Point &point, MouseButton button, EventState state)
 {
