@@ -752,10 +752,13 @@ void CanvasView::paint_item_cache(CairoCtx *cr, double x, double y,
     cr->paint();
 
 #ifdef DEBUG
-  cr->rectangle(0, 0, cairo_image_surface_get_width(cached_item)-1,
-    cairo_image_surface_get_height(cached_item)-1);
-  cr->set_color(Color((rand()%100)/100.0, (rand()%100)/100.0, (rand()%100)/100.0));
-  cr->stroke();
+  if (getenv("DEBUG_CANVAS"))
+  {
+    cr->rectangle(0, 0, cairo_image_surface_get_width(cached_item)-1,
+      cairo_image_surface_get_height(cached_item)-1);
+    cr->set_color(Color((rand()%100)/100.0, (rand()%100)/100.0, (rand()%100)/100.0));
+    cr->stroke();
+  }
 #endif
 
   cr->restore();
@@ -1475,6 +1478,7 @@ void CanvasView::handle_mouse_move(int x, int y, EventState state)
 
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void CanvasView::handle_mouse_button(MouseButton button, bool press, int x, int y, EventState state)
 {
@@ -1541,7 +1545,32 @@ void CanvasView::handle_mouse_button(MouseButton button, bool press, int x, int
     _ilayer->handle_mouse_button_bottom(button, press, point, state);
 }
 
+//--------------------------------------------------------------------------------------------------
 
+void CanvasView::handle_mouse_double_click(MouseButton button, int x, int y, EventState state)
+{
+  if (_destroying || _ui_lock > 0)
+    return;
+
+  Point point= window_to_canvas(x, y);
+  bool handled= false;
+
+  // Must lock after the external callback otherwise we may get a deadlock.
+  CanvasAutoLock lock(this);
+
+  CanvasItem* item= get_leaf_item_at(point);
+
+  // send click if the button was released over the same item it was pressed
+  handled= propagate_event(item,
+                           sigc::mem_fun(&CanvasItem::on_double_click),
+                           point, button, state);
+
+  set_last_click_item(item);
+  _last_mouse_pos= point;
+  _last_click_info[button].pos= point;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 bool CanvasView::handle_key(const KeyInfo &key, bool press, EventState state)
 {
