@@ -104,7 +104,13 @@ using namespace mforms;
     [self setBorderType: NSBezelBorder];
     [self setHasVerticalScroller: scrolls & mforms::VerticalScrollBar];
     [self setHasHorizontalScroller: scrolls & mforms::HorizontalScrollBar];
-    
+    [self setAutohidesScrollers: YES];
+    if (scrolls & mforms::SmallScrollBars)
+    {
+      [[self verticalScroller] setControlSize: NSSmallControlSize];
+      [[self horizontalScroller] setControlSize: NSSmallControlSize];
+    }
+
     frame.size= [self minimumSize];
     mContentView= [[[InternalTextView alloc] initWithFrame: frame AndBackend: mOwner] autorelease];
     [self setDocumentView:mContentView];
@@ -112,6 +118,7 @@ using namespace mforms;
     [[mContentView textContainer] setContainerSize: NSMakeSize(1000000000, 1000000000)];
     [mContentView setHorizontallyResizable: YES];
     [mContentView setVerticallyResizable: YES];
+
     if (scrolls & mforms::VerticalScrollBar)
     {
       [[mContentView textContainer] setHeightTracksTextView: NO];
