@@ -361,10 +361,6 @@ static bool treeview_create(mforms::TreeView *self, mforms::TreeOptions options)
 {
   MFTreeViewImpl *tree= [[[MFTreeViewImpl alloc] initWithObject: self] autorelease];
   
-  if (!(options & mforms::TreeShowHeader))
-  {
-    [tree->mOutline setHeaderView:nil];
-  }
   if (options & mforms::TreeAllowReorderRows)
   {
     [tree enableRowReordering];
