@@ -25,7 +25,7 @@
 
 #include "mforms/fs_object_selector.h"
 #include "mforms/utilities.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace mforms;
 
@@ -43,7 +43,7 @@ FsObjectSelector::FsObjectSelector(bool horizontal)
   
   set_spacing(10);
   add(_edit, true, true);
-  add(_browse_button, false, false);
+  add(_browse_button, false, true);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -81,18 +81,21 @@ void FsObjectSelector::initialize(const std::string& initial_path, FileChooserTy
   _show_hidden = show_hidden;
   _extensions= extensions;
   
-  char **parts = g_strsplit(extensions.c_str(), "|", -1);
-  if (parts)
+  if (extensions.size() > 0)
   {
-    if (parts[0] && parts[1])
-      _default_extension = parts[1];
-    else if (parts[0])
-      _default_extension = parts[0];
-    
-    if (_default_extension[0] == '*')
-      _default_extension = _default_extension.substr(1);
+    char **parts = g_strsplit(extensions.c_str(), "|", -1);
+    if (parts)
+    {
+      if (parts[0] && parts[1])
+        _default_extension = parts[1];
+      else if (parts[0])
+        _default_extension = parts[0];
 
-    g_strfreev(parts);
+      if (_default_extension[0] == '*')
+        _default_extension = _default_extension.substr(1);
+
+      g_strfreev(parts);
+    }
   }
 
   _edit->set_value(initial_path);
