@@ -1,11 +1,11 @@
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include "../lf_code_editor.h"
 #include "tinyxml.h"
 #include <memory>
 #include <ctype.h>
 #include <stdlib.h>
-#include "wb_memory.h"
-#include "wb_iterators.h"
+#include "base/wb_memory.h"
+#include "base/wb_iterators.h"
 
 #include "Scintilla.h"
 #include "SciLexer.h"
@@ -84,7 +84,7 @@ mforms::gtk::CodeEditorImpl::CodeEditorImpl(CodeEditor* self)
   _sci = SCINTILLA(_sci_gtk_widget);
   _owner = self;
   gtk_signal_connect(GTK_OBJECT(_sci_gtk_widget), SCINTILLA_NOTIFY, GTK_SIGNAL_FUNC(notify_signal), this);
-
+  _sci_gtkmm_widget->show();
 }
 
 //------------------------------------------------------------------------------
@@ -577,7 +577,7 @@ void mforms::gtk::CodeEditorImpl::set_read_only(CodeEditor* self, bool flag)
   CodeEditorImpl* ce = self->get_data<CodeEditorImpl>();
   if (ce)
   {
-    ce->send_editor(SCI_SETREADONLY, 1);
+    ce->send_editor(SCI_SETREADONLY, flag);
   }
 }
 
@@ -671,6 +671,8 @@ void mforms::gtk::CodeEditorImpl::set_font(CodeEditor* self, const std::string &
     bool italic = 0;
     if (base::parse_font_description(fontDescription, name, size, bold, italic))
     {
+      if (!name.empty() && name[0] != '!')
+        name = "!"+name;
       ce->send_editor(SCI_STYLESETFONT, STYLE_DEFAULT, (sptr_t)name.c_str());
       ce->send_editor(SCI_STYLESETSIZE, STYLE_DEFAULT, size);
       ce->send_editor(SCI_STYLESETBOLD, STYLE_DEFAULT, bold);
