@@ -42,6 +42,8 @@
 #include "../lf_popup.h"
 #include "../lf_menu.h"
 #include "../lf_code_editor.h"
+#include "../lf_menubar.h"
+#include "../lf_toolbar.h"
 //#include "lf_grttreeview.h"
 
 sigc::slot<void,std::string> mforms::gtk::UtilitiesImpl::open_url_slot;
@@ -75,6 +77,8 @@ void ::mforms::gtk::init()
   PopupImpl::init();
   MenuImpl::init();
   CodeEditorImpl::init();
+  lf_menubar_init();
+  mforms::gtk::lf_toolbar_init();
 //  GRTTreeViewImpl::init(); initialized in main program
 }
 
