@@ -100,8 +100,8 @@ TreeViewImpl::TreeViewImpl(TreeView *self, mforms::TreeOptions opts)
   _tree->signal_button_press_event().connect_notify(sigc::mem_fun(this, &TreeViewImpl::on_button_event));
 
   _swin->add(*_tree);
-
-  _tree->set_headers_visible(opts & mforms::TreeShowHeader);
+  _swin->show();
+  //_tree->set_headers_visible(opts & mforms::TreeShowHeader);
   _tree->set_reorderable(opts & mforms::TreeAllowReorderRows);
 }
 
@@ -403,33 +403,40 @@ bool TreeViewImpl::get_check(TreeView *self, int row, int column)
 std::string TreeViewImpl::get_row_tag(TreeView *self, int row)
 {
   TreeViewImpl* tree = self->get_data<TreeViewImpl>();
-  Gtk::TreeRow tree_row;
-  Gtk::TreePath path;
-
-  path.append_index(row);
+  Glib::ustring ret;
 
   if (tree->_list_store)
-    tree_row= *tree->_list_store->get_iter(path);
-  else
-    return "";
+  {
+    Gtk::TreePath path;
+    path.append_index(row);
 
-  return Glib::ustring(tree_row[tree->_columns.tag_column()]);
+    const Gtk::TreeIter iter = tree->_list_store->get_iter(path);
+    if (iter)
+    {
+      const Gtk::TreeRow row = *iter;
+      ret = row[tree->_columns.tag_column()];
+    }
+  }
+
+  return ret;
 }
 
 void TreeViewImpl::set_row_tag(TreeView *self, int row, const std::string &value)
 {
   TreeViewImpl* tree = self->get_data<TreeViewImpl>();
-  Gtk::TreeRow tree_row;
-  Gtk::TreePath path;
-
-  path.append_index(row);
 
   if (tree->_list_store)
-    tree_row= *tree->_list_store->get_iter(path);
-  else
-    return;
+  {
+    Gtk::TreePath path;
+    path.append_index(row);
 
-  tree_row[tree->_columns.tag_column()]= value;
+    Gtk::TreeIter iter = tree->_list_store->get_iter(path);
+    if (iter)
+    {
+      Gtk::TreeRow tree_row = *tree->_list_store->get_iter(path);
+      tree_row[tree->_columns.tag_column()] = value;
+    }
+  }
 }
 
 void TreeViewImpl::set_allow_sorting(TreeView* self, bool flag)
