@@ -1,20 +1,20 @@
 /* 
- * (c) 2008-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
  
 #include "config.h"
@@ -251,6 +251,42 @@ int UtilitiesImpl::show_warning(const std::string &title, const std::string &tex
   return r;
 }
 
+static void handle_click(Gtk::CheckButton* btn, bool* state)
+{
+  *state = btn->get_active();
+}
+
+int show_message_with_checkbox(const std::string &title, const std::string &text,
+                               const std::string &ok, const std::string &cancel,
+                               const std::string &other,
+                               const std::string &checkbox_text, // empty text = default "Don't show this message again" text
+                               bool &remember_checked)
+{
+  Gtk::Dialog dlg("<b>"+title+"</b>\n"+text, true, true);
+
+  Gtk::VBox *vbox = dlg.get_vbox();
+  vbox->add(*Gtk::manage(new Gtk::Label(text, true)));
+
+  Gtk::CheckButton* btn = Gtk::manage(new Gtk::CheckButton(checkbox_text.empty() ? "Don't show this message again" : checkbox_text));
+  vbox->add(*btn);
+  btn->set_active(remember_checked);
+  btn->signal_clicked().connect(sigc::bind(sigc::ptr_fun(handle_click), btn, &remember_checked));
+
+  Gtk::Button *ok_btn = Gtk::manage(new Gtk::Button(ok));
+  dlg.add_action_widget(*ok_btn, mforms::ResultOk);
+
+  if (!cancel.empty())
+    dlg.add_action_widget(*Gtk::manage(new Gtk::Button(cancel)), mforms::ResultCancel);
+  if (!other.empty())
+    dlg.add_action_widget(*Gtk::manage(new Gtk::Button(other)), mforms::ResultOther);
+
+  dlg.set_transient_for(*get_mainwindow());
+  int r= dlg.run();
+  if (r == Gtk::RESPONSE_DELETE_EVENT)
+    return mforms::ResultCancel;
+  return r;
+}
+
 void UtilitiesImpl::set_clipboard_text(const std::string &text)
 {
   Gtk::Clipboard::get()->set_text(text);
@@ -644,7 +680,7 @@ void TransparentMessage::show_message(const std::string& title, const std::strin
   layout = create_pango_layout(text);
   layout->set_font_description(Pango::FontDescription("Bitstream Vera Sans,Helvetica, 9"));
   layout->set_width((MESSAGE_WINDOW_WIDTH-icon->get_width()-30-20)*Pango::SCALE);
-  pixmap->draw_layout(gc, icon->get_width()+30, 80, layout);
+  pixmap->draw_layout(gc, icon->get_width()+30, 90, layout);
 
   get_style()->set_bg_pixmap(Gtk::STATE_NORMAL, pixmap);
 
