@@ -20,7 +20,7 @@
 #include "text_list_columns_model.h"
 #include "gtk_helpers.h"
 #include "../lf_view.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #define DEFAULT_WIDTH 700
 #define DEFAULT_HEIGHT 500
