@@ -23,7 +23,7 @@
 #include <mforms/box.h>
 #include <mforms/app.h>
 
-#include "ui_form.h"
+#include "base/ui_form.h"
 #include <sigc++/sigc++.h>
 
 namespace bec {
@@ -31,6 +31,8 @@ namespace bec {
 };
 
 namespace mforms {
+  class ToolBar;
+  class MenuBar;
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 #ifndef SWIG
@@ -56,6 +58,8 @@ namespace mforms {
     sigc::slot<bool> _on_close_slot;
     std::string _context_name;
     std::string _identifier;
+    mforms::MenuBar *_menubar;
+    mforms::ToolBar *_toolbar;
     bool _is_main;
     
   public:
@@ -69,9 +73,10 @@ namespace mforms {
     AppView(bool horiz, const std::string &context_name, bool is_main);
 #else
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
-    AppView(bool horiz, const std::string &context_name, bool is_main) : Box(horiz), _context_name(context_name), _is_main(is_main) {}
+    AppView(bool horiz, const std::string &context_name, bool is_main);
 #endif
 #endif
+    virtual ~AppView();
     
     /** Sets the title of this view, when docked. Alias for App::set_view_title()
      */
@@ -96,6 +101,12 @@ namespace mforms {
     void set_on_close(const sigc::slot<bool> &slot) { _on_close_slot= slot; }
 #endif
 
+    mforms::MenuBar *get_menubar() { return _menubar; }
+    void set_menubar(mforms::MenuBar *menu);
+
+    mforms::ToolBar *get_toolbar() { return _toolbar; }
+    void set_toolbar(mforms::ToolBar *toolbar);
+
     /** Internal use */
     virtual bool on_close() { return _on_close_slot(); }
   protected:
