@@ -85,7 +85,7 @@ namespace mforms {
     }
 
     void set_data(id data) { objc_msgSend(data, sel_getUid("retain")); /* calls [data retain] */ _data= data; }
-    id get_data() { return _data; }
+    id get_data() const { return _data; }
     
     virtual ~Object() { /*objc_msgSend(_data, sel_getUid("release"));*/ /* calls [_data release] */ }
   private:
@@ -101,9 +101,9 @@ namespace mforms {
     void set_data(void *data) { _data= data; }
 
     template<class C>
-      C* get_data() { return reinterpret_cast<C*>(_data); }
+      C* get_data() const { return reinterpret_cast<C*>(_data); }
 
-    void *get_data_ptr() { return _data; }
+    void *get_data_ptr() const { return _data; }
 
   private:
     void *_data;
