@@ -86,6 +86,7 @@ namespace mforms {
       bool fullwidth; // TODO: similar this value
     };
     std::list<Row> _rows;
+    std::string _ok_caption; // STL type, produces the same DLL-interface warning.
 #ifdef _WIN32
 #pragma warning(pop)
 #endif // _WIN32
@@ -95,7 +96,6 @@ namespace mforms {
     Box *_button_box;
     Button *_ok_button;
     Button *_cancel_button;
-    std::string _ok_caption;
     int _caption_width;
     int _view_width;
     int _title_width;
