@@ -23,7 +23,7 @@
 #include <mforms/base.h>
 #include <sigc++/sigc++.h>
 
-#include "geometry.h"
+#include "base/geometry.h"
 
 namespace mforms {
 
