@@ -26,7 +26,7 @@ using namespace mforms;
   
 
 TreeView::TreeView(TreeOptions options)
-  : _updating(false), _context_menu(0)
+  : _context_menu(0), _updating(false)
 {
   _treeview_impl= &ControlFactory::get_instance()->_treeview_impl;
 
