@@ -26,14 +26,20 @@
 
 #include <mforms/utilities.h>
 #include <mforms/mforms.h>
-#include "string_utilities.h"
-
+#include "base/string_utilities.h"
+#include "base/file_functions.h"
+#include <string.h>
 #include <math.h>
+#include "base/log.h"
+ENABLE_LOG("mforms.utils")
 
 using namespace mforms;
 
 GThread *_mforms_main_thread=0;
 
+static std::map<std::string, int> remembered_message_answers;
+static std::string remembered_message_answer_file;
+
 //--------------------------------------------------------------------------------------------------
 
 int Utilities::show_message(const std::string &title, const std::string &text,
@@ -63,6 +69,70 @@ int Utilities::show_warning(const std::string &title, const std::string &text,
 
 //--------------------------------------------------------------------------------------------------
 
+int Utilities::show_message_and_remember(const std::string &title, const std::string &text,
+                                         const std::string &ok, const std::string &cancel,
+                                         const std::string &other,
+                                         const std::string &answer_id, const std::string &checkbox_text)
+{
+  if (remembered_message_answers.find(answer_id) != remembered_message_answers.end())
+    return remembered_message_answers[answer_id];
+
+  bool remember = false;
+  int rc = ControlFactory::get_instance()->_utilities_impl.show_message_with_checkbox(title, text, ok, cancel, other, checkbox_text, remember);
+  if (remember)
+  {
+    remembered_message_answers[answer_id] = rc;
+    save_message_answers();
+  }
+  return rc;
+}
+
+
+void Utilities::set_message_answers_storage_path(const std::string &path)
+{
+  remembered_message_answer_file = path;
+  
+  FILE *f = base_fopen(remembered_message_answer_file.c_str(), "r");
+  if (f)
+  {
+    char line[1024];
+    
+    while (fgets(line, sizeof(line), f))
+    {
+      char *ptr = strrchr(line, '=');
+      if (ptr)
+      {
+        *ptr= 0;
+        remembered_message_answers[line] = atoi(ptr+1);
+      }
+    }
+    fclose(f);
+  }
+}
+
+
+void Utilities::save_message_answers()
+{
+  if (!remembered_message_answer_file.empty())
+  {
+    FILE *f = base_fopen(remembered_message_answer_file.c_str(), "w+");
+
+    for (std::map<std::string, int>::const_iterator iter = remembered_message_answers.begin();
+         iter != remembered_message_answers.end(); ++iter)
+      fprintf(f, "%s=%i\n", iter->first.c_str(), iter->second);
+    fclose(f);
+  }
+}
+
+
+void Utilities::forget_message_answers()
+{
+  remembered_message_answers.clear();
+  save_message_answers();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void Utilities::show_wait_message(const std::string &title, const std::string &text)
 {
   ControlFactory::get_instance()->_utilities_impl.show_wait_message(title, text);
@@ -177,6 +247,7 @@ bool Utilities::request_input(const std::string &title, const std::string &descr
 
 void Utilities::store_password(const std::string &service, const std::string &account, const std::string &password)
 {
+  log_debug("Storing password for '%s'@'%s'", account.c_str(), service.c_str());
   ControlFactory::get_instance()->_utilities_impl.store_password(service, account, password);
 }
 
@@ -184,13 +255,16 @@ void Utilities::store_password(const std::string &service, const std::string &ac
 
 bool Utilities::find_password(const std::string &service, const std::string &account, std::string &password)
 {
-  return ControlFactory::get_instance()->_utilities_impl.find_password(service, account, password);
+  const bool ret = ControlFactory::get_instance()->_utilities_impl.find_password(service, account, password);
+  log_debug("Looking up password for '%s'@'%s' has %s", account.c_str(), service.c_str(), ret ? "succeeded" : "failed");
+  return ret;
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void Utilities::forget_password(const std::string &service, const std::string &account)
 {
+  log_debug("Forgetting password for '%s'@'%s'", account.c_str(), service.c_str());
   ControlFactory::get_instance()->_utilities_impl.forget_password(service, account);
 }
 
@@ -202,7 +276,7 @@ static bool _ask_for_password(const std::string &title, const std::string &servi
   mforms::Form password_form(NULL, (FormFlag) (FormDialogFrame | FormStayOnTop));
   mforms::Table content;
   mforms::ImageBox icon;
-  mforms::Label title_label("");
+  mforms::Label description("");
   mforms::Label service_description_label("");
   mforms::Label service_label("");
   mforms::Label user_description_label("");
@@ -213,7 +287,14 @@ static bool _ask_for_password(const std::string &title, const std::string &servi
   mforms::Button ok_button;
   mforms::Button cancel_button;
   
-  password_form.set_title(title.empty() ? _("MySQL Workbench Authentication") : title);
+  // Since we cannot simply change the function's signature (I only say: python) we have to use
+  // a different way to pass an additional value in. The description used for the login details
+  // request is passed in the title parameter as well, separated by the pipe symbol.
+  std::vector<std::string> title_parts = base::split(title, "|", 2);
+  if (title_parts.size() == 0 || title_parts[0].empty())
+    password_form.set_title(_("MySQL Workbench Authentication"));
+  else
+    password_form.set_title(title_parts[0]);
   
   content.set_padding(12);
   content.set_row_count(6);
@@ -224,11 +305,14 @@ static bool _ask_for_password(const std::string &title, const std::string &servi
   icon.set_image("message_wb_lock.png");
   content.add(&icon, 0, 1, 0, 6, HFillFlag | VFillFlag);
   
-  title_label.set_text(_("Please enter password for the following service:"));
-  title_label.set_wrap_text(true);
-  title_label.set_style(BigBoldStyle);
-  title_label.set_size(250, -1);
-  content.add(&title_label, 1, 3, 0, 1, HFillFlag | HExpandFlag | VFillFlag);
+  if (title_parts.size() < 2 || title_parts[1].empty())
+    description.set_text(_("Please enter password for the following service:"));
+  else
+    description.set_text(title_parts[1]);
+  description.set_wrap_text(true);
+  description.set_style(BigBoldStyle);
+  description.set_size(300, -1);
+  content.add(&description, 1, 3, 0, 1, HFillFlag | HExpandFlag | VFillFlag);
   
   service_description_label.set_text(_("Service:"));
   service_description_label.set_text_align(MiddleRight);
