@@ -382,9 +382,9 @@ void BaseWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int areah
   }
 
   // Erase background behind the description.
-  cairo_set_source_rgb(cr, BK_RED, BK_GREEN, BK_BLUE); // Same as the widget box.
-  cairo_rectangle(cr, 0, get_height() - _description_height, get_width(), _description_height);
-  cairo_fill(cr);
+//  cairo_set_source_rgb(cr, BK_RED, BK_GREEN, BK_BLUE); // Same as the widget box.
+//  cairo_rectangle(cr, 0, get_height() - _description_height, get_width(), _description_height);
+//  cairo_fill(cr);
 
   if (_description != "")
   {
@@ -471,9 +471,9 @@ void BaseWidget::create_context_for_layout()
 
 void WidgetSeparator::repaint(cairo_t *cr, int areax, int areay, int areaw, int areah)
 {
-  cairo_set_source_rgb(cr, BK_RED, BK_GREEN, BK_BLUE); // Same as the widget box.
-  cairo_rectangle(cr, 0, 0, get_width(), get_height());
-  cairo_fill(cr);
+//  cairo_set_source_rgb(cr, BK_RED, BK_GREEN, BK_BLUE); // Same as the widget box.
+//  cairo_rectangle(cr, 0, 0, get_width(), get_height());
+//  cairo_fill(cr);
 
   double offset= get_width() / 2;
 
@@ -495,7 +495,7 @@ void WidgetSeparator::repaint(cairo_t *cr, int areax, int areay, int areaw, int
 WidgetContainer::WidgetContainer(const std::string& title)
 : Box(false), _title(title), _content(true)
 {
-  set_back_color(BACKGROUND_COLOR);
+//  set_back_color(BACKGROUND_COLOR);
 
   set_padding(4);
   //set_spacing(4);
@@ -812,8 +812,8 @@ void ServerInfoWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int
   cairo_save(cr);
 
   // Erase background.
-  cairo_set_source_rgb(cr, BK_RED, BK_GREEN, BK_BLUE);
-  cairo_paint(cr);
+//  cairo_set_source_rgb(cr, BK_RED, BK_GREEN, BK_BLUE);
+//  cairo_paint(cr);
 
   cairo_surface_t* icon;
   std::string status_text;
