@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,8 @@
 #define _WF_BASE_H_
 
 #include <string>
+#include <assert.h>
+
 #include <mforms/base.h>
 #include <mforms/view.h>
 #include "ConvUtils.h"
@@ -39,6 +41,8 @@ struct _cairo_surface
 struct _GMutex
 {};
 
+#pragma make_public(mforms::Object)
+
 using namespace System;
 using namespace Windows::Forms;
 using namespace Collections::Generic;
@@ -49,12 +53,12 @@ using namespace Runtime::InteropServices;
 #define DEFAULT_FONT_SIZE 11
 
 namespace MySQL {
-  
+
 // Callback definitions.
 public ref class MCallback
 {
 public:
-  ~MCallback() {};
+  ~MCallback();
 };
 
 template<typename R> public ref class MCallback0 : public MCallback
@@ -387,6 +391,7 @@ namespace Forms {
     Windows::Forms::Control^ control;
     Windows::Forms::CommonDialog^ dialog;
     Windows::Forms::Menu^ menu;
+    Windows::Forms::ToolStripItem^ toolStripItem;
 
     // Returns a fixed pointer to this object that will not be modified by the GC
     IntPtr GetFixedId()
@@ -409,6 +414,8 @@ namespace Forms {
       dialog= nullptr;
       menu= nullptr;
 
+      toolStripItem = nullptr;
+
       // Get a fixed pointer to this object.
       IntPtr ip = this->GetFixedId();
 
@@ -433,101 +440,141 @@ namespace Forms {
     }
 
   public:
-    static ObjectImpl^ FromUnmanaged(::mforms::Object *object)
+    static ObjectImpl^ FromUnmanaged(const mforms::Object *object)
     {
       return GetFromFixedId((IntPtr)object->get_data_ptr());
     }
 
-      /**
-       * Creates a new instance of a .NET control of type T for the given backend object and returns it.
-       * The backend object pointer is stored in the Tag property of the .NET control for later back reference.
-       * For the same reason the wrapper gets a reference of the .NET control.
-       */
-      template<class T>
-      static T^ create(::mforms::Object* backend, ObjectImpl^ wrapper)
-      {
-        assert(wrapper->dialog == nullptr);
-        assert(wrapper->menu == nullptr);
+    /**
+      * Creates a new instance of a .NET control of type T for the given backend object and returns it.
+      * The backend object pointer is stored in the Tag property of the .NET control for later back reference.
+      * For the same reason the wrapper gets a reference of the .NET control.
+      */
+    template<class T>
+    static T^ create(::mforms::Object* backend, ObjectImpl^ wrapper)
+    {
+      assert(wrapper->dialog == nullptr);
+      assert(wrapper->menu == nullptr);
+      assert(wrapper->toolStripItem == nullptr);
 
-        T^ control= gcnew T();
-        control->Tag= gcnew IntPtr(backend);
-        wrapper->control= control;
+      T^ control= gcnew T();
+      control->Tag= gcnew IntPtr(backend);
+      wrapper->control= control;
 
-        // Do some common setup for the control.
-        control->AutoSize= false;
-        control->Size= control->MinimumSize;
-        control->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE, FontStyle::Regular, GraphicsUnit::Pixel);
+      // Do some common setup for the control.
+      control->AutoSize= false;
+      control->Size= control->MinimumSize;
+      control->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE, FontStyle::Regular, GraphicsUnit::Pixel);
 
-        return control;
-      }
+      return control;
+    }
 
-      template<>
-      static ContextMenu^ create<ContextMenu>(::mforms::Object* backend, ObjectImpl^ wrapper)
-      {
-        assert(wrapper->dialog == nullptr);
-        assert(wrapper->control == nullptr);
+    template<>
+    static ContextMenu^ create<ContextMenu>(::mforms::Object* backend, ObjectImpl^ wrapper)
+    {
+      assert(wrapper->dialog == nullptr);
+      assert(wrapper->control == nullptr);
+      assert(wrapper->toolStripItem == nullptr);
 
-        ContextMenu^ menu= gcnew ContextMenu();
-        menu->Tag= gcnew IntPtr(backend);
-        wrapper->menu= menu;
+      ContextMenu^ menu= gcnew ContextMenu();
+      menu->Tag= gcnew IntPtr(backend);
+      wrapper->menu= menu;
 
-        return menu;
-      }
+      return menu;
+    }
 
-      /**
-       * Creates a new instance of a common dialog of type T for the given backend View and returns it.
-       * Otherwise it is the same as the create<T> function.
-       */
-      template<class T>
-      static T^ create_dialog(::mforms::Object* backend, ObjectImpl^ wrapper)
-      {
-        assert(wrapper->control == nullptr);
-        assert(wrapper->menu == nullptr);
+    template<class T>
+    static T^ create(::mforms::Object* backend, ObjectImpl^ wrapper, ToolStripItem^)
+    {
+      assert(wrapper->dialog == nullptr);
+      assert(wrapper->control == nullptr);
+      assert(wrapper->menu == nullptr);
 
-        T^ dialog= gcnew T();
-        dialog->Tag= gcnew IntPtr(backend);
-        wrapper->dialog= dialog;
+      ToolStripItem^ item= gcnew T();
+      item->Tag= gcnew IntPtr(backend);
+      wrapper->toolStripItem= item;
 
-        return dialog;
-      }
+      return (T^) item;
+    }
 
-      // Returns the .NET control or dialog connected to this wrapper.
-      template<class T>
-      T^ get_control()
-      {
-        if (control != nullptr)
-          return dynamic_cast<T^>(control);
+    /**
+      * Creates a new instance of a common dialog of type T for the given backend View and returns it.
+      * Otherwise it is the same as the create<T> function.
+      */
+    template<class T>
+    static T^ create_dialog(::mforms::Object* backend, ObjectImpl^ wrapper)
+    {
+      assert(wrapper->control == nullptr);
+      assert(wrapper->menu == nullptr);
+      assert(wrapper->toolStripItem == nullptr);
+
+      T^ dialog= gcnew T();
+      dialog->Tag= gcnew IntPtr(backend);
+      wrapper->dialog= dialog;
+
+      return dialog;
+    }
+
+    // Returns the .NET control or dialog connected to this wrapper.
+    template<class T>
+    T^ get_control()
+    {
+      if (control != nullptr)
+        return dynamic_cast<T^>(control);
+      else
+        if (menu != nullptr)
+          return dynamic_cast<T^>(menu);
         else
-          if (menu != nullptr)
-            return dynamic_cast<T^>(menu);
+          if (toolStripItem != nullptr)
+            return dynamic_cast<T^>(toolStripItem);
           else
             return dynamic_cast<T^>(dialog);
-      }
+    }
 
-      // Returns the backend object which is wrapped by the wrapper class the given
-      // control was created for.
-      template<class T>
-      static T* get_backend_control(Object^ object)
+    // This function is the same as get_control() but without a template.
+    // It is used to return the wrapped .NET for other libraries.
+    Object^ get_object()
+    {
+      if (control != nullptr)
+        return control;
+      else
+        if (menu != nullptr)
+          return menu;
+        else
+          if (toolStripItem != nullptr)
+            return toolStripItem;
+          else
+            return dialog;
+    }
+
+    // Returns the backend object which is wrapped by the wrapper class the given
+    // control was created for.
+    template<class T>
+    static T* get_backend_control(Object^ object)
+    {
+      Control^ control= dynamic_cast<Control^>(object);
+      if (control != nullptr)
+        return reinterpret_cast<T*>(((IntPtr^)control->Tag)->ToPointer());
+      else
       {
-        Control^ control= dynamic_cast<Control^>(object);
-        if (control != nullptr)
-          return reinterpret_cast<T*>(((IntPtr^)control->Tag)->ToPointer());
+        ContextMenu^ menu= dynamic_cast<ContextMenu^>(object);
+        if (menu != nullptr)
+          return reinterpret_cast<T*>(((IntPtr^)menu->Tag)->ToPointer());
         else
         {
-          ContextMenu^ menu= dynamic_cast<ContextMenu^>(object);
-          if (menu != nullptr)
-            return reinterpret_cast<T*>(((IntPtr^)menu->Tag)->ToPointer());
+          CommonDialog^ dialog= dynamic_cast<CommonDialog^>(object);
+          if (dialog != nullptr)
+            return reinterpret_cast<T*>(((IntPtr^)dialog->Tag)->ToPointer());
           else
           {
-            CommonDialog^ dialog= dynamic_cast<CommonDialog^>(object);
-            if (dialog != nullptr)
-              return reinterpret_cast<T*>(((IntPtr^)dialog->Tag)->ToPointer());
-            else
-              return NULL;
+            ToolStripItem^ item = dynamic_cast<ToolStripItem^>(object);
+            if (item != nullptr)
+              return reinterpret_cast<T*>(((IntPtr^) item->Tag)->ToPointer());
+            return NULL;
           }
         }
       }
-
+    }
   };
 };
 
