@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -43,12 +43,12 @@ namespace MySQL {
 
     private ref struct ColumnEntry
     {
-      BindableControl^ nodeControl;
+      NodeControl^ nodeControl;
       int column;
       bool editable;
       bool isIcon;
 
-      ColumnEntry(BindableControl^ control, int column, bool editable, bool isIcon)
+      ColumnEntry(NodeControl^ control, int column, bool editable, bool isIcon)
       {
         this->nodeControl= control;
         this->column= column;
@@ -58,14 +58,43 @@ namespace MySQL {
     };
 
     /**
-     * Helper class to forward double clicks for certain tree columns.
+     * Helper class to forward double clicks on the node icon.
      */
-    private ref class SwitchNodeIcon: public NodeIcon
+    private ref class SwitchNodeIcon : public NodeIcon
     {
     public:
       virtual void MouseDoubleClick(TreeNodeAdvMouseEventArgs^ args) override;
     };
 
+    /**
+     * Helper class to show our own expand icons.
+     */
+    private ref class TriangleNodeControl : public NodeControl
+    {
+    private:
+      System::Drawing::Bitmap^ _expanded_icon;
+      System::Drawing::Bitmap^ _collapsed_icon;
+      System::Drawing::Size _size; 
+    public:
+      TriangleNodeControl();
+
+      virtual Drawing::Size MeasureSize(TreeNodeAdv^ node, DrawContext context) override;
+      virtual void Draw(TreeNodeAdv^ node, DrawContext context) override;
+      virtual void MouseDown(TreeNodeAdvMouseEventArgs^ args) override;
+      virtual void MouseDoubleClick(TreeNodeAdvMouseEventArgs^ args) override;
+    };
+
+    /**
+     * Helper class to forward double clicks for non-editable node texts.
+     */
+    private ref class SwitchNodeText : public NodeTextBox
+    {
+    protected:
+      virtual void OnDrawText(DrawEventArgs^ args) override;
+    public:
+      virtual void MouseDoubleClick(TreeNodeAdvMouseEventArgs^ args) override;
+    };
+
     /**
      * Wrapper for a native .NET treeview control with GRT bec::TreeModel data.
      * We use the TreeViewAdv control as we need multicolumn support.
@@ -77,27 +106,28 @@ namespace MySQL {
       MySQL::Grt::SimpleGrtTreeModel^ _model;
       List<ColumnEntry^> _columns;
 
-      void add_node_control(BindableControl^ control, int column, bool editable, bool isIcon);
+      void add_node_control(NodeControl^ control, int column, bool editable, bool isIcon);
 
       // Event handlers.
       static void SelectionChanged(System::Object^ sender, EventArgs^ e);
       static void OnMouseDown(System::Object^ sender, MouseEventArgs^ e);
 
     protected:
-      static bool create(::mforms::GRTTreeView *self, bool show_headers);
+      static bool create(::mforms::GRTTreeView *self, mforms::TreeOptions options);
 
-      static void set_model(::mforms::GRTTreeView*, bec::TreeModel*);
+      static void set_model(::mforms::GRTTreeView* self, bec::TreeModel* model);
 
       static int add_column(::mforms::GRTTreeView *self, ::mforms::GRTTreeColumnType type, int model_column, const std::string &title);
       static int add_column_editable(::mforms::GRTTreeView *self, ::mforms::GRTTreeColumnType type, int model_column, const std::string &title);
 
-      static void refresh(mforms::GRTTreeView*, const bec::NodeId &);
-      static void set_column_width(mforms::GRTTreeView*, int, int);
-      static bool get_selected_node(mforms::GRTTreeView*, bec::NodeId&);
-      static int get_selection(mforms::GRTTreeView*, std::vector<bec::NodeId>&);
-      static void set_allow_multi_selection(::mforms::GRTTreeView *self, bool flag);
+      static void refresh(mforms::GRTTreeView* self, const bec::NodeId& node);
+      static void set_column_width(mforms::GRTTreeView* self, int column, int width);
+      static bool get_selected_node(mforms::GRTTreeView* self, bec::NodeId& node);
+      static int get_selection(mforms::GRTTreeView* self, std::vector<bec::NodeId>& nodes);
+      static void set_expanded(mforms::GRTTreeView* self, const bec::NodeId& node, bool expanded);
 
-      static void row_count_changed(::mforms::GRTTreeView *self, int old_count);
+      static void set_allow_multi_selection(mforms::GRTTreeView *self, bool flag);
+      static void row_count_changed(mforms::GRTTreeView *self, const bec::NodeId& node, int old_count);
 
       void set_model(bec::TreeModel*);
     public:
@@ -110,7 +140,7 @@ namespace MySQL {
       {
         ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();
 
-        DEF_CALLBACK2(bool, ::mforms::GRTTreeView*, bool, mgr, f->_grttreeview_impl, GRTTreeViewImpl, create);
+        DEF_CALLBACK2(bool, ::mforms::GRTTreeView*, mforms::TreeOptions, mgr, f->_grttreeview_impl, GRTTreeViewImpl, create);
         DEF_CALLBACK2(void, ::mforms::GRTTreeView*, bec::TreeModel*, mgr, f->_grttreeview_impl, GRTTreeViewImpl, set_model);
         DEF_CALLBACK4(int, ::mforms::GRTTreeView*, ::mforms::GRTTreeColumnType, int, const std::string&, mgr, f->_grttreeview_impl, GRTTreeViewImpl, add_column);
         DEF_CALLBACK4(int, ::mforms::GRTTreeView*, ::mforms::GRTTreeColumnType, int, const std::string&, mgr, f->_grttreeview_impl, GRTTreeViewImpl, add_column_editable);
@@ -119,8 +149,8 @@ namespace MySQL {
         DEF_CALLBACK2(bool, ::mforms::GRTTreeView*, bec::NodeId&, mgr, f->_grttreeview_impl, GRTTreeViewImpl, get_selected_node);
         DEF_CALLBACK2(int, ::mforms::GRTTreeView*, std::vector<bec::NodeId>&, mgr, f->_grttreeview_impl, GRTTreeViewImpl, get_selection);
         DEF_CALLBACK2(void, ::mforms::GRTTreeView*, bool, mgr, f->_grttreeview_impl, GRTTreeViewImpl, set_allow_multi_selection);
-        DEF_CALLBACK2(void, ::mforms::GRTTreeView*, int, mgr, f->_grttreeview_impl, GRTTreeViewImpl, row_count_changed);
-        
+        DEF_CALLBACK3(void, ::mforms::GRTTreeView*, const bec::NodeId&, int, mgr, f->_grttreeview_impl, GRTTreeViewImpl, row_count_changed);
+        DEF_CALLBACK3(void, ::mforms::GRTTreeView*, const bec::NodeId&, bool, mgr, f->_grttreeview_impl, GRTTreeViewImpl, set_expanded);
       }
     };
   }
