@@ -124,6 +124,11 @@ namespace MySQL {
           OnStructureChanged(gcnew TreePathEventArgs(TreePath::Empty));
         }
       }
+
+      void Sort()
+      {
+        OnStructureChanged(gcnew TreePathEventArgs(TreePath::Empty));
+      }
     };
 
     /**
