@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,6 +32,14 @@ using namespace Windows::Forms;
 namespace MySQL {
   namespace Forms {
 
+    // Message type for the C# interface.
+    public enum class MessageType
+    {
+      MessageInfo,
+      MessageWarning,
+      MessageError
+    };
+
     /**
      * A custom message box for the output as there is no predefined dialog which allows to
      * have custom button captions.
@@ -48,11 +56,27 @@ namespace MySQL {
       Windows::Forms::Button^ _button2;
       Windows::Forms::Button^ _button3;
       Windows::Forms::PictureBox^ _picture;
+      Windows::Forms::CheckBox^ _checkbox;
 
       void ComputeLayout();
       void ButtonClick(System::Object^ sender, EventArgs^ e);
+
+      static mforms::DialogResult ShowVistaStyle(const std::string& title, const std::string& text,
+        PCWSTR mainIcon, const std::string& buttonOK, const std::string& buttonCancel,
+        const std::string& buttonOther, const std::string& checkbox, bool& checked);
+
+      static mforms::DialogResult ShowTraditionalStyle(String^ title, String^ text, Drawing::Image^ image,
+        String^ button1, String^ button2, String^ button3, String^ checkbox, bool& checked);
     public:
-      static mforms::DialogResult Show(String^ title, String^ text, Drawing::Image^ image, String^ button1, String^ button2, String^ button3);
+      // C++ interface
+      static mforms::DialogResult Show(const std::string& title, const std::string& text,
+        PCWSTR mainIcon, const std::string& buttonOK, const std::string& buttonCancel,
+        const std::string& buttonOther, const std::string& checkbox, bool& checked);
+
+      // C# interface
+      static Windows::Forms::DialogResult Show(MessageType type, String^ title, String^ text, String^ buttonOK,
+        String^ buttonCancel, String^ buttonOther, String^ checkbox, bool& checked);
+      static Windows::Forms::DialogResult Show(MessageType type, String^ title, String^ text, String^ buttonOK);
     };
 
     public ref class UtilitiesImpl
@@ -69,6 +93,8 @@ namespace MySQL {
         const string &cancel, const string &other);
       static int show_warning(const string &title, const string &text, const string &ok, 
         const string &cancel, const string &other);
+      static int show_message_with_checkbox(const string &title, const string &text, const string &ok, 
+        const string &cancel, const string &other, const std::string &checkbox_text, bool &isChecked);
       static void show_wait_message(const string &title, const string &text);
       static bool hide_wait_message();
 
@@ -95,6 +121,8 @@ namespace MySQL {
           mgr, f->_utilities_impl, UtilitiesImpl, show_error);
         DEF_CALLBACK5(int, const string&, const string&, const string&, const string&, const string&, 
           mgr, f->_utilities_impl, UtilitiesImpl, show_warning);
+        DEF_CALLBACK7(int, const string&, const string&, const string&, const string&, const string&, 
+          const string&, bool&, mgr, f->_utilities_impl, UtilitiesImpl, show_message_with_checkbox);
         DEF_CALLBACK2(void, const string&, const string&, mgr, f->_utilities_impl, UtilitiesImpl,
           show_wait_message);
         DEF_CALLBACK0(bool, mgr, f->_utilities_impl, UtilitiesImpl, hide_wait_message);
