@@ -1,20 +1,20 @@
 /* 
- * © 2008 MySQL AB, 2008-2009 Sun Microsystems, Inc.
+ * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -28,12 +28,12 @@
 
 #include "grtpp_util.h"
 #include "grts/structs.h"
-#include "util_functions.h"
+#include "base/util_functions.h"
 
 namespace grt
 {
 
-DiffChange *diff_make(const ValueRef &source, const ValueRef &target, const Omf *omf,const sigc::slot<bool, ValueRef , ValueRef ,std::string> sqlDefinitionCmp)
+DiffChange *diff_make(const ValueRef &source, const ValueRef &target, const Omf *omf,const TSlotNormalizerSlot sqlDefinitionCmp)
 {
   return GrtDiff().diff(source, target, omf, sqlDefinitionCmp);
 }
@@ -250,8 +250,8 @@ DiffChange *GrtDiff::on_object(DiffChange *parent, const ObjectRef &source, cons
 
           changes.append(change);
 #if 0
-         if (name1 == "indices")
-            continue;
+//         if (name1 == "indices")
+//            continue;
           if (name1 == "formattedRawType")
           {
             //          std::cout<<"v1("<<GrtObjectRef::cast_from(target)->name().c_str()<<") v2("<<GrtObjectRef::cast_from(source)->name().c_str()<<")";
