@@ -19,17 +19,17 @@
 
 #include "stdafx.h"
 
+#include "grtpp_shell_python.h"
 #include "grtpp.h"
 #include "grtpp_util.h"
 #include "grtpp_shell.h"
 #include "grtpp_module_cpp.h"
 #include "grtpp_undo_manager.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #include <algorithm>
 
 #include "grtpp_shell_lua.h"
-#include "grtpp_shell_python.h"
 
 #include <glib.h>
 
