@@ -23,8 +23,8 @@
 #include "grtpp.h"
 #include "grtpp_module_cpp.h"
 #include "grtpp_util.h"
-#include "string_utilities.h"
-#include "file_functions.h"
+#include "base/string_utilities.h"
+#include "base/file_functions.h"
 
 using namespace grt;
 
