@@ -21,8 +21,8 @@
 
 #include "grtpp_module_python.h"
 #include "grtpp_util.h"
-#include "string_utilities.h"
-#include "file_functions.h"
+#include "base/string_utilities.h"
+#include "base/file_functions.h"
 
 #ifdef ENABLE_PYTHON_MODULES
 
@@ -444,23 +444,13 @@ bool PythonModuleLoader::load_library(const std::string &file)
 
 bool PythonModuleLoader::run_script_file(const std::string &path)
 {
-  FILE *f= base_fopen(path.c_str(), "r");
-  if (!f)
+  if (!g_file_test(path.c_str(), G_FILE_TEST_EXISTS))
     return false;
-  
+
   WillEnterPython lock;
   
-  // execute in the global environment will make it available to the interactive shell
-  if (PyRun_SimpleFile(f, path.c_str()) < 0)
-  {
-    fclose(f);
-    PyErr_Print();
-    return false;
-  }
-  
-  fclose(f);
-  
-  return true;
+  // Execute in the global environment will make it available to the interactive shell.
+  return _pycontext.run_file(path, true) == 0;
 }
 
 bool PythonModuleLoader::run_script(const std::string &script)
