@@ -29,7 +29,7 @@
 #ifdef ENABLE_LUA_MODULES
 
 #include "grtpp_shell_lua.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #include <math.h>
 
