@@ -1,41 +1,47 @@
 /* 
- * (c) 2007-2008 MySQL AB, 2008-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 
 #include "stdafx.h"
 
-#include "grtpp_util.h"
-#include "string_utilities.h"
+#include "grtpp_module_python.h"
 
-#if defined(_WIN32)
-#include <objbase.h>
-#elif defined(__APPLE__)
-#include <CoreFoundation/CoreFoundation.h>
-#else
-// Linux specific
-#include <uuid/uuid.h>
-#endif
+#include "grtpp_util.h"
+#include "base/string_utilities.h"
+
+#include <glib/gbase64.h>
 
 using namespace grt;
 
+#ifdef HAVE_CONFIG_H
+# include "../../../config.h"
+#endif
 
 
+#if defined(HAVE_LIBUUID) || defined(__APPLE__)
+
+#ifdef __APPLE__
+# include <CoreFoundation/CoreFoundation.h>
+#else
+# include <uuid/uuid.h>
+#endif
+
 std::string grt::get_guid()
 {
   /* GUIDs must be no more than 50 chars */
@@ -86,6 +92,61 @@ std::string grt::get_guid()
 }
 
 
+#else
+#include <boost/uuid/uuid.hpp>
+#include <boost/uuid/uuid_generators.hpp>
+
+// not really base64 alphabet, since base64 uses /, which we dont want since the guid's are also used as filenames
+static const char base64_chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+=";
+
+static boost::mt19937 random_gen;
+static boost::uuids::basic_random_generator<boost::mt19937> uuid_gen(&random_gen);
+static GStaticMutex uuid_gen_mutex = G_STATIC_MUTEX_INIT;
+
+std::string grt::get_guid()
+{
+  g_static_mutex_lock(&uuid_gen_mutex);
+  boost::uuids::uuid uuid = uuid_gen();
+  g_static_mutex_unlock(&uuid_gen_mutex);
+  
+  const unsigned char *ptr = (const unsigned char*)&uuid;
+ 
+  // 16 bytes encoded as base64 = 21 bytes + 2 bytes for {}
+  std::string uuid_str(23, ' ');
+
+  uuid_str[0] = '{';
+  uuid_str[1] = base64_chars[ptr[0] >> 2];
+  uuid_str[2] = base64_chars[((ptr[0] & 0x03) << 4) | (ptr[0+1] >> 4)];
+  uuid_str[3] = base64_chars[((ptr[0+1] & 0x0f) << 2) | (ptr[0+2] >> 6)];
+  uuid_str[4] = base64_chars[ptr[0+2] & 0x3f];
+  
+  uuid_str[5] = base64_chars[ptr[3] >> 2];
+  uuid_str[6] = base64_chars[((ptr[3] & 0x03) << 4) | (ptr[3+1] >> 4)];
+  uuid_str[7] = base64_chars[((ptr[3+1] & 0x0f) << 2) | (ptr[3+2] >> 6)];
+  uuid_str[8] = base64_chars[ptr[3+2] & 0x3f];
+  
+  uuid_str[9] = base64_chars[ptr[6] >> 2];
+  uuid_str[10] = base64_chars[((ptr[6] & 0x03) << 4) | (ptr[6+1] >> 4)];
+  uuid_str[11] = base64_chars[((ptr[6+1] & 0x0f) << 2) | (ptr[6+2] >> 6)];
+  uuid_str[12] = base64_chars[ptr[6+2] & 0x3f];
+  
+  uuid_str[13] = base64_chars[ptr[9] >> 2];
+  uuid_str[14] = base64_chars[((ptr[9] & 0x03) << 4) | (ptr[9+1] >> 4)];
+  uuid_str[15] = base64_chars[((ptr[9+1] & 0x0f) << 2) | (ptr[9+2] >> 6)];
+  uuid_str[16] = base64_chars[ptr[9+2] & 0x3f];
+  
+  uuid_str[17] = base64_chars[ptr[12] >> 2];
+  uuid_str[18] = base64_chars[((ptr[12] & 0x03) << 4) | (ptr[12+1] >> 4)];
+  uuid_str[19] = base64_chars[((ptr[12+1] & 0x0f) << 2) | (ptr[12+2] >> 6)];
+  uuid_str[20] = base64_chars[ptr[12+2] & 0x3f];
+  
+  uuid_str[21] = base64_chars[ptr[15] >> 2];
+  uuid_str[22] = '}';
+
+  return uuid_str;
+}
+#endif // !HAVE_LIBUUID
+
 std::string grt::fmt_simple_type_spec(const SimpleTypeSpec &type)
 {
   switch (type.type)
@@ -1003,7 +1064,6 @@ void grt::remove_list_items_matching(ObjectListRef list, const sigc::slot<bool,g
 
 
 // temporary code, should be replaced with dynamic loading once langauge support is pluginized
-#include "grtpp_module_python.h"
 
 bool grt::init_python_support(grt::GRT *grt, const std::string &module_path)
 {
@@ -1011,9 +1071,9 @@ bool grt::init_python_support(grt::GRT *grt, const std::string &module_path)
   PythonModuleLoader *loader= new PythonModuleLoader(grt);
   if (!module_path.empty())
   {
-    loader->get_python_context()->add_module_path(module_path);
+    loader->get_python_context()->add_module_path(module_path, true);
 #ifdef _WIN32
-    loader->get_python_context()->add_module_path(module_path + "/python/site-packages");
+    loader->get_python_context()->add_module_path(module_path + "/python/site-packages", true);
 #endif
   }
   grt->add_module_loader(loader);
