@@ -22,7 +22,7 @@
 #include "grtpp.h"
 #include "grtpp_util.h"
 #include "grtpp_undo_manager.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #include <glib.h>
 
@@ -922,6 +922,7 @@ bool process_reset_references_for_member(const MetaClass::Member* m, Object* obj
       if (m->owned_object)
         member_value.valueptr()->reset_references();
       
+      obj->signal_changed().clear();
       // set the member value to null
       obj->get_metaclass()->set_member_internal(obj, m->name, grt::ValueRef(), true);
     }
