@@ -1,20 +1,20 @@
 /* 
- * (c) 2007-2008 MySQL AB, 2008-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 
@@ -22,12 +22,12 @@
 
 #include "lua_context.h"
 
-#include "util_functions.h"
+#include "base/util_functions.h"
 
 #include "grtpp_shell.h"
 #include "grtpp_util.h"
-#include "string_utilities.h"
-#include "file_functions.h"
+#include "base/string_utilities.h"
+#include "base/file_functions.h"
 
 #include <lauxlib.h>
 #include <lualib.h>
@@ -1326,7 +1326,7 @@ static const char* read_lua_file(lua_State* state, void* data, size_t* size)
     // Since we load the whole file in one run, if we come here, we are done and can close the file.
     fclose(file_data->file);
     file_data->file= NULL;
-    delete file_data->data;
+    delete []file_data->data;
     file_data->data= 0;
     
     *size= 0;
@@ -1800,11 +1800,11 @@ static int l_set_global(lua_State *l)
   }
   else
   {
-    if (strcmp2(path, "/") == 0)
+    if (path && strcmp(path, "/") == 0)
     {
       ctx->get_grt()->set_root(value);
     }
-    else
+    else if (path)
     {
       std::string fpath;
 
