@@ -23,8 +23,8 @@
 #include "python_context.h"
 #include "grtpp.h"
 #include "grtpp_util.h"
-#include "string_utilities.h"
-#include "file_functions.h"
+#include "base/string_utilities.h"
+#include "base/file_functions.h"
 
 #ifdef ENABLE_PYTHON_MODULES
 
@@ -48,6 +48,9 @@
 #include "python_grtlist.h"
 #include "python_grtdict.h"
 
+#include "base/log.h"
+ENABLE_LOG("grt")
+
 using namespace grt;
 using namespace base;
 
@@ -114,7 +117,7 @@ PythonContext::~PythonContext()
 }
 
 
-void PythonContext::add_module_path(const std::string &modpath)
+void PythonContext::add_module_path(const std::string &modpath, bool prepend)
 {
   // add the path to the search path so that it can be imported
   PyObject *path_list;
@@ -134,8 +137,12 @@ void PythonContext::add_module_path(const std::string &modpath)
   }
   
   if (i < 0) // not found
-    PyList_Append(path_list, path);
-  
+  {
+    if (prepend)
+      PyList_Insert(path_list, 0, path);
+    else
+      PyList_Append(path_list, path);
+  }
   Py_DECREF(path);
 }
 
@@ -228,6 +235,7 @@ static PyObject *grt_print(PyObject *self, PyObject *args)
 #else
   g_print("%s", text.c_str()); // g_print is not routed to g_log
 #endif
+  base::Logger::log(base::Logger::LogInfo, "grt.python", text.c_str());
   ctx->get_grt()->send_output(text);
   
   Py_INCREF(Py_None);
@@ -915,17 +923,22 @@ ValueRef PythonContext::from_pyobject(PyObject *object, const grt::TypeSpec &exp
 
 int PythonContext::run_file(const std::string &file, bool interactive)
 {
-  FILE *f= base_fopen(file.c_str(), "r");
+  PyObject *f = PyFile_FromString((char*)base::string_to_path(file).c_str(), (char*)"r");
   if (!f)
-    return -1;
-  
-  if (PyRun_SimpleFile(f, file.c_str()) != 0)
   {
-    fclose(f);
+    log_error("Could not open file %s", file.c_str());
     PyErr_Print();
     return -1;
   }
-  fclose(f);
+
+  log_debug2("About to pyrun '%s'", file.c_str());
+  if (PyRun_SimpleFile(PyFile_AsFile(f), file.c_str()) != 0)
+  {
+    Py_DECREF(f);
+    PyErr_Print();
+    return -1;
+  }
+  Py_DECREF(f);
   
   return 0;
 }
