@@ -1,27 +1,26 @@
 /* 
  * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
- * Boston, MA 02110-1301  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 
 #ifndef _GRT_PYTHON_CONTEXT_H_
 #define _GRT_PYTHON_CONTEXT_H_
 
-#include "grtpp.h"
 
 #ifdef ENABLE_PYTHON_MODULES
 
@@ -31,6 +30,8 @@
 # include <Python.h>
 #endif
 
+#include "grtpp.h"
+
 #if (PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION < 5)
 // stuff that doesn't exist in 2.4
 typedef int Py_ssize_t;
@@ -54,10 +55,11 @@ namespace grt {
     {
     }
 
-    AutoPyObject(PyObject *py)
+    AutoPyObject(PyObject *py, bool retain=true)
     : object(py)
     {
-      Py_INCREF(object);
+      if (retain)
+        Py_XINCREF(object);
     }
     
     ~AutoPyObject()
@@ -74,10 +76,20 @@ namespace grt {
       return *this;
     }
     
+    operator bool()
+    {
+      return object != 0;
+    }
+    
     operator PyObject*()
     {
       return object;
-    }    
+    }
+    
+    PyObject* operator->()
+    {
+      return object;
+    }
   };
   
   class MYSQLGRT_PUBLIC PythonContext
@@ -92,7 +104,7 @@ namespace grt {
     static PyObject *internal_cobject_from_value(const ValueRef &value);
     static ValueRef value_from_internal_cobject(PyObject *value);
 
-    void add_module_path(const std::string &path);
+    void add_module_path(const std::string &path, bool prepend = false);
     bool import_module(const std::string &name);
     
     PyObject *from_grt(const ValueRef &value);
@@ -111,18 +123,20 @@ namespace grt {
     PyObject *get_grt_module();
     
     PyObject *get_global(const std::string &value);
-    
+    bool set_global(const std::string &name, PyObject *value);
+        
     int refresh();
         
     GRT *get_grt() const { return _grt; }
     
     bool set_cwd(const std::string &path);
     std::string get_cwd() const { return _cwd; }
-        
+
+    sigc::slot<std::string> stdin_readline_slot;
     
-    static void set_python_error(const grt::type_error &exc);
-    static void set_python_error(const grt::bad_item &exc);
-    static void set_python_error(const std::exception &exc);
+    static void set_python_error(const grt::type_error &exc, const std::string &location="");
+    static void set_python_error(const grt::bad_item &exc, const std::string &location="");
+    static void set_python_error(const std::exception &exc, const std::string &location="");
     
   protected:
     GRT *_grt;    
