@@ -24,7 +24,7 @@
 #include "grtsqlparser/module_utils.h"
 #include "mysql_sql_parser_utils.h"
 #include "grtdb/db_object_helpers.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include <sstream>
 #include <iterator>
 
