@@ -27,7 +27,7 @@
 #include "grts/structs.db.mysql.h"
 #include "grtdb/charset_utils.h"
 
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace grt;
 
