@@ -23,7 +23,7 @@
 #include "mysql_sql_syntax_check.h"
 #include "mysql_sql_parser_fe.h"
 #include "grtdb/charset_utils.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 
 using namespace grt;
