@@ -284,15 +284,19 @@ void DiffSQLGeneratorBE::generate_create_stmt(db_mysql_TableRef table)
     {
       db_mysql_IndexRef index= indices.get(i);
 
-      bool separate_index= _gen_create_index && (index->isPrimary() == 0);
-      // not sure if indices for FKs should be skipped as well when skip FKs is requested
-      // maybe someone with MyISAM still uses these indices for something? --alfredo 10/04/16
-      if (_skip_foreign_keys && (stricmp(index->indexType().c_str(), "FOREIGN") == 0
+      if (index->isPrimary())
+      {
+        callback->create_table_index(index, false);
+        continue;      
+      }
+      
+      if (_skip_fk_indexes && (stricmp(index->indexType().c_str(), "FOREIGN") == 0
                                  || fk_indexes.find(index) != fk_indexes.end()))
         continue;
+
       if ((fk_skip_indexes.find(index) != fk_skip_indexes.end())&&(fk_create_indexes.find(index) == fk_create_indexes.end()))
         continue;
-      callback->create_table_index(index, separate_index);
+      callback->create_table_index(index, _gen_create_index);
     }
 
     callback->create_table_indexes_end(table);
@@ -717,6 +721,9 @@ void DiffSQLGeneratorBE::generate_alter(grt::ListRef<db_mysql_Index> indices, co
     case grt::ListItemOrderChanged: // DROP/ADD INDEX
       {
         const grt::ListItemOrderChange *order_change= static_cast<const grt::ListItemOrderChange *>(index_change);
+        //Only position changed nothing to do
+        if (!order_change->get_subchange())
+          break;
         callback->alter_table_drop_index(db_mysql_IndexRef::cast_from(order_change->get_old_value()));
         callback->alter_table_add_index(db_mysql_IndexRef::cast_from(order_change->get_new_value()));
       }
@@ -1542,7 +1549,7 @@ static void fill_set_from_list(grt::StringListRef string_list, std::set<std::str
 }
 
 DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, DiffSQLGeneratorBEActionInterface *cb)
-  : padding(2), callback(cb), _gen_create_index(false), _use_filtered_lists(true), _skip_foreign_keys(false)
+  : padding(2), callback(cb), _gen_create_index(false), _use_filtered_lists(true), _skip_foreign_keys(false), _skip_fk_indexes(false)
 {
   if(options.is_valid())
   {
@@ -1552,8 +1559,12 @@ DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, DiffSQLGeneratorBEA
       grt::IntegerRef SkipForeignKeys= grt::IntegerRef::cast_from(options.get("SkipForeignKeys"));
       if(SkipForeignKeys.is_valid())
         _skip_foreign_keys = (SkipForeignKeys.operator long() != 0);
+    }if(options.has_key("SkipFKIndexes"))
+    {
+      grt::IntegerRef skip_fk_indexes= grt::IntegerRef::cast_from(options.get("SkipFKIndexes"));
+      if(skip_fk_indexes.is_valid())
+        _skip_fk_indexes = (skip_fk_indexes.operator long() != 0);
     }
-
     if(options.has_key("UseFilteredLists"))
     {
       grt::IntegerRef use_filtered_lists= grt::IntegerRef::cast_from(options.get("UseFilteredLists"));
