@@ -35,7 +35,7 @@
 
 #include "db_mysql_diffsqlgen_grant.h"
 #include "db_mysql_catalog_report.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 #include "grtsqlparser/sql_specifics.h"
 #include "sqlide/recordset_table_inserts_storage.h"
@@ -288,12 +288,8 @@ static std::string generate_create(db_mysql_IndexRef index, std::string table_q_
     index_sql.append("INDEX ");
   }
   
-  if(!pk)
-  {
-    const char *name= index->name().c_str();
-    if (name && ('\0' != name[0]))
-      index_sql.append(strfmt("`%s` ", name));
-  }
+  if (!pk && !index->name().empty())
+    index_sql.append(strfmt("`%s` ", index->name().c_str()));
 
   if(strlen(index->indexKind().c_str()))
     index_sql.append("USING ").append(index->indexKind().c_str()).append(" ");
@@ -415,11 +411,10 @@ static std::string generate_drop_index(db_mysql_IndexRef index)
 
   std::string index_name;
   {
-    const char *name= index->oldName().c_str();
-    if (name && ('\0' != name[0]))
-      index_name= strfmt("`%s` ", name);
-    else
+    if (index->oldName().empty())
       index_name= " ";
+    else
+      index_name= strfmt("`%s` ", index->oldName().c_str());
   }    
   return strfmt("DROP INDEX %s", index_name.c_str());
 }
@@ -757,7 +752,12 @@ void ActionGenerateSQL::create_table_max_rows(grt::StringRef value)
 
 void ActionGenerateSQL::create_table_comment(grt::StringRef value)
 {
-  sql.append("\nCOMMENT = '").append(escape_sql_string(value).c_str()).append("'");
+  std::string comment = escape_sql_string(bec::TableHelper::get_sync_comment(value)).append("'");
+  if (!bec::TableHelper::get_document_comment(value).empty())
+  {
+    comment.append(" /* comment truncated */");  
+  };
+  alter_table_property(sql, "\nCOMMENT = '", comment);
 }
 
 void ActionGenerateSQL::create_table_data_dir(grt::StringRef value)
@@ -908,7 +908,12 @@ void ActionGenerateSQL::alter_table_collate(db_mysql_TableRef table, grt::String
 
 void ActionGenerateSQL::alter_table_comment(db_mysql_TableRef table, grt::StringRef str)
 {
-  alter_table_property(sql, "COMMENT = '", escape_sql_string(str.c_str()).append("'"));
+  std::string comment = escape_sql_string(bec::TableHelper::get_sync_comment(str)).append("'");
+  if (!bec::TableHelper::get_document_comment(str).empty())
+  {
+    comment.append(" /* comment truncated */");  
+  };
+  alter_table_property(sql, "\nCOMMENT = '", comment);
 }
 
 void ActionGenerateSQL::alter_table_merge_union(db_mysql_TableRef table, grt::StringRef str)
@@ -1406,9 +1411,11 @@ void ActionGenerateSQL::create_routine(db_mysql_RoutineRef routine, bool for_alt
 {
 
   std::string routine_sql;
-//  if(!_use_short_names)
-  routine_sql = "\nDELIMITER ";
-  routine_sql.append(_non_std_sql_delimiter).append("\n");
+  if(!_use_short_names)
+  {
+    routine_sql = "\nDELIMITER ";
+    routine_sql.append(_non_std_sql_delimiter).append("\n");
+  }
   if (!_use_short_names || _gen_use)
   {
     routine_sql.append("USE `");
@@ -1423,8 +1430,8 @@ void ActionGenerateSQL::create_routine(db_mysql_RoutineRef routine, bool for_alt
       renamer->rename_schema_references(routine_sql, routine->owner()->name(), "");
   }
   
-//  if(!_use_short_names)
-  routine_sql.append("\nDELIMITER ;");
+  if(!_use_short_names)
+    routine_sql.append("\nDELIMITER ;\n");
 
   //remove_delims(routine_sql);
   if(for_alter)
@@ -1555,12 +1562,14 @@ int DbMySQLImpl::generateSQL(GrtNamedObjectRef org_object,
   
   if(result.type() == DictType)
   {
-    DiffSQLGeneratorBE(options, new ActionGenerateSQL(result, obj_list, get_grt()))
+    ActionGenerateSQL generator = ActionGenerateSQL(result, obj_list, get_grt());
+    DiffSQLGeneratorBE(options, &generator)
       .process_diff_change(org_object, p, grt::DictRef::cast_from(result));
   }
   else if(result.type() == ListType)
   {
-    DiffSQLGeneratorBE(options, new ActionGenerateSQL(result, obj_list, get_grt()))
+    ActionGenerateSQL generator = ActionGenerateSQL(result, obj_list, get_grt());
+    DiffSQLGeneratorBE(options, &generator)
       .process_diff_change(org_object, p, grt::StringListRef::cast_from(result), obj_list);
   }
 
@@ -1864,7 +1873,7 @@ static std::string get_export_sql(db_mysql_CatalogRef cat,
           inserts_sql
             .append("\n-- -----------------------------------------------------\n-- Data for table ")
             .append(get_name(table,use_short_names))
-            .append("\n-- -----------------------------------------------------\nSET AUTOCOMMIT=0;\n")
+            .append("\n-- -----------------------------------------------------\nSTART TRANSACTION;\n")
             .append(use_code)
             .append(table_inserts_sql)
             .append("\nCOMMIT;\n");
@@ -2282,12 +2291,9 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
 
   if (diff)
   {
-    ActionGenerateSQL* generator = new ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>(),get_grt());
-    generator->set_put_if_exists(false);
-    DiffSQLGeneratorBE *gen= new DiffSQLGeneratorBE(
-      options, generator);
-
-    gen->process_diff_change(source, diff, result);
+    ActionGenerateSQL generator = ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>(),get_grt());
+    generator.set_put_if_exists(false);
+    DiffSQLGeneratorBE(options, &generator).process_diff_change(source, diff, result);
     delete diff;
     std::string objname = get_old_object_name_for_key(obj);
     ValueRef change = result.get(objname, StringRef(""));
@@ -2432,10 +2438,8 @@ std::string DbMySQLImpl::makeCreateScriptForObject(GrtNamedObjectRef object)
 
   if (diff)
   {
-    DiffSQLGeneratorBE *gen= new DiffSQLGeneratorBE(
-      options, new ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>(),get_grt()));
-
-    gen->process_diff_change(ValueRef(), diff, result);
+    ActionGenerateSQL generator = ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>(),get_grt());
+    DiffSQLGeneratorBE(options, &generator).process_diff_change(ValueRef(), diff, result);
     delete diff;
     sql= result.get_string(get_full_object_name_for_key(object), "");
   }
@@ -2446,7 +2450,9 @@ std::string DbMySQLImpl::makeCreateScriptForObject(GrtNamedObjectRef object)
 
 grt::ListRef<db_mysql_StorageEngine> DbMySQLImpl::getKnownEngines()
 {
-  return dbmysql::get_known_engines(this->get_grt());
+  if (!_known_engines.is_valid())
+    _known_engines = dbmysql::get_known_engines(this->get_grt());
+  return _known_engines;
 }
 
 
