@@ -243,6 +243,9 @@ public:
   virtual grt::ListRef<db_mysql_StorageEngine> getKnownEngines();
   
   grt::ListRef<db_UserDatatype> getDefaultUserDatatypes(db_mgmt_RdbmsRef rdbms);
+  
+private:
+  grt::ListRef<db_mysql_StorageEngine> _known_engines;
 };
 
 #endif // _MODULE_DB_MYSQL_H_
