@@ -38,7 +38,7 @@
 #include "grtsqlparser/module_utils.h"
 #include "sqlide/table_inserts_loader_be.h"
 #include "grts/structs.workbench.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace base;
 
