@@ -103,6 +103,20 @@ def upperCaseCell(rs, row, column):
 #  return 0
 
 
+
+class ASTHelper:
+    def __init__(self, text):
+        self.max_range = len(text)
+
+    def get_ast_range(self, ast):
+        offset = ast[3]
+        b = ast[4]+offset if ast[4] is not None else self.max_range
+        e = ast[5]+offset if ast[5] is not None else 0
+        for c in ast[2]:
+            b_, e_ = self.get_ast_range(c)
+            b = min(b_, b)
+            e = max(e_, e)
+        return b, e        
   
 
 @ModuleInfo.plugin("wb.sqlide.enbeautificate", caption = "Reformat SQL Query", input=[wbinputs.currentQueryBuffer()], pluginMenu="SQL/Utilities")
@@ -116,29 +130,30 @@ def enbeautificate(editor):
         selectionOnly = False
         text = editor.script
 
-    def get_ast_range(ast):
-        b = ast[4]
-        e = ast[5]
-        for c in ast[2]:
-            b_, e_ = get_ast_range(c)
-            b = b_ if b is None else min(b_, b)
-            e = max(e_, e)
-        return b, e
+    helper = ASTHelper(text)
+
+    ok_count = 0
+    bad_count = 0
     
     curpos = 0
     new_text = ""
     ast_list = MysqlSqlFacade.parseAstFromSqlScript(text)
     for ast in ast_list:
-        s, v, c, _line, _begin, _end = ast
-        begin, end = get_ast_range(ast)
-        new_text += text[curpos:begin]
-        curpos = end
-
         if type(ast) is str:
             # error
             print ast
+            mforms.App.get().set_status_text("Cannot format invalid SQL: %s"%ast)
             return 1
         else:
+            if 0: # debug
+                from sql_reformatter import dump_tree
+                import sys
+                dump_tree(sys.stdout, ast)
+            s, v, c, _base, _begin, _end = ast
+            begin, end = helper.get_ast_range(ast)
+            new_text += text[curpos:begin].rstrip(" ") # strip spaces that would come before statement
+            curpos = end
+
             def trim_ast(node):
                 s = node[0]
                 v = node[1]
@@ -147,28 +162,35 @@ def enbeautificate(editor):
                 for i in c:
                     l.append(trim_ast(i))
                 return (s, v, l)
-        
 
             formatter = formatter_for_statement_ast(ast)
             if formatter:
+                ok_count += 1
                 p = formatter(trim_ast(ast))
                 fmted = p.run()
             else:
-                fmted = text[begin:end-begin]
+                bad_count += 1
+                fmted = text[begin:end]
             new_text += fmted
     new_text += text[curpos:]
 
     if selectionOnly:
-        editor.replaceSelection()
+        editor.replaceSelection(new_text)
     else:
         editor.replaceContents(new_text)
 
+    if bad_count > 0:
+        mforms.App.get().set_status_text("Formatted %i statements, %i unsupported statement types skipped."%(ok_count, bad_count))
+    else:
+        mforms.App.get().set_status_text("Formatted %i statements."%ok_count)
+
     return 0
 
 
 def apply_to_keywords(editor, callable):
     from grt.modules import MysqlSqlFacade
-    non_keywords = ["ident", "TEXT_STRING", "text_string", "TEXT_STRING_filesystem", "TEXT_STRING_literal", "TEXT_STRING_sys"]
+    non_keywords = ["ident", "TEXT_STRING", "text_string", "TEXT_STRING_filesystem", "TEXT_STRING_literal", "TEXT_STRING_sys",
+                    "part_name"]
 
     text = editor.selectedText
     selectionOnly = True
@@ -178,27 +200,43 @@ def apply_to_keywords(editor, callable):
     
     new_text = ""
     ast_list = MysqlSqlFacade.parseAstFromSqlScript(text)
+    bb = 0
     for ast in ast_list:
         if type(ast) is str:
             # error
             print ast
+            mforms.App.get().set_status_text("Cannot format invalid SQL: %s"%ast)
+            return 1
         else:
-            def traverse(script, node):
-                s, v, c, l, b, e = node
-                if v and s not in non_keywords:
-                    script = script[:b] + callable(script[b:e]) + script[e:]
+            if 0: # debug
+                from sql_reformatter import dump_tree
+                import sys
+                dump_tree(sys.stdout, ast)
+
+            def get_keyword_offsets(offsets, script, node):
+                result = ""
+                s, v, c, base, b, e = node
+                if v:
+                    b += base
+                    e += base
+                    if s not in non_keywords:
+                        offsets.append((b, e))
                 for i in c:
-                    script = traverse(script, i)
+                    get_keyword_offsets(offsets, script, i)
 
-                return script
-
-            new_text = traverse(text, ast)
+            offsets = []
+            get_keyword_offsets(offsets, text, ast)
+            for b, e in offsets:
+                new_text += text[bb:b] + callable(text[b:e])
+                bb = e
+    new_text += text[bb:]
 
     if selectionOnly:
-        editor.replaceSelection()
+        editor.replaceSelection(new_text)
     else:
         editor.replaceContents(new_text)
 
+    mforms.App.get().set_status_text("SQL code reformatted.")
     return 0
 
 
@@ -214,29 +252,53 @@ def lowercaseKeywords(editor):
     return apply_to_keywords(editor, lambda s: s.lower())
 
 
+def get_lines_in_range(text, range_start, range_end):
+    lines = []
+    offs = 0
+    while True:
+        end = text.find("\n")
+        if end >= 0:
+            end += 1
+            line = text[0:end]
+            if offs >= range_start and offs < range_end:
+                lines.append(line)
+            offs += end
+            text = text[end:]
+        else:
+            if offs >= range_start and offs < range_end:
+                lines.append(text)
+            break
+    return lines
+
 
 @ModuleInfo.plugin("wb.sqlide.indent", caption = "Indent Selected Lines", input=[wbinputs.currentQueryBuffer()], pluginMenu="SQL/Utilities")
 @ModuleInfo.export(grt.INT, grt.classes.db_query_QueryBuffer)
 def indent(editor):
+    # indent and unindent handle selection a bit differently:
+    # - if there is no selection, only the line where the cursor is should be indented
+    # - if there is a selection, all selected lines should be indented, even if selected partially
+        
     indentation = " "*4
-    text = editor.selectedText
-    if text:
-        lines = text.split("\n")
-        fullText = editor.script
-        start = editor.selectionStart
-        end = editor.selectionEnd
-        if start > 0 and fullText[start-1] != "\n":
-            line0 = lines[0]
-            del lines[0]
-            new_text = line0 + ("\n%s"%indentation).join(text.split("\n"))
-            editor.replaceSelection(new_text)
-        else:
-            new_text = indentation + ("\n%s"%indentation).join(text.split("\n"))
-            editor.replaceSelection(new_text)
+    start = editor.selectionStart
+    end = editor.selectionEnd
+    full_text = editor.script
+
+    if end > start:
+        lines = get_lines_in_range(full_text, start, end)
+        new_text = indentation + indentation.join(lines)
+        last_line_end = end
+        while last_line_end < len(full_text) and full_text[last_line_end-1] != "\n":
+            last_line_end += 1
+        if last_line_end != end:
+            new_text = new_text[:-(last_line_end-end)]
+        editor.replaceSelection(new_text)
     else:
-        text = editor.script
-        new_text = indentation + ("\n%s"%indentation).join(text.split("\n"))
-        editor.replaceContents(new_text)
+        line_start = pos = editor.insertionPoint
+        while line_start > 0 and full_text[line_start-1] != "\n":
+            line_start -= 1
+        editor.replaceContents(full_text[:line_start] + indentation + full_text[line_start:]) 
+        editor.insertionPoint = pos
+
     return 0
 
 
@@ -244,36 +306,49 @@ def indent(editor):
 @ModuleInfo.plugin("wb.sqlide.unindent", caption = "Unindent Selected Lines", input=[wbinputs.currentQueryBuffer()], pluginMenu="SQL/Utilities")
 @ModuleInfo.export(grt.INT, grt.classes.db_query_QueryBuffer)
 def unindent(editor):
-    def _unindent(text):
-        lines = text.split("\n")
-        if any(not line.startswith(indentation) for line in lines if len(line.strip()) > 0):
-            return None
-        return "\n".join(line[len(indentation):] if line.strip() else line for line in lines)
-
     indentation = " "*4
-    text = editor.selectedText
-    if text:
-        lines = text.split("\n")
-        fullText = editor.script
-        start = editor.selectionStart
-        end = editor.selectionEnd
-        if start > 0 and fullText[start-1] != "\n":
-            line0 = lines[0]
-            del lines[0]
-            tmp = _unindent(text)
-            if tmp:
-                new_text = line0 + tmp
-                editor.replaceSelection(new_text)
-        else:
-            tmp = _unindent(text)
-            if tmp:
-                new_text = tmp
-                editor.replaceSelection(new_text)
+    start = editor.selectionStart
+    end = editor.selectionEnd
+    full_text = editor.script
+
+    if end > start:
+        lines = get_lines_in_range(full_text, start, end)
+        for i in range(len(lines)):
+            if lines[i].startswith(indentation):
+                lines[i] = lines[i][len(indentation):]
+        new_text = "".join(lines)
+        last_line_end = end
+        while last_line_end < len(full_text) and full_text[last_line_end-1] != "\n":
+            last_line_end += 1
+        if last_line_end != end:
+            new_text = new_text[:-(last_line_end-end)]
+        editor.replaceSelection(new_text)
     else:
-        text = editor.script
-        new_text = _unindent(text)
-        if new_text:
-            editor.replaceContents(new_text)
+        line_start = pos = editor.insertionPoint
+        while line_start > 0 and full_text[line_start-1] != "\n":
+            line_start -= 1
+        if full_text[line_start:].startswith(indentation):
+            editor.replaceContents(full_text[:line_start] + full_text[line_start+len(indentation):]) 
+            editor.insertionPoint = pos
+
     return 0
 
 
+@ModuleInfo.plugin("wb.sqlide.comment", caption = "Un/Comment Selection", input=[wbinputs.currentQueryBuffer()], pluginMenu="SQL/Utilities")
+@ModuleInfo.export(grt.INT, grt.classes.db_query_QueryBuffer)
+def commentText(editor):
+    text = editor.selectedText
+    if text:
+        lines = text.split("\n")
+        if lines[0].startswith("-- "):
+            new_text = "\n".join((line[3:] if line.startswith("-- ") else line) for line in lines)
+        else:
+            new_text = "\n".join("-- "+line for line in lines)
+        editor.replaceSelection(new_text)
+    else:
+        editor.replaceSelection("-- ")
+    return 0
+
+
+
+
