@@ -26,7 +26,7 @@
 
 #include "db_mysql_diff_reporting.h"
 #include "db.mysql/backend/db_plugin_be.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 #include "mforms/fs_object_selector.h"
 
 using namespace grtui;
