@@ -59,7 +59,7 @@ private:
       box.add(&browse_box, false, false);
       
       browse_box.set_spacing(8);
-      browse_box.add(file_radio, false);
+      browse_box.add(file_radio, false,true);
       browse_box.add(&file_selector, true, true);
 
       file_selector.initialize("", mforms::OpenFile, "SQL Files (*.sql)|*.sql", _("Browse..."));
