@@ -19,7 +19,7 @@
 
 #include "stdafx.h"
 #include "mysql_relationship_editor.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace bec;
 using namespace base;
@@ -124,7 +124,10 @@ void RelationshipEditorBE::set_left_mandatory(bool flag)
 
 bool RelationshipEditorBE::get_left_mandatory()
 {
-  return _relationship->foreignKey()->mandatory() != 0;
+  if (_relationship->foreignKey().is_valid())
+    return _relationship->foreignKey()->mandatory() != 0;
+  
+  return false;
 }
 
 
@@ -149,7 +152,10 @@ void RelationshipEditorBE::set_right_mandatory(bool flag)
 
 bool RelationshipEditorBE::get_right_mandatory()
 {
-  return _relationship->foreignKey()->referencedMandatory() != 0;
+  if (_relationship->foreignKey().is_valid())
+    return _relationship->foreignKey()->referencedMandatory() != 0;
+  
+  return false;
 }
 
 
@@ -166,7 +172,10 @@ void RelationshipEditorBE::set_to_many(bool flag)
 
 bool RelationshipEditorBE::get_to_many()
 {
-  return _relationship->foreignKey()->many() != 0;
+  if (_relationship->foreignKey().is_valid())
+    return _relationship->foreignKey()->many() != 0;
+  
+  return false;
 }
 
 
@@ -226,13 +235,17 @@ void RelationshipEditorBE::set_visibility(VisibilityType type)
 
 std::string RelationshipEditorBE::get_left_table_name()
 {
-  return *_relationship->foreignKey()->owner()->name();
+  if (_relationship->foreignKey().is_valid())
+    return *_relationship->foreignKey()->owner()->name();
+  return std::string();
 }
 
 
 std::string RelationshipEditorBE::get_right_table_name()
 {
-  return *_relationship->foreignKey()->referencedTable()->name();
+  if (_relationship->foreignKey().is_valid())
+    return *_relationship->foreignKey()->referencedTable()->name();
+  return std::string();
 }
 
 
@@ -240,12 +253,13 @@ std::string RelationshipEditorBE::get_left_table_info()
 {
   std::string text;
   db_ForeignKeyRef fk(_relationship->foreignKey());
-  for (size_t c= fk->columns().count(), i= 0; i < c; i++)
-  {
-    text.append(strfmt("%s: %s%s\n", fk->columns()[i]->name().c_str(), 
-                       fk->columns()[i]->formattedRawType().c_str(),
-                       fk->owner()->isPrimaryKeyColumn(fk->columns()[i])?" (PK)":""));
-  }
+  if (fk.is_valid())
+    for (size_t c= fk->columns().count(), i= 0; i < c; i++)
+    {
+      text.append(strfmt("%s: %s%s\n", fk->columns()[i]->name().c_str(), 
+                         fk->columns()[i]->formattedRawType().c_str(),
+                         fk->owner()->isPrimaryKeyColumn(fk->columns()[i])?" (PK)":""));
+    }
   return text;
 }
 
@@ -254,12 +268,13 @@ std::string RelationshipEditorBE::get_right_table_info()
 {
   std::string text;  
   db_ForeignKeyRef fk(_relationship->foreignKey());
-  for (size_t c= fk->columns().count(), i= 0; i < c; i++)
-  {
-    text.append(strfmt("%s: %s%s\n", fk->referencedColumns()[i]->name().c_str(), 
-                       fk->referencedColumns()[i]->formattedRawType().c_str(),
-                       fk->referencedTable()->isPrimaryKeyColumn(fk->referencedColumns()[i])?" (PK)":""));
-  }
+  if (fk.is_valid())
+    for (size_t c= fk->columns().count(), i= 0; i < c; i++)
+    {
+      text.append(strfmt("%s: %s%s\n", fk->referencedColumns()[i]->name().c_str(), 
+                         fk->referencedColumns()[i]->formattedRawType().c_str(),
+                         fk->referencedTable()->isPrimaryKeyColumn(fk->referencedColumns()[i])?" (PK)":""));
+    }
   return text;
 }
 
@@ -329,14 +344,18 @@ void RelationshipEditorBE::invert_relationship()
 
 bool RelationshipEditorBE::get_is_identifying()
 {
-  db_TableRef table(_relationship->foreignKey()->owner());
-
-  GRTLIST_FOREACH(db_Column, _relationship->foreignKey()->columns(), column)
+  if (_relationship->foreignKey().is_valid())
   {
-    if (!table->isPrimaryKeyColumn(*column))
-      return false;
+    db_TableRef table(_relationship->foreignKey()->owner());
+    
+    GRTLIST_FOREACH(db_Column, _relationship->foreignKey()->columns(), column)
+    {
+      if (!table->isPrimaryKeyColumn(*column))
+        return false;
+    }
+    return true;
   }
-  return true;
+  return false;
 }
 
 
