@@ -40,7 +40,7 @@
 #import "MResultsetViewer.h"
 
 #import "DbPrivilegeEditorTab.h"
-#import "string_utilities.h"
+#include "base/string_utilities.h"
 
 const NSString* shouldRaiseException = @"should raise exception";
 
