@@ -20,7 +20,7 @@
 #import "MySQLRoutineGroupEditor.h"
 #import "MCPPUtilities.h"
 #include "grtdb/db_object_helpers.h" // get_rdbms_for_db_object()
-#import "string_utilities.h"
+#include "base/string_utilities.h"
 
 @implementation DbMysqlRoutineGroupEditor
 
