@@ -99,6 +99,8 @@ void DbMySQLSQLExport::set_option(const std::string& name, bool value)
     _generate_use= value;
   else if(name.compare("SkipForeignKeys") == 0)
     _skip_foreign_keys= value;
+  else if(name.compare("SkipFKIndexes") == 0)
+    _skip_fk_indexes= value;
 }
 
 void DbMySQLSQLExport::set_option(const std::string& name, const std::string& value)
@@ -163,6 +165,7 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
   options.set("UseShortNames", grt::IntegerRef(_omit_schemata ? 1 : 0));
   options.set("GenerateUse", grt::IntegerRef(_generate_use ? 1 : 0));
   options.set("SkipForeignKeys", grt::IntegerRef(_skip_foreign_keys ? 1 : 0));
+  options.set("SkipFKIndexes", grt::IntegerRef(_skip_fk_indexes ? 1 : 0));
 
   std::vector<std::string> schemata_names;
   db_mysql_CatalogRef cat(get_model_catalog());
