@@ -32,7 +32,7 @@
 
 #include "grtpp.h"
 #include "db_mysql_sql_export.h"
-#include "string_utilities.h"
+#include "base/string_utilities.h"
 
 using namespace grt;
 
@@ -733,6 +733,8 @@ void DbMySQLScriptSync::copy_table_children(db_mysql_TableRef from, db_mysql_Tab
     to->triggers().insert(trigger);
     trigger->owner(to);
   }
+  to->comment(bec::TableHelper::get_sync_comment(to->comment())+
+    bec::TableHelper::get_document_comment(from->comment()));
 }
 
 void DbMySQLScriptSync::copy_schema_children(db_mysql_SchemaRef from, db_mysql_SchemaRef to)
