@@ -1,3 +1,20 @@
+# Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; version 2 of the
+# License.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+# 02110-1301  USA
+
 import threading
 from threading import currentThread
 import sys
@@ -29,22 +46,31 @@ def get_local_ip_list():
       out = os.popen(cmd)
     except Exception, e:
       # We do not care if execution failed, the list will just be empty
-      print e
+      #print e
       out = None
     except:
       print "Unknown exception while running '%s'"%cmd
 
-    ret = []
-    if out:
-        result = out.read()
-        m = re.findall(regexp, result)
+    try:
+      ret = []
+      if out:
+          result = out.read()
+          m = re.findall(regexp, result)
+
+          for ip in m:
+              try:
+                  t_ip = socket.gethostbyname(ip)
+                  ret.append(ip)
+              except socket.gaierror,e:
+                  print ip, str(e)
+    except Exception, e:
+      # We do not care if execution failed, the list will just be empty
+      #print e
+      ret = []
+    except:
+      print "Unknown exception while running '%s'"%cmd
+      ret = []
 
-        for ip in m:
-            try:
-                t_ip = socket.gethostbyname(ip)
-                ret.append(ip)
-            except socket.gaierror,e:
-                print ip, str(e)
     return ret
 
 local_ip_list = get_local_ip_list()
@@ -167,7 +193,7 @@ class ServerProfile:
 
     @property
     def remote_admin_enabled(self):
-        return (self.uses_ssh) and not self.disable_remote_admin
+        return (self.uses_ssh or (not self.is_local and self.uses_wmi)) and not self.disable_remote_admin
     
     @property
     def admin_enabled(self):
@@ -217,8 +243,7 @@ class ServerProfile:
     # If WMI admin was selected as mgmt mean (can be for local or remote)
     @property
     def uses_wmi(self):
-        return False
-        #return self._int_server_info("windowsAdmin")
+        return self._int_server_info("windowsAdmin")
 
     @property
     def ssh_username(self):
@@ -305,7 +330,7 @@ class PasswordHandler:
     def get_password_parameters(self, service_type):
         # known service types: remoteshell, ssh, sshkey, file, service.startstop, service.status
         # these are mapped to
-        # known password types: ssh, sshkey, sudo, UAC
+        # known password types: ssh, sshkey, sudo, UAC, wmi
         # which are then mapped to (title, service, account) or "UAC" or None
 
         profile = self.server_profile
@@ -319,9 +344,7 @@ class PasswordHandler:
                 password_type = "sudo"
         elif service_type == "service.startstop":
             if profile.target_is_windows:
-                # windows to windows uses WMI, which gets password from elsewhere at startup
-                # others to windows don't need anything since remote ssh server runs as admin
-                password_type = None
+                password_type = "wmi"
             else:
                 if profile.use_sudo:
                     password_type = "sudo"
@@ -381,6 +404,9 @@ class PasswordHandler:
 
             return (title, service, account)
 
+        elif password_type == "wmi":
+            return ("WMI Password Required", "wmi@%s" % profile.wmi_hostname, profile.wmi_username)
+
         elif password_type == "UAC":
             return "UAC"
         
@@ -524,13 +550,12 @@ class ServerControlShell(ServerControlBase):
 
     def start(self, password):
         """ Can also throw InvalidPasswordError"""
-        self.info("Starting server...")        
-        self.info("%s %s" % (self.profile.start_server_cmd, self.profile.use_sudo and "(with sudo)"))
+        self.info("Starting server...") # TODO: This does not come to info output
         try:
             r = self.helper.execute_command(self.profile.start_server_cmd, 
                                     as_admin=self.profile.use_sudo, 
                                     admin_password=password,
-                                    output_handler=lambda s:self.info("Start server: %s"%(s or "").replace(password, "")))
+                                    output_handler=lambda s:self.info("Start server: %s"%(s if ((type(s) is str) or (type(s) is unicode)) else "").replace(password, "")))
         except InvalidPasswordError, exc:
             # forget password, so that next time it will be requested
             self.password_delegate.reset_password_for("service.startstop")
@@ -544,12 +569,19 @@ class ServerControlShell(ServerControlBase):
     def stop(self, password):
         """ Can also throw InvalidPasswordError"""
         self.info("Stopping server...")
-        self.info("%s %s" % (self.profile.stop_server_cmd, self.profile.use_sudo and "(with sudo)"))
         try:
+            def strip_pwd(output):
+                if output is not None and (type(output) is str or type(output) is unicode):
+                    if password is not None and password != "":
+                        output = output.replace(password, "")
+                else:
+                    output = ""
+                self.info("Stop server: %s"%str(output))
+
             r = self.helper.execute_command(self.profile.stop_server_cmd, 
                                     as_admin=self.profile.use_sudo, 
                                     admin_password=password,
-                                    output_handler=lambda s:self.info("Stop server: %s"%(s or "").replace(password, "")))
+                                    output_handler=strip_pwd)
         except InvalidPasswordError, exc:
             # forget password, so that next time it will be requested
             self.password_delegate.reset_password_for("service.startstop")
@@ -633,12 +665,19 @@ class ServerControlWMI(ServerControlBase):
     def __init__(self, profile, helper, password_delegate):
         ServerControlBase.__init__(self, profile, helper, password_delegate)
         self.wmi = grt.modules.Workbench
-        server = self.profile.wmi_hostname
-        user = self.profile.wmi_username
         self.wmi_session_ids = {}
+        self.shell = None
         if self.profile.is_local:
+            user = ""
+            server = ""
             password = ""
+            self.check_and_fix_profile_for_local_windows(profile)
+            self.shell = ServerControlShell(profile, helper, password_delegate)
+            # Forse usage of as_admin for local windows
+            self.info("Workbench will use cmd shell commands to start/stop this instance")
         else:
+            user = self.profile.wmi_username
+            server = self.profile.wmi_hostname
             password = self.get_password()
         try:
             sess = self.wmi.wmiOpenSession(server, user, password or "")
@@ -647,26 +686,63 @@ class ServerControlWMI(ServerControlBase):
             import traceback
             traceback.print_exc()
             raise RuntimeError("Could not initialize WMI interface: %s"%exc)
-        if self.wmi_session_id <= 0:
+        if self.wmi_session_id_for_current_thread <= 0:
             raise RuntimeError("Could not initialize WMI interface")
-    
+
+
+    def check_and_fix_profile_for_local_windows(self, profile):
+        settings = profile.get_settings_object()
+        serverInfo = settings.serverInfo
+
+        serverInfo["sys.usesudo"] = 1 # Force this in any case
+        # Check commands
+        service = profile.wmi_service_name
+        #  return self._str_server_info("sys.mysqld.service_name")
+        if service == "":
+            serverInfo["sys.mysqld.service_name"] = "MySQL"
+            self.info("MySQL service was empty. Set to 'MySQL'. Check this in 'Manage Server Instances' from 'Home'.")
+
+        if profile.start_server_cmd == "":
+            serverInfo["sys.mysqld.start"] = "sc start " + service
+            print "WMIShell: start command set to '%s'"%profile.start_server_cmd
+
+        if profile.stop_server_cmd == "":
+            serverInfo["sys.mysqld.stop"] = "sc stop " + service
+            print "WMIShell: stop command set to '%s'"%profile.stop_server_cmd
+
+        if profile.check_server_status_cmd == "":
+            serverInfo["sys.mysqld.status"] = "sc query %s | wba_filter(RUNNING)"%service
+            print "WMIShell: status command set to '%s'"%profile.check_server_status_cmd
+
+    def set_filter_functions(self, prepare_f, apply_f):
+        if self.profile.is_local and self.shell:
+          print "WMIShell: Set filters for local windows cli"
+          self.shell.prepare_filter = prepare_f
+          self.shell.apply_filter = apply_f
+
     def get_password(self):
         if self.profile.is_local:
             return ""
         return self.password_delegate.get_password_for("service.startstop")
-    
+
     # WMI sessions are thread specific. To make things easier, we create one for
     # each thread that uses it
     @property
-    def wmi_session_id(self):
+    def wmi_session_id_for_current_thread(self):
         thr = currentThread()
         s = self.wmi_session_ids.get(thr)
-        print "%s = %s" % (thr, s)
         if s is not None:
             return s
-        password = self.password_delegate.get_password_for("service.startstop")
-        sess = self.wmi.wmiOpenSession(self.profile.wmi_hostname, self.profile.wmi_username, password or "")
-        self.wmi_session_ids[thr] = sess
+        password = self.get_password()
+
+        if self.profile.is_local:
+            sess = self.wmi.wmiOpenSession('', '', '')
+        else:
+            sess = self.wmi.wmiOpenSession(self.profile.wmi_hostname, self.profile.wmi_username, password or "")
+
+        if sess:
+            self.wmi_session_ids[thr] = sess
+
         return sess
 
     def close(self):
@@ -676,35 +752,48 @@ class ServerControlWMI(ServerControlBase):
         ServerControlBase.close() # not certain if the call is needed.
 
     def start(self, password):
-        # Profile may hold the actual command passed
-        service = self.profile.wmi_service_name
-        action = "start"
-        self.info("Starting service '%s' through WMI..." % service)
-        result = self.wmi.wmiServiceControl(self.wmi_session_id, service, action)
-        self.info("Service start result: %s" % result)
-        if result.startswith("error"):
-            raise RuntimeError("Error stopping service %s through WMI\n%s" % (service, result))
-        return not result.startswith("error")
+        self.info("Starting server...");
+        if self.shell:
+            return self.shell.start(password)
+        else:
+            # Profile may hold the actual command passed
+            service = self.profile.wmi_service_name
+            action = "start"
+            self.info("Starting service '%s' through WMI..." % service)
+            result = self.wmi.wmiServiceControl(self.wmi_session_id_for_current_thread, service, action)
+            self.info("Service start result: %s" % result)
+            if result.startswith("error"):
+                raise RuntimeError("Error stopping service %s through WMI\n%s" % (service, result))
+            return not result.startswith("error")
 
 
     def stop(self, password):
-        service = self.profile.wmi_service_name
-        action = "stop"
-        self.info("Stopping service '%s' through WMI..." % service)
-        result = self.wmi.wmiServiceControl(self.wmi_session_id, service, action)
-        self.info("Service stop result: %s" % result)
-        if result.startswith("error"):
-            raise RuntimeError("Error stopping service %s through WMI\n%s" % (service, result))
-        return not result.startswith("error")
-
+        if self.shell:
+            return self.shell.stop(password)
+        else:
+            service = self.profile.wmi_service_name
+            action = "stop"
+            self.info("Stopping service '%s' through WMI..." % service)
+            result = self.wmi.wmiServiceControl(self.wmi_session_id_for_current_thread, service, action)
+            self.info("Service stop result: %s" % result)
+            if result.startswith("error"):
+                raise RuntimeError("Error stopping service %s through WMI\n%s" % (service, result))
+            return not result.startswith("error")
 
     def get_status(self, verbose=1, password=None):
         "Returned value is one of running, stopping, starting, stopped, unknown"
         service = self.profile.wmi_service_name
         action = "status"
-        result = self.wmi.wmiServiceControl(self.wmi_session_id, service, action)
+        if verbose > 1:
+            self.info("Checking service status of instance %s..." % service)
+        result = self.wmi.wmiServiceControl(self.wmi_session_id_for_current_thread, service, action)
         if verbose:
             self.info("Status check of service '%s' returned %s" % (service, result))
+        # translate status to something more displayable
+        if result == "stop pending":
+            result = "stopping"
+        elif result == "start pending":
+            result = "starting"
 
         return result
 
