@@ -1,3 +1,20 @@
+# Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; version 2 of the
+# License.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+# 02110-1301  USA
+
 import platform
 import socket
 import threading
@@ -95,6 +112,11 @@ class Source:
 
 #===============================================================================
 class SqlDataSource(DataSource):
+  stopped = 0
+  started = 1
+  skip_poll = 2
+  polling   = 3
+
   def __init__(self, sql, mon_be):
     DataSource.__init__(self, "sql_source", mon_be, None)
     self.sql = sql
@@ -105,24 +127,33 @@ class SqlDataSource(DataSource):
     self.ctrl_be.add_me_for_event("server_started", self)
     self.ctrl_be.add_me_for_event("server_stopped", self)
     self.ctrl_be.add_me_for_event("shutdown", self)
+    self.poll_control = self.stopped
+    dprint_ex(3, "SqlDataSource created. poll_control =", self.poll_control)
 
   def server_started_event(self):
+    self.poll_control = self.started
     if self.dbconn:
       self.dbconn.disconnect()
       self.dbconn = None
-
     self.dbconn = self.ctrl_be.get_new_sql_connection(True)
-      
+    self.poll_control = self.polling
+    dprint_ex(3, "SqlDataSource server started. poll_control =", self.poll_control)
 
   def server_stopped_event(self):
+    self.poll_control = self.skip_poll
     if self.dbconn:
       self.dbconn.disconnect()
     self.dbconn = None
+    self.poll_control = self.stopped
+    dprint_ex(3, "SqlDataSource server stopped. poll_control =", self.poll_control)
   
   def shutdown_event(self):
+    self.poll_control = self.skip_poll
     if self.dbconn:
       self.dbconn.disconnect()
     self.dbconn = None
+    self.poll_control = self.stopped
+    dprint_ex(3, "SqlDataSource shutdown. poll_control =", self.poll_control)
 
   def add_source(self, name, definition, widget):
     # Source is a dict which contains names of status variables, formula to calc
@@ -153,7 +184,8 @@ class SqlDataSource(DataSource):
       widget[0].set_value_range(definition['min'], definition['max'])
 
   def poll(self):
-    if self.dbconn:
+    dprint_ex(5, "SqlDataSource poll. poll_control =", self.poll_control)
+    if self.poll_control == self.polling and self.dbconn:
       result = None
       try:
         result = self.dbconn.executeQuery(self.sql) #ctrl_be from DataSource
@@ -201,10 +233,11 @@ class WBAWinRemoteStats:
 
     self.ssh   = ctrl_be.open_ssh_session_for_monitoring()
 
-    (dirpath, code) = self.ssh.exec_cmd("cmd /C echo %APPDATA%")
+    (dirpath, code) = self.ssh.exec_cmd("cmd /C echo %USERPROFILE%") # %APPDATA% is n/a for LocalService
+                                                                     # which is a user sshd can be run
     dirpath = dirpath.strip(" \r\t\n")
 
-    if code == 0 and dirpath is not None:
+    if code == 0 and dirpath is not None and dirpath != "%USERPROFILE%":
       script_path = App.get().get_resource_path("mysql_system_status_rmt.vbs")
       filename = "\"" + dirpath + "\\mysql_system_status_rmt.vbs\""
       dprint_ex(1, "script local path is ", script_path, ". Will be uploaded to", filename)
@@ -247,6 +280,9 @@ class WBAWinRemoteStats:
           self.ssh.close()
           self.ssh = None
           raise e
+    else:
+      print "Can't find a place to upload script dirpath='%s'"%dirpath
+
 
   #-----------------------------------------------------------------------------
   def shutdown_event(self):
@@ -473,6 +509,56 @@ class WBAWinLocalStats:
 
     return ret
 
+
+#===============================================================================
+class WMIStats:
+  def __init__(self, ctrl_be, server_profile, cpu_widget, mem_widget):
+    if not hasattr(ctrl_be.server_control, 'wmi'):
+      raise Exception("Current profile has no WMI enabled") # TODO Should be better message
+
+    self.ctrl_be = ctrl_be
+    self.server_profile = server_profile
+    self.cpu_widget = cpu_widget
+    self.mem_widget = mem_widget
+    self.cpu_mon_id = None
+    self.total_mem  = None
+
+    self.wmi = ctrl_be.server_control.wmi
+
+  def query(self, session, attr, query):
+    value = None
+    res = self.wmi.wmiQuery(session, query)
+    if len(res) > 0:
+      if hasattr(res[0], attr):
+        value = getattr(res[0], attr)
+        try:
+          value = int(value)
+        except:
+          print "Wmi query: can't cast '%s' to int" % str(value)
+          value = 1
+      else:
+        print "Wmi query: expected '%s' result attribute, got:" % attr
+        print res
+        value = 1
+    return value
+
+  def poll(self):
+    wmi_session = self.ctrl_be.server_control.wmi_session_id_for_current_thread
+    # Calc CPU
+    value = self.query(wmi_session, 'PercentProcessorTime', "SELECT PercentProcessorTime FROM Win32_PerfFormattedData_PerfOS_Processor WHERE Name = '_Total'")
+    if value is not None:
+      self.cpu_widget[0].set_value(value/100.0)
+      self.ctrl_be.uitask(self.cpu_widget[0].set_description ,"CPU: " + str(int(value)) + "%")
+    # Calc Memory
+    if self.total_mem is None:
+      self.total_mem = float(self.query(wmi_session, 'TotalVisibleMemorySize', "Select TotalVisibleMemorySize FROM Win32_OperatingSystem"))
+
+    value = self.query(wmi_session, 'AvailableKBytes', "Select AvailableKBytes FROM Win32_PerfFormattedData_PerfOS_Memory")
+    if value is not None:
+      mem_perc = round(100 - ((value/self.total_mem) * 100), 1)
+      self.mem_widget[0].set_value(mem_perc)
+      self.ctrl_be.uitask(self.mem_widget[0].set_description ,"Mem: " + str(int(mem_perc)) + "%")
+
 #===============================================================================
 class WBAdminMonitorBE:
   def __init__(self, interval, server_profile, ctrl_be, widgets, sql):
@@ -486,35 +572,39 @@ class WBAdminMonitorBE:
     self.ctrl_be.add_me_for_event("server_started", self)
     self.ctrl_be.add_me_for_event("server_stopped", self)
 
-    # First read all script commands that were specified for the given instance.
-    script = server_profile.sys_script
-    lines = script.split("\n")
-    for line in lines:
-      line = line.strip(" \r\t\n")
-      idx = line.find("=")
-      if idx > 0:
-        name = line[:idx].strip(" \t")
-        cmds = line[idx+1:].strip(" \t")
-        self.commands[name] = cmds
+    if server_profile.target_os != wbaOS.windows:
+      if server_profile.is_local or server_profile.remote_admin_enabled:
+        # First read all script commands that were specified for the given instance.
+        script = server_profile.sys_script
+        lines = script.split("\n")
+        for line in lines:
+          line = line.strip(" \r\t\n")
+          idx = line.find("=")
+          if idx > 0:
+            name = line[:idx].strip(" \t")
+            cmds = line[idx+1:].strip(" \t")
+            self.commands[name] = cmds
 
-        widget = None
-        if name in widgets:
-          widget = widgets[name]
+            widget = None
+            if name in widgets:
+              widget = widgets[name]
 
-        cmd = ShellDataSource(name, cmds, self, widget)
+            cmd = ShellDataSource(name, cmds, self, widget)
 
-        self.sources.append(cmd)
-
-    # Next, if we are reading from Windows targets replace sources to use a special script.
-    if server_profile.target_os == wbaOS.windows:
-      self.sources = []
-      if server_profile.is_local:
+            self.sources.append(cmd)
+      else:
+        print "WBAMonBE: Data sources were not added. Profile set to non-local or remote admin is disabled."
+    else:
+      if server_profile.is_local: # or server_profile.uses_wmi: # Force WMI for local boxes
+        self.wmimon = WMIStats(ctrl_be, server_profile, widgets['get_cpu_info'], widgets['get_mem_info'])
+        self.sources.append(self.wmimon)
+        #Next, if we are reading from Windows targets replace sources to use a special script.
         # Create special source for windows. This stats will open a script that will output values of mem
         # and cpu usage upon request. The data obtained from the script is passed directly to widgets
-        stats = WBAWinLocalStats(ctrl_be, server_profile, self.running, widgets['get_cpu_info'], widgets['get_mem_info'])
-        self.sources.append(stats)
+        #stats = WBAWinLocalStats(ctrl_be, server_profile, self.running, widgets['get_cpu_info'], widgets['get_mem_info'])
+        #self.sources.append(stats)
       else:
-        if server_profile.connect_method == 'ssh':
+        if server_profile.remote_admin_enabled and server_profile.connect_method == 'ssh':
           stats = WBAWinRemoteStats(ctrl_be, server_profile, self.running, widgets['get_cpu_info'], widgets['get_mem_info'])
           self.sources.append(stats)
 
