@@ -1,4 +1,21 @@
-from mforms import TabView, newBox, newTable, newPanel, TitledBoxPanel, newScrollPanel, newCheckBox, newTextEntry, newTextBox
+# Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; version 2 of the
+# License.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+# 02110-1301  USA
+
+from mforms import newTabView, newBox, newTable, newPanel, TitledBoxPanel, newScrollPanel, newCheckBox, newTextEntry, newTextBox
 from mforms import newLabel, HFillFlag, VFillFlag, HExpandFlag, Utilities, newSelector
 from mforms import newButton, SmallHelpTextStyle, FileChooser, OpenDirectory
 from mforms import SelectorPopup, SelectorCombobox
@@ -7,6 +24,7 @@ import wb_admin_config_file_be
 import os
 from wb_common import dprint_ex, debug_level
 from wb_admin_config_file_be import multi_separator
+from wb_admin_utils import no_remote_admin_warning_label
 
 CATOPTS = os.getenv("WB_CATOPTS")
 if CATOPTS is not None:
@@ -15,9 +33,6 @@ if CATOPTS is not None:
 
   def handle_cat_opt(cat, grp, enabled):
     print "CATOPT", "'" + enabled + "', '" + cat.get_string_value() + "', '" + grp.get_string_value()
-    #f = open("cats.txt", "a")
-    #f.write("'" + enabled + "', '" + cat.get_string_value() + "', '" + grp.get_string_value() + "'\n")
-    #f.close()
 
 #===============================================================================
 class Page:
@@ -31,20 +46,31 @@ class Page:
   def set_update_cb(self, update_cb):
     self.update_cb = update_cb
 
+
+
 #===============================================================================
-class WbAdminConfigFileUI(TabView):
+class WbAdminConfigFileUI(mforms.Box):
     #---------------------------------------------------------------------------
     def page_activated(self):
+      self.main_view.set_content_label(" Options File")
       if not self.ui_created:
         self.ui_created = True
         self.suspend_layout()
         self.create_ui()
         self.resume_layout()
-
+      else:
+        on = self.server_profile.admin_enabled
+        self.file_name_ctrl.set_enabled(on)
+        self.section_ctrl.set_enabled(on)
+        self.bottom_box.set_enabled(on)
+        self.tab_view.set_enabled(on)
+        self.search_panel.set_enabled(on)
 
     #---------------------------------------------------------------------------
-    def __init__(self, server_profile, file_name_ctrl, section_ctrl, ctrl_be, version="5.1"):
-      TabView.__init__(self)
+    def __init__(self, server_profile, ctrl_be, main_view, version="5.1"):
+      mforms.Box.__init__(self, False)
+      self.tab_view = newTabView()
+      self.main_view = main_view
 
       self.myopts = None
       self.opt2ctrl_map = {}
@@ -56,12 +82,42 @@ class WbAdminConfigFileUI(TabView):
       self.version = version
       self.ctrl_be = ctrl_be
       self.server_profile = server_profile
-      self.file_name_ctrl = file_name_ctrl
-      self.file_name_ctrl.set_enabled(False)
-      self.section_ctrl = section_ctrl
       self.cfg_be = None # Will be created later in self.create_ui
       self.version = version
 
+      self.pack_to_top()
+      container = self.main_view.add_content_page(self, "CONFIGURATION", "Options File", "admin_option_file_win", True)
+
+      self.search_panel = self.create_search_panel()
+      container.add_end(self.search_panel, False, False);
+
+    #---------------------------------------------------------------------------
+    def create_search_panel(self):
+      search_label = newLabel("Locate option:")
+      self.option_lookup_entry  = newTextEntry()
+      self.option_lookup_entry.set_size(200,-1)
+      search_btn = newButton()
+      search_btn.set_text("Find")
+      search_btn.set_size(70, -1)
+
+      search_box = newBox(True)
+      search_box.set_padding(2)
+      search_box.set_spacing(4)
+      #search_box.set_size(300, -1)
+      search_box.add(search_label, False, True)
+      search_box.add(self.option_lookup_entry, False, True)
+      search_box.add(search_btn, False, True)
+      search_panel = newPanel(mforms.FilledPanel)
+      search_panel.add(search_box)
+      self.main_view.ui_profile.apply_style(search_panel, 'option-search-panel')
+
+      def lookup_option_wrapper():
+          self.locate_option(self.option_lookup_entry.get_string_value())
+
+      search_btn.add_clicked_callback(lookup_option_wrapper)
+
+      return search_panel
+
     #---------------------------------------------------------------------------
     def create_page(self, page_number):
       self.loading = True
@@ -173,6 +229,7 @@ class WbAdminConfigFileUI(TabView):
       box.resume_layout()
       self.loading = False
 
+    #---------------------------------------------------------------------------
     def locate_option(self, option_name_fragment):
       ret = None
       for opt in self.cfg_be.get_options_containing(option_name_fragment):
@@ -181,7 +238,7 @@ class WbAdminConfigFileUI(TabView):
           for page_idx, page in self.pages.iteritems():
             if page.page_name == tab_name:
               self.create_page(page_idx)
-              self.set_active_tab(page_idx - 1)
+              self.tab_view.set_active_tab(page_idx - 1)
               page.panel.flush_events()
               ctrl_tuple = self.opt2ctrl_map.get(opt)
               # ctrl_tuple is in for of ('<type>', <ui controls tuple>, <opt def from xml file>)
@@ -190,14 +247,16 @@ class WbAdminConfigFileUI(TabView):
                 page.panel.scroll_to_view(ctrl[0]) # scroll to enabled checkbox
                 return
 
+    #---------------------------------------------------------------------------
     def tab_changed(self):
-      page_number = self.get_active_tab() + 1
+      page_number = self.tab_view.get_active_tab() + 1
       page = self.pages.get(page_number)
       if page is not None and page.update_cb is not None:
         page.update_cb(page)
       else:
         self.create_page(page_number)
 
+    #---------------------------------------------------------------------------
     def create_ui(self):
       self.loading = True
 
@@ -229,7 +288,7 @@ class WbAdminConfigFileUI(TabView):
       for page_pos in page_positions:
         page = self.pages[page_pos]
         page.panel = newScrollPanel(mforms.ScrollPanelNoFlags)
-        self.add_page(page.panel, page.page_name)
+        self.tab_view.add_page(page.panel, page.page_name)
 
       if debug_level > 0:
         # Create file page
@@ -237,21 +296,23 @@ class WbAdminConfigFileUI(TabView):
         page.panel = newTextBox(mforms.BothScrollBars)
         page.set_update_cb(self.update_file_content_tab)
         self.pages[max(self.pages.keys()) + 1] = page
-        self.add_page(page.panel, page.page_name)
+        self.tab_view.add_page(page.panel, page.page_name)
 
       # Create first page, so we display something from start
       self.create_page(1)
       self.loading = True # create_page resets loading flag
 
-      self.add_tab_changed_callback(self.tab_changed)
+      self.tab_view.add_tab_changed_callback(self.tab_changed)
 
       self.loading = False
 
+    #---------------------------------------------------------------------------
     def update_file_content_tab(self, page):
       page.panel.clear()
       for line in self.cfg_be.get_file_content():
         page.panel.append_text(line)
 
+    #---------------------------------------------------------------------------
     def create_textedit(self, name, ctrl_def):
       te = newTextEntry()
       te.set_enabled(False)
@@ -259,6 +320,7 @@ class WbAdminConfigFileUI(TabView):
       te.set_tooltip("To convert option to a multi-line one, use " + multi_separator + " to separate values. The symbol " + multi_separator + " should not be the first char")
       return te
 
+    #---------------------------------------------------------------------------
     def create_dir_file_edit(self, name, ctrl_def):
       dir_box = newBox(True)
       dir_box.set_spacing(4)
@@ -271,18 +333,20 @@ class WbAdminConfigFileUI(TabView):
       btn.add_clicked_callback(lambda: self.open_file_chooser(OpenDirectory, te, name))
 
       dir_box.add(te, True, True)
-      dir_box.add(btn, False, False)
+      dir_box.add(btn, False, True)
       te.set_enabled(False)
       btn.set_enabled(False)
 
       return (dir_box, te, btn)
 
+    #---------------------------------------------------------------------------
     def create_numeric(self, name, ctrl_def):
       #spin_box = newBox(True)
       #spin_box.set_spacing(5)
       te = newTextEntry()
       #spin_box.add(te, True, True)
       te.set_enabled(False)
+      te.set_tooltip("For numeric values, you may specify a K, M or G size suffix, if appropriate.")
 
       te.add_changed_callback(lambda: self.control_action(name))
 
@@ -306,6 +370,7 @@ class WbAdminConfigFileUI(TabView):
       #return (spin_box, te, unitcontrol, unit_items)
       return te
 
+    #---------------------------------------------------------------------------
     def create_dropdownbox(self, name, ctrl_def, ctype):
       items = None
       if 'choice' in ctrl_def:
@@ -365,7 +430,7 @@ class WbAdminConfigFileUI(TabView):
         table.add(te, 1, 2, row, row+1, HExpandFlag | HFillFlag)
         ctrl = ('txt', (enabled, te), ctrl_def)
         self.opt2ctrl_map[name] = ctrl
-      elif ctype == "directory" or ctype == "filename":
+      elif ctype == "directory" or ctype == "filename" or ctype == "dirname":
         (dir_box, te, btn) = self.create_dir_file_edit(name, ctrl_def)
         table.add(dir_box, 1, 2, row, row + 1, HExpandFlag | HFillFlag)
         te.add_changed_callback(lambda: self.control_action(name))
@@ -731,3 +796,49 @@ class WbAdminConfigFileUI(TabView):
               self.set_string_value_to_control(ctrl_tupple, "")
 
       self.loading = False
+
+
+    #-------------------------------------------------------------------------------
+    def pack_to_top(self):
+      self.suspend_layout()
+      self.main_view.ui_profile.apply_style(self, 'page')
+
+      #if self.server_profile.admin_enabled:
+      self.file_name_ctrl = newTextEntry()
+      sys_config_path = self.server_profile.config_file_path
+      if sys_config_path is None:
+        sys_config_path = ""
+      self.file_name_ctrl.set_value(sys_config_path)
+      self.file_name_ctrl.set_size(300, -1)
+
+      self.section_ctrl = newSelector()
+      self.section_ctrl.set_size(150, -1)
+
+      #spacer = newPanel(mforms.TransparentPanel)
+      #spacer.set_size(100, 10)
+
+      self.bottom_box = newBox(True)
+
+      accept_btn = newButton()
+      accept_btn.set_text("Apply ...")
+
+      discard_btn = newButton()
+      discard_btn.set_text("Discard")
+
+      #self.add(self.search_panel, False, True)
+      self.add(self.tab_view, True, True)
+      self.add(self.bottom_box, False, False)
+
+      self.bottom_box.add(newLabel("Configuration File:"), False, True)
+      self.bottom_box.add(self.file_name_ctrl, True, True)
+      self.bottom_box.add(self.section_ctrl, False, True)
+
+      Utilities.add_end_ok_cancel_buttons(self.bottom_box, accept_btn, discard_btn)
+
+      self.bottom_box.set_spacing(8)
+      self.bottom_box.set_padding(12)
+
+      accept_btn.add_clicked_callback(self.config_apply_changes_clicked)
+      discard_btn.add_clicked_callback(self.config_discard_changes_clicked)
+
+      self.resume_layout()
