@@ -24,8 +24,7 @@ import time
 from mforms import Utilities, AppView, newSectionBox, App
 import mforms
 
-import wb_admin_monitor
-import wb_admin_configuration
+import wb_admin_main
 import wb_admin_utils
 import wba_ssh_ui
 import wb_admin_ssh
@@ -44,54 +43,20 @@ class DBError(Exception):
 
 tab_references = []
 
-
-
-
-##def get_db_connection(server_instance_settings):
-#  if server_instance_settings.connection:
-#    db_connection = MySQLConnection(server_instance_settings.connection)
-#    ignore_error = False
-#    error_location = None
-#    the_error = None
-#    try:
-#      db_connection.connect()
-#    except MySQLError, exc:
-#     # errors that probably just mean the server is down can be ignored (ex 2013)
-#     # errors from incorrect connect parameters should raise an exception
-#     # ex 1045: bad password
-#      if exc.code in (1045,):
-#        raise exc
-#      elif exc.code in (2013,):
-#        ignore_error = True
-#      error_location = exc.location
-#      the_error = str(exc)
-#
-#      if not ignore_error:
-#        if Utilities.show_warning("Could not Connect to MySQL Server at %s" % error_location,
-#                "%s\nYou can continue but some functionality may be unavailable." % the_error,
-#                "Continue Anyway", "Cancel", "") != ResultOk:
-#          raise MySQLError("")
-#    return db_connection
-#  else:
-#    Utilities.show_warning("WB Admin", "Server Profile has no database connection specified.\nSome functionality will not be available.", "OK", "", "")
-#
-#
-#  return None
-
 #===============================================================================
 #
 #===============================================================================
-
 class AdministratorTab(AppView):
     #top_box holds all sections
     top_vbox = None
     monitor = None
     configuration = None
     ctrl_be = None
-    
+
+    #---------------------------------------------------------------------------
     def __init__(self, server_instance_settings):
         AppView.__init__(self, False, "admin", True)
-        
+
         server_profile = ServerProfile(server_instance_settings, False)
 
         self.ctrl_be = wb_admin_control.WbAdminControl(server_profile, 
@@ -110,15 +75,12 @@ class AdministratorTab(AppView):
         self.on_close(wb_admin_utils.weakcb(self, "handle_on_close"))
 
         # Create sections and add them to the admin page.
-        self.monitor = wb_admin_monitor.WbAdminMonitor(server_profile, self.ctrl_be)
-        section = self.add_section("Server Status", True, True)
-        section.set_content(self.monitor)
-
-        self.configuration = wb_admin_configuration.WbAdminConfiguration(server_profile, self.ctrl_be, self.monitor)
-        section = self.add_section("Configuration", False, False)
-        section.set_content(self.configuration)
-
+        self.configuration = wb_admin_main.WbAdminMainView(server_profile, self.ctrl_be, self.monitor)
+        self.add(self.configuration, True, True)
+        #section = self.add_section("Configuration", False, False)
+        #section.set_content(self.configuration)
 
+    #---------------------------------------------------------------------------
     def handle_on_close(self):
         App.get().set_status_text("Closing Administator.")
         self.configuration.shutdown()
@@ -126,14 +88,14 @@ class AdministratorTab(AppView):
         tab_references.remove(self)
         return True
 
-
+    #---------------------------------------------------------------------------
     def wait_server_check(self, timeout):
         # wait for the server status check to be performed until the given timeout
         t = time.time()
-        while self.configuration.last_is_running_check is None and time.time() - t < timeout:
+        while self.ctrl_be.is_server_running() and time.time() - t < timeout:
             time.sleep(0.5)
 
-    
+    #---------------------------------------------------------------------------
     def add_section(self, label_text, is_content_expandable, header_mode):
         section = newSectionBox(is_content_expandable, label_text, header_mode)
         section.set_name(label_text)
@@ -141,7 +103,7 @@ class AdministratorTab(AppView):
         return section
 
 
-
+#-------------------------------------------------------------------------------
 def do_open_administrator(server_instance):
     validate_setting(server_instance.serverInfo, "sys.usesudo", norm_to_switch, None)#"Server profile has no indication of sudo usage")
     validate_setting(server_instance.serverInfo, "sys.usesudostatus", norm_to_switch, None)
@@ -267,6 +229,7 @@ def selectServer(title):
     
     listbox = mforms.newListBox(False)
     box.add(listbox, True, True)
+    listbox.show();
 
     for inst in grt.root.wb.rdbmsMgmt.storedInstances:
       listbox.add_item(inst.name)
