@@ -114,7 +114,7 @@ struct = { 'Advanced': { 'General': { 'default_week_format': 'server',
                'Networking': { 'enable-named-pipe': 'server',
                                'port': 'server',
                                'skip-networking': 'security'}},
-  'InnoDB Parameters': { 'Activate InnoDB': { 'innodb': 'innodb'
+  'InnoDB': { 'Activate InnoDB': { 'innodb': 'innodb'
                                             },
                          'Datafiles': { 'innodb_data_file_path': 'innodb',
                                         'innodb_data_home_dir': 'innodb',
@@ -215,7 +215,7 @@ struct = { 'Advanced': { 'General': { 'default_week_format': 'server',
                                              'long_query_time': 'server',
                                              'slow-query-log': 'server',
                                              'slow_query_log_file': 'server'}},
-  'MyISAM Parameters': { 'Advanced Settings': { 'myisam-block-size': 'myisam',
+  'MyISAM': { 'Advanced Settings': { 'myisam-block-size': 'myisam',
                                                 'myisam-recover': 'myisam',
                                                 'myisam_data_pointer_size': 'myisam',
                                                 'myisam_max_extra_sort_file_size': 'myisam',
@@ -238,7 +238,7 @@ struct = { 'Advanced': { 'General': { 'default_week_format': 'server',
                                       'key_buffer_size': 'myisam',
                                       'key_cache_age_threshold': 'server',
                                       'skip-locking': 'server'}},
-  'NDB Parameters': { 'General': { 'ndb-batch-size': 'ndb',
+  'NDB': { 'General': { 'ndb-batch-size': 'ndb',
                                    'ndb-cluster-connection-pool': 'ndb',
                                    'ndb-connectstring': 'ndb',
                                    'ndb-log-update-as-write': 'ndb',
