@@ -19,7 +19,7 @@
 
 #import "WbModelStoredNoteEditor.h"
 #import "MCPPUtilities.h"
-#import "string_utilities.h"
+#include "base/string_utilities.h"
 
 @implementation StoredNoteEditor
 
